#include <rw/rwtime.h>
#include <rw/rwdate.h>

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>



#include "abng_core_management.hh"
#include "bootstrap_n.hh"

#include "whlib.hh"

char *defaultview = "viewroom";  
int running = 1;

class abtrace_args{
public:
  abtrace_args();
  ~abtrace_args();

  const char *getName() const { return username; };
  const char *getView() const { return viewname; };
  int getVerbose() const { return verbose; };
  int display(const char* prop) const;
  void setName(const char *newstr);
  void setView(const char *newstr);
  void setVerbose(int varg) { verbose = varg; };
protected:
  char *viewname;
  char *username;
  int verbose;
};

abtrace_args::abtrace_args() 
{
  viewname = username = NULL;
  verbose = 0;
};

abtrace_args::~abtrace_args()
{
  if (viewname)
    free(viewname);
  if (username)
    free(username);
}

void abtrace_args::setName(const char *newstr)
{
  if (username) 
    free(username);
  username=(newstr) ? strdup(newstr) : NULL;
}

void abtrace_args::setView(const char *newstr)
{
  if (viewname)
    free(viewname);
  viewname=(newstr) ? strdup(newstr) : NULL;
}

void usage( char *prog_name) {
  cout << prog_name << endl
       << "\t[-a] [-v view_name] username" << endl;
}

void my_parseargs(int argc, char *argv[], abtrace_args *abtargs)
{
  int c;
  char *prgname = argv[0];
  abtargs->setView(defaultview);
  abtargs->setName(NULL);

  while (( c = getopt( argc, argv, "av:")) != EOF)
  {
    switch (c)
    {
      case 'v':
	abtargs->setName(strdup(optarg));
	break;
      case 'a':
	abtargs->setVerbose(1);
	break;
      default:
	usage(prgname);
	exit(1);
    }
  }

  if ( optind < argc )
    abtargs->setName(argv[optind]);
  else
  {
    usage(prgname);
    exit(2);
  }
}

void notify(whereis_result *result)
{
  const char *str;
  //cout << "notify in abtrace" << endl;
   
  str=result->getLastSeenStr();
  cout << "Last seen:\t" << (str ? str : "(null)\n");
  cout << "Quality:\t" << result->getQuality() << endl;
  str=result->getLocType();
  cout << "At " << (str ? str : "(null)") << ":\t(";
  str=result->getRoomNum();
  cout << (str ? str : "(null)") << ") ";
  str=result->getRoomName();
  cout << (str ? str : "(null)") << endl; 
}

void sig_handler(int signr)
{
  running=0; // wymusza wyjscie
//  cerr << "program shuting down" << endl;
}


int main(int argc, char *argv[])
{
  whereis_result *result;
  abtrace_args *abtargs = new abtrace_args();

  my_parseargs(argc, argv, abtargs);

  whereis_tools *wtools = new whereis_tools(argc, argv);

  signal(SIGHUP, sig_handler);
  signal(SIGINT, sig_handler);
  signal(SIGTERM, sig_handler);

  wtools->connect(abtargs->getView());
  if ( (result = wtools->traceUser(abtargs->getName(), notify, 
				   abtargs->getVerbose() ? WH_FU_VERBOSE : WH_FU_NULL )) )
  {
    const char *str;
    str=result->getName();
    cout << "uid:\t\t" << (str ? str : "(null)") << endl;
    str=result->getForeName();
    cout << "Name:\t\t" << (str ? str : "(null)") << endl;
    str=result->getSurName();
    cout << "SurName:\t" << (str ? str : "(null)") << endl;
    str=result->getPosition();
    cout << "Position:\t" << (str ? str : "(null)") << endl;

    str=result->getLastSeenStr();
    cout << "Last seen:\t" << (str ? str : "(null)\n");
    cout << "Quality:\t" << result->getQuality() << endl;
    str=result->getLocType();
    cout << "At " << (str ? str : "(null)") << ":\t(";
    str=result->getRoomNum();
    cout << (str ? str : "(null)") << ") ";
    str=result->getRoomName();
    cout << (str ? str : "(null)") << endl;  
    if (result->getAway())
      cout << "Message:\t" << result->getAway() << endl; 
    cout << "People in room:\t" << result->getRoomPeople() << endl; 
    for (int i=0; i < result->getRoomPeople()-1; i++)
    {
      const whereis_result *result2;
      if ( (result2=result->getRoomPeople(i)) )
      {
	str=result2->getName();
	cout << "\tuid:\t\t" << (str ? str : "(null)") << endl;  
	str=result2->getForeName();
	cout << "\tName:\t\t" << (str ? str : "(null)") << endl;  
	str=result2->getSurName();
	cout << "\tSurName:\t" << (str ? str : "(null)") << endl;  
	str=result2->getPosition();
	cout << "\tPosition:\t" << (str ? str : "(null)") << endl;  
      }
      cout << endl;
    }
  }
 
  while (running && result)
  {
    //cerr << result->getLastSeenStr();
    //cerr << ".";
    sleep (1);
  }

  if (result)
    delete result;
  wtools->disconnect();
  delete wtools;
  delete abtargs;

  //cerr << "exit soon" << endl;
  exit(0);
}


