/*
 * (c) 1998 Dariusz Zbik <zbik@dwd.interkom.pl>
 */

#include  "whlib.hh"
#include <unistd.h>


whereis_result::whereis_result()
{
  viewname = username    = forename = surname  = NULL;
  position = lastseenStr = location = roomname = NULL;
  roomnum  = away        = NULL;
  roomlist = NULL;
  view = NULL;
  whobs = NULL;
  loct_descr_item_id=0;
  trace = 0;
  wht = NULL;
}

whereis_result::~whereis_result ()
{
  if (viewname) free (viewname);
  if (username) free (username);
  if (forename) free (forename);
  if (surname)  free (surname);
  if (position) free (position);
  if (lastseenStr) free (lastseenStr);
  if (location) free (location);
  if (roomname) free (roomname);
  if (roomnum)  free (roomnum);
  if (away)     free (away);
  if (trace)
    wht->detach_it(this);
  freeroomlist();
}

void whereis_result::freeroomlist()
{
  if (roomlist)
  {
    int i=0;
    while (roomlist[i])
      delete roomlist[i];

    delete[] roomlist;
    roomlist=NULL;
  }  
}

void whereis_result::setView(const char *viewname_arg)
{
  if (viewname)
    free(viewname);
  viewname = (viewname_arg) ? strdup(viewname_arg) : NULL;
}

void whereis_result::setName(const char *name)
{
  if (username)
    free(username);
  username = (name) ? strdup(name) : NULL;
}

void whereis_result::setForeName(const char *fore)
{
  if (forename)
    free(forename);
  forename = (fore) ? strdup(fore) : NULL;
}

void whereis_result::setSurName(const char *sur)
{
  if (surname)
    free(surname);
  surname = (sur) ? strdup(sur) : NULL;
}

void whereis_result::setPosition(const char *pos)
{
  if (position)
    free(position);
  position = (pos) ? strdup(pos) : NULL;
}

void whereis_result::setAway(const char *aw)
{
  if (away)
    free(away);
  away = (aw) ? strdup(aw) : NULL;
}

void whereis_result::setLocType(const char *ltype)
{
  if (location)
    free(location);
  location = (ltype) ? strdup(ltype) : NULL;
}

void whereis_result::setRoomNum(const char *num)
{
  if (roomnum)
    free(roomnum);
  roomnum = (num) ? strdup(num) : NULL;
}

void whereis_result::setRoomName(const char *rname)
{
  if (roomname)
    free(roomname);
  roomname = (rname) ? strdup(rname) : NULL;
}

void whereis_result::setLastSeen(const time_t seen)
{
  if (lastseenStr)
    free(lastseenStr);
  lastseen = seen;
  lastseenStr = strdup(ctime(&seen));
}

void whereis_result::setQuality(short q)
{
  quality=q;
}

void whereis_result::setRoomPeople(short num)
{
  freeroomlist();

  if ( num>0 )
  {
    roomlist = new whereis_result*[num];
    for (int i=0; i< num; i++)
      roomlist[i] = NULL;
  }
  roompeople=num;
}

void whereis_result::setRoomPeople(short idx, class whereis_result *res)
{
  if ( idx >=0 && idx < roompeople )
    roomlist[idx] = res; 
}

int whereis_tools::attach_it(whereis_result *res,
			     void (*notify)(whereis_result *),
			     int flags)
{ 
  if (res->getID() == 0)
    return 1; // error;

  detach_it(res); // najperw usun stare powiazanie
  
  if ( ! (res->whobs = new whereis_observer(res, notify,
					    (flags & WH_FU_VERBOSE) ? 1 : 0)) )
    return 3;

  Domain_Repository::Repository_Item_Ids observed_id_list(1);
  observed_id_list.length(1);
  observed_id_list[0]=res->loct_descr_item_id;  

  //cerr << "attempt to observe " << res->loct_descr_item_id << endl;
  
  try {
    //cerr << " connecting ... " << endl;
    //cerr << "view name... " << view->view_name() << endl;
    view->attach_locatable_dynamic_data_observer ( res->whobs, observed_id_list );
  } catch( Domain_Repository :: Unknown_Repository_Item_Ids ) {
    cerr << "unknown error " << endl;
    return 2;
  } 
  
  res->trace = 1;
  res->wht = this;
  return 0; // no error
}

void whereis_tools::detach_it(whereis_result *res)
{
  if (res->loct_descr_item_id != 0 && res->whobs)
  {
    Domain_Repository::Repository_Item_Ids observed_id_list(1);
    observed_id_list.length(1);
    observed_id_list[0]=res->loct_descr_item_id;
 
    try {
      view->detach_locatable_dynamic_data_observer(res->whobs, observed_id_list);
    }
    catch (...) { 
      cerr << "unknown error " << endl;
    }
  }

  res->trace = 0;

  sleep(2);
  if ( res->whobs ) 
    delete res->whobs;
  res->whobs = NULL;
}

whereis_tools::whereis_tools(int argc, char *argv[])
{
  orb= CORBA::ORB_init( argc, argv, ORB_IDENTIFIER);
  boa= orb->BOA_init( argc, argv, BOA_IDENTIFIER);
  orb->setDiagnostics(0);
  Bootstrap::init(orb);
  state = WH_ST_NULL;
}

whereis_tools::~whereis_tools()
{
  disconnect();
}

int whereis_tools::connect(const char *viewname)
{
  disconnect();

  CORBA::String_var corbaviewname = CORBA::string_dup(viewname);
  CORBA::Object_var tmp_obj(NULL);
  ABng_Core_Management::View_Manager_var view_manager;

  try {
    tmp_obj= Bootstrap::resolve( 
	     Bootstrap::CTX_ABng_Core_Management_View_Managers()+
	     corbaviewname);
    view_manager= ABng_Core_Management::View_Manager::_narrow( tmp_obj);
    view= view_manager->my_view();
  }
  catch ( CORBA::Exception &ex) {
    cerr << "Cannot bind to view/view manager" << endl;
    cerr << ex << endl;
    return 0;
  }

  try {
    tmp_obj= Bootstrap::resolve( Bootstrap::SRV_Away_Repository());
    away_rep = ABng_Core::Away_Rep::_narrow( tmp_obj);
  }
  catch ( CORBA::Exception &ex) {
//    cerr << "Cannot bind to away repository" << endl;
//    cerr << ex << endl;
//    return 0;
  }

  state |= WH_ST_CONNECT;
  return 1;
}

void whereis_tools::disconnect()
{
  if ( state & WH_ST_CONNECT )
  {
    away_rep=NULL;
    view=NULL;
  }

  state ^= WH_ST_CONNECT;
}

struct trace_body_struct
{
  CORBA::BOA_ptr boa;
  class whereis_result *ret;
};

void trace_body(void *arg)
{
  struct trace_body_struct *tbs = (struct trace_body_struct *)arg;

  while (tbs->ret->getTrace())
  {
#ifdef ORBIX
  tbs->boa->processEvents(1000);
#elif defined OMNIORB
  tbs->impl_is_ready();
#endif
  }
  cerr << "thread going down" << endl;
  delete tbs;
}

class whereis_result *whereis_tools::traceUser(const char *userid, 
					       void (*notify)(whereis_result *),
					       int flags)
{
  struct trace_body_struct *tbs = new struct trace_body_struct;
  tbs->boa = boa;

  if ( (tbs->ret = findUser(userid, WH_FU_NULL)) )
  {
    if (attach_it(tbs->ret, notify, flags))
    {
      cerr << "unable to attach " << endl;
      delete tbs->ret;
      delete tbs;
      return NULL;
    }
    
    (new omni_thread( trace_body, tbs))->start();
  }
  return tbs->ret;
}

class whereis_result *whereis_tools::findUser(
      Domain_Repository::Repository_Item_Id usernr,
      const char *userid,
      int flags,
      whereis_result *ret)
{
  CORBA::String_var str;

  Domain_Repository::User::Pattern user_pattern;

  ABng_Core::Locatables_var loct_list;
  ABng_Core::Locatable::Pattern loct_pattern;
  
  ABng_Core::Locations_var loc_list;
  ABng_Core::Location::Pattern loc_pattern;
  ABng_Core::Location::Pattern lpattern_empty;

  lpattern_empty.is_any = 1;
  lpattern_empty.item_id_pattern.is_any = 1;
  lpattern_empty.loc_descr_pattern.is_any = 1;  
  lpattern_empty.loc_descr_pattern.item_id_pattern.is_any = 1;
  lpattern_empty.loc_descr_pattern.type_name_pattern.is_any = 1;
  lpattern_empty.loc_descr_pattern.property_list_pattern.is_any = 1;
  lpattern_empty.loc_descr_pattern.property_list_pattern.constraints =
    Domain_Repository::UNRESTRICTED;
  
  ABng_Core::Sighting_Description sd;
  ABng_Core::Sighting_Description *sd_ptr;
  ABng_Core::Sighting_Descriptions *sd_list;
  
  user_pattern.is_any = 0;
  user_pattern.item_id_pattern.is_any = 1;
  user_pattern.uid_pattern.is_any = 0;
  user_pattern.uid_pattern.value = (const char *) userid;
  user_pattern.forename_pattern.is_any = 1;
  user_pattern.surname_pattern.is_any = 1;
  user_pattern.position_pattern.is_any = 1;
  user_pattern.email_pattern.is_any = 1;
  user_pattern.phone_pattern.is_any = 1;
  
  loct_pattern.is_any = 0;
  loct_pattern.item_id_pattern.is_any = 1;
  loct_pattern.item_id_pattern.value = usernr;
  loct_pattern.home_location_pattern.is_any = 1;
  loct_pattern.home_location_pattern = lpattern_empty;
  loct_pattern.type_specific_object_pattern.is_any = 1;
  loct_pattern.type_specific_object_pattern.object_pattern.user_obj_pattern(user_pattern);
  loct_pattern.assigned_pattern.is_any = 1;
  loct_pattern.visitor_pattern.is_any = 1;

  if ( userid )
    loct_pattern.type_specific_object_pattern.is_any = 0;
  else
    loct_pattern.item_id_pattern.is_any = 0;

  loct_list = view->find_locatable(loct_pattern);

  if (loct_list->length() <= 0)
    return NULL; // user not found

  class whereis_result *ret_free=NULL;
  if (!ret)
    ret = ret_free = new whereis_result();
  ret->setView(str=view->view_name());
  ret->view = view;

  if (flags & WH_FU_AWAY && !CORBA::is_nil(away_rep))
  {
    CORBA::String_var str2;
    ABng_Core::Away::Pattern away_pattern;
    ABng_Core::Aways_var away_list;
    ABng_Core::Away_var away;

    away_pattern.is_any = 0;
    away_pattern.item_id_pattern.is_any = 1;
    away_pattern.type_specific_object_pattern.is_any = 0;
    away_pattern.type_specific_object_pattern.object_pattern.user_obj_pattern(user_pattern);
    away_pattern.start_time_pattern.is_any = 1;
    away_pattern.stop_time_pattern.is_any = 1;
    away_pattern.cancel_from_time_pattern.is_any = 1;
    away_pattern.message_pattern.is_any = 1;
    away_pattern.contact_pattern.is_any = 1;
    away_pattern.cancel_by_badge_pattern.is_any = 1;
    away_pattern.active_pattern.is_any = 1;

    away_list=away_rep->find(away_pattern);

    for (int i=0; i <away_list->length(); i++)
    {
      away=away_list[i];
      str=away->message();
      ret->setAway(str);
    } 
  }

  ABng_Core::Locatable::Description_var desc;
  desc = loct_list[0]->descr();

  ret->loct_descr_item_id=desc->item_id;
  ret->setName(str=desc->type_specific_object.user_obj()->uid());
  ret->setForeName(str=desc->type_specific_object.user_obj()->forename());
  ret->setSurName(str=desc->type_specific_object.user_obj()->surname());
  ret->setPosition(str=desc->type_specific_object.user_obj()->position());

  sd_ptr=loct_list[0]->get_sighting();

  ret->setQuality(sd_ptr->q);
  ret->setLastSeen((time_t)(sd_ptr->update_time.seconds));

  loc_pattern.is_any = 0;
  loc_pattern.item_id_pattern.is_any = 0;	
  loc_pattern.item_id_pattern.value = sd_ptr->my_location;
  ret->current_location = sd_ptr->my_location;
  loc_pattern.loc_descr_pattern.is_any = 1;
  loc_list = view->find_location(loc_pattern);
  
  if ( loc_list->length()>0 )
  {
    ret->setLocType(str=loc_list[0]->loc_descr()->type_name());
    
    Domain_Repository::Location_Description_var ldesc;

    Domain_Repository::Properties_var prop;
      
    ldesc=loc_list[0]->loc_descr();
    prop = ldesc->property_list();
    
    
    for (int l=0; l < prop->length(); l++)
    {
      if ( !strcmp((prop)[l].name, PROP_NUMBER) )
	ret->setRoomNum(str=to_string((prop)[l].value));
      
      if ( !strcmp((prop)[l].name, PROP_DESCR) )
	ret->setRoomName(str=to_string((prop)[l].value));
    }

    Domain_Repository::Time tm;
    tm.seconds = time(NULL)-TIME_PERIOD;
    tm.microseconds = 0;
    sd_list = loc_list[0]->get_sightings(tm);

    int people=0;

    for (int i=0; i < sd_list->length(); i++)
    {
      sd=(*sd_list)[i];
      if ( sd.my_locatable_type == ABng_Core::LOCATABLE_USER )
	people++;
    }
     
    ret->setRoomPeople(people);

    if ( (flags & WH_FU_NEIGHB) && people)
    {
      whereis_result *ret2;
      int idx=0;
      for (int i=0; i < (*sd_list).length(); i++)
	if ( (*sd_list)[i].my_locatable !=
	     desc->item_id )
	{
	  if ( (ret2=this->findUser((*sd_list)[i].my_locatable,
				    NULL,
				    WH_FU_NULL)) )
	    ret->setRoomPeople(idx++, ret2); 
	}
    }
  }
  return ret;
}

char *whereis_tools::to_string(CORBA::Any &any_value)
{
  char *ret;
  switch ( any_value.type()->kind())
  {
    case CORBA::tk_string:
      {
#ifdef OMNIORB
	char *str= NULL;
	CORBA::Any::to_string ts( str, any_value.type()->length());
	any_value >>= ts;
	if (str)
	{
	  ret = strdup(str);
	  CORBA::string_free( str);
	}
#else
	char *str=NULL;
	any_value >>= str;
	ret = strdup(str);
#endif
      }
      break;

    default:
      ret = strdup("Unsupported type code kind");
      break;
  }
  return ret;
}


/* fragment do wywolan z jezyka C (bez plusow) */

whlib_handle whereis_init(int argc, char *argv[])
{
  class whereis_tools *whobj;
  whobj = new whereis_tools(argc, argv);
  return whobj;
}

void whereis_destroy(whlib_handle wh)
{
  delete ((class whereis_tools*) wh);
}

int whereis_connect(whlib_handle wh, const char *viewname)
{
  return ((class whereis_tools*) wh)->connect(viewname);
}

void whereis_disconnect(whlib_handle wh)
{
  ((class whereis_tools*) wh)->disconnect();
}

whlib_result whereis_findUser(whlib_handle wh,
			      const char *userid)
{
  class whereis_result *wh_res;  
  wh_res = ((class whereis_tools*) wh)->findUser(userid);
  return wh_res;
}

whlib_result whereis_findUserFL(whlib_handle wh,
				const char *userid,
				int flags)
{
  class whereis_result *wh_res;  
  wh_res = ((class whereis_tools*) wh)->findUser(userid, flags);
  return wh_res;
}

void whereis_free_result(whlib_result wr)
{
  if (wr)
    delete ((class whereis_result*) wr);
}


const char *get_Name(whlib_result wr)
{
  return ((class whereis_result*) wr)->getName();
}

const char *get_ForeName(whlib_result wr)
{
  return ((class whereis_result*) wr)->getForeName();
}

const char *get_SurName(whlib_result wr)
{
  return ((class whereis_result*) wr)->getSurName();
}

const char *get_Away(whlib_result wr)
{
  return ((class whereis_result*) wr)->getAway();
}

const char *get_LastSeenStr(whlib_result wr)
{
  return ((class whereis_result*) wr)->getLastSeenStr();
}

const char *get_RoomName(whlib_result wr)
{
  return ((class whereis_result*) wr)->getRoomName();
}

const char *get_RoomNum(whlib_result wr)
{
  return ((class whereis_result*) wr)->getRoomNum();
}
