package jabba;
// klasy potrzebne do odbierania informacji o przemieszczaniu sie
// osoby (przedmiotu) obserwowanej
// po ludzku: odbieranie sightingow
import ABng_Core._Dynamic_Data_ObserverOperations;
import ABng_Core._tie_Dynamic_Data_Observer;
import ABng_Core.Sighting_Description;
import ABng_Core.View;
import ABng_Core._View_Data_ObserverOperations;
import ABng_Core._tie_View_Data_Observer;
import ABng_Core.Locatable;
import ABng_Core.Locatable_Type_Specific_Object;
import ABng_Core.Location;

import ABng_Core.View_Data_ObserverPackage.View_Data_Object; 

import Domain_Repository.User;
import Domain_Repository.Equipment;
import Domain_Repository.Destination_Unreachable;
import Domain_Repository.Unknown_Repository_Item_Ids;
import Domain_Repository.Unknown_Observer;
import Domain_Repository.Unmatched_Observer_Item_Ids;
import Domain_Repository.Location_Description;
import Domain_Repository.Property;
import Domain_Repository.Search_Not_Implemented;
import Domain_Repository.Change_Type;
import Domain_Repository.User_Rep;
import Domain_Repository.Equipment_Rep;
import Domain_Repository.Location_Description_Rep;
import Domain_Repository._Static_Data_ObserverOperations;
import Domain_Repository._tie_Static_Data_Observer;

import Domain_Repository.Static_Data_ObserverPackage.Static_Data_Object;

import org.omg.CORBA.portable.InputStream;

// tutaj bedziemy przechowywac obserwowane rzeczy
import java.util.Vector;
import java.util.Date;
import java.util.Locale;
import java.util.Calendar;

import java.text.DateFormat;

// watek postarzania bedzie odpalany co iles-tam sekund
import com.sun.java.swing.Timer;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import CosTypedEventComm._TypedPushConsumerImplBase;

import pl.edu.agh.ics.abngcore.RepTools;


/**
 * Receives all the messages that are sent to the JABBA application.
 */
class Receiver 
implements _Dynamic_Data_ObserverOperations, _View_Data_ObserverOperations, _Static_Data_ObserverOperations {
  // klasa zapewnia segregacje ludzi i sprzetu wewnatrz SYNCHRONIZOWANYCH list
  Vector peopleObserved;
  Vector equipmentObserved;
  Vector locationsObserved;


  // do odswiezania zawartosci modeli w tle
  //  ModelUpdater modelUpdater;

  /**
   * What type of messages do you want to receive?
   */
  int messagesSent;
  int singlePersonIndex;
  int singleLocationIndex;

  /** User-indexed messages. */
  public static final int USER_MESSAGES = 1;

  /** Location-indexed messages. */
  public static final int LOCATION_MESSAGES = 2;

  /** Single-user messages. */
  public static final int SINGLE_USER_MESSAGES = 8;

  /** Single-location messages. */
  public static final int SINGLE_LOCATION_MESSAGES = 16;

  /** Equipment messages. */
  public static final int EQUIPMENT_MESSAGES = 4;

  private static final int USER_REP = 0;
  private static final int EQUIPMENT_REP = 1;
  private static final int LOCATION_DESCRIPTION_REP = 2;

  /**
   * Real receivers (TIE approach).
   */
  _tie_Dynamic_Data_Observer sightingReceiver;
  _tie_View_Data_Observer viewMessagesReceiver;
  _tie_Static_Data_Observer staticMessagesReceiver;

  /**
   * An object to send messages to.
   */
  JabbaDataModel dataModel;

  /** Observed ABng_Core.View */
  View view;

  /** The user repository reference. */
  User_Rep user_rep;

  /** The equipment repository reference. */
  Equipment_Rep equipment_rep;

  /** The location description repository reference. */
  Location_Description_Rep location_description_rep;

  /** The main program object */
  Jabba jabba;

  /** Used to check if someone has left */
  private Timer timer;

  /** Indicates whether the event channel is present or not. */
  boolean eventChannelPresent;


  /**
   * Who am i?
   */
  public java.lang.Object _deref() {
    jabba.console.messageFrom( this, "_deref() called" );
    return this;
  }

  /**
   * Sets up a new data model.
   */
  public void registerDataModel( JabbaDataModel jdm, int messages ) {
    registerDataModel( jdm,messages,-1 );
  }

  public void registerDataModel( JabbaDataModel jdm, int messages, int index ){
    jabba.console.messageFrom( this, "registerDataModel() called." );
    synchronized( jdm ) {
      dataModel = jdm;
      messagesSent = messages;
      if( (messages & SINGLE_USER_MESSAGES) != 0 ) singlePersonIndex = index;
      if( (messages & SINGLE_LOCATION_MESSAGES) != 0 ) singleLocationIndex = index;
    }
  }

  /**
   * Switches to a previously registered data model.
   */
  public synchronized void updateDataModel( JabbaDataModel jdm, int messages ) {
    updateDataModel( jdm,messages,-1 );
  }

  public synchronized void updateDataModel( JabbaDataModel jdm, int messages, int index ) {
    jabba.console.messageFrom( this, "updateDataModel() called." );
    dataModel = jdm;
    messagesSent = messages;
    if( (messages & SINGLE_USER_MESSAGES) != 0 ) singlePersonIndex = index;
    if( (messages & SINGLE_LOCATION_MESSAGES) != 0 ) singleLocationIndex = index;
  }

  /**
   * Implementation of a method declared in _Dynamic_Data_ObserverOperations.
   * @param sd incoming sighting description message
   */
  public void receive_sighting( Sighting_Description sd ) {
    int index=-1;
    Sighting_Description old_sighting;
    Object pom=null;
    LocationRecord lr;

    jabba.console.messageFrom( this,"Received sighting: locatable "+sd.my_locatable );

    for( int i=0; i<peopleObserved.size(); i++ )
      if( userLocatableId( pom=peopleObserved.elementAt(i) ) == sd.my_locatable ) {
	index=i;
	jabba.console.messageFrom( this, "User sighting." );
	break;
      }
    // sighting dotyczy uzytkownika
    if( index>=0 ) {
      // zamiana starego sightingu na nowy (UserRecord)
      ((UserRecord)pom).setSighting(sd);
    }
    else { // nie znaleziono rekordu uzytkownika o podanym id
      for( int i=0; i<equipmentObserved.size(); i++ )
	if( equipmentLocatableId( pom=equipmentObserved.elementAt(i) ) == sd.my_locatable ) {
	  index=i;
	  jabba.console.messageFrom( this, "Equipment sighting." );
	  break;
	}
      if( index>=0 ) {
	((EquipmentRecord)pom).setSighting(sd);
      }
      else {
	jabba.console.messageFrom( this, "Unknown locatable id (receive_sighting)." );
	return;
      }
    }
  }


  public void receive_sightings( Sighting_Description[] sds) {
    for( int i=0; i<sds.length; i++ ) receive_sighting( sds[i] );
  }


  public void abort() {
    jabba.console.messageFrom( this, "abort() called" );
  }


  /**
   * Constructs a new Receiver object.
   * CAUTION! View reference must not be null!
   * @param jabba main program object handle
   * @param view  view, the messages to be received from 
   * @see #receive_sighting
   */
  Receiver( Jabba jabba, View view, User_Rep urep, Equipment_Rep eqrep, Location_Description_Rep ldrep, boolean evtChPresent ) {
    synchronized( this ) {
      this.jabba = jabba;
      jabba.console.addWriter( this, "Receiver" );
      eventChannelPresent = evtChPresent;
      attachToServers( view,urep,eqrep,ldrep );
      jabba.console.messageFrom( this,"Got "+peopleObserved.size()+" users." );
      // modelUpdater = new ModelUpdater();
      // modelUpdater.setPriority( Thread.MIN_PRIORITY );
      // modelUpdater.start();
    }
    
    timer = new Timer( Jabba.CHECK_FOR_ABSENCES_MILLIS, new ActionListener(){
      public void actionPerformed( ActionEvent ev ){
	UserRecord ur;
	EquipmentRecord eq;

// NASZE
dataModel.maskChanged();
// NASZE

for (int i=0; i<peopleObserved.size(); i++ ) {
	  ur = (UserRecord)peopleObserved.elementAt(i);

//// NASZE
ur.mask = ur.setMask();
/// NASZE

	  if( ur.present )
	    if( !ur.checkPresence() ) {
	      ur.location_record.decreasePresentNumber(ur.index);
	      if( (messagesSent & USER_MESSAGES) != 0 ) {
		dataModel.user_qualityChanged( ur.index );
		// TU		dataModel.user_timeChanged( ur.index );
	      }
	      if( (messagesSent & SINGLE_USER_MESSAGES) != 0 ) {
		if( singlePersonIndex==ur.index ) {
		  dataModel.single_user_qualityChanged( ur.index );
		}
	      }
	    }
	}	      
	for( int i=0; i<equipmentObserved.size(); i++ ) {
	  eq = (EquipmentRecord)equipmentObserved.elementAt(i);

/// NASZE
eq.mask = eq.setMask();
//// NASZE

	  if( eq.present ) {
	    if( !eq.checkPresence() ) {
	      if( (messagesSent & EQUIPMENT_MESSAGES) != 0 ) {
		dataModel.equipment_qualityChanged( eq.index );
		// TU		dataModel.equipment_timeChanged( eq.index );
	      }
	    }
	  }
	}
      }
    } );
    timer.start();
    
    System.err.println( "Timer started." );
    // impl_is_ready();
  }

  void impl_is_ready() {
    jabba.orb_wrapper.impl_is_ready();
  }


  /** 
   * Attaches to the specified View, User Repository, Equipment Repository and Location Description Repository. Automatically detaches from an old one.
   * @param v view to be observed
   */
  void attachToServers( View v, User_Rep urep, Equipment_Rep eqrep, Location_Description_Rep ldrep ) {
    jabba.console.messageFrom( this, "attachToView() called" );
    if( view != null ) detachFromView();
    view = v;
    user_rep = urep;
    equipment_rep = eqrep; 
    location_description_rep = ldrep;

    ABng_Core.LocatablePackage.Value_Description[] ldsc=null;
    ABng_Core.LocatablePackage.Pattern lptn=null;

    try {
      lptn = RepTools.get_locatable_pattern();
      ldsc = view.find_locatable_values(lptn);
    }
    catch( Destination_Unreachable du ) {
      jabba.console.exception( this,du );
    }
    catch( Search_Not_Implemented sni ) {
      jabba.console.exception( this,sni );
    }

    peopleObserved = new Vector();
    equipmentObserved = new Vector();
    
    ABng_Core.Locatable_Type_Specific_Object_Value tso;
    Domain_Repository.UserPackage.Description uDescr=null;
    Domain_Repository.EquipmentPackage.Value_Description eqDescr=null;

    if( ldsc!=null )
      for( int i=0,uCounter=0,eqCounter=0; i<ldsc.length; i++ ) {
	tso = ldsc[i].type_specific_object_value;
	if( tso.is_user_description() ) {
	  if( (uDescr = tso.user_description())!=null )
	    peopleObserved.addElement( new UserRecord( ldsc[i].item_id,uDescr,uCounter++ ) );
	}
	else if( tso.is_equipment_description() ) {
	  if( (eqDescr = tso.equipment_description())!=null )
	    equipmentObserved.addElement( new EquipmentRecord( ldsc[i].item_id,eqDescr,eqCounter++ ) );
	}
      }

    // Przetworzenie ostatnich sightingow
    Sighting_Description[] sdsc=view.get_sightings( new Domain_Repository.Time() );
    for( int i=0; i<sdsc.length; i++ )
      receive_sighting( sdsc[i] );

    // Koniec inicjalizacji uzytkownikow...
    for( int i=0; i<peopleObserved.size(); i++ ) 
      ((UserRecord)(peopleObserved.elementAt(i))).checkPresence();

    // ... i sprzetu
    for( int i=0; i<equipmentObserved.size(); i++ ) 
      ((EquipmentRecord)(equipmentObserved.elementAt(i))).checkPresence();

    jabba.console.messageFrom( this, "Got "+peopleObserved.size()+" users and "+equipmentObserved.size()+" pieces of equipment." );

    // teraz lokacje
    ABng_Core.LocationPackage.Value_Description[] ds=null;
    ABng_Core.LocationPackage.Pattern lpatt=null;

    try {
      lpatt = RepTools.get_location_pattern();
      ds = view.find_location_values(lpatt);
    } 
    catch( Destination_Unreachable du ) {
      jabba.console.exception( this, du );
    }
    catch( Search_Not_Implemented sni ) {
      jabba.console.exception( this, sni );
    }

    locationsObserved = new Vector();

    if( ds != null )
      for( int i=0,j=0; i<ds.length; i++ ) {
	if( ds[i]!=null )
	  locationsObserved.addElement( new LocationRecord( ds[i],j++ ) );
      }

    connectUsersToLocations( peopleObserved, locationsObserved );
    connectEquipmentToLocations( equipmentObserved, locationsObserved );

    jabba.console.messageFrom( this, "Got "+locationsObserved.size()+" locations." );

    jabba.orb_wrapper.connect_object( 
      sightingReceiver = new _tie_Dynamic_Data_Observer( this ) );

    jabba.orb_wrapper.connect_object(
      viewMessagesReceiver = new _tie_View_Data_Observer( this ) );

    jabba.orb_wrapper.connect_object(
      staticMessagesReceiver = new _tie_Static_Data_Observer( this ) );

    jabba.console.messageFrom( this,"Connected to the ORB." );

    
    if( !eventChannelPresent ) {
      // obserwujemy wszystkich uzytkownikow ...
      int[] list1loc = listAvailableUsersLocatableIds();
      int[] list1rep = listAvailableUsersRepositoryIds();
      attachToLocatables( list1loc );
      attachToRepositoryItems( list1loc,list1rep,USER_REP );

      // ... i caly sprzet
      int[] list2loc = listAvailableEquipmentLocatableIds();
      int[] list2rep = listAvailableEquipmentRepositoryIds();
      attachToLocatables( list2loc );
      attachToRepositoryItems( list2loc,list2rep,EQUIPMENT_REP );

      System.err.println( "Attached to locatables." );
    }
    else {
      connect_events( view,sightingReceiver );
      System.err.println( "Connected to the event channel." );
    }

    // Chcemy otrzymywac komunikaty o skasowaniu uzytkownikow / lokacji
    view.attach_locatable_add_view_data_observer( viewMessagesReceiver );
    view.attach_location_add_view_data_observer( viewMessagesReceiver );

    // Chcemy tez widziec zmiany parametrow lokacji
    int[] list3view = listAvailableLocationsViewIds();
    int[] list3rep = listAvailableLocationsRepositoryIds();
    attachToRepositoryItems( list3view,list3rep,LOCATION_DESCRIPTION_REP );
  }

  /**
   */
  private void connectUsersToLocations( Vector users, Vector locations ){
    for( int i=0; i<users.size(); i++ ) {
      connectSingleUserToLocation( users.elementAt(i),locations );
    }
  }

  /**
   */
  private void connectSingleUserToLocation( Object user, Vector locations ) {
    UserRecord ur = (UserRecord)user;
    LocationRecord lr;
    int loc_id = ur.sighting.my_location;
    for( int j=0; j<locations.size(); j++ ) {
      lr = (LocationRecord)locations.elementAt(j);
      if( lr.view_id == loc_id ) {
	ur.setLocation( lr );
	break;	  
      }
    }
  }

  /**
   */
  private void connectEquipmentToLocations( Vector equipment, Vector locations ) {
    for( int i=0; i<equipment.size(); i++ ) {
      connectPieceOfEquipmentToLocation( equipment.elementAt(i),locations );
    }
  }


  /**
   */
  private void connectPieceOfEquipmentToLocation( Object piece, Vector locations ) {
    EquipmentRecord eq = (EquipmentRecord)piece;
    LocationRecord lr;
    int loc_id = eq.sighting.my_location;
    for( int j=0; j<locations.size(); j++ ) {
      lr = (LocationRecord)locations.elementAt(j);
      if( lr.view_id == loc_id ) {
	eq.setLocation(lr);
	break;
      }
    }
  }


  /**
   * Detaches Jabba from the current view.
   */
  void detachFromView() {
    jabba.console.messageFrom( this, "detachFromView() called" );

    // <TU WSTAWIC> wyrejestrowanie z View </TU>

    view = null;
  }

  /**
   * Returns a list of identifiers of all the users registered in current View.
   */
  private int[] listAvailableUsersLocatableIds() {
    jabba.console.messageFrom( this, "listAvailableUsersLocatableIds() called." );
    int i;
    int[] retval = new int[ i=peopleObserved.size() ];

    for( int j=0; j<i; j++ ) 
      retval[j] = userLocatableId( peopleObserved.elementAt(j) );

    return retval;
  }

  /**
   * Returns a list of user repository identifiers of all the users registered
   * in current View.
   */
  private int[] listAvailableUsersRepositoryIds() {
    int i;
    int[] retval = new int[ i=peopleObserved.size() ];

    for( int j=0; j<i; j++ )
      retval[j] = userRepositoryId( peopleObserved.elementAt(j) );

    return retval;
  }

  /** 
   * Returns a list of identifiers of all equipment registered in current View.
   */
  private int[] listAvailableEquipmentLocatableIds() {
    jabba.console.messageFrom( this, "listAvailableEquipmentLocatableIds() called." );
    int i;
    int[] retval = new int[ i=equipmentObserved.size() ];

    for( int j=0; j<i; j++ )
      retval[j] = equipmentLocatableId( equipmentObserved.elementAt(j) );

    return retval;
  }


  /** 
   * Returns a list of repository identifiers of all equipment registered
   * in the current View.
   */
  private int[] listAvailableEquipmentRepositoryIds() {
    int i;
    int[] retval = new int[ i=equipmentObserved.size() ];
    
    for( int j=0; j<i; j++ ) 
      retval[j] = equipmentRepositoryId( equipmentObserved.elementAt(j) );

    return retval;
  }

  /**
   * Turns on watching locatables.
   * @param locatableIdList list of locatable identifiers
   * @param userRepIdList list of the User Repository identifiers
   * @see #detachFromLocatables
   */
  void attachToLocatables( int[] locatableIdList ) {
    jabba.console.messageFrom( this, "attachToLocatables() called" );

    if( locatableIdList.length<1 ) {
      jabba.console.messageFrom( this, "attachToLocatables(): Empty id List!" );
      return;
    }
    else {
      for( int i=0; i<locatableIdList.length; i++ ) {
	System.err.print( " "+locatableIdList[i] );
      }
      System.err.println( "" );
    }


    if( view != null && sightingReceiver != null ) {
      try{
        view.attach_locatable_dynamic_data_observer( sightingReceiver,locatableIdList );
      }
      catch( Unknown_Repository_Item_Ids ex ) {
	jabba.console.exception( this, ex );
	int[] uds = ex.unknown_items;
	for( int i=0; i<uds.length; i++ ) {
	  jabba.console.messageFrom( this, "Item "+uds[i]+" unknown." );
	}
      }
    }
  }


  void attachToRepositoryItems( int[] viewIdList, int[] repIdList, int rep ){
    try{
      if( rep==USER_REP ) {
	view.attach_locatable_view_data_observer( viewMessagesReceiver,viewIdList );
	user_rep.attach_static_data_observer( staticMessagesReceiver,repIdList );
      }
      else if( rep==EQUIPMENT_REP ) {
	equipment_rep.attach_static_data_observer( staticMessagesReceiver,repIdList );
	view.attach_locatable_view_data_observer( viewMessagesReceiver,viewIdList );
      }
      else if( rep==LOCATION_DESCRIPTION_REP ) {
	view.attach_location_view_data_observer( viewMessagesReceiver,viewIdList );
	location_description_rep.attach_static_data_observer( staticMessagesReceiver,repIdList );
      }
    }
    catch( Unknown_Repository_Item_Ids ex ) {
      jabba.console.exception( this, ex );
      int[] uds = ex.unknown_items;
      for( int i=0; i<uds.length; i++ ) {
	jabba.console.messageFrom( this, "Item "+uds[i]+" unknown." );
      }
    }
  }
  



  /**
   * Turns off watching locatables.
   * @param idList list of identifiers
   * @see #attachToLocatables
   */
  void detachFromLocatables( int[] idList ){
    jabba.console.messageFrom( this, "detachFromLocatables() called" );
    if( view != null && sightingReceiver != null ) {
      try{
	view.detach_locatable_dynamic_data_observer( sightingReceiver,idList );
      }
      catch( Unknown_Observer ex ) {
	jabba.console.exception( this, ex );
      }
      catch( Unmatched_Observer_Item_Ids ex ) {
	jabba.console.exception( this, ex );
      }
      catch( Unknown_Repository_Item_Ids ex ) {
	jabba.console.exception( this, ex );
      }
    }
  }

  /** 
   * Holds data about an User object.
   */
  class UserRecord {
    User user;
    Sighting_Description sighting;
    LocationRecord location_record;
    boolean present;

    // cache 
    String forename, surname, fullName; // imie,nazwisko,imie i nazwisko
    String position,email;              // stanowisko i adres mailowy
    Integer sighting_quality;           // jakosc sightingu
    ReceiverDate sighting_date;         // data ostatniego sightingu
    int user_repository_id;             // identyfikator uzytkownika
    int locatable_id;

////////////// NASZE
int mask;
///////////// NASZE

    // czy jestem uzytkownikiem wybranym?
    boolean selected;

    // indeks wewnatrz wektora uzytkownikow
    int index;

    UserRecord( Locatable loc, User user, int index ) {
      this.user = user;
      forename = user.forename();
      surname = user.surname();
      position = user.position();
      email = user.email();
      user_repository_id = user.item_id();
      fullName = surname+" "+forename;
      sighting = loc.get_sighting();
      sighting_quality = new Integer( sighting.q );
      sighting_date = new ReceiverDate( ((long)sighting.update_time.seconds)*1000L );

      locatable_id = sighting.my_locatable;
      selected = false;
      this.index = index;
      checkPresence();
    }

    UserRecord( int locatable_id, Domain_Repository.UserPackage.Description desc, int index ) {
      user = null;
      this.locatable_id = locatable_id;
      sighting = null;
      setStaticData( desc );
      sighting_quality = new Integer(-100);
      sighting_date = new ReceiverDate( jabba.clock.currentTimeSecs()*1000L );
      present = false;
      selected = false;
      this.index = index;
      location_record=null;
    }

    private void setStaticData( Domain_Repository.UserPackage.Description desc ) {
      user_repository_id = desc.item_id;
      forename = desc.forename;
      surname = desc.surname;
      position = desc.position;
      email = desc.email;
      fullName = surname+" "+forename;
    }

    void updateStaticData( Domain_Repository.UserPackage.Description desc ) {
      setStaticData( desc );
      
      if( dataModel!=null ) {
	if( (messagesSent & USER_MESSAGES) != 0 ) {
	  dataModel.user_nameChanged(index);
	  dataModel.user_surnameChanged(index);
	  dataModel.user_positionChanged(index);
	  dataModel.user_emailChanged(index);
	}
	if( (messagesSent & SINGLE_USER_MESSAGES) != 0 ) 
	  if( singlePersonIndex==index ) {
	    dataModel.single_user_nameChanged(index);
	    dataModel.single_user_surnameChanged(index);
	    dataModel.single_user_positionChanged(index);
	    dataModel.single_user_emailChanged(index);
	  }
	  else { // TU WSTAWIC rozeslanie do userow w tej samej lokacji
	  }
      }
    }
          
    void setLocation( LocationRecord loc_rc ) {
      if( loc_rc!=location_record ) { // TEN SAM
	if( location_record!=null ) {
	  System.err.println( "Remove user 1" );
	  location_record.removeUser( this );
	}
	location_record = loc_rc;
	System.err.println( "Add user 1" );
	location_record.addUser( this );
      }
    }

    void setLocation( int loc_id ) {
      LocationRecord lr = findLocationByViewId(loc_id);
      if( lr!=null ) {
	setLocation(lr);
      }
      // <TU WSTAWIC> rzucanie wyjatkiem - podany nieprawidlowy identyfikator lokacji </TU>
    }

    void setSighting( Sighting_Description sd ) {
      Sighting_Description old_sighting = sighting;
      sighting = sd;
      sighting_quality = new Integer( sd.q );
      sighting_date.setTime( ((long)sd.update_time.seconds)*1000L );

      // to tylko inicjalizacja
      if( old_sighting==null ) 

///////////// NASZE
{ mask=setMask();
return;
}
/////////////// NASZE

      if( sd.my_location != old_sighting.my_location ) {
	setLocation( sd.my_location );
      }

      // rozsylanie wiadomosci
      if( dataModel != null ) {
	if( (messagesSent & USER_MESSAGES) != 0 ) {
	  if( sd.q != old_sighting.q ) 
	    dataModel.user_qualityChanged(index);

	  if( sd.update_time.seconds != old_sighting.update_time.seconds ) {
	    dataModel.user_timeChanged(index);

///////////////////////// NASZE
if (mask != setMask()) {mask=setMask(); 
dataModel.user_locationChanged(index);
}
////////////// NASZE

	    if( !present ) 
	      if( checkPresence() ) {
		location_record.increasePresentNumber( index );
		dataModel.user_qualityChanged( index );
	      }
	  }

	  if( sd.my_location != old_sighting.my_location ) {
	    dataModel.user_locationChanged(index);
	    dataModel.user_phoneChanged(index);
	  }
	}
	if( (messagesSent & LOCATION_MESSAGES) != 0 ) {
	  if( sd.update_time.seconds != old_sighting.update_time.seconds ){
	    if( !present ) 
	      if( checkPresence() ) {
		location_record.increasePresentNumber( index );
	      }
	  }
	} // LOCATION_MESSAGES
	if( (messagesSent & SINGLE_USER_MESSAGES) != 0 ) {
	  jabba.console.trace( "SINGLE user message: user " +index +'(' +singlePersonIndex +')' );
	  if( singlePersonIndex==index ) {
	    if( sd.my_location != old_sighting.my_location ) {
	      setLocation( sd.my_location );
	      dataModel.single_user_locationChanged(index);
	      dataModel.single_user_phoneChanged(index);
	    }
	    if( sd.q != old_sighting.q ) {
	      jabba.console.trace( "Changing sighting quality..." );
	      dataModel.single_user_qualityChanged(index);
	    }
	    if( sd.update_time.seconds != old_sighting.update_time.seconds ) {
	      dataModel.single_user_timeChanged(index);
	      if( !present ) {
		if( checkPresence() ) {
		  location_record.increasePresentNumber(index);		  
		  dataModel.single_user_qualityChanged(index);
		}
	      }
	    }
	  }
	  /* if ( tu warunek: pierwsza lub druga lokacja taka, jak obserwowanego ) */
	  else {
	    if( !present ) 
	      if( checkPresence() ) {
		location_record.increasePresentNumber(index);
	      }
	  }
	} // SINGLE_USER_MESSAGES
	if( (messagesSent&SINGLE_LOCATION_MESSAGES)!=0 ) {
	  if( sd.my_location!=old_sighting.my_location ) {
	    LocationRecord lr = (LocationRecord)(locationsObserved.elementAt(singleLocationIndex));
	    LocationRecord lr2 = findLocationByViewId(old_sighting.my_location);
	  }
	} // SINGLE_LOCATION_MESSAGES
      }
    }

    boolean checkPresence() {
      return checkPresence( jabba.clock.currentTimeSecs() );
    }

    boolean checkPresence( long seconds ) {
      if( !(present=((seconds - sighting.update_time.seconds)<Jabba.ABSENT_SECONDS)) ) 
	sighting_quality = Jabba.ABSENT_QUALITY;
      return present;
    }


///////////////// ASIA

int setMask() {
return setMask (jabba.clock.currentTimeSecs() ) ;
}

int setMask (long seconds) {
int time=sighting.update_time.seconds;
if ((seconds - time) > Jabba.ABSENT_SECONDS) return 3;
if ((seconds - time) > Jabba.PERHAPS_SECONDS) return 2;
if ((seconds - time) > Jabba.MIGHT_SECONDS) return 1;
return 0;
}

///////////////////////////// ASIA

  }

  ///////////////////////////// END OF CLASS UserRecord ///////////////////////

  Boolean getUserRecordSelected(int userIndex) {
    boolean sel = ((UserRecord)peopleObserved.elementAt(userIndex)).selected;
    return (sel)?Boolean.TRUE:Boolean.FALSE;
  }

  void setUserRecordSelected(int userIndex,Boolean value) {
    ((UserRecord)peopleObserved.elementAt(userIndex)).selected = value.booleanValue();
  }

  private UserRecord findUserByRepositoryId( int id ) {
    UserRecord retval;
    for( int i=0; i<peopleObserved.size(); i++ ) {
      retval = (UserRecord)(peopleObserved.elementAt(i));
      if( retval.user_repository_id == id ) return retval;
    }
    return null;
  }

  private UserRecord findUserByViewId( int id ) {
    UserRecord retval;
    for( int i=0; i<peopleObserved.size(); i++ ) {
      retval = (UserRecord)(peopleObserved.elementAt(i));
      if( retval.locatable_id == id ) return retval;
    }
    return null;
  }
    


  private void updateUserStaticData( int id, User user ) {
    jabba.console.messageFrom( this, "updateUserStaticData() called" );
    UserRecord ur = findUserByRepositoryId( id );
    Domain_Repository.UserPackage.Description dsc=user.value_descr();

    if( ur!=null ) ur.updateStaticData(dsc);
  }

  /**
   * Helps retrieving user id from UserRecord.
   * @see UserRecord
   */
  private int userLocatableId( Object o ) {
    return ((UserRecord)o).locatable_id;
  }

  /**
   * Returns the repository id of specified user.
   */
  private int userRepositoryId( Object o ) {
    return ((UserRecord)o).user_repository_id;
  }


  /**
   * Returns the quality of last sighting.
   * @param userIndex user position in the list
   */
  Integer getUserSightingQuality( int userIndex ) {
    return ((UserRecord)(peopleObserved.elementAt(userIndex))).sighting_quality;
  } 

  /**
   * Returns user's name.
   * @param userIndex user position in the list
   */
  String getUserName(int userIndex) {
    return ((UserRecord)(peopleObserved.elementAt(userIndex))).forename;
  }

  /**
   * Returns user's surname.
   * @param userIndex user position in the list
   */
  String getUserSurname(int userIndex) {
    return ((UserRecord)(peopleObserved.elementAt(userIndex))).surname;
  }

  /**
   * Returns string created by concatenation of forename and surname.
   */
  String getUserFullName(int userIndex) {
    return ((UserRecord)(peopleObserved.elementAt(userIndex))).fullName;
  }

  /**
   * Returns the user's position (e.g. student).
   */
  String getUserPosition(int userIndex) {
    return ((UserRecord)(peopleObserved.elementAt(userIndex))).position;
  }

  /**
   * Returns the user's email address.
   */
  String getUserEmail(int userIndex) {
    return ((UserRecord)(peopleObserved.elementAt(userIndex))).email;
  }

  /**
   * Returns the user's record reference.
   */
  Object getUserRecordReference(int userIndex) {
    return peopleObserved.elementAt(userIndex);
  }


// NASZE

  /**
   * Returns the user's mask.
   */
  int getUserMask(int userIndex) {
    return ((UserRecord)(peopleObserved.elementAt(userIndex))).mask;
  }

// NASZE

  /**
   * Returns ids of the people in the same location the specified user is.
   */
  int[] peopleIdsForUser(int userIndex) {
    LocationRecord lr = locationOfUser(userIndex);
    if( lr!=null ) 
      return lr.getPresentUsersIndexes();
    else return null;
  }

  /**
   * Returns ids of the people in the specified location.
   */
  int[] peopleIdsForLocation(int locationIndex) {
    LocationRecord lr = ((LocationRecord)locationsObserved.elementAt(locationIndex));
    if( lr!=null ) 
      return lr.getPresentUsersIndexes();
    else return null;
  }


  /** 
   * Holds data about an Equipment object.
   */
  class EquipmentRecord {
    Equipment equipment;
    Sighting_Description sighting;
    LocationRecord location_record;

    // cache 
    String name,type;         // nazwa i typ sprzetu
    Integer sighting_quality; 
    ReceiverDate sighting_date;
    int equipment_repository_id;
    int locatable_id;

///////////////////// NASZE
int mask;
//////////////////// NASZE


    // czy jestem sprzetem wybranym?
    boolean selected;

    // czy jestem obecny?
    boolean present;

    // indeks wewnatrz wektora sprzetu
    int index;

    EquipmentRecord( Locatable loc, Equipment equipment, int index ) {
      this.equipment = equipment;
      this.name = equipment.name();

      try {
	this.type = equipment.meta_data().type();
      } 
      catch( Destination_Unreachable du ) {
	jabba.console.exception( this, du );
      }

      equipment_repository_id = equipment.item_id();

      sighting = loc.get_sighting();
      sighting_quality = new Integer( sighting.q );
      sighting_date = new ReceiverDate( ((long)sighting.update_time.seconds)*1000L );
      locatable_id = sighting.my_locatable;
      selected = false;
      this.index = index;
      checkPresence();
    }

    EquipmentRecord( int locatable_id, Domain_Repository.EquipmentPackage.Value_Description desc, int index ) {
      equipment = null;
      this.locatable_id = locatable_id;
      sighting = null;
      setStaticData( desc );
      sighting_quality=new Integer(-100);
      sighting_date = new ReceiverDate( jabba.clock.currentTimeSecs()*1000L );
      present = false;
      selected = false;
      this.index = index;
      location_record=null;
    }

    private void setStaticData( Domain_Repository.EquipmentPackage.Value_Description desc ) {
      equipment_repository_id = desc.item_id;
      name = desc.name;
      type = desc.meta_data.type;
    }    

    void updateStaticData( Domain_Repository.EquipmentPackage.Value_Description desc ) {
      setStaticData( desc );
      if( dataModel!=null ) {
	if( (messagesSent & EQUIPMENT_MESSAGES) != 0 ) {
	  dataModel.equipment_nameChanged(index);
	  dataModel.equipment_typeChanged(index);
	}
      }

      // TU ROZESLAC wiadomosci o update'ach (PIECE_OF_EQ)
    }



    void setLocation( LocationRecord loc_rc ) {
      if( loc_rc!=location_record ) { // TEN SAM
	location_record = loc_rc;
      }
    }

    void setLocation( int loc_id ) {
      LocationRecord lr = findLocationByViewId(loc_id);
      if( lr!=null ) {
	setLocation(lr);
      }
      // <TU WSTAWIC> rzucanie wyjatkiem - podany nieprawidlowy identyfikator lokacji </TU>
    }

    void setSighting( Sighting_Description sd ) {
      Sighting_Description old_sighting = sighting;
      sighting = sd;
      sighting_quality = new Integer( sd.q );
      sighting_date.setTime( ((long)sd.update_time.seconds)*1000L );

      // to tylko inicjalizacja
      if( old_sighting==null ) 

////////////// NASZE
{ mask=setMask();
return;
}
/////////// NASZE

      if( sd.my_location != old_sighting.my_location ) {
	setLocation( sd.my_location );
      }

      // rozsylanie wiadomosci
      if( dataModel != null ) {
	if( (messagesSent & EQUIPMENT_MESSAGES) != 0 ) {
	  if( sd.q != old_sighting.q ) 
	    dataModel.equipment_qualityChanged(index);

	  if( sd.update_time.seconds != old_sighting.update_time.seconds ) {
	    dataModel.equipment_timeChanged(index);

////////////////// NASZE
if (mask != setMask()) {mask=setMask();
dataModel.equipment_locationChanged(index); 
}
////////////////// NASZE


	    if( !present ) 
	      if( checkPresence() ) {
		dataModel.equipment_qualityChanged( index );
	      }
	  }

	  if( sd.my_location != old_sighting.my_location ) {
	    dataModel.equipment_locationChanged(index);
	  }
	}
	else {
	  if( !present ) checkPresence(); 
	}
      } // EQUIPMENT_MESSAGES
    }

    boolean checkPresence() {
      return checkPresence( jabba.clock.currentTimeSecs() );  
    }

    boolean checkPresence( long seconds ) {
      if( !(present=((seconds - sighting.update_time.seconds)<Jabba.EQUIPMENT_NOT_PRESENT_SECONDS)) ) 
	sighting_quality = Jabba.ABSENT_QUALITY;
      return present;
    }


//////////////////////////// ASIA

int setMask() {
return setMask (jabba.clock.currentTimeSecs() ) ;
}

int setMask (long seconds) {
int time=sighting.update_time.seconds;
if ((seconds - time) > Jabba.EQUIPMENT_NOT_PRESENT_SECONDS) return 3;
return 0;
}


///////////////////////////// ASIA


  } // end class EquipmentRecord

  private int equipmentLocatableId(Object o) {
    return ((EquipmentRecord)o).locatable_id;
  }

  private int equipmentRepositoryId( Object o ) {
    return ((EquipmentRecord)o).equipment_repository_id;
  }

  public String getEquipmentName(int equipmentIndex) {
    return ((EquipmentRecord)(equipmentObserved.elementAt(equipmentIndex))).name;
  }

  public String getEquipmentType(int equipmentIndex) {
    return ((EquipmentRecord)(equipmentObserved.elementAt(equipmentIndex))).type;
  }

  public String getEquipmentLocationDescription(int equipmentIndex) {
    EquipmentRecord eqr = (EquipmentRecord)(equipmentObserved.elementAt(equipmentIndex));
    LocationRecord lr = eqr.location_record;
    if( lr!=null ) {
      return lr.numberName;
    }
    else {
      return null;
    }
  }

///////////////////////// ASIA

public String getEquipmentLocationNumber(int equipmentIndex) {
EquipmentRecord eqr = (EquipmentRecord)(equipmentObserved.elementAt(equipmentIndex));
    LocationRecord lr = eqr.location_record;
    if( lr!=null ) {
      return lr.number;
    }
    else {
      return null;
    }
  }

////////////////////////////////// ASIA


  Boolean getEquipmentRecordSelected(int eqIndex) {
    boolean sel = ((EquipmentRecord)equipmentObserved.elementAt(eqIndex)).selected;
    return (sel)?Boolean.TRUE:Boolean.FALSE;
  }

  /**
   * Returns the equipment's record reference.
   */
  Object getEquipmentRecordReference(int eqIndex) {
    return equipmentObserved.elementAt(eqIndex);
  }

  public Integer getEquipmentSightingQuality(int equipmentIndex) {
    return ((EquipmentRecord)(equipmentObserved.elementAt(equipmentIndex))).sighting_quality;
  }

  public Date getEquipmentSightingTime(int equipmentIndex) {
    return ((EquipmentRecord)(equipmentObserved.elementAt(equipmentIndex))).sighting_date;
  }

  private EquipmentRecord findEquipmentByRepositoryId( int id ) {
    EquipmentRecord retval;
    for( int i=0; i<equipmentObserved.size(); i++ ) {
      retval=(EquipmentRecord)(equipmentObserved.elementAt(i));
      if( retval.equipment_repository_id==id ) return retval;
    }
    return null;
  }

  private EquipmentRecord findEquipmentByViewId( int id ) {
    EquipmentRecord retval;
    for( int i=0; i<equipmentObserved.size(); i++ ) {
      retval=(EquipmentRecord)(equipmentObserved.elementAt(i));
      if( retval.locatable_id==id ) return retval;
    }
    return null;
  }
    


  // NASZE

  /**
   * Returns the equipment's mask.
   */
  int getEquipmentMask(int userIndex) {
    return ((EquipmentRecord)(equipmentObserved.elementAt(userIndex))).mask;
  }

// NASZE

  private void updateEquipmentStaticData( int id,Equipment eq ) {
    jabba.console.messageFrom( this, "updateEquipmentStaticData() called" );
    EquipmentRecord eqr = findEquipmentByRepositoryId( id );
    Domain_Repository.EquipmentPackage.Value_Description dsc=null;
    try {
      dsc = eq.value_descr();
    }
    catch( Destination_Unreachable du ) {
      jabba.console.exception( this, du );
    }

    if( eqr!=null && dsc!=null ) eqr.updateStaticData(dsc);
  }

  /**
   * A record that holds some location data.
   */
  class LocationRecord {
    // docelowo nie ma byc odwolan do tych pol
    Location location;
    Location_Description description;

    // identyfikator opisu lokacji
    int location_description_repository_id;

    // identyfikator lokacji w View
    int view_id;

    // czy wybrana, czy nie
    boolean selected;

    // indeks w wektorze lokacji obserwowanych
    int index;

    // ilu obecnych 
    int peopleNumber;

    // szczegolowe pola
    String number,name,numberName,phones,color;
    int rgbColor,origX,origY,sizeX,sizeY;
    int[] doorsData;    // a moze theDoors ;)

    // kto tu jest
    Vector userList;

    private void adjustProperties( Property[] props ) {
      if( props!=null ) {
	for( int i=0; i<props.length; i++ ) {
	  if( props[i] != null ) {
	    if( props[i].name.equals( "description" ) ) {
	      if( props[i].value!=null ) name = props[i].value.extract_string();
	    }
	    else if( props[i].name.equals( "number" ) ) {
	      if( props[i].value!=null ) number = props[i].value.extract_string();
	    }
	    else if( props[i].name.equals( "phones" ) ) {
	      if( props[i].value!=null ) {
		InputStream instream = props[i].value.create_input_stream();
		long numphones = instream.read_long();
		phones = null;
		for( int j=0; j<numphones; j++ ) {
		  if( phones != null ) phones = phones+", "+instream.read_string();
		  else phones = instream.read_string();
		}
	      }
	    }
	    else if( props[i].name.equals( "color" ) ) {
	      if( props[i].value!=null ) color = props[i].value.extract_string();
	    }
	    else if( props[i].name.equals( "rgbcolor" ) ) {
	      if( props[i].value!=null ){
		//		InputStream instream = props[i].value.create_input_stream();
		//		rgbColor = instream.read_ulong();
		rgbColor = props[i].value.extract_ulong();
	      }
	    }
	    else if( props[i].name.equals( "orig_x" ) ) {
	      if( props[i].value!=null ) {
		//		InputStream instream = props[i].value.create_input_stream();
		//		origX = instream.read_ulong();
		origX = props[i].value.extract_ulong();
	      }
	    }
	    else if( props[i].name.equals( "orig_y" ) ) {
	      if( props[i].value!=null ) {
		//		InputStream instream = props[i].value.create_input_stream();
		//		origY = instream.read_ulong();
		origY = props[i].value.extract_ulong();
	      }
	    }
	    else if( props[i].name.equals( "size_x" ) ) {
	      if( props[i].value!=null ) {
		//		InputStream instream = props[i].value.create_input_stream();
		//		sizeX = instream.read_ulong();
		sizeX = props[i].value.extract_ulong();
	      }
	    }
	    else if( props[i].name.equals( "size_y" ) ) {
	      if( props[i].value!=null ) {
		//		InputStream instream = props[i].value.create_input_stream();
		//		sizeY = instream.read_ulong();
		sizeY = props[i].value.extract_ulong();
	      }
	    }
	    else if( props[i].name.equals( "doors" ) ) {
	      if( props[i].value!=null ) {
		InputStream instream = props[i].value.create_input_stream();

		try {
		  if( instream.available()>0 ) {
		    int count=instream.read_ulong();
		    doorsData = new int[count];
		    instream.read_ulong_array(doorsData,0,count);
		  }
		  else 
		    doorsData = new int[0];
		}
		catch (java.io.IOException ex) {
		  doorsData = new int[0];
		}
		catch (org.omg.CORBA.BAD_OPERATION ex) {
		  doorsData = new int[0];
		}
	      }
	    }
	  }
	}
      }
      if( name==null ) name = "none";
      if( number==null ) number = "none";

      numberName = "("+number+") "+name;
    }



    LocationRecord( Location loc, int index ) throws Destination_Unreachable {
      location = loc;
      description = loc.loc_descr();
      view_id = loc.item_id();
      location_description_repository_id = description.item_id();
      adjustProperties( description!=null ? description.property_list() : null );
      userList = new Vector();
      this.index = index;
      peopleNumber=0;
      selected = false;
    }

    LocationRecord( ABng_Core.LocationPackage.Value_Description vdesc, int index ) {
      location = null;
      description = null;

      location_description_repository_id = vdesc.loc_value_descr.item_id;
      view_id = vdesc.item_id;
      adjustProperties( vdesc.loc_value_descr.property_list );
      userList = new Vector();
      peopleNumber=0;
      this.index = index;
      selected = false;
    }

    void updateStaticData( Domain_Repository.Location_DescriptionPackage.Description desc ) {
      adjustProperties( desc.property_list );

      if( dataModel!=null ) {
	if( (messagesSent & LOCATION_MESSAGES) != 0 ) {
	  dataModel.location_numberChanged(index);
	  dataModel.location_descriptionChanged(index);
	  dataModel.location_phonesChanged(index);
	}
	if( (messagesSent & SINGLE_LOCATION_MESSAGES) != 0 ) {
	  if( singleLocationIndex==index ) {
	    dataModel.single_location_numberChanged(index);
	    dataModel.single_location_descriptionChanged(index);
	    dataModel.single_location_phonesChanged(index);
	  }
	}
	if( (messagesSent & USER_MESSAGES) != 0 ) {
	  for( int i=0; i<userList.size(); i++ ) {
	    dataModel.user_locationChanged(((UserRecord)userList.elementAt(i)).index);
	  }
	}
	if( (messagesSent & SINGLE_USER_MESSAGES) != 0 ) {
	  UserRecord ur = (UserRecord)(peopleObserved.elementAt(singlePersonIndex));
	  if( ur.location_record == this ) dataModel.single_user_locationChanged(ur.index);
	}
	if( (messagesSent & EQUIPMENT_MESSAGES) != 0 ) {
	  // TU powiadamiac equipment wewnatrz lokacji o zmianie nazwy
	}
      }
    }
  
//      private int findUserOfIndex(int ind) {
//        System.err.println( "Find user of index: "+ind );
//        System.err.println( "Location: "+numberName );
//        int j=userList.size();
//        System.err.println( "User list size: "+j );
//        UserRecord ur;
//        for( int i=0; i<j; i++ ) {
//  	ur = (UserRecord)(userList.elementAt(i));
//  	System.err.println( "Comparing "+ind+" with "+ur.index );
//  	if( ur.index==ind ) {
//  	  System.err.println( "Returning: "+i );
//  	  return i;
//  	}
//        }

//        // search failed
//        System.err.println( "Returning -1" );
//        return -1;
//      }

    void removeUser( UserRecord rec ) {
      System.err.println( "Removing user: "+rec.present );
      userList.removeElement(rec);
      if( rec.present ) decreasePresentNumber(rec.index);  // !!!!!
    }

    void addUser( UserRecord rec ) {
      System.err.println( "Adding user: "+rec.present );
      userList.addElement(rec);
      if( rec.present ) {
	increasePresentNumber(rec.index);
      }
    }

    void firePeopleNumberChanged() {
      if( dataModel!=null ) {
	if( (messagesSent & USER_MESSAGES) != 0 ) {
	  for( int i=0; i<userList.size(); i++ ) // <TUTAJ>
	    dataModel.user_peopleNumberChanged( ((UserRecord)userList.elementAt(i)).index );
	}
	if( (messagesSent & LOCATION_MESSAGES) != 0 ) {
	  dataModel.location_peopleNumberChanged( index );
	}
      }
    }

    void increasePresentNumber(int ind) {
      if( (messagesSent&SINGLE_LOCATION_MESSAGES)!=0 
	  && singleLocationIndex==index ) {
	System.err.println( "Person entered: "+(ind>=0) );
	if( ind>=0 ) dataModel.single_location_personEntered(ind);
      }
      if( (messagesSent&SINGLE_USER_MESSAGES)!=0 ) {
	UserRecord ur = (UserRecord)(peopleObserved.elementAt(singlePersonIndex));
	System.err.println( "Person joined: "+(ind>=0) );
	if( ur.location_record==this && ind>=0 ) dataModel.single_user_personJoined(ind);
      }

      peopleNumber++;
      firePeopleNumberChanged();
    }
    
    void decreasePresentNumber(int ind) {
      System.err.println( "Decrease present number..." );
      if( (messagesSent&SINGLE_LOCATION_MESSAGES)!=0 
	  && singleLocationIndex==index ) {
	System.err.println( "Person left: "+(ind>=0) );
	if( ind>=0 ) dataModel.single_location_personLeft(ind);
      }

      if( (messagesSent&SINGLE_USER_MESSAGES)!=0 ) {
	UserRecord ur = (UserRecord)(peopleObserved.elementAt(singlePersonIndex));
	System.err.println( "Person disjoined: "+(ind>=0) );
	if( ur.location_record==this ) {
	  if( ind>=0 ) dataModel.single_user_personDisjoined(ind);
	}
      }

      peopleNumber--;
      firePeopleNumberChanged();
    }

//      private int presentIndex( int uInd ) {
//        int j=userList.size(),retval=-1;
//        for( int i=0; i<=uInd && i<j; i++ ) 
//  	if( ((UserRecord)(userList.elementAt(i))).present ) retval++;
//        System.err.println( "Returning: "+retval );
//        return retval;
//      }

      int getPeopleNumber() {
        return peopleNumber;
      }


    // mozliwa modyfikacja (przechowywanie id, ale czy oplacalna czasowo...?)
    int[] getPresentUsersIndexes() {
      int[] retval = new int[peopleNumber];
      System.err.println( "people number: "+peopleNumber );
      int j=0;
      UserRecord ur;
      for( int i=0; i<userList.size(); i++ ) {
	ur = (UserRecord)userList.elementAt(i);
	if( ur.present ) {
	  retval[j++]=ur.index;
	}
      }
      System.err.println( "Reported: "+j+" present of total "+userList.size()+" users in location." );
      return retval;
    }
  }

  ////////////////////////////////// END OF CLASS LocationRecord //////////////

  private LocationRecord findLocationByRepositoryId( int id ) {
    LocationRecord retval;
    for( int i=0; i<locationsObserved.size(); i++ ) {
      retval = (LocationRecord)(locationsObserved.elementAt(i));
      if( retval.location_description_repository_id == id ) return retval;
    }
    return null;
  }


  private void updateLocationStaticData( int id, Location_Description loc ) {
    jabba.console.messageFrom( this, "updateLocationStaticData() called" );
    LocationRecord lr = findLocationByRepositoryId( id );
    Domain_Repository.Location_DescriptionPackage.Description dsc = loc.value_descr();

    if( lr!=null ) lr.updateStaticData(dsc);
  }


  private int[] listAvailableLocationsRepositoryIds() {
    jabba.console.messageFrom( this, "listAvailableLocationsRepositoryIds() called" );

    int i;
    int[] retval = new int[ i=locationsObserved.size() ];
    for( int j=0; j<i; j++ )
      retval[j] = ((LocationRecord)(locationsObserved.elementAt(j))).location_description_repository_id;
    return retval;
  }

  private int[] listAvailableLocationsViewIds() {
    jabba.console.messageFrom( this,"listAvailableLocationsViewIds() called" );

    int i;
    int[] retval = new int[ i=locationsObserved.size() ];
    for( int j=0; j<i; j++ )
      retval[j] = ((LocationRecord)(locationsObserved.elementAt(j))).view_id;
    return retval;
  }

  LocationRecord findLocationByViewId( int id ) {
    LocationRecord retval;
    for( int i=0; i<locationsObserved.size(); i++ ) {
      retval = (LocationRecord)locationsObserved.elementAt(i);
      if( retval.view_id == id ) return retval;
    }
    return null;
  }

  /**
   * Returns reference to the record describing location the user is currently in.
   */
  private LocationRecord locationOfUser(int userIndex) {
    UserRecord ur = (UserRecord)peopleObserved.elementAt(userIndex);
    if( ur!=null ) return ur.location_record;
    return null;
  }


  /**
   * Returns description of the specified location.
   * @param userIndex user position in the list
   */
  String getUserLocationDescription( int userIndex ) {
    LocationRecord lr = locationOfUser(userIndex);
    if( lr!=null )
      return lr.numberName;
    else return null;
  }

  /**
   *
   */
 
/////////// ASIA

 String getUserLocationNumber( int userIndex ) {
    LocationRecord lr = locationOfUser(userIndex);
    if( lr==null ) return null;
    else {
      String retval = lr.number;
      // <LATA>
//      if( retval==null ) return "   ";
      // </LATA>
      return retval;
    }
  }

////////////////// ASIA
  /**
   * Returns number of people present in the specified location.
   * @param userIndex user position in the list
   */
  Integer getUserLocationPeopleNumber( int userIndex ) {
    LocationRecord lr = locationOfUser(userIndex);
    // <LAG>
    if( lr!=null )
      return new Integer( lr.getPeopleNumber() );
    else return null;
    // </LAG>
  }

  /**
   * Returns number of the phone(s) that is nearest to the user.
   * @param userIndex user position in the list
   */
  String getUserLocationPhones( int userIndex ) {
    LocationRecord lr = locationOfUser(userIndex);
    if( lr!=null )
      return lr.phones;
    else return null;
  }

  /**
   * Returns the number of the specified location.
   * @param locIndex location position in the list
   */
  String getLocationNumber( int locIndex ) {
    LocationRecord lr = (LocationRecord)(locationsObserved.elementAt(locIndex));
    if( lr != null ) 
      return lr.number;
    else return null;
  }

  /**
   * Returns color of the location (string representation).
   * @param locIndex location position in the list.
   */
  String getLocationColor( int locIndex ) {
    LocationRecord lr = (LocationRecord)(locationsObserved.elementAt(locIndex));
    if( lr != null ) 
      return lr.color;
    else return null;
  }
    
  /**
   * Returns color of the location (int representation).
   * @param locIndex location position in the list.
   */

  Integer getLocationRGBColor( int locIndex ) {
    LocationRecord lr = (LocationRecord)(locationsObserved.elementAt(locIndex));
    if( lr != null ) 
      return new Integer(lr.rgbColor);
    else return null;
  }

  /**
   * Returns X origin coordinate of the location (int representation).
   * @param locIndex location position in the list.
   */
  Integer getLocationOriginX( int locIndex ) {
    LocationRecord lr = (LocationRecord)(locationsObserved.elementAt(locIndex));
    if( lr != null ) 
      return new Integer(lr.origX);
    else return null;
  }

  /**
   * Returns Y origin coordinate of the location (int representation).
   * @param locIndex location position in the list.
   */
  Integer getLocationOriginY( int locIndex ) {
    LocationRecord lr = (LocationRecord)(locationsObserved.elementAt(locIndex));
    if( lr != null ) 
      return new Integer(lr.origY);
    else return null;
  }

  /**
   * Returns X size of the location (int representation).
   * @param locIndex location position in the list.
   */
  Integer getLocationSizeX( int locIndex ) {
    LocationRecord lr = (LocationRecord)(locationsObserved.elementAt(locIndex));
    if( lr != null ) 
      return new Integer(lr.sizeX);
    else return null;
  }

  /**
   * Returns Y size of the location (int representation).
   * @param locIndex location position in the list.
   */
  Integer getLocationSizeY( int locIndex ) {
    LocationRecord lr = (LocationRecord)(locationsObserved.elementAt(locIndex));
    if( lr != null ) 
      return new Integer(lr.sizeY);
    else return null;
  }

  /**
   * Returns the doors of the location's data (int representation).
   * @param locIndex location position in the list.
   */
  int[] getLocationDoors( int locIndex ) {
    LocationRecord lr = (LocationRecord)(locationsObserved.elementAt(locIndex));
    if( lr != null ) 
      return lr.doorsData;
    else return null;
  }

  /**
   * Returns description of the specified location.
   * @param locIndex location position in the list
   */
  String getLocationDescription( int locIndex ) {
    LocationRecord lr = (LocationRecord)(locationsObserved.elementAt(locIndex));
    if( lr != null ) 
      return lr.name;
    else return null;
  }

  /**
   * Lists numbers of phones placed in the specified location.
   * @param locIndex location position in the list.
   */
  String getLocationPhones( int locIndex ) {
    LocationRecord lr = (LocationRecord)(locationsObserved.elementAt(locIndex));
    if( lr != null ) 
      return lr.phones;
    else return null;
  }

  /**
   * Returns the number of people currently present in the specified location.
   * @param locIndex location position in the list.
   */
  Integer getLocationPeopleNumber( int locIndex ) {
    LocationRecord lr = (LocationRecord)(locationsObserved.elementAt(locIndex));
    if( lr != null ) 
      return new Integer( lr.getPeopleNumber() );
    else return null;
  }

  Boolean getLocationRecordSelected(int locIndex) {
    boolean sel = ((LocationRecord)locationsObserved.elementAt(locIndex)).selected;
    return (sel)?Boolean.TRUE:Boolean.FALSE;
  }

  /**
   * Returns the location's record reference.
   */
  Object getLocationRecordReference(int locIndex) {
    return peopleObserved.elementAt(locIndex);
  }

  /*
   * A subclass of java.util.Date with more useful toString() method.
   */
  private static Locale locpl = new Locale( "pl","PL" );
  private static DateFormat dform = DateFormat.getDateInstance( DateFormat.SHORT,locpl );
  private static DateFormat tform = DateFormat.getTimeInstance( DateFormat.MEDIUM,locpl );
  private static Calendar calendar = Calendar.getInstance( locpl );
  private static Date referenceDate = new Date();

  private class ReceiverDate extends Date {
    
    ReceiverDate( long millis ) {
      super( millis );
    }

    public String toString() {
      referenceDate.setTime( jabba.clock.currentTimeSecs()*1000L );
      calendar.setTime( referenceDate );
      int y1 = calendar.get( Calendar.YEAR );
      int d1 = calendar.get( Calendar.DAY_OF_YEAR );

      calendar.setTime( this );
      int y2 = calendar.get( Calendar.YEAR );
      int d2 = calendar.get( Calendar.DAY_OF_YEAR );

      if( y1==y2 && d1==d2 ) return tform.format(this);
      else return dform.format(this);
    }
  } // end class ReceiverDate

  /**
   * Returns the date the user was last seen at.
   * @param userIndex user position in the list
   */
  Date getUserSightingTime( int userIndex ) {
    return ((UserRecord)(peopleObserved.elementAt(userIndex))).sighting_date;
  }

  int getObservedObjectsCount() {
    return getObservedObjectsCount( messagesSent );
  }

  int getObservedObjectsCount( int msg ) {
    int retval=0;
    if( (msg&USER_MESSAGES)!=0 ) retval += peopleObserved.size();
    if( (msg&LOCATION_MESSAGES)!=0 ) retval += locationsObserved.size();
    if( (msg&EQUIPMENT_MESSAGES)!=0 ) retval += equipmentObserved.size();
    if( (msg&SINGLE_USER_MESSAGES)!=0 ) retval++;
    if( (msg&SINGLE_LOCATION_MESSAGES)!=0 ) retval++;
    return retval;
  }

  ///////////////////////////// View Data Observer ////////////////////////////
  public void _notify( int id, View_Data_Object obj, Change_Type cht ) {
    jabba.console.messageFrom( this, "_notify(view) called" );
    switch( cht.value() ) {
    case Change_Type._OBJECT_UPDATED:
      jabba.console.messageFrom( this,"Object updated (view), id: " +id );
      updateViewObject( obj,id );
      break;
    case Change_Type._OBJECT_ADDED:
      jabba.console.messageFrom( this,"Object added (view), id: " +id );
      addViewObject( obj,id );
      break;
    case Change_Type._OBJECT_REMOVED:
      jabba.console.messageFrom( this,"Object removed (view), id: " +id );
      removeViewObject( obj,id );
      break;
    }
  }

  private void addViewObject( View_Data_Object obj, int id ) {
    if( obj.is_locatable_obj() ) {
      Locatable locobj = obj.locatable_obj();
      Locatable_Type_Specific_Object tso=null;
      try {
	tso=locobj.type_specific_object();
      }
      catch( Destination_Unreachable de ) {
	return;
      }

      if( tso.is_user_obj() ) {
	User user = tso.user_obj();
	UserRecord ur = new UserRecord( locobj,user,peopleObserved.size() );
	peopleObserved.addElement( ur );
	connectSingleUserToLocation( ur,locationsObserved );

	int[] idloc = new int[1],idr = new int[1];
	idloc[0]=ur.locatable_id;
	try {
	  view.attach_locatable_dynamic_data_observer( sightingReceiver,idloc );
	}
	catch( Unknown_Repository_Item_Ids ex ) {}
	
	idr[0] = ur.user_repository_id;
	attachToRepositoryItems( idloc,idr,USER_REP );

	if( (messagesSent&USER_MESSAGES)!=0 ) dataModel.user_added(ur.index);
      }
      else if( tso.is_equipment_obj() ) {
	Equipment eq = tso.equipment_obj();
	EquipmentRecord eqr = new EquipmentRecord( locobj,eq,peopleObserved.size() );
	equipmentObserved.addElement( eqr );
	connectPieceOfEquipmentToLocation( eqr,locationsObserved );

	int[] idloc = new int[1],idr = new int[1];
	idloc[0]=eqr.locatable_id;
	try {
	  view.attach_locatable_dynamic_data_observer( sightingReceiver,idloc );
	}
	catch( Unknown_Repository_Item_Ids ex ) {}

	idr[0] = eqr.equipment_repository_id;
	attachToRepositoryItems( idloc,idr,EQUIPMENT_REP );

	if( (messagesSent&EQUIPMENT_MESSAGES)!=0 ) dataModel.equipment_added( eqr.index );
      }
    }
    else if( obj.is_location_obj() ) {
      Location loc = obj.location_obj();
      LocationRecord lr;
      try {
	lr = new LocationRecord( loc,locationsObserved.size() );
	locationsObserved.addElement( lr );
      }
      catch( Destination_Unreachable du ) {
	return;
      }

      int[] idloc = new int[1],idr = new int[1];
      idloc[0] = lr.view_id;
      idr[0] = lr.location_description_repository_id;
      attachToRepositoryItems( idloc,idr,LOCATION_DESCRIPTION_REP );

      if( (messagesSent&LOCATION_MESSAGES)!=0 ) dataModel.location_added( lr.index );
    }
  }
	  
  private void removeViewObject( View_Data_Object obj, int id ) {
    if( obj.is_locatable_obj() ) {
      Locatable_Type_Specific_Object tso=null;
      try {
        tso=obj.locatable_obj().type_specific_object();
      }
      catch( Destination_Unreachable du ) {
	return;
      }
      
      if( tso.is_user_obj() ) {
	UserRecord ur = findUserByViewId(id);
	removeUser( ur );
      }
      else if( tso.is_equipment_obj() ) {
	EquipmentRecord eq = findEquipmentByViewId(id);
	removeEquipment(eq);
      }
    }
    else if( obj.is_location_obj() ) {
      LocationRecord lr = findLocationByViewId(id);
      removeLocation(lr);
    }
  }

  private void updateViewObject( View_Data_Object obj, int id ) {
    if( obj.is_locatable_obj() ) {
      Locatable_Type_Specific_Object tso=null;
      try {
	tso = obj.locatable_obj().type_specific_object();
      }
      catch( Destination_Unreachable du ) {
	return;
      }

      if( tso.is_user_obj() ) {
	UserRecord ur = findUserByViewId(id);
	ur.updateStaticData( tso.user_obj().value_descr() );
      }
      else if( tso.is_equipment_obj() ) {
	EquipmentRecord eqr = findEquipmentByViewId(id);
	if( eqr!=null ) {
	  try {
	    eqr.updateStaticData( tso.equipment_obj().value_descr() );
	  }
	  catch( Destination_Unreachable du ) {
	    return;
	  }
	}
      }
    }
    else if( obj.is_location_obj() ) {
      LocationRecord lr = findLocationByViewId(id);
      if( lr!=null ) {
	try {
	  lr.updateStaticData( obj.location_obj().loc_descr().value_descr() );
	}
	catch( Destination_Unreachable du ) {
	  return;
	}
      }
    }
  }
  
  ///////////////////////////// Static Data Observer //////////////////////////
  public void _notify( int id, Static_Data_Object obj, Change_Type cht ) {
    jabba.console.messageFrom( this, "_notify(static) called" );
    switch( cht.value() ) {
    case Change_Type._OBJECT_UPDATED:
      jabba.console.messageFrom( this,"Object updated (static), id: " +id );
      if( obj.is_user_obj() ) 
	updateUserStaticData( id,obj.user_obj() );
      else if( obj.is_equipment_obj() ) 
	updateEquipmentStaticData( id,obj.equipment_obj() );
      else if( obj.is_location_descr_obj() )
	updateLocationStaticData( id,obj.location_descr_obj() );
      break;
    case Change_Type._OBJECT_ADDED:
      jabba.console.messageFrom( this,"Object added (static), id: " +id );
      break;
    case Change_Type._OBJECT_REMOVED:
      jabba.console.messageFrom( this,"Object removed (static), id: " +id );
      if( obj.is_user_obj() ) 
	removeUser( id,obj.user_obj() );
      else if( obj.is_equipment_obj() ) 
	removeEquipment( id,obj.equipment_obj() );
      else if( obj.is_location_descr_obj() )
	removeLocation( id,obj.location_descr_obj() );
      break;
    default:
      break;
    }      
  }

  private void removeUser( int id, User user ) {
    UserRecord ur = findUserByRepositoryId(id);
    removeUser(ur);
  }

  private void removeUser( UserRecord ur ) {
    if( ur==null ) return;
    synchronized( peopleObserved ) {
      peopleObserved.removeElementAt(ur.index);
      int j=peopleObserved.size();
      for( int i=0; i<j; i++ ) 
	((UserRecord)(peopleObserved.elementAt(i))).index = i;
    }

    if( (messagesSent&USER_MESSAGES)!=0 ) dataModel.user_removed(ur.index);

  }


  private void removeEquipment( int id, Equipment eq ) {
    EquipmentRecord eqr = findEquipmentByRepositoryId(id);
    removeEquipment(eqr);
  }

  private void removeEquipment( EquipmentRecord eq ) {
    if( eq==null ) return;
    synchronized( equipmentObserved ) {
      equipmentObserved.removeElementAt(eq.index);
      int j=equipmentObserved.size();
      for( int i=0; i<j; i++ ) 
	((EquipmentRecord)(equipmentObserved.elementAt(i))).index = i;
    }
    
    if( (messagesSent&EQUIPMENT_MESSAGES)!=0 ) dataModel.equipment_added(eq.index);
    
  }    


  private void removeLocation( int id, Location_Description ld ) {
    LocationRecord lr = findLocationByRepositoryId(id);
    removeLocation(lr);
  }

  private void removeLocation( LocationRecord lr ) {
    if( lr==null ) return;
    synchronized( locationsObserved ) {
      locationsObserved.removeElementAt(lr.index);
      int j=locationsObserved.size();
      for( int i=0; i<j; i++ ) 
	((LocationRecord)(locationsObserved.elementAt(i))).index = i;
    }
    
    if( (messagesSent&LOCATION_MESSAGES)!=0 ) dataModel.location_removed(lr.index);
    
  }    





  ///////////////////////////// Event Channel /////////////////////////////////
  class ReceiverPushConsumer extends CosTypedEventComm._TypedPushConsumerImplBase {
    
    private org.omg.CORBA.Object typed_cons;
    
    public ReceiverPushConsumer( org.omg.CORBA.Object typed_cons) {
      this.typed_cons= typed_cons;
    }
    
    public org.omg.CORBA.Object get_typed_consumer() {
      return typed_cons;
    }

    public void push( org.omg.CORBA.Any v) {
      System.err.println("Receiver::ReceiverPushConsumer::push() called.");
    }

    public void disconnect_push_consumer() {
      System.err.println("Receiver::ReceiverPushConsumer disconnected");
    }
  }


  void connect_events( ABng_Core.View view,
		       ABng_Core.Dynamic_Data_Observer ddo) {
    
    CosTypedEventChannelAdmin.TypedEventChannel channel = view.sighting_channel();
    CosEventChannelAdmin.ProxyPushSupplier pps = channel.for_consumers().obtain_push_supplier();
    
    try {
      pps.connect_push_consumer( new ReceiverPushConsumer( ddo ) );
    }
    catch( CosEventChannelAdmin.AlreadyConnected ac ) {}
  }


}








