package jabba;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.InputEvent;

import com.sun.java.swing.JTable;
import com.sun.java.swing.ListSelectionModel;
import com.sun.java.swing.DefaultListSelectionModel;

import com.sun.java.swing.event.TableModelEvent;

import com.sun.java.swing.table.TableModel;
import com.sun.java.swing.table.JTableHeader;
import com.sun.java.swing.table.TableColumn;
import com.sun.java.swing.table.TableColumnModel;

import java.util.Date;

/**
 * Based on example taken from the Swing 1.0.1 release.
 */
class TableSorter extends TableMap implements JabbaConstants {
  int[] indices,rev_indices;
  int sortingColumn;
  boolean ascending = true;

  SelModel selModel;

  public TableSorter() {}

  public TableSorter(TableModel model) {
    selModel = new SelModel(this);
    setModel(model);
  }

  public void setModel(TableModel model) {
    super.setModel(model); 
    reallocateIndices(); 
  }

  public ListSelectionModel getSelectionModel() {
    return selModel;
  }

  // reusable
  private static Object o1,o2;
  private static double d1,d2;
  private static long l1,l2;
  private static String s1,s2;
  private static boolean b1,b2;
  private static Class type;
  public int compareRowsByColumn(int row1, int row2, int column) {
    type = model.getColumnClass(column);
   
    o1 = model.getValueAt(row1, column);
    o2 = model.getValueAt(row2, column); 

    if(o1==null) 
      return (o2==null)?0:-1;
    else if(o2 == null) return 1; 

    if( type.getSuperclass()==java.lang.Number.class) {
      d1 = ((Number)model.getValueAt(row1, column)).doubleValue();
      d2 = ((Number)model.getValueAt(row2, column)).doubleValue();

      if( d1<d2 ) return -1;
      else if( d1>d2 ) return 1;
      else return 0;
    }
    else if( type==java.util.Date.class || 
	     type.getSuperclass()==java.util.Date.class ) {
      l1 = ((Date)model.getValueAt(row1, column)).getTime();
      l2 = ((Date)model.getValueAt(row2, column)).getTime();

      if( l1<l2 ) return -1;
      else if( l1>l2 ) return 1;
      else return 0;
    }
    else if( type==String.class ) {
      s1 = (String)model.getValueAt(row1, column);
      s2 = (String)model.getValueAt(row2, column);
      return s1.compareTo(s2);
    }
    else if( type==Boolean.class ) {
      b1 = ((Boolean)model.getValueAt(row1, column)).booleanValue();
      b2 = ((Boolean)model.getValueAt(row2, column)).booleanValue();
      if (b1 == b2) return 0;
      else if (b1)  return 1;
      else return -1;
    }
    else {
      s1 = model.getValueAt(row1, column).toString();
      s2 = model.getValueAt(row2, column).toString();
      return s1.compareTo(s2);
    }
  }

  public int compare(int row1, int row2) {
    int result = compareRowsByColumn(row1, row2, sortingColumn);
    return ascending ? result : -result;
  }

  public void  reallocateIndices() {
    int rowCount = model.getRowCount();
    System.err.println( "Reallocate indices: "+rowCount+" rows." );
    indices = new int[rowCount];
    rev_indices = new int[rowCount];

    for(int row = 0; row < rowCount; row++){
      indices[row] = row;
      rev_indices[row] = row;
    }
  }

  public void tableChanged(TableModelEvent ev) {
    if( ev.getType()!=TableModelEvent.UPDATE || ev.getFirstRow()-ev.getLastRow() > 0 ) {  // bezpiecznik
      System.err.println( "Sorter: Insert/Delete/multiple update" );
      reallocateIndices();
      super.tableChanged(ev);
    }
    else {
      TableModelEvent event = new TableModelEvent( this,rev_indices[ev.getFirstRow()],rev_indices[ev.getFirstRow()],ev.getColumn() );
      super.tableChanged(event);
    }
  }

  private void adjustReverseIndices() {
    for( int i=0; i<indices.length; i++ ) {
      rev_indices[ indices[i] ] = i;
    }
  }

  // sorts the data and adjust selection
  public void  sort(Object sender) {
    // shuttlesort((int[])indices.clone(), indices, 0, indices.length);
    n2sort();
    adjustReverseIndices();
    if( selModel!=null ) {
      selModel.adjustSelection();
    }
  }

  public void swap( int x,int y ) {
    int pom = indices[x];
    indices[x] = indices[y];
    indices[y] = pom;
  }

  // sortowanie przez prosty wybor - powinno byc szybsze, niz rekurencyjne
  public void n2sort() {
    int pozmin,i,j;

    for( i=0; i<indices.length-1; i++ ) {
      pozmin = i;
      for( j=i+1; j<indices.length; j++ ) {
	if( compare(indices[pozmin],indices[j]) > 0 ) pozmin = j;
      }
      swap( i,pozmin );
    }      
  }

  public void shuttlesort(int from[], int to[], int low, int high) {
    if (high - low < 2) return;
    int middle = (low + high)/2;
    shuttlesort(to, from, low, middle);
    shuttlesort(to, from, middle, high);

    int p = low;
    int q = middle;

    if (high - low >= 4 && compare(from[middle-1], from[middle]) <= 0) {
      for (int i = low; i < high; i++) to[i] = from[i];
      return;
    }

    for(int i = low; i < high; i++) {
      if (q >= high || (p < middle && compare(from[p], from[q]) <= 0)) {
	to[i] = from[p++];
      }
      else to[i] = from[q++];
    }
  }

  public Object getValueAt(int row, int clmn) {
    return model.getValueAt(indices[row], clmn);
  }

  public int convertIndexToLowerLevel( int index ) {
    return indices[index];
  }

  public int convertIndexToHigherLevel( int index ) {
    return rev_indices[index];
  }

  public void setValueAt(Object val, int row, int clmn) {
    model.setValueAt(val,indices[row],clmn);
  }

  public void sortByColumn(int column) {
    sortByColumn(column, true);
  }

  public void sortByColumn(int column, boolean ascending) {
    this.ascending = ascending;
    sortingColumn = column;
    sort(this);
    super.tableChanged(new TableModelEvent(this)); 
  }

  public void addMouseListenerToHeaderInTable(JTable table) { 
    final TableSorter sorter = this; 
    final JTable tableView = table; 
    tableView.setColumnSelectionAllowed(false); 
    MouseAdapter listMouseListener = new MouseAdapter() {
      public void mouseClicked(MouseEvent ev) {
	TableColumnModel columnModel = tableView.getColumnModel();
	int viewColumn = columnModel.getColumnIndexAtX(ev.getX()); 
	int column = tableView.convertColumnIndexToModel(viewColumn); 
	if(ev.getClickCount() == 1 && column != -1) {
	  int shiftPressed = ev.getModifiers()&InputEvent.SHIFT_MASK; 
	  boolean ascending = (shiftPressed == 0); 
	  sorter.sortByColumn(column, ascending); 
	}
      }
    };
    JTableHeader th = tableView.getTableHeader(); 
    th.addMouseListener(listMouseListener); 
  }

  ////////////////////////////////////////////////////////////////////
  // Niezbyt ladna implementacja ListSelectionModel-u
  class SelModel extends DefaultListSelectionModel implements JabbaConstants {
    TableSorter parent;

    public SelModel( TableSorter parent ) {
      super();
      this.parent = parent;
    }    

    private boolean isSelected(int row) {
      return ((Boolean)parent.getValueAt(row,ATTRIBUTE_SELECTED)).booleanValue();
    }

    public void insertIndexInterval( int index, int len, boolean before ) {
      super.insertIndexInterval( index,len,before );
    }

    public void removeIndexInterval( int ind1, int ind2 ) {
      super.removeIndexInterval( ind1,ind2 );
    }

    public void addSelectionInterval( int ind0, int ind1 ) {
      if( ind0<0 || ind1<0 ) return;
      for( int i=ind0; i<=ind1; i++ ) {
	parent.setValueAt(Boolean.TRUE,i,ATTRIBUTE_SELECTED);
	super.addSelectionInterval(i,i);
      }
    }

    public void setSelectionInterval( int ind0,int ind1 ) {
      if( ind0>=0 && ind1>=0 ) removeSelectionInterval(getMinSelectionIndex(),getMaxSelectionIndex());
      addSelectionInterval( ind0,ind1 );
    }       


    public void removeSelectionInterval( int ind0, int ind1 ) {
      if( ind0<0 || ind1<0 ) return;
      for( int i=ind0; i<=ind1; i++ ) {
	parent.setValueAt(Boolean.FALSE,i,ATTRIBUTE_SELECTED);
	super.removeSelectionInterval(i,i);
      }
    }
    
    public void clearSelection() {
      removeSelectionInterval(getMinSelectionIndex(),getMaxSelectionIndex());
    }

    public boolean isSelectedIndex(int ind) {
      return isSelected(ind);
    }

    public void adjustSelection() {
	//      System.err.println( "adjustSelection() called." );
      for( int i=0; i<parent.indices.length; i++ ) 
	if( isSelected(i) ) {
	  addSelectionInterval(i,i);
	}
	else {
	  removeSelectionInterval(i,i);
	}
    }
  }

}
