package jabba;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.util.Vector;

import com.sun.java.swing.JFrame;
import com.sun.java.swing.JPanel;
import com.sun.java.swing.JLabel;
import com.sun.java.swing.JMenuBar;
import com.sun.java.swing.JMenu;
import com.sun.java.swing.JMenuItem;
import com.sun.java.swing.JSeparator;
import com.sun.java.swing.JScrollPane;
import com.sun.java.swing.JTable;
import com.sun.java.swing.JTabbedPane;
import com.sun.java.swing.UIManager;
import com.sun.java.swing.JOptionPane;
import com.sun.java.swing.WindowConstants;

import com.sun.java.swing.event.ChangeListener;
import com.sun.java.swing.event.ChangeEvent;

import com.sun.java.swing.UnsupportedLookAndFeelException;

class Viewer extends JPanel implements JabbaInterface {
  private Jabba jabba; // referencja do silnika

  // ramka glowna
  static JFrame frame=null;

  // komendy dla watku menu
  public static final int CMD_NO_COMMAND = 0;
  public static final int CMD_OPEN_CONSOLE = 1;

  private MenuThread menuThread;  // coby menu obslugiwane sprawniej bylo...
  private int menuCommand;

  // model danych
  private ViewerMainTableDataModel dataModel;

  // <TU> modele danych wyzszego poziomu (filtry) oraz (najwyzej) sorter </TU>

  // zakladki
  Vector tabs;
  ViewerTab currentTab;

  // content pane
  JTabbedPane tabbedPane;

  // fonty uzywane przez interfejs
  public static final Font defaultFont = new Font( "Dialog",Font.PLAIN,12 );
  public static final Font romanFont = new Font( "Serif", Font.PLAIN, 12 );
  public static final Font helveticaFont = new Font( "SansSerif", Font.PLAIN, 12 );
  public static final Font courierFont = new Font( "Monospaced", Font.PLAIN, 12 );

  Viewer( Jabba jabba ) {
    this( jabba, "Jabba 2.1" );
  }

  Viewer( Jabba jabba, String title ) {
    super( true );
    System.err.println( "Creating viewer..." );

    this.jabba = jabba;
    
    frame.setTitle( title );
    setName( "Viewer main panel" );
    setFont( defaultFont );

    jabba.registerInterface( this );
    System.err.println( "Registered user interface." );

    // rejestracja w konsoli sledzenia
    jabba.console.addWriter( this, "Viewer" );

    // tworzymy okno glowne programu
    frame.setJMenuBar( createMenu() );

    // tworzymy potrzebne watki
    menuThread = new MenuThread();
    menuThread.setPriority( Thread.MIN_PRIORITY );
    menuCommand = CMD_NO_COMMAND;
    menuThread.start();

    // tworzymy zakladki
    ViewerTab vt;
    tabs = new Vector();

    // dolaczamy glowna zakladke
    vt = createMainTab(null);
    tabs.addElement( vt );
    jabba.console.messageFrom( this,"Main tab added." );

    // zakladka opisu poszczegolnych pomieszczen
    vt = createLocationsTab(null);
    tabs.addElement( vt );
    jabba.console.messageFrom( this,"Locations tab added." );

    // zakladka - zestawienie sprzetu
    vt = createEquipmentTab(null);
    tabs.addElement( vt );
    jabba.console.messageFrom( this,"Equipment tab added." );

    // zakladka - mapka 
    vt = createMapTab(null);
    tabs.addElement( vt );
    jabba.console.messageFrom( this,"Map tab added." );

    // skladamy zakladki razem
    tabbedPane = new JTabbedPane();
    for( int i=0; i<tabs.size(); i++ ) {
      vt = (ViewerTab)(tabs.elementAt(i));
      tabbedPane.addTab( vt.getName(),vt.getPanel() );
    }

    // przelaczanie pomiedzy zakladkami
    tabbedPane.addChangeListener( createTabSelectionChangeListener() );

    setLayout( new BorderLayout() );
    add( tabbedPane, BorderLayout.CENTER  );

    switchToTab( (ViewerTab)(tabs.elementAt(0)) );
  }

  ViewerTab createMainTab( String name ) {
    if( name==null )
      return new ViewerMainTab( jabba,this );
    else return new ViewerMainTab( jabba,this,name );
  }

  ViewerTab createLocationsTab( String name ) {
    if( name==null ) 
      return new ViewerLocationsTab( jabba,this );
    else return new ViewerLocationsTab( jabba,this,name );
  }

  ViewerTab createEquipmentTab( String name ) {
    if( name==null ) 
      return new ViewerEquipmentTab( jabba,this );
    else return new ViewerEquipmentTab( jabba,this,name );
  }

  ViewerTab createMapTab( String name ) {
    if( name==null )
      return new ViewerMapTab( jabba,this );
    else return new ViewerMapTab( jabba,this,name );
  }

  DismissableViewerTab createPersonTab( String name, int person_index, Object user ) {
    if( name==null ) 
      return new ViewerPersonTab( jabba,this,person_index );
    else return new ViewerPersonTab( jabba,this,name,person_index );
  }

  DismissableViewerTab createSingleLocationTab( String name, int loc_index ) {
    if( name==null ) 
      return new ViewerSingleLocationTab( jabba,this,loc_index );
    else return new ViewerSingleLocationTab( jabba,this,name,loc_index );
  }

  void addTab( ViewerTab vt ) {
    tabs.addElement( vt );
    tabbedPane.addTab( vt.getName(),vt.getPanel() );
    tabbedPane.repaint();
  }

  void deleteTabAt( int vt ) {
    jabba.console.messageFrom( this, "destroyTab() called." );
    tabbedPane.removeTabAt( vt );
  }

  void selectTab( ViewerTab vt ) {
    jabba.console.messageFrom( this, "selectTab() called." );
    tabbedPane.setSelectedComponent( (Component)vt );
  }

  ViewerTab getMainTab() {
    return (ViewerTab)(tabbedPane.getComponentAt(0));
  }

  int getCurrentTabIndex() {
    return tabbedPane.getSelectedIndex();
  }

  ChangeListener createTabSelectionChangeListener() {
    return new ChangeListener() {
      public void stateChanged( ChangeEvent ev ) {
	JTabbedPane tabbedPane = (JTabbedPane)(ev.getSource());
	Component current = tabbedPane.getSelectedComponent();
	switchToTab( (ViewerTab)current );

	// <TEMP>
	// System.err.println( "Change Event: "+ev );
	// System.err.println(((ViewerTab)current).getName());
	// </TEMP>
      }
    }; 
  }
  
  void switchToTab( ViewerTab tab ) {
    currentTab = tab;
    currentTab.update();
  }

  // tworzy pasek menu glownego
  private JMenuBar createMenu() {
    JMenuBar retval = new JMenuBar();
    JMenu menu;
    JMenuItem mi;

    // tworzymy menu "Options"
    menu = new JMenu( "Options" );
    menu.setMnemonic( 'O' );
    
    // Options-->Show console...
    mi = new JMenuItem( "Show console..." );
    mi.setMnemonic( 'S' );
    mi.addActionListener( new ActionListener() {
      public void actionPerformed( ActionEvent ev ) {
        menuCommand = CMD_OPEN_CONSOLE;
	synchronized( menuThread ) {
          menuThread.notify();
	}
      }
    } );
    menu.add( mi ); 

    // Options--> ----------------
    menu.add( new JSeparator() );

    // Options--> Exit
    mi = new JMenuItem( "Exit Jabba" );
    mi.setMnemonic( 'x' );
    mi.addActionListener( new ActionListener() {
      public void actionPerformed( ActionEvent ev ) {
	exit( 0 );
      }
    } );
    menu.add( mi );

    // dodajemy menu "Options"
    retval.add( menu );

    return retval;
  }

  // odpowiada za pokazanie interfejsu
  public void display() {
    frame.pack();
    frame.validate();
    frame.setVisible(true);
    System.err.println( "Displayed." );
  }

  // reakcja na informacje o zadaniu wyjscia z programu
  void exit( int kod ) {
    jabba.exit( kod );
  }


  // watek sluzacy do realizacji bardziej czasochlonnych polecen menu 
  // (np. otwarcie konsoli)
  private class MenuThread extends Thread {
    public void run() {
      boolean stop=false;
      while( !stop ) {
        while( menuCommand==CMD_NO_COMMAND ) {
          try{
	    synchronized(this) {
	      wait();
	    }
	  }
          catch( InterruptedException e ) {}
        }

	// jesli juz wiemy, ze zbudzono nas nie bez powodu...
	switch( menuCommand ) {
	case CMD_OPEN_CONSOLE:
	  jabba.showConsole();
	  break;
	default:
	  break;
	}
	menuCommand = CMD_NO_COMMAND;
      }
    }
  }

  // do zmiany / wyrzucenia
  JabbaDataModel getDataModel() {
    return dataModel;
  }

  private static final int FRAME_WIDTH = 500;
  private static final int FRAME_HEIGHT = 400;
  // ta funkcja powinna byc uzywana do tworzenia nowego okna
  public static JFrame createFrame( Jabba jabba ) {
    System.err.println( "Creating frame..." );
    
    try {
      UIManager.setLookAndFeel(
	  UIManager.getCrossPlatformLookAndFeelClassName()
      );
    }
    catch( UnsupportedLookAndFeelException ex ) {
      jabba.console.exception( null,ex );
    }
    catch( IllegalAccessException ex ) {
      jabba.console.exception( null,ex );
    }
    catch( InstantiationException ex ) {
      jabba.console.exception( null,ex );
    }
    catch( ClassNotFoundException ex ) {
      jabba.console.exception( null,ex );
    }

    frame = new JFrame( );

    JOptionPane.setRootFrame( frame );
    
    frame.setCursor( Cursor.getPredefinedCursor(
			    Cursor.WAIT_CURSOR ) );

    Viewer viewer = new Viewer( jabba );

    frame.getContentPane().setLayout( new BorderLayout() );
    frame.getContentPane().add( viewer, BorderLayout.CENTER );

    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    frame.setLocation( (screenSize.width-FRAME_WIDTH)/2,
		       (screenSize.height-FRAME_HEIGHT)/2 );
    
    frame.setSize( FRAME_WIDTH,FRAME_HEIGHT );             
    frame.setCursor( Cursor.getPredefinedCursor(
			      Cursor.DEFAULT_CURSOR ) );

    frame.addWindowListener( new WindowAdapter() {
	public void windowClosing( WindowEvent ev ) {
	    System.exit(0);
	}
    } );

    return frame;
  }
}





