package jabba;

import com.sun.java.swing.table.AbstractTableModel;
import com.sun.java.swing.event.TableModelEvent;

/** 
 * Receives messages from Receiver object and directly accesses it.
 */
class ViewerMainTableDataModel extends AbstractJabbaTableDataModel implements JabbaConstants {
  private final static String[] columnNames = {
    "Name", "Nearest Phone", "Last seen", "Quality", "Seen at", "People #"
  };
  
  // przykladowe dlugie wartosci
  private final static Object[] longValues = {
    "Maria Komendera-Wasowicz", "9999999", "00-00-00", 
    QualityRenderer.getLongValue(), "ICS corridor", new Integer(77) };

  // ktora kolumna gdzie (indeksy w tablicy columnNames)
  final static int COLUMN_NAME = 0;
  final static int COLUMN_LOCATION_NAME = 4;
  final static int COLUMN_PHONE = 1;
  final static int COLUMN_QUALITY = 3;
  final static int COLUMN_TIME = 2;
  final static int COLUMN_PEOPLE = 5;

  // glowny obiekt programu
  Jabba jabba;

  // obiekt przechowujacy dane
  Receiver receiver;

  ViewerMainTableDataModel( Jabba jabba ) {
    super();
    this.jabba = jabba;
    receiver = jabba.receiver;
    jabba.console.addWriter( this, "1st main table model" );
  }

  // pomocnicza w okresleniu szerokosci kolumn
  public Object getLongValue( int clmn ) {
    return longValues[clmn];
  }

  public int getColumnCount() {
    return columnNames.length;
  }

  public int getRowCount() {
    return receiver.getObservedObjectsCount();
  }

  public Class getColumnClass(int col) {
    Object o = getValueAt(0,col);
    if( o!=null ) return o.getClass();
    else return java.lang.String.class;
  }

  public Object getValueAt( int row, int clmn ) {
    switch( clmn ) {
    case COLUMN_NAME: 
      return receiver.getUserFullName(row);
    case COLUMN_LOCATION_NAME: 
      return receiver.getUserLocationDescription(row);
    case COLUMN_PHONE:
      return receiver.getUserLocationPhones(row);
    case COLUMN_QUALITY: 
      return receiver.getUserSightingQuality(row);
    case COLUMN_PEOPLE: 
      return receiver.getUserLocationPeopleNumber(row);
    case COLUMN_TIME:
      return receiver.getUserSightingTime(row);

    case ATTRIBUTE_INDEX:            
      return new Integer(row);
    case ATTRIBUTE_REFERENCE:
      return receiver.getUserRecordReference(row);
    case ATTRIBUTE_SELECTED:
      return receiver.getUserRecordSelected(row);

    default: break;
    }

    return new String("N.A.");
  }

  public void setValueAt( Object value, int row, int clmn ) {
    switch( clmn ) {
    case ATTRIBUTE_SELECTED:
      receiver.setUserRecordSelected(row,(Boolean)value);
      break;
    default: 
      break;
    }
  }

  public String getColumnName( int clmn ) {
    return columnNames[clmn];
  }

  // implementacja JabbaDataModel
  public void user_nameChanged(int pos) {
    jabba.console.messageFrom( this, "nameChanged("+pos+") called." );
    fireTableCellUpdated( pos,COLUMN_NAME );
  }

  public void user_surnameChanged(int pos) {
    jabba.console.messageFrom( this, "surnameChanged("+pos+") called." );
    fireTableCellUpdated( pos,COLUMN_NAME );
  }

  public void user_locationChanged(int pos) {
    jabba.console.messageFrom( this, "locationChanged("+pos+") called." );
    fireTableCellUpdated( pos,COLUMN_LOCATION_NAME );
  }

  public void user_phoneChanged(int pos) {
    jabba.console.messageFrom( this, "phoneChanged("+pos+") called." );
    fireTableCellUpdated( pos,COLUMN_PHONE );
  }

  public void user_qualityChanged(int pos) { 
    jabba.console.messageFrom( this, "qualityChanged("+pos+") called." );
    fireTableCellUpdated( pos,COLUMN_QUALITY );
  }

  public void user_peopleNumberChanged(int pos) {
    jabba.console.messageFrom( this, "peopleNumberChanged("+pos+") called." );
    fireTableCellUpdated( pos,COLUMN_PEOPLE );
  }
  
  public void user_timeChanged(int pos) {
    jabba.console.messageFrom(this, "timeChanged("+pos+") called." );
    fireTableCellUpdated( pos,COLUMN_TIME );
  }

  public void user_added(int pos) {
    fireTableRowsInserted(pos-1,pos);
  }

  public void user_removed(int pos) {
    fireTableRowsDeleted(pos,pos);
  }

}




