package jabba;

//
// DO ZROBIENIA:
// - model danych odbierajacy komunikaty od receivera
//   - update'owanie listy uzytkownikow bedacych w tym samym pomieszczeniu
// - rejestracja modelu w metodzie regenerate()
//

import com.sun.java.swing.JPanel;
import com.sun.java.swing.Box;
import com.sun.java.swing.BoxLayout;
import com.sun.java.swing.JLabel;
import com.sun.java.swing.JButton;
import com.sun.java.swing.JTable;
import com.sun.java.swing.JScrollPane;
import com.sun.java.swing.table.TableModel;
import com.sun.java.swing.event.TableModelEvent;
import com.sun.java.swing.event.TableModelListener;

import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

class ViewerPersonTab extends JPanel implements DismissableViewerTab {
  Jabba jabba;
  Receiver receiver;
  Viewer viewer;
  String name;
  ViewerTab parentTab;
  int personIndex;

  // model danych 
  ViewerPersonTabDataModel dataModel;

  // elementy interfejsu (miedzymordzia, jak kto woli)
  JButton dismissButton;

  // elementy zmieniajace sie
  JLabel nameLabel, surnameLabel, positionLabel, emailLabel, phoneLabel, timeLabel, qualityLabel;
  JLabel locationLabel;
  JTable peopleTable;
  Box insidePanel;

  ViewerPersonTab( Jabba jabba, Viewer viewer, int index ) {
    this( jabba,viewer,null,"Person",index );
  }

  ViewerPersonTab( Jabba jabba, Viewer viewer, ViewerTab parent, int index ) {
    this( jabba,viewer,parent,"Person",index );
  }

  ViewerPersonTab( Jabba jabba, Viewer viewer, String name, int index ) {
    this( jabba,viewer,null,name,index );
  }

  ViewerPersonTab( Jabba jabba, Viewer viewer, ViewerTab parent, 
		   String name, int index ) {
    super();
    this.jabba = jabba;
    this.viewer = viewer;
    this.receiver = jabba.receiver;
    this.name = name;
    parentTab = parent;
    personIndex = index;

    this.setLayout( new BorderLayout() );
  }

  public String getName() {
    return name;
  }

  public JPanel getPanel() {
    return this;
  }

  public void regenerate() {
    // model
    dataModel = new ViewerPersonTabDataModel( jabba,this );
    jabba.receiver.registerDataModel( dataModel, Receiver.SINGLE_USER_MESSAGES, personIndex );

    // elementy interfejsu
    removeAll();
    add( createLocationDataPanel(), BorderLayout.NORTH );
    add( insidePanel=createInsidePanel(), BorderLayout.CENTER );
    add( createControlPanel(), BorderLayout.SOUTH );
  }

  public void update() {
    // ...temp
    jabba.console.messageFrom( this, "update() called." );
    regenerate();
  }

  public JabbaDataModel getJabbaDataModel() {
    return dataModel;
  }

  Box createInsidePanel() {
    Box retval = new Box( BoxLayout.X_AXIS );
    TableModel stm = new SimpleTableModel( jabba,jabba.receiver.peopleIdsForUser(personIndex), "People here: " );

    stm.addTableModelListener( new TableModelListener() {
      public void tableChanged(TableModelEvent ev) {
	if( ev.getType()==TableModelEvent.DELETE && insidePanel!=null )
	  insidePanel.paintAll(insidePanel.getGraphics());
      }
    } );

    TableSorter ts = new TableSorter(stm);
    peopleTable = new JTable(ts);
    ts.addMouseListenerToHeaderInTable(peopleTable);
    JScrollPane scrTable = JTable.createScrollPaneForTable(peopleTable);

    // UWAGA! Lista powinna byc update'owana przez model

    retval.add( scrTable );
    return retval;
  }

  Box createLocationDataPanel() {
    Box retval = new Box( BoxLayout.X_AXIS );
    Box categories1 = new Box( BoxLayout.Y_AXIS );
    Box categories2 = new Box( BoxLayout.Y_AXIS );
    Box values1 = new Box( BoxLayout.Y_AXIS );
    Box values2 = new Box( BoxLayout.Y_AXIS );


    categories1.add( new JLabel( "Name: " ) );
    categories1.add( new JLabel( "Surname: " ) );
    categories1.add( new JLabel( "Position: " ) );
    categories1.add( new JLabel( "Email: " ) );
    categories1.add( new JLabel( "Nearest phone: " ) );

    categories2.add( new JLabel( "Last location:" ) );
    categories2.add( new JLabel( "Sighting time:" ) );
    categories2.add( new JLabel( "Sighting quality:" ) );

    values1.add( nameLabel = new JLabel( receiver.getUserName(personIndex)+"   " ) );
    values1.add( surnameLabel = new JLabel( receiver.getUserSurname(personIndex)+"   " ) );
    values1.add( positionLabel = new JLabel( receiver.getUserPosition(personIndex)+ "    " ) );
    values1.add( emailLabel = new JLabel( receiver.getUserEmail(personIndex)+"    " ) );
    values1.add( phoneLabel = new JLabel( "617 "+ receiver.getUserLocationPhones(personIndex)+"   " ) );

    values2.add( locationLabel = new JLabel( receiver.getUserLocationDescription(personIndex)+"            " ) );
    values2.add( timeLabel = new JLabel( ""+receiver.getUserSightingTime(personIndex)+"    " ) );
    values2.add( qualityLabel = new JLabel( ""+receiver.getUserSightingQuality(personIndex) +"%    " ) );

    retval.add( categories1 );
    retval.add( values1 );
    retval.add( categories2 );
    retval.add( values2 );
    return retval;
  }

  JPanel createControlPanel() {
    JPanel retval = new JPanel( new FlowLayout( FlowLayout.RIGHT ) );

    dismissButton = new JButton( "Dismiss" );
    dismissButton.addActionListener( new ActionListener() {
      public void actionPerformed( ActionEvent ev ) {
	dismiss();
      }
    } );
    retval.add( dismissButton );

    return retval;
  }

  public void setParentTab( ViewerTab vt ) {
    parentTab = vt;
  }

  public void dismiss() {
    ViewerTab nextTab = (parentTab!=null)? parentTab : viewer.getMainTab();
    int my_index = viewer.getCurrentTabIndex();

    viewer.selectTab( nextTab );
    viewer.deleteTabAt( my_index );
  }



}
