package jabba;

import com.sun.java.swing.JButton;
import com.sun.java.swing.JPanel;
import com.sun.java.swing.JLabel;
import com.sun.java.swing.Box;
import com.sun.java.swing.BoxLayout;
import com.sun.java.swing.JTable;
import com.sun.java.swing.JScrollPane;
import com.sun.java.swing.table.TableModel;
import com.sun.java.swing.event.TableModelEvent;
import com.sun.java.swing.event.TableModelListener;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

class ViewerSingleLocationTab extends JPanel implements DismissableViewerTab {
  Jabba jabba;
  Viewer viewer;
  Receiver receiver;
  String name;
  ViewerTab parentTab;
  int locationIndex;

  // elementy interfejsu
  JButton dismissButton;

  // elementy danych zmiennych
  JLabel numberLabel;
  JLabel descriptionLabel;
  JLabel phonesLabel;

  JTable peopleTable;
  Box insidePanel;

  // model danych
  ViewerSingleLocationTabDataModel dataModel;

  ViewerSingleLocationTab( Jabba jabba, Viewer viewer, int index ) {
    this( jabba,viewer,null,"Location",index );
  }

  ViewerSingleLocationTab( Jabba jabba, Viewer viewer, ViewerTab parent, int index ) {
    this( jabba,viewer,parent,"Location",index );
  }

  ViewerSingleLocationTab( Jabba jabba, Viewer viewer, String name, int index ) {
    this( jabba,viewer,null,name,index );
  }

  ViewerSingleLocationTab( Jabba jabba, Viewer viewer, ViewerTab parent,
			   String name, int index ) {
    super();
    this.jabba = jabba;
    this.viewer = viewer;
    receiver = jabba.receiver;
    this.name = name;
    parentTab = parent;
    locationIndex = index;

    setLayout( new BorderLayout() );
  }

  public String getName() {
    return name;
  }

  public JPanel getPanel() {
    return this;
  }

  public void regenerate() {
    dataModel = new ViewerSingleLocationTabDataModel(jabba,this);
    jabba.receiver.registerDataModel( dataModel, Receiver.SINGLE_LOCATION_MESSAGES, locationIndex );

    removeAll();
    add( createLocationDataPanel(), BorderLayout.NORTH );
    add( insidePanel=createInsidePanel(), BorderLayout.CENTER );
    add( createControlPanel(), BorderLayout.SOUTH );
  }

  public void update() {
    // ...tmp
    regenerate();
  }

  public JabbaDataModel getJabbaDataModel() {
    return null;
  }


  // ponizsze nieladne - duplikacja kodu z ViewerPersonTab
  Box createInsidePanel() {
    Box retval = new Box( BoxLayout.X_AXIS );
    TableModel stm = new SimpleTableModel( jabba,jabba.receiver.peopleIdsForLocation(locationIndex), "People here: " );
    stm.addTableModelListener( new TableModelListener() {
      public void tableChanged(TableModelEvent ev) {
	if( ev.getType()==TableModelEvent.DELETE && insidePanel!=null )
	  insidePanel.paintAll(insidePanel.getGraphics());
      }
    } );
    TableSorter ts = new TableSorter(stm);
    peopleTable = new JTable(ts);
    ts.addMouseListenerToHeaderInTable(peopleTable);
    JScrollPane scrTable = JTable.createScrollPaneForTable(peopleTable);

    // UWAGA! Lista powinna byc update'owana przez model

    retval.add( scrTable );
    return retval;
  }

  Box createLocationDataPanel() {
    Box retval = new Box( BoxLayout.X_AXIS );
    Box categories = new Box( BoxLayout.Y_AXIS );
    Box values = new Box( BoxLayout.Y_AXIS );

    categories.add( new JLabel( "Number: " ) );
    categories.add( new JLabel( "Description: " ) );
    categories.add( new JLabel( "Phones: " ) );

    values.add( numberLabel = new JLabel( receiver.getLocationNumber(locationIndex) +"    " ) );
    values.add( descriptionLabel = new JLabel( receiver.getLocationDescription(locationIndex) +"        " ) );
    values.add( phonesLabel = new JLabel( receiver.getLocationPhones(locationIndex) +"            " ) );

    retval.add( categories );
    retval.add( values );
    return retval;
  }

  JPanel createControlPanel() {
    JPanel retval = new JPanel( new FlowLayout( FlowLayout.RIGHT ) );

    dismissButton = new JButton( "Dismiss" );
    dismissButton.addActionListener( new ActionListener() {
      public void actionPerformed( ActionEvent ev ) {
	dismiss();
      }
    } );
    retval.add( dismissButton );

    return retval;
  }
  
  public void setParentTab( ViewerTab vt ) {
    parentTab = vt;
  }

  public void dismiss() {
    ViewerTab nextTab = (parentTab!=null)? parentTab: viewer.getMainTab();
    int my_index = viewer.getCurrentTabIndex();

    viewer.selectTab( nextTab );
    viewer.deleteTabAt( my_index );
  }


}


