#include <rw/rwtime.h>
#include <rw/rwdate.h>

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>


#include "abng_core_management.hh"
#include "bootstrap_n.hh"

#include "whlib.hh"

class whereis_args{
public:
  whereis_args();
  ~whereis_args();

  const char *getName() const { return username; };
  const char *getView() const { return viewname; };
  int      getMessage() const { return msg;      };
  int      getLong()    const { return lformat;  };  
  void setName(const char *newstr);
  void setView(const char *newstr);
  void setMessage(int val) { msg = val; };
  void setLong(int val) { lformat =val; };
protected:
  char *viewname;
  char *username;
  int lformat;
  int msg;
};

char *defaultview = "viewroom";

whereis_args::whereis_args() 
{
  viewname = username = NULL;
  lformat = 0;
  msg = 0;
};

whereis_args::~whereis_args()
{
  if (viewname)
    free(viewname);
  if (username)
    free(username);
}

void whereis_args::setName(const char *newstr)
{
  if (username) 
    free(username);
  username=(newstr) ? strdup(newstr) : NULL;
}

void whereis_args::setView(const char *newstr)
{
  if (viewname)
    free(viewname);
  viewname=(newstr) ? strdup(newstr) : NULL;
}

void usage( char *prog_name) {
  cout << prog_name << endl
       << "\t[-m] [-l] [-v view_name] username" << endl;
}

void my_parseargs(int argc, char *argv[], whereis_args *wargs)
{
  int c;
  char *prgname = argv[0];
  wargs->setView(defaultview);
  wargs->setName(NULL);

  while (( c = getopt( argc, argv, "mlv:")) != EOF)
  {
    switch (c)
    {
      case 'm':
	wargs->setMessage(1);
	break;
      case 'l':
	wargs->setLong(1);
	break;
      case 'v':
	wargs->setView(strdup(optarg));
	break;
      default:
	usage(prgname);
	exit(1);
    }
  }

  if ( optind < argc )
    wargs->setName(argv[optind]);
  else
  {
    usage(prgname);
    exit(2);
  }
}

int main(int argc, char *argv[])
{
  whereis_result *result = NULL;
  whereis_args *wargs = new whereis_args();

  my_parseargs(argc, argv, wargs);

  whereis_tools *wtools = new whereis_tools(argc, argv);

  if ( wtools->connect(wargs->getView()) &&
       (result = wtools->findUser(wargs->getName(),
				  WH_FU_NULL | 
				  (wargs->getMessage() ? WH_FU_AWAY : 0) |
				  (wargs->getLong() ? WH_FU_NEIGHB : 0))) )
  {
    const char *str;
    str=result->getName();
    cout << "uid:\t\t" << (str ? str : "(null)") << endl;
    str=result->getForeName();
    cout << "Name:\t\t" << (str ? str : "(null)") << endl;
    str=result->getSurName();
    cout << "SurName:\t" << (str ? str : "(null)") << endl;
    str=result->getPosition();
    cout << "Position:\t" << (str ? str : "(null)") << endl;

    str=result->getLastSeenStr();
    cout << "Last seen:\t" << (str ? str : "(null)");
    cout << "Quality:\t" << result->getQuality() << endl;
    str=result->getLocType();
    cout << "At " << (str ? str : "(null)") << ":\t(";
    str=result->getRoomNum();
    cout << (str ? str : "(null)") << ") ";
    str=result->getRoomName();
    cout << (str ? str : "(null)") << endl;  
    if (result->getAway())
      cout << "Message:\t" << result->getAway() << endl; 
    cout << "People in room:\t" << result->getRoomPeople() << endl; 

    for (int i=0; i < result->getRoomPeople()-1; i++)
    {
      const whereis_result *result2;
      if ( (result2=result->getRoomPeople(i)) )
      {
	str=result2->getName();
	cout << "\tuid:\t\t" << (str ? str : "(null)") << endl;  
	str=result2->getForeName();
	cout << "\tName:\t\t" << (str ? str : "(null)") << endl;  
	str=result2->getSurName();
	cout << "\tSurName:\t" << (str ? str : "(null)") << endl;  
	str=result2->getPosition();
	cout << "\tPosition:\t" << (str ? str : "(null)") << endl;  
      }
      cout << endl;
    }
  }

  exit(0);

  delete result;
  wtools->disconnect();
  delete wtools;
  delete wargs;
  
  exit (0);
  return 0;
}


