// -*- Mode: C++; -*-
// $Id: basic_observed_i.h,v 1.2 1998/07/29 13:14:46 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/common/basic_observed_i.h,v $
//
// $Log: basic_observed_i.h,v $
// Revision 1.2  1998/07/29 13:14:46  mencnar
// time synchronisation,
// persistent observers,
// notify calls moved to another thread ( fixed deadlock)
// internal server names changed
//
// Revision 1.1  1998/06/16 12:56:16  mencnar
// Initial revision
//
//
// File:          basic_observed.h
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  02.02.98
//
// General Description:
//  Declaration of the template Basic_Observed_I class which implements
//  operations common for all observed ABng Repositories.

#ifndef __BASIC_OBSERVED_I_H__
#define __BASIC_OBSERVED_I_H__

#include <rw/tvhasht.h>
#include <rw/tvhdict.h>
#include "abng_guarded.hh"
#include "compatibility.hh"


#include "DO_ptr.h"


template <class Observer_Types>
class Basic_Observed_I : public virtual ABng_Guarded  {
  ABng_Guarded ic_guard;
  Incarnation_Number ic;
  int loading_observers;

  static void static_gc_worker( void *args);

  const int save_observers_latency;
  int save_observers_needed;
  void do_save_observers();
  static void static_save_observers_worker( void *args);
  ABng_S_Guarded save_observers_guard;
  omni_condition save_observers_cond;
protected:
  virtual void gc_worker();
public:

  Basic_Observed_I(
#ifdef FilePersistency
    const char *repository_name_p
#endif
  );

  virtual ~Basic_Observed_I();


  typedef Observer_Types::Observed Observed;
  typedef Observer_Types::Observer Observer;
  typedef Observer_Types::Observers Observers;
  typedef Observer_Types::Observer_ptr Observer_ptr;
  typedef Observed::Observer_Entries Observer_Entries;
  typedef DO_ptr<Observer_Types::Observer_ptr> SDO_ptr;

  typedef RWTValHashTable<Observer_ptr> Observer_List;
  typedef RWTValHashTableIterator<Observer_ptr> Observer_List_Iterator;

  Incarnation_Number get_new_ic() {
    auto_lock local_lock( ic_guard);
    return ++ic;
  } 

  virtual Observer_Entries *
    observer_list (
    ENV_DECL1(env))
    throw (CORBA::SystemException);
             
  virtual void attach_observer (
    Observer_ptr obs,  
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env))
    throw (CORBA::SystemException, 
	   Domain_Repository::Unknown_Repository_Item_Ids);
 
  virtual void detach_observer (
    Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) 
    throw (CORBA::SystemException, 
	   Domain_Repository::Unknown_Repository_Item_Ids, 
	   Domain_Repository::Unknown_Observer,
	   Domain_Repository::Unmatched_Observer_Item_Ids);

  virtual void private_attach_observer (
    Observer_ptr obs,  
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env))
    throw (CORBA::SystemException, 
	   Domain_Repository::Unknown_Repository_Item_Ids);

  virtual void private_detach_observer (
    Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) 
    throw (CORBA::SystemException, 
	   Domain_Repository::Unknown_Repository_Item_Ids, 
	   Domain_Repository::Unknown_Observer,
	   Domain_Repository::Unmatched_Observer_Item_Ids);

  virtual void private_detach_add_observer (
    Observer_ptr obs
    ENV_DECL(env)) {}

  virtual Domain_Repository::Repository_Item_Ids get_item_ids_list(
    Observer_ptr obs);

  virtual Observers* get_item_observers (
    Domain_Repository::Repository_Item_Id item_id_p
    ENV_DECL(env)) 
    throw (CORBA::SystemException);

  virtual Observer_List *get_item_observer_list (
    Domain_Repository::Repository_Item_Id item_id_p);

  virtual void insert_new_item(
    Domain_Repository::Repository_Item_Id id_p);

  virtual void detach_item_from_observers(
    Domain_Repository::Repository_Item_Id id_p);

  void save_observers();
  virtual void save_observers_to_stream( ostream &ostr);
  void load_observers();
  virtual void load_observers_from_stream( istream &istr);
protected:

  // Calculate the hash value of the observer pointer.
  static unsigned 
    hash_observer(const Observer_ptr& observer_p);

  // Calculate the hash value of the item id.
  static unsigned hash_item_id(
    const Domain_Repository::Repository_Item_Id& item_id_p);

  void add_proxy(Observer_ptr &obs);
  virtual void remove_proxy(Observer_ptr obs);

  typedef RWTValHashTable<Domain_Repository::Repository_Item_Id> 
    Item_Id_List;
  typedef RWTValHashTableIterator<Domain_Repository::Repository_Item_Id> 
    Item_Id_List_Iterator;
  typedef RWTValHashDictionary <Observer_ptr, Item_Id_List*> 
    Observer_Item_Dictionary;
  typedef RWTValHashDictionaryIterator <Observer_ptr, Item_Id_List*> 
    Observer_Item_Dictionary_Iterator;
  typedef RWTValHashDictionary<Domain_Repository::Repository_Item_Id,
    Observer_List*> Item_Observer_Dictionary;
  typedef RWTValHashDictionaryIterator<Domain_Repository::Repository_Item_Id,
    Observer_List*> Item_Observer_Dictionary_Iterator;

  // Mapping of observers to items.
  Observer_Item_Dictionary observers_to_item_ids_list;
  // Mapping of items to observers 
  Item_Observer_Dictionary item_ids_to_observers_list;

  typedef RWTValHashTable<SDO_ptr> Proxy_List;
  typedef RWTValHashTableIterator<SDO_ptr> Proxy_List_Iterator;
  Proxy_List proxy_table;
  char *persistent_observers_rep;
};

#endif /* __BASIC_OBSERVED_I_H__ */








