// $Id: basic_rep_i.cc,v 1.2 1998/07/29 13:14:47 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/common/basic_rep_i.cc,v $
//
// $Log: basic_rep_i.cc,v $
// Revision 1.2  1998/07/29 13:14:47  mencnar
// time synchronisation,
// persistent observers,
// notify calls moved to another thread ( fixed deadlock)
// internal server names changed
//
// Revision 1.1  1998/06/16 12:55:00  mencnar
// Initial revision
//
//
// File:          basic_i.cc
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  02.02.98
//
// General Description:
//  Definition of the template implementaition base class for 
//  ABng repositories.

#define EXCEPTIONS

#include "basic_rep_i.h"
#include <rw/tvdlist.h>

template <class Item_Types, class Observer_Types>
No_Update_Basic_Rep_I<Item_Types, Observer_Types>::No_Update_Basic_Rep_I(
  unsigned (*hash_item_p)(const Item_I& item),
#ifdef ObjectStore
    os_database* db_p
#elif FilePersistency
    const char *repository_name_p
#endif
  ) : hash_item(hash_item_p),
  my_item_list(hash_item_p),
  my_item_deref(hash_item_id),
  item_counter(1),
  notifier_cond( & (notifier_guard.mutex())),
  Observed_I<Observer_Types>(
#ifdef FilePersistency
     repository_name_p
#endif
     )
{
#ifdef ObjectStore
  my_db =  db_p;
#elif FilePersistency
  my_repository_name = new char[strlen(repository_name_p) + 1];
  (void) strcpy(my_repository_name, repository_name_p);
#endif

  (new omni_thread( static_notify_worker, this))->start();
}

template <class Item_Types, class Observer_Types>
void No_Update_Basic_Rep_I<Item_Types, Observer_Types>::init(int cold_start) {

  // This cannot be done in the constructor, because it would have called
  // pure virtual functions redefined in a derived class.

#ifdef FilePersistency
  // open (or create) repository file
  ifstream my_rep(my_repository_name);
  if (!my_rep.rdbuf()->is_open()) {
    if(cold_start) {
      ofstream create_rep(my_repository_name);
      save();
      if(!create_rep) {
	cerr << "Cannot write the database file:" << endl;
	cerr << "\t" << my_repository_name << endl;
	exit(1);
      }
      my_rep.open(my_repository_name, ios::in );
      DB_Header db_header;
      if (!(my_rep >> db_header)) {
	cerr << "Cannot read the created database file:" << endl;
	cerr << "\t" << my_repository_name << endl;
	cerr << "It has improper access rigts or has been corrupted." << endl;
	exit(1); // after startup, the database will be empty
      }
    }
    else {
      cerr << "Cannot open the database file:" << endl;
      cerr << "\t" << my_repository_name << endl;
      cerr << "Use the cold start option to create the database."<< endl;
      exit(1);
    }
  }
  else {
    if(cold_start) {
     cerr << "Cannot perform the cold start because the database file:" << endl;
      cerr <<"\t" << my_repository_name << endl;
      cerr << "already exists. Remove the file or start the server without the cold start" << endl;
      cerr << "option." << endl;
      exit(1);
    }
    else {
      restore(my_rep);
      if (!my_rep) {
	cerr << "Cannot read the database file:" << endl;
	cerr << "\t" << my_repository_name << endl;
	cerr << "It has improper access rigts or has been corrupted." << endl;
	exit(1); // after startup, the database will be empty    
      }
    }
  }
#endif
}

template <class Item_Types, class Observer_Types>
No_Update_Basic_Rep_I<Item_Types, Observer_Types>::~No_Update_Basic_Rep_I() {

#ifdef FilePersistency
  delete [] my_repository_name;
#endif
}

// Implementation of the accessor for the item_list IDL attribute

template <class Item_Types, class Observer_Types>
No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Items 
  *No_Update_Basic_Rep_I<Item_Types, Observer_Types>::item_list (
  ENV_DEF1(env)) {

  // initialize the sequence for collected location_types
  Items* collected_items =
    new Items(my_item_list.entries());

  // browse through hash table to collect location_types
  my_item_list.apply(gather_item, (void*)collected_items);
  return collected_items;
}

// Implementation of the find IDL operation

template <class Item_Types, class Observer_Types>
No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Items
  *No_Update_Basic_Rep_I<Item_Types, Observer_Types>::find(
  const No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Pattern& data_record_pattern
  ENV_DEF(env)) {

  auto_lock local_lock(*this);
  return private_find(data_record_pattern USE_ENV(env));
}

template <class Item_Types, class Observer_Types>
No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Value_Descriptions
  *No_Update_Basic_Rep_I<Item_Types, Observer_Types>::find_values (
  const No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Pattern& data_record_pattern
  ENV_DEF(env)) {

  auto_lock local_lock(*this);
  return private_find_values(data_record_pattern USE_ENV(env));
}

template <class Item_Types, class Observer_Types>
No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Value_Descriptions
  *No_Update_Basic_Rep_I<Item_Types, Observer_Types>::private_find_values (
  const No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Pattern& data_record_pattern
  ENV_DEF(env)) {

  // Collect references of matching items 
  Items* matching_items = private_find(data_record_pattern USE_ENV(env));

  try {
    Value_Descriptions* collected_items_values=
      collect_value_descriptions( *matching_items);
    
    // delete the seqeance of matching items
    
    delete matching_items;
    return collected_items_values;
  }
  catch (...) {
    delete matching_items;
    throw;
  }
}

template <class Item_Types, class Observer_Types>
No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Value_Descriptions
  *No_Update_Basic_Rep_I<Item_Types, Observer_Types>::collect_value_descriptions( 
	    No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Items &matching_items) 
{
  // Number of matching items
  int matching_items_length = matching_items.length();

  // Initialize the sequence for collected item values
  Value_Descriptions* collected_items_values =
    new Value_Descriptions(matching_items_length);

  // Length of collected_items is equal to matching_itemss
  collected_items_values->length(matching_items_length);
  try {
    // Copy values from items to collected_items sequence
    Item_I *cpp_item;
    for(int i = 0; i < matching_items_length; i++){
      cpp_item = my_item_deref[matching_items[i]->item_id()];
      if (!cpp_item) {
	continue;
      }
      collect_values(cpp_item, (*collected_items_values)[i]);
    }
  }
  catch (...) {
    delete collected_items_values;
    throw;
  }
  return collected_items_values;
}

template <class Item_Types, class Observer_Types>
No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Query_Results*
 No_Update_Basic_Rep_I<Item_Types, Observer_Types>::complex_query (
  const No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Entity_Set& es,
  const Domain_Repository::Query_Options &qo,
  No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Observer_ptr obs
  ENV_DEF(env)) {

  auto_lock local_lock(*this);

#ifdef DEBUG
  cerr << "complex_query called" << endl;
#endif

  RWTValDlist<Item_I*> my_set;

  switch (es._d()) {
  case Domain_Repository::ESC_Pattern:
    {
      Items *fres= find( es.s_pattern());
      for (int i=0; i<fres->length(); i++)
	my_set.insert( deref_item( (*fres)[i]));
      delete fres;
    }
  break;
  case Domain_Repository::ESC_Ids: 
    {
      const Domain_Repository::Repository_Item_Ids &ids= es.s_ids();
      for (int i=0; i<ids.length(); i++) 
	if (my_item_deref.contains( ids[i]))
	  my_set.insert( my_item_deref[ ids[i]]);
	else
	  my_set.insert( NULL);
    }
  break;
  case Domain_Repository::ESC_Refs:
    {
      const Items &fres= es.s_refs();
      for (int i=0; i<fres.length(); i++) {
	try {
	  my_set.insert( deref_item( fres[i]));
	}
	catch (...) {
	  my_set.insert( NULL);
	}
      }
    }
  break;
  default:
    return new Query_Results();
    break;
  }

  Query_Results *ret= new Query_Results();
  try {
    int len= my_set.entries();

    if (qo.return_references)
      ret->r_references.length( len);
    if (qo.return_descriptions) {
      ret->r_descriptions.length( len);
      ret->r_valid_descriptions.length( len);
    }
    if (qo.return_value_descriptions) {
      ret->r_value_descriptions.length( len);
      ret->r_valid_value_descriptions.length( len);
    }
    if (qo.do_attach)
      ret->r_attach_results.length( len);
    if (qo.do_detach)
      ret->r_detach_results.length( len);

    Domain_Repository::Repository_Item_Ids tmpids(1);
    tmpids.length(1);

    int i=0;
    for (RWTValDlistIterator<Item_I*> iter( my_set);
	 iter(); 
	 i++) {
      Item_I *item= iter.key();

      if (qo.return_references) {
	if (item) ret->r_references[i]= Item::_duplicate( item->get_corba_object());
	else ret->r_references[i]= Item::_nil();
      }
      if (qo.return_descriptions) {
	ret->r_valid_descriptions[i]= 0;
	if (item) {
	  try {
	    Description *desc= item->descr();
	    ret->r_descriptions[i]= *desc;
	    ret->r_valid_descriptions[i]= 1;
	    delete desc;
	  }
	  catch (...) {
	    ret->r_descriptions[i]= get_null_description( item);
	  }
	}
	else ret->r_descriptions[i]= get_null_description( item);
      }
      if (qo.return_value_descriptions) {
	ret->r_valid_value_descriptions[i]= 0;
	if (item) {
	  try {
	    Value_Description *v_desc= item->value_descr();
	    ret->r_value_descriptions[i]= *v_desc;
	    ret->r_valid_value_descriptions[i]= 1;
	    delete v_desc;
	  }
	  catch (...) {
	    ret->r_value_descriptions[i]= get_null_value_description( item);
	  }
	}
	else ret->r_value_descriptions[i]= get_null_value_description( item);
      }

      if (qo.do_attach) {
	ret->r_attach_results[i]= (CORBA::Boolean) 0;
	try {
	  if (item) {
	    tmpids[0]= item->item_id();
	    attach_observer( obs,
			     tmpids);
	    ret->r_attach_results[i]= (CORBA::Boolean) 1;
#ifdef DEBUG
	    cerr << "Attach succeeded" << endl;
#endif
	  }
	}
	catch (Domain_Repository::Unknown_Repository_Item_Ids ) {
#ifdef DEBUG
	  cerr << "Attach failed" << endl;
#endif
	}
      }
      if (qo.do_detach) {
	ret->r_detach_results[i]= (CORBA::Boolean) 0;
	try {
	  if (item) {
	    tmpids[0]= item->item_id();
	    detach_observer( obs,
			     tmpids);
	    ret->r_detach_results[i]= (CORBA::Boolean) 1;
	  }
	}
	catch (Domain_Repository::Unknown_Repository_Item_Ids ) {}
	catch (Domain_Repository::Unknown_Observer ) {}
      }
    }
  }
  catch (...) {
    delete ret;
#ifdef DEBUG
    cerr << "complex_query exception" << endl;
#endif
    throw;
  }
  return ret;
}

// Implementation of the add IDL operation

template <class Item_Types, class Observer_Types>
No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Item_ptr 
  No_Update_Basic_Rep_I<Item_Types, Observer_Types>::add (
  const No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Initial_Data& data_record
  ENV_DEF(env)) {

  auto_lock local_lock(*this);
  return private_add(data_record USE_ENV(env));
}

template <class Item_Types, class Observer_Types>
No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Item_ptr 
  No_Update_Basic_Rep_I<Item_Types, Observer_Types>::private_add (
  const No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Initial_Data& data_record
  ENV_DEF(env)) {

  validate_add_data(data_record);
  Item_I *new_item = create_item(data_record);
  add_permitted(new_item);
  init_corba_object(new_item);
  // assign a unique item id to it
  new_item->set_item_id(assign_id());
  insert_item(new_item);
  notify_observers(my_add_observer_list, new_item, Domain_Repository::OBJECT_ADDED);
#ifdef FilePersistency
  save();
#endif
  return Item::_duplicate(new_item->get_corba_object());
}

// Implementation of add helper methods

template <class Item_Types, class Observer_Types>
No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Item_I 
  *No_Update_Basic_Rep_I<Item_Types, Observer_Types>::create_item(
  const No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Initial_Data& data_record) {

  // create the new Item_I object (C++)
#ifdef ObjectStore
  Item_I* new_item = 
    new (my_db, Item_I::get_os_typespec()
	 ) Item_I (data_record);
#else
  Item_I* new_item = new Item_I (data_record);
#endif
  return new_item;
}

template <class Item_Types, class Observer_Types>
int No_Update_Basic_Rep_I<Item_Types, Observer_Types>::add_permitted(
  No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Item_I* new_item) {

  // check if if such an item already exists
  if (my_item_list.contains(new_item)){
    delete new_item;
    throw Domain_Repository::Duplicate_Data_Record();
  }
  return 1;
}

template <class Item_Types, class Observer_Types>
void No_Update_Basic_Rep_I<Item_Types, Observer_Types>::insert_item(
  No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Item_I* new_item) {

  // insert the new item (C++) object 
  my_item_list.insert(new_item);
  my_item_deref.insertKeyAndValue(new_item->item_id(), new_item);
  // insert this new item item_ids_to_observers_list
  item_ids_to_observers_list.insertKeyAndValue(new_item->item_id(),
     new Observer_List(hash_observer));
}

// Implementation of the remove IDL operation

template <class Item_Types, class Observer_Types>
void No_Update_Basic_Rep_I<Item_Types, Observer_Types>::remove (
  No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Item_ptr object_to_remove
  ENV_DEF(env)) {

  auto_lock local_lock(*this);
  private_remove(object_to_remove USE_ENV(env));
}

template <class Item_Types, class Observer_Types>
void No_Update_Basic_Rep_I<Item_Types, Observer_Types>::private_remove (
  No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Item_ptr object_to_remove
  ENV_DEF(env)) {

  if(!validate_remove_data(object_to_remove)) {
    return;
  }
  Item_I *item = deref_item(object_to_remove);
  if(!remove_permitted(item)) {
    return;
  }
  Observer_List *observer_list_p = 
    get_item_observer_list(item->item_id());
  notify_observers(*observer_list_p, item, Domain_Repository::OBJECT_REMOVED);
  remove_item(item);
  uninit_corba_object(item);
  delete item;
#ifdef FilePersistency
  // store updated state of the repository
  save();
#endif
}

// Implementation of remove helper methods

template <class Item_Types, class Observer_Types>
int No_Update_Basic_Rep_I<Item_Types, Observer_Types>::validate_remove_data(
  No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Item_ptr item_p) {

  // check if reference to the object to be removed is not nil or local
  if(CORBA::is_nil(item_p)) {
    throw Domain_Repository::Unknown_Object_Ref();
  }
#ifdef ORBIX
  if(item_p->_isRemote()) {
    throw Domain_Repository::Unknown_Object_Ref();
  }
#endif
  return 1;
}

template <class Item_Types, class Observer_Types>
int No_Update_Basic_Rep_I<Item_Types, Observer_Types>::remove_permitted(
  Item_I* item_p) {

  if (!my_item_list.contains(item_p)) {
    throw Domain_Repository::Unknown_Object_Ref();
  }
  return 1;
}

template <class Item_Types, class Observer_Types>
No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Item_I 
  *No_Update_Basic_Rep_I<Item_Types, Observer_Types>::deref_item(
  No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Item_ptr item_p) {

  // Map the item reference into a pointer to the item implementation
  Item_I* item_i_p = my_item_deref[item_p->item_id()];
  if (!item_i_p) {
    throw Domain_Repository::Unknown_Object_Ref();
  }
  return item_i_p;
}

template <class Item_Types, class Observer_Types>
void No_Update_Basic_Rep_I<Item_Types, Observer_Types>::remove_item(
  No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Item_I* item_p) {

  if (!my_item_list.remove(item_p)) {
    throw Domain_Repository::Unknown_Object_Ref();
  }
  my_item_deref.remove(item_p->item_id());
  detach_item_from_observers(item_p->item_id());
}

template <class Item_Types, class Observer_Types>
void No_Update_Basic_Rep_I<Item_Types, Observer_Types>::uninit_corba_object(
  Item_I *item_p) {

  CORBA::release(item_p->get_corba_object());
}


// Common helper functions

template <class Data_Object, class Observer_List, class Observer_List_Iterator> 
T_Notify_Params<Data_Object, Observer_List, Observer_List_Iterator>::
 T_Notify_Params( Observer_List &_obs_list) :
  obs_list( _obs_list) {

  // Iterator of the observer list
  Observer_List_Iterator iterator(obs_list);
  while (iterator())
    iterator.key()->_duplicate( iterator.key());
}

template <class Data_Object, class Observer_List, class Observer_List_Iterator> 
T_Notify_Params<Data_Object, Observer_List, Observer_List_Iterator>::~T_Notify_Params() {

  // Iterator of the observer list
  Observer_List_Iterator iterator(obs_list);
  while (iterator())
    CORBA::release( iterator.key());
}

template <class Item_Types, class Observer_Types>
void No_Update_Basic_Rep_I<Item_Types, Observer_Types>::notify_observers (
  Observer_List& obs_list,
  No_Update_Basic_Rep_I<Item_Types, Observer_Types>::Item_I *item_p, 
  Domain_Repository::Change_Type type_p) {

  Notify_Params *ntf= new Notify_Params( obs_list);

  Data_Object data_object_p;
  if (type_p == Domain_Repository::OBJECT_REMOVED)
    construct_data_object(data_object_p, NULL);
  else
    construct_data_object(data_object_p, item_p->get_corba_object());
  ntf->data_object_p= data_object_p;
  ntf->item_id= item_p->item_id();
  ntf->type_p= type_p;

#ifdef DEBUG
  cerr << ntf->obs_list.entries() << " observers" << endl;
#endif

  // Iterator of the observer list
  Observer_List cp_obs= ntf->obs_list;

  Observer_List_Iterator iterator( cp_obs);
  while (iterator()){
#ifdef OMNIORB
    if (!iterator.key()->is_proxy()) // a local object
#elif defined ORBIX
    if (!iterator.key()->_isRemote())
#elif
#error __LINE__ __FILE__
#endif
      {
	try {
	  iterator.key()->notify( ntf->item_id, 
				  ntf->data_object_p, 
				  ntf->type_p);
	}
	catch (...) {}
	ntf->obs_list.remove( iterator.key());
      }
  }
#ifdef DEBUG
  cerr << ntf->obs_list.entries() << " remote observers" << endl;
#endif
  insert_into_notifier( ntf);
}

template <class Item_Types, class Observer_Types>
void No_Update_Basic_Rep_I<Item_Types, Observer_Types>::insert_into_notifier(	
  Notify_Params *ntf) {

  ABng_S_Guarded::auto_lock local_lock( notifier_guard);
  
  notifier_queue.insert( ntf);
  notifier_cond.signal();
}

template <class Item_Types, class Observer_Types>
void No_Update_Basic_Rep_I<Item_Types, Observer_Types>::static_notify_worker (
  void *args) {

  ((No_Update_Basic_Rep_I<Item_Types, Observer_Types>*) args)->notify_worker();
}

template <class Item_Types, class Observer_Types>
void No_Update_Basic_Rep_I<Item_Types, Observer_Types>::notify_worker () {

  while (1) {
    Notify_Params *ntf;
    
    {
      ABng_S_Guarded::auto_lock local_lock( notifier_guard);
	
      while (notifier_queue.isEmpty())
	notifier_cond.wait();
      ntf= notifier_queue.get();
    }
    
    {
      // Iterator of the observer list
      Observer_List_Iterator iterator( ntf->obs_list);

      // Notify all observers of this entity
      while (iterator()){
	try {
	  iterator.key()->notify( ntf->item_id, 
				  ntf->data_object_p, 
				  ntf->type_p);
	}
	catch (...) {
#ifdef DEBUG
	  cerr << "The observer is dead" << endl;
#endif
	  {
	    auto_lock local_lock( *this);

	    // TO DO : add "Incarnation_Number" checking here
	    //  that would prevent detaching observers that crashed
	    //  and attached ( in the meantime) once again
	    //

	    try {
	      private_detach_add_observer( iterator.key());
	    } catch (...) {}
	    try {
	      private_detach_observer( iterator.key(),
				       get_item_ids_list( iterator.key()));
	    } catch(...) {}
	  }
	}
      }
    }
    delete ntf;
  }
}

#ifdef FilePersistency

template <class Item_Types, class Observer_Types>
void No_Update_Basic_Rep_I<Item_Types, Observer_Types>::save() {
  FileTransaction ft( my_repository_name,
		      1 /* create backup */);

  ostream *ostr= ft.prepare();
  save( *ostr);
  ft.commit();
}

template <class Item_Types, class Observer_Types>
void No_Update_Basic_Rep_I<Item_Types, Observer_Types>::save( ostream & iostr) {


  DB_Header *db_header = create_db_header();
  iostr << *db_header;
  delete db_header;

  iostr << get_item_counter() << endl;

  iostr << my_item_list.entries() << endl;
  Item_List_Iterator iterator(my_item_list);
  while (iterator()) {
    iterator.key()->save(iostr);
  }
}

template <class Item_Types, class Observer_Types>
void No_Update_Basic_Rep_I<Item_Types, Observer_Types>::restore ( istream & iostr) {

  DB_Header db_header;

  if ( !( iostr >> db_header)) return;

  // Restore item counter
  if ( !( iostr >> item_counter)) return;

  int number_of_items;
  iostr >>  number_of_items;

  for (int i = 0; i < number_of_items; i++) {
    // Create and restore the state of the item object
    Item_I *new_item = new Item_I (iostr);
    if (!iostr) {
      return;
    }
    init_corba_object(new_item);
    insert_item(new_item);
  }
}

#endif /* FilePersistency */

template <class Item_Types, class Observer_Types>
void No_Update_Basic_Rep_I<Item_Types, Observer_Types>::gather_item(
  Item_I* current_item, void* apply_data) {

  // Project apply_data (void*) to Items*
  Items *collected_items = (Items*) apply_data;

  // increase the length of the container
  collected_items->length(collected_items->length() + 1);

  // Put duplicated item (CORBA object) into the container
  (*collected_items)[collected_items->length()-1] =
    Item::_duplicate(current_item->get_corba_object());
}

template <class Item_Types, class Observer_Types>
void No_Update_Basic_Rep_I<Item_Types, Observer_Types>::write_ior(
  const CORBA::Object_ptr obj_p, 
  const char *file_name_p) {

#ifdef ORBIX
  char * str_ref;
  try {
    str_ref = obj_p->_object_to_string(
		CORBA::IT_INTEROPERABLE_OR_KIND
		);
  } catch(CORBA::SystemException &sysEx) {
      cerr <<"Unexpected system exception " << endl; 
      cerr << &sysEx ;
  }
  cout << str_ref << endl;

  ofstream strm(file_name_p);

  if(!strm){
    cerr << "Unable to open file - ";
    cerr << file_name_p << endl;
  }

  strm << str_ref;
  strm.close();

  CORBA::string_free(str_ref);
#endif
}




// ----- Basic_Rep_I
// Implementation of the update IDL operation

template <class Item_Types, class Observer_Types>
void Basic_Rep_I<Item_Types, Observer_Types>::update(
  Basic_Rep_I<Item_Types, Observer_Types>::Item_ptr object_to_update, 
  const Basic_Rep_I<Item_Types, Observer_Types>::Initial_Data& 
    data_record_pattern
  ENV_DEF(env)) {
  
  auto_lock local_lock(*this);
  private_update(object_to_update, data_record_pattern USE_ENV(env));
}

template <class Item_Types, class Observer_Types>
void Basic_Rep_I<Item_Types, Observer_Types>::private_update(
  Basic_Rep_I<Item_Types, Observer_Types>::Item_ptr object_to_update, 
  const Basic_Rep_I<Item_Types, Observer_Types>::Initial_Data& 
    data_record_pattern
  ENV_DEF(env)) {
  
  if(!validate_update_data(object_to_update, data_record_pattern)) {
    return;
  }
  Item_I *item = deref_item(object_to_update);
  if(!update_permitted(item, data_record_pattern,item)) {
    return;
  }
  my_item_list.remove(item);
  update_item(item, data_record_pattern);
  my_item_list.insert(item);
  
  Observer_List *observer_list_p = 
    get_item_observer_list(item->item_id());
 
  notify_observers(*observer_list_p, item, Domain_Repository::OBJECT_UPDATED);
 
#ifdef FilePersistency
  // store updated state of the repository
  save();
#endif

}

// Implementation of update helper methods

template <class Item_Types, class Observer_Types>
int Basic_Rep_I<Item_Types, Observer_Types>::validate_update_data(
  Basic_Rep_I<Item_Types, Observer_Types>::Item_ptr item_p, 
  const Basic_Rep_I<Item_Types, Observer_Types>::Initial_Data& 
    data_record_pattern) {

  // check if reference to the object to be removed is not nil or local
  if(CORBA::is_nil(item_p)) {
    throw Domain_Repository::Unknown_Object_Ref();
  }
#ifdef ORBIX
  if(item_p->_isRemote()) {
    throw Domain_Repository::Unknown_Object_Ref();
  }
#endif
  return 1;
}

template <class Item_Types, class Observer_Types>
int Basic_Rep_I<Item_Types, Observer_Types>::update_permitted(
  Basic_Rep_I<Item_Types, Observer_Types>::Item_I* item_p,
  const Basic_Rep_I<Item_Types, Observer_Types>::Initial_Data& 
    data_record_pattern,
  Basic_Rep_I<Item_Types, Observer_Types>::Item_I* objtu
  ) {

  // check if such an item already exists
  Item_I *temporary_item = create_item(data_record_pattern);
  Item_I *duplicated_item;

  if (duplicated_item = my_item_list.find(temporary_item)){
    if (duplicated_item != objtu) {
      delete temporary_item;
      throw Domain_Repository::Duplicate_Data_Record();
    }
  }

  delete temporary_item;
  return 1;
}

template <class Item_Types, class Observer_Types>
void Basic_Rep_I<Item_Types, Observer_Types>::update_item(
  Basic_Rep_I<Item_Types, Observer_Types>::Item_I* item_p, 
  const Basic_Rep_I<Item_Types, Observer_Types>::Initial_Data& 
    data_record_pattern) {

  //  Update, but do not change its item id!!! - 
  //  see implementation of set_descr()
  item_p->set_descr(data_record_pattern);
}








