//
// $Id: bootstrap_base.cc,v 1.3 1998/07/29 13:14:50 mencnar Exp $
//
// $Log: bootstrap_base.cc,v $
// Revision 1.3  1998/07/29 13:14:50  mencnar
// time synchronisation,
// persistent observers,
// notify calls moved to another thread ( fixed deadlock)
// internal server names changed
//
// Revision 1.2  1998/06/16 12:55:00  mencnar
// *** empty log message ***
//
// Revision 1.1  1998/02/20 15:32:21  mencnar
// Initial revision
//
//
//-----------------------------------------------------------------------------
// 
// Author:      Daniel Mencnarowski  (mencnar@ics.agh.edu.pl)
//
// Description: Implementation file
//              A base class for "bootstrap" interface
//

#include "bootstrap_base.hh"
#include <iostream.h>
#include "compatibility.hh"
#include <string.h>
#include <stdlib.h>
#include <fstream.h>

///////////////////////////////////////////////////////////////////Bootstrap_base//
CORBA::ORB_ptr Bootstrap_base::orb;

Bootstrap_base::Bootstrap_base( CORBA::ORB_ptr _orb) {
  orb= _orb;

  char buf[2048];

  const char *ns_ior_file= getenv( NS_IOR_FILE);
  if ( ! ns_ior_file) {
    cerr << NS_IOR_FILE " not defined," << endl
         << "\tplease set it to a file name with a valid IOR of Name Service" << endl;
    exit( 1);
  }
  
  ifstream ior_file( ns_ior_file);
  if ( ! ior_file) {
    cerr << "Could not open Name Service IOR file " << ns_ior_file << endl;
    exit( 1);
  }
  ior_file >> buf;
#ifdef DEBUG
  //  cerr << "NS IOR:" << endl  << buf << endl;
#endif

  try {
    CORBA::Object_var tmp_obj= orb->string_to_object( buf);
    if (CORBA::is_nil(tmp_obj))
      cerr << "Nil stringified NS reference" << endl;
#ifdef DEBUG
#endif
    root_ns= CosNaming::NamingContext::_narrow( tmp_obj);
#ifdef DEBUG
    //    cerr << "Root NS resolved" << endl;
    if (CORBA::is_nil( (CosNaming::NamingContext_ptr) root_ns))
      cerr << "Nil root NS" << endl;
    //    root_ns->_non_existent();
#endif
  }
  catch ( CORBA::Exception &ex) {
    cerr << "Could not narrow stringified reference (NS)" << endl << ex << endl;
    exit( 1);
  }
  const char *root_ns_name= getenv( ABNG_CONTEXT_NAME);
  if ( !root_ns_name) root_ns_name= ABNG_DEFAULT_CONTEXT_NAME;

  abng_root_ns= CORBA::string_dup( root_ns_name);
#ifdef OMNIORB
  // Turn off garbage collecting of unused connections
  //
  omniORB::idleConnectionScanPeriod( omniORB::idleIncoming, 0);
  omniORB::idleConnectionScanPeriod( omniORB::idleOutgoing, 0);
#endif
}


//----------------------------------------------------------incremental_bind--
//
void Bootstrap_base::incremental_bind( const ABng_Server_Manager::Name &name,
				       CORBA::Object_var object) {
 
  CosNaming::Name tmpName( name.length());

  for ( int i=1; i<name.length(); i++) {
    tmpName.length( i);

    for ( int j=0; j<i; j++) {
      tmpName[ j].id=    name[ j];
      tmpName[ j].kind=  name[ j];
    }
    try {
      CosNaming::NamingContext_var nc( root_ns->bind_new_context( tmpName));
    }
    catch ( CosNaming::NamingContext::AlreadyBound &) {}
    catch ( CORBA::Exception &ex) {
      cerr << "Caught exception while creating name context " << ex << endl;
      exit(1);
    }
    catch (...) {}
  }

  int len= name.length();
  tmpName.length( len);
  tmpName[ len-1].id=    name[ len-1];
  tmpName[ len-1].kind=  name[ len-1];
  try {
    root_ns->unbind( tmpName);
  }
  catch (...) {
#ifdef DEBUG
    cerr << "unbind exception" << endl;
#endif
  }
  try {
    root_ns->bind( tmpName, object);
  }
  catch (CORBA::SystemException &se) {
    cerr << "Caught system exception during (NS)bind " << endl;
#ifdef OMNIORB
    cerr << se.NP_RepositoryId() << endl;
#endif
    exit(1);
  }
  catch (CORBA::Exception &ex) {
    cerr << "Caught exception during (NS)bind " << ex << endl;
    for (CORBA::ULong xi=0; xi<tmpName.length(); xi++)
      cerr << (const char *) (tmpName[xi].id) << '-'
           << (const char *) (tmpName[xi].kind) << '.';
    cerr << endl; 
    exit(1);
  }
}



//-----------------------------------------------------------------------resolve--
//
CORBA::Object_var Bootstrap_base::resolve( const ABng_Server_Manager::Name &name) {

  CosNaming::Name tmpName( name.length());
  tmpName.length( name.length());
  
  for ( int i=0; i<name.length(); i++) {
    tmpName[ i].id=    name[ i];
    tmpName[ i].kind=  name[ i];
  }
  CORBA::Object_var ret= root_ns->resolve( tmpName);
#ifdef ORBIX
  if ( CORBA::is_nil( ret)) ret= NULL;
#endif
  return ret;
}

//------------------------------------------------------------------------unbind--
//
void Bootstrap_base::unbind( const ABng_Server_Manager::Name &name) {

  CosNaming::Name tmpName( name.length());
  tmpName.length( name.length());

  for ( int i=0; i<name.length(); i++) {
    tmpName[ i].id=    name[ i];
    tmpName[ i].kind=  name[ i];
  }
  root_ns->unbind( tmpName);
}








