//
// $Id: bootstrap_n.cc,v 1.2 1998/07/29 13:14:50 mencnar Exp $
//
// $Log: bootstrap_n.cc,v $
// Revision 1.2  1998/07/29 13:14:50  mencnar
// time synchronisation,
// persistent observers,
// notify calls moved to another thread ( fixed deadlock)
// internal server names changed
//
// Revision 1.1  1998/06/16 12:55:00  mencnar
// Initial revision
//
//
//-----------------------------------------------------------------------------
// 
// Author:      Daniel Mencnarowski  (mencnar@ics.agh.edu.pl)
//
// Description: Bootstrap class implementation
//              Hides functionality of ABng_Server_Manager::Register interface
//

#include <rw/cstring.h>
#include <rw/rwtime.h>
#include <rw/rwdate.h>
#include "bootstrap_n.hh"
#include "mtime.hh"
#include "domain_rep.hh"
#include "db_utils.hh"
#include <iostream.h>
#include <stdlib.h>
#include <unistd.h>

ostream & operator << ( ostream & ostr, 
			const ABng_Server_Manager::Name &name) {
  ostr << name.length() << ' ';
  for ( int i= 0; i<name.length(); i++)
    ostr << name[i] << ' ';
  ostr << endl;
  return ostr;
}

istream & operator >> ( istream &istr,
			ABng_Server_Manager::Name &name) {
  int len;
  istr >> len;
  if ( ! istr) return istr;
  name.length( len);

  RWCString buf;
  for ( int i= 0; i<len; i++) {
    istr >> buf;
    name[ i]= ( const char *) buf;
  }
  return istr;
}

int operator == ( const ABng_Server_Manager::Name &name1,
		  const ABng_Server_Manager::Name &name2) {

  if ( name1.length() == name2.length()) {
    for ( int i=0; i<name1.length(); i++)
      if ( strcmp( ( const char *) ( name1[ i]),
		   ( const char *) ( name2[ i]))) return 0;
    return 1;
  }
  return 0;
}

int operator == ( const ABng_Server_Manager::Name &name1,
		  const char *name2) {

  if ( name1.length() == 1)
    return ( ! strcmp( ( const char *) name1[ 0], name2));
  else
    return 0;
}

int operator >= ( const ABng_Server_Manager::Name &name1,
		  const ABng_Server_Manager::Name &name2) {
  
  if ( name1.length() >= name2.length()) {
    for ( int i=0; i<name2.length(); i++)
      if ( strcmp( ( const char *) ( name1[ i]),
		   ( const char *) ( name2[ i]))) return 0;
    return 1;
  }
  return 0;
}

ABng_Server_Manager::Name operator + ( const ABng_Server_Manager::Name &name,
				       const char *add_name) {

  ABng_Server_Manager::Name ret( name);
  ret.length( ret.length()+1);
  ret[ name.length()]= add_name;
  return ret;
}

ABng_Server_Manager::Name operator - ( const ABng_Server_Manager::Name &name1,
				       const ABng_Server_Manager::Name &name2) {
  ABng_Server_Manager::Name ret;

  int size2= name2.length();
  int size1= name1.length();
  if ( size1 > size2) {
    ret.length( size1 - size2);
    
    for ( int i=size2; i<size1; i++)
      ret[ i-size2]= name1[ i];
  }
  return ret;
}

//=======================================================Bootstrap_listener==
//
Bootstrap_listener::Bootstrap_listener() :
  callbackimpl( *this) {

  register_callback( Bootstrap::SRV_Bootstrap());
  register_callback( Bootstrap::SRV_Bootstrap_Client());
}

ABng_Server_Manager::uid Bootstrap_listener::register_callback( 
     			        const ABng_Server_Manager::Name &name) {

  if (CORBA::is_nil( (ABng_Server_Manager::Register_Manager_ptr)
		     Bootstrap::register_manager_object))
    return Bootstrap::register_object->
      register_restricted_callback( &callbackimpl,
				    name);
  else 
    return Bootstrap::register_manager_object->
      register_callback( &callbackimpl,
			 name);    
}

ABng_Server_Manager::uid Bootstrap_listener::register_context_callback( 
				 const ABng_Server_Manager::Name &name) {

  if (CORBA::is_nil( (ABng_Server_Manager::Register_Manager_ptr)
		     Bootstrap::register_manager_object))
    return Bootstrap::register_object->
      register_restricted_context_callback( &callbackimpl,
					    name);
  else 
    return Bootstrap::register_manager_object->
      register_context_callback( &callbackimpl,
				 name);    
}

void Bootstrap_listener::unregister_callback( ABng_Server_Manager::uid identifier) {
  if (CORBA::is_nil( (ABng_Server_Manager::Register_Manager_ptr)
		     Bootstrap::register_manager_object))
    Bootstrap::register_object->unregister_restricted_callback( identifier);
  else
    Bootstrap::register_manager_object->unregister_callback( identifier);
}


//============================================================SM_CallbackImpl==
//
void SM_CallbackImpl::server_startup( CORBA::Object_ptr server_reference,
				      const ABng_Server_Manager::Name &name,
				      ABng_Server_Manager::uid callback_id
				      ENV_DEF( env) ) {
#ifdef DEBUG
  cerr << "server_startup " << name << endl;
  RWTime now;
  cerr << now << endl;
#endif

  if ( name == Bootstrap::SRV_Bootstrap()) {
    try {
      Bootstrap::register_manager_object= 
	ABng_Server_Manager::Register_Manager::_narrow( server_reference);
    }
    catch ( CORBA::Exception &ex) {
    }
  }
  if ( name == Bootstrap::SRV_Bootstrap_Client()) {
    try {
      Bootstrap::register_object= 
	ABng_Server_Manager::Register::_narrow( server_reference);
    }
    catch ( CORBA::Exception &ex) {
    }
  }

  listener.reference_resolved( name, 
			       CORBA::Object::_duplicate( server_reference), 
			       callback_id);
}

void SM_CallbackImpl::server_shutdown( const ABng_Server_Manager::Name &name,
				       ABng_Server_Manager::uid callback_id
				       ENV_DEF( env) ) {

  listener.server_shutdown( name, 
			    callback_id);
}

//===================================================================Bootstrap==
//
Bootstrap_base * Bootstrap::base_impl= NULL;
ABng_Server_Manager::Register_var Bootstrap::register_object;
ABng_Server_Manager::Register_Manager_var Bootstrap::register_manager_object;

void Bootstrap::init( CORBA::ORB_ptr orb_ptr,
		      int client_only) {

  base_impl= new Bootstrap_base( orb_ptr);
  try {
    CORBA::Object_var tmp_obj= resolve( Bootstrap::SRV_Bootstrap_Client());

    register_object= ABng_Server_Manager::Register::_narrow( tmp_obj);
    if (CORBA::is_nil((ABng_Server_Manager::Register_ptr) register_object)) {
      cerr << "Nil register object" << endl;
  //    exit(1);
    }

    if (!client_only) {
      CORBA::Object_var tmp_obj= resolve( Bootstrap::SRV_Bootstrap());
      
      register_manager_object= 
	ABng_Server_Manager::Register_Manager::_narrow( tmp_obj);
      if (CORBA::is_nil((ABng_Server_Manager::Register_Manager_ptr) 
			register_manager_object)) {
	cerr << "Nil register manager object" << endl;
//	exit(1);
      }
    }
  }
  catch ( CORBA::Exception &ex) {
    cerr << "Could not contact ABng_Server_Manager::Register object" << endl
         << ex << endl;
    exit( 1);
  }
}

//-------------------------------------------------------------------------------
/* time synchronisation stuff goes here */

ABng_Guarded Bootstrap::time_guard;
unsigned long Bootstrap::next_synch= 0;
omni_thread * Bootstrap::synch_thread= NULL;
int Bootstrap::synch_required= -1;

ABng_Server_Manager::Time Bootstrap::gettimeofday() {
  return ::currtime();
}

int Bootstrap::is_synch_required() {
  if (synch_required>=0) return synch_required;
  try {
    ABng_Server_Manager::ip_address_var srv_ip=
      register_object->get_ip_address();
    if (srv_ip->length() != 4) return (synch_required= 1);

    struct in_addr myaddr=  get_host_address();
    
    // IP address ver. 4
    //
    ABng_Server_Manager::ip_address r(4);
    r.length( 4);
    
    r[0]= (CORBA::Octet)((ntohl(myaddr.s_addr) & 0xff000000) >> 24);
    r[1]= (CORBA::Octet)((ntohl(myaddr.s_addr) & 0x00ff0000) >> 16);
    r[2]= (CORBA::Octet)((ntohl(myaddr.s_addr) & 0x0000ff00) >> 8);
    r[3]= (CORBA::Octet)(ntohl(myaddr.s_addr) & 0x000000ff);
    for (CORBA::ULong i=0; i<4; i++) {
      if (r[i] != srv_ip[i]) {
#ifdef DEBUG
	cerr << "Current process and time server located on different machines"
	     << endl
	     << " time synchronization required" << endl;
#endif
	return (synch_required= 1);
      }
    }
#ifdef DEBUG
    cerr << "Processes located on the same machine, no synchronization required" << endl;
    return (synch_required= 0);
#endif
  }
  catch (...) {
    return 0;
  }
}

void Bootstrap::need_synch_time( unsigned long accuracy_microseconds) {
  if (!synch_thread && is_synch_required()) {
    unsigned long *par= new unsigned long;
    *par= accuracy_microseconds;
    
    (synch_thread= new omni_thread( synch_time_worker, (void*) par))->start();
  }
}

void Bootstrap::synch_time( unsigned long accuracy_microseconds) {
  if (!is_synch_required()) return;

  auto_lock local_lock( time_guard);
  ABng_Server_Manager::TimeServer_var ts=
    ABng_Server_Manager::TimeServer::_duplicate(
       ((ABng_Server_Manager::TimeServer_ptr) 
	(ABng_Server_Manager::Register_ptr) register_object));

  next_synch= ::synch_time( accuracy_microseconds,
			    ts);
}

void Bootstrap::synch_time_worker( void *args ) {
  unsigned long accuracy_microseconds= *((unsigned long *)args);

  while (1) {
    unsigned long nexts;
    {
      auto_lock local_lock( time_guard);
      
      nexts= next_synch;
    }
    if (nexts) {
      cerr << "Next time synchronisation in " << nexts << " seconds" << endl;
      sleep( nexts);
    }
    synch_time( accuracy_microseconds);
  }
}


void Bootstrap::register_server_in_NS( const ABng_Server_Manager::Name &name,
				       CORBA::Object_ptr server_ref) {

  assert(!CORBA::is_nil( (ABng_Server_Manager::Register_Manager_ptr)
		     register_manager_object));
    
  register_manager_object->register_server( server_ref,
					    name);
}

void Bootstrap::unregister_server_from_NS( const ABng_Server_Manager::Name &name) {
  assert(!CORBA::is_nil( (ABng_Server_Manager::Register_Manager_ptr)
		     register_manager_object));
    
  register_manager_object->unregister_server( name);
}

CORBA::Object_var Bootstrap::resolve( const ABng_Server_Manager::Name &name) {
  return base_impl->resolve( name);
}

ABng_Server_Manager::Name Bootstrap::CTX_Root() {
  return ABng_Server_Manager::Name( 0);
}

ABng_Server_Manager::Name Bootstrap::CTX_ABng() {
  return CTX_Root()+"ABng";
}

ABng_Server_Manager::Name Bootstrap::CTX_Bootstrap_Client() {
  return CTX_ABng()+"Bootstrap_Client";
}

ABng_Server_Manager::Name Bootstrap::CTX_Domain_Repository() {
  return CTX_ABng()+"Domain_Repository";
}

ABng_Server_Manager::Name Bootstrap::CTX_Domain_Repository_Public() {
  return CTX_Domain_Repository()+"Public";
}

ABng_Server_Manager::Name Bootstrap::CTX_Domain_Repository_Management() {
  return CTX_Domain_Repository()+"Management";
}

ABng_Server_Manager::Name Bootstrap::CTX_ABng_Core() {
  return CTX_ABng()+"ABng_Core";
}

ABng_Server_Manager::Name Bootstrap::CTX_ABng_Core_Public() {
  return CTX_ABng_Core()+"Public";
}

ABng_Server_Manager::Name Bootstrap::CTX_ABng_Core_Public_Views() {
  return CTX_ABng_Core_Public()+"Views";
}

ABng_Server_Manager::Name Bootstrap::CTX_ABng_Core_Management() {
  return CTX_ABng_Core()+"Management";
}

ABng_Server_Manager::Name Bootstrap::CTX_ABng_Core_System() {
  return CTX_ABng_Core()+"System";
}

ABng_Server_Manager::Name Bootstrap::CTX_ABng_Core_System_Pollers() {
  return CTX_ABng_Core_System()+"Pollers";
}

ABng_Server_Manager::Name Bootstrap::CTX_ABng_Core_System_Vsps() {
  return CTX_ABng_Core_System()+"Vsps";
}

ABng_Server_Manager::Name Bootstrap::CTX_ABng_Core_Management_View_Managers() {
  return CTX_ABng_Core_Management()+"View_Managers";
}

ABng_Server_Manager::Name Bootstrap::SRV_Bootstrap() {
  return CTX_ABng()+"Bootstrap@";
}

ABng_Server_Manager::Name Bootstrap::SRV_Bootstrap_Client() {
  return CTX_Bootstrap_Client()+"Client";
}

ABng_Server_Manager::Name Bootstrap::SRV_Scheduler() {
  return CTX_ABng_Core_System()+"Scheduler@";
}

ABng_Server_Manager::Name Bootstrap::SRV_Sensor_Repository() {
  return CTX_ABng_Core_Management()+"Sensor_Repository@";
}

ABng_Server_Manager::Name Bootstrap::SRV_Badge_Repository() {
  return CTX_ABng_Core_Management()+"Badge_Repository@";
}

ABng_Server_Manager::Name Bootstrap::SRV_Badge_Holder_Repository() {
  return CTX_ABng_Core_Management()+"Badge_Holder_Repository@";
}

ABng_Server_Manager::Name Bootstrap::SRV_Location_Type_Repository() {
  return CTX_Domain_Repository_Public()+"Location_Type_Repository";
}

ABng_Server_Manager::Name Bootstrap::SRV_Location_Type_Repository_Manager() {
  return CTX_Domain_Repository_Management()+"Location_Type_Repository_Manager@";
}

ABng_Server_Manager::Name Bootstrap::SRV_Location_Description_Repository() {
  return CTX_Domain_Repository_Public()+"Location_Description_Repository";
}

ABng_Server_Manager::Name Bootstrap::SRV_Location_Description_Repository_Manager() {
  return CTX_Domain_Repository_Management()+"Location_Description_Repository_Manager@";
}

ABng_Server_Manager::Name Bootstrap::SRV_ABng_Location_Description_Repository() {
  return CTX_ABng_Core_Management()+"ABng_Location_Description_Repository@";
}

ABng_Server_Manager::Name Bootstrap::SRV_User_Repository() {
  return CTX_Domain_Repository_Public()+"User_Repository";
}

ABng_Server_Manager::Name Bootstrap::SRV_User_Repository_Manager() {
  return CTX_Domain_Repository_Management()+"User_Repository_Manager@";
}

ABng_Server_Manager::Name Bootstrap::SRV_Equipment_Repository() {
  return CTX_Domain_Repository_Public()+"Equipment_Repository";
}

ABng_Server_Manager::Name Bootstrap::SRV_Equipment_Repository_Manager() {
  return CTX_Domain_Repository_Management()+"Equipment_Repository_Manager@";
}

ABng_Server_Manager::Name Bootstrap::SRV_Meta_Equipment_Repository() {
  return CTX_Domain_Repository_Public()+"Meta_Equipment_Repository";
}

ABng_Server_Manager::Name Bootstrap::SRV_Meta_Equipment_Repository_Manager() {
  return CTX_Domain_Repository_Management()+"Meta_Equipment_Repository_Manager@";
}

ABng_Server_Manager::Name Bootstrap::SRV_Audio_Devices_Repository() {
  return CTX_Domain_Repository_Public()+"Audio_Devices_Repository";
}

ABng_Server_Manager::Name Bootstrap::SRV_Audio_Devices_Repository_Manager() {
  return CTX_Domain_Repository_Management()+"Audio_Devices_Repository_Manager@";
}

ABng_Server_Manager::Name Bootstrap::SRV_Property_Record_Repository() {
  return CTX_Domain_Repository_Public()+"Property_Record_Repository";
}

ABng_Server_Manager::Name Bootstrap::SRV_Property_Record_Repository_Manager() {
  return CTX_Domain_Repository_Management()+"Property_Record_Repository_Manager";
}

ABng_Server_Manager::Name Bootstrap::SRV_Main_Sighting_Processor() {
  return CTX_ABng_Core_System()+"Main_Sighting_Processor@";
}

ABng_Server_Manager::Name Bootstrap::SRV_Away_Repository() {
  return CTX_ABng_Core_Public()+"Away_Repository";
}

ABng_Server_Manager::Name Bootstrap::SRV_Away_Repository_Manager() {
  return CTX_ABng_Core_Management()+"Away_Repository_Manager@";
}

ABng_Server_Manager::Name Bootstrap::SRV_Local_Dependence() {
  return CTX_Domain_Repository()+"Local_Dependence";
}

