//
// $Id: process_status.cc,v 1.2 1998/07/29 13:14:54 mencnar Exp $
//
// $Log: process_status.cc,v $
// Revision 1.2  1998/07/29 13:14:54  mencnar
// time synchronisation,
// persistent observers,
// notify calls moved to another thread ( fixed deadlock)
// internal server names changed
//
// Revision 1.1  1998/06/16 12:55:00  mencnar
// Initial revision
//
//
//-----------------------------------------------------------------------------
// 
// Author:      Daniel Mencnarowski  (mencnar@ics.agh.edu.pl)
//
// Description: Implemenation file
//              Implementation class for ABng_Server_Manager::ProcessControl
//


#include <iostream.h>
#include <strstream.h>
#include "Naming.hh"
#include "compatibility.hh"
#include "process_status.hh"
#include "bootstrap_n.hh"
#include <sys/types.h>
#include <sys/param.h>
#include <unistd.h>
#include <stdlib.h>
#include <netdb.h>
#include <pwd.h>

#if !defined (DECLARED_GETHOSTNAME)
extern "C" int          gethostname (char *, int);
#endif

void ServerStatusInterface::get_server_status( ostream &ostr) {
  ostr << "Not implemented" << endl;
}

ProcessControlImpl::ProcessControlImpl( const char *_process_name,
					ServerStatusInterface *_status_impl) :
  status_impl( _status_impl),
  process_name( CORBA::string_dup( _process_name))
{
  gettimeofday( &start_time, NULL);
  CONNECT_IMPL( ABng_Server_Manager::ProcessControl_var, this);
  Bootstrap::register_server_in_NS( (Bootstrap::CTX_ABng()+"ps_dev")+process_name,
				    this);
}

ProcessControlImpl::~ProcessControlImpl() {
  Bootstrap::unregister_server_from_NS( (Bootstrap::CTX_ABng()+"ps_dev")+process_name);
  CORBA::string_free( process_name);
}

void time_bomb_wrapper( void *args) {
  sleep(2);
  exit(1);
}

void ProcessControlImpl::kill_server( ENV_DEF1(env)) {

  cerr << "ABng_Server::ProcessControl::kill_server called, exiting in 2 seconds";
  (new omni_thread( time_bomb_wrapper, NULL))->start();
}

CORBA::Long ProcessControlImpl::get_pid( ENV_DEF1(env)) {
  return (CORBA::Long) getpid();
}

char * ProcessControlImpl::get_host_name( ENV_DEF1(env)) {
  char buf[MAXHOSTNAMELEN];
  gethostname( buf, MAXHOSTNAMELEN);

  return CORBA::string_dup( buf);
}

char * ProcessControlImpl::get_process_owner( ENV_DEF1(env)) {
  struct passwd* pe= getpwuid( getuid());
  return CORBA::string_dup( pe ? pe->pw_name : "unknown");
}

CORBA::Long ProcessControlImpl::uptime( ENV_DEF1(env)) {
  struct timeval now;
  gettimeofday( &now, NULL);
  return now.tv_sec - start_time.tv_sec;
}

char * ProcessControlImpl::get_server_status( ENV_DEF1(env)) {
  if ( status_impl) {
    ostrstream ostr;   
    status_impl->get_server_status( ostr);
    ostr << '\0'; // terminate string
    char *str= ostr.str();
    char *ret= CORBA::string_dup( str);
    free(str);
    return ret;
  }
  else return CORBA::string_dup( "Not implemented\n");
}







