// -*- Mode: C++; -*-
//
//  File:         location_type_i.h
//  Author:       Malgorzata Steinder (gosia@ics.agh.edu.pl)
//  Date Created: 12.12.97
//
// General Description:
//   Header file for the implementation class of the Abng_Location_Description interface
//

#ifndef abng_location_description_i_h
#define abng_location_description_i_h

#include <stdio.h>
#include <fstream.h>
#include <errno.h>
#include <string.h>

#ifdef ObjectStore
#include <oosa.h>
#endif

#ifdef FilePersistency
#include <rw/rwfile.h>
#endif

#include "abng_core_management.hh"
#include "abng_guarded.hh"
#include "compatibility.hh"


class ABng_Loc_Descr_I  : public virtual ABng_Guarded {
 private:
  //
  // the state of the IDL ABng_Location_Description object
  //
  ABng_Core_System::ABng_Location_Description::Description my_descr;

  //
  // the reference of CORBA object encapsulated this C++ object
  //
  ABng_Core_System::ABng_Location_Description_ptr my_corba_object;

  //
  // Sensors Repository_Item_Id cache
  //
  Domain_Repository::Repository_Item_Ids sensor_ids;

  //
  // Location Description Repository_Item_id
  //
  Domain_Repository::Repository_Item_Id loc_descr_id;


 public:
   //
   // ctors
   //
   ABng_Loc_Descr_I (
     const ABng_Core_System::ABng_Location_Description::Description& descr_init
   );

   ABng_Loc_Descr_I (const ABng_Loc_Descr_I&);

#ifdef FilePersistency
   ABng_Loc_Descr_I (istream & from);
#endif

  virtual 
   ~ABng_Loc_Descr_I ();

  operator==(const ABng_Loc_Descr_I&);

  //
  // methods enabling access to the state
  //
  void set_descr (
    const ABng_Core_System::ABng_Location_Description::Description& descr_update
  );
 
  const ABng_Core_System::ABng_Location_Description::Description&
    get_descr () const;

  void set_corba_object (
	const ABng_Core_System::ABng_Location_Description_ptr corba_object_p
  );

  const ABng_Core_System::ABng_Location_Description_ptr get_corba_object ();

  void set_item_id(const Domain_Repository::Repository_Item_Id item_id_p);

  Domain_Repository::Repository_Item_Ids& get_sensor_ids() {
        return sensor_ids;
  }

  Domain_Repository::Repository_Item_Id get_loc_descr_id() const {
        return loc_descr_id;
  }

  void set_loc_descr(Domain_Repository::Location_Description_ptr);

  void remove_sensor(Domain_Repository::Repository_Item_Id id);

  int contains_sensor(Domain_Repository::Repository_Item_Id id);

  //
  // IDL operations of the System_Static_Data_Observed_E interface
  //                       _____________________________

  virtual 
  ABng_Core_System::System_Static_Data_Observed_E::System_Static_Data_Observers*
    system_static_data_observer_list (
       ENV_DECL1(env)
  );
                
  virtual void attach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs ENV_DECL(env)
  );

  virtual void detach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs ENV_DECL(env)
  );


  //
  // IDL operations of the ABng_Location_Description interface
  //                       ______
 
  virtual ABng_Core_System::ABng_Location_Description::Description* descr (
	ENV_DECL1(env)
  );

  virtual ABng_Core_System::ABng_Location_Description::Value_Description* 
    value_descr (ENV_DECL1(env)
  );

  virtual ABng_Core_System::ABng_Location_Description::Value_Description* 
    _private_value_descr (ENV_DECL1(env)
  );
 
  virtual Domain_Repository::Repository_Item_Id item_id (
	ENV_DECL1(env)
  );

  virtual Domain_Repository::Location_Description* loc_descr(
	ENV_DECL1(env)
  );

  virtual ABng_Core_System::Sensors* sensor_list(
	ENV_DECL1(env)
  );

  virtual void add(ABng_Core_System::Sensor_ptr ENV_DECL(env)
  );

  virtual void remove(ABng_Core_System::Sensor_ptr ENV_DECL(env)
  );


#ifdef ObjectStore
  //
  // method for ObjectStore
  //

  static os_typespec* get_os_typespec();

#elif defined FilePersistency
  // methods to store and restore object of this class
  virtual void
   save (
	ostream & istr
   );

  virtual void
   restore (
	istream &ostr
   );
#endif
};   


#include "abng_location_descr_rep_i.h"

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_ABng_Core_System_ABng_Location_Description(ABng_Loc_Descr_I)
#else
DEF_TIE_ABng_Core_System_ABng_Location_Description(ABng_Loc_Descr_I)
#endif
#elif defined OMNIORB
// DEF_TIE__tie_ABng_Location_Description
#endif



#endif /* abng_location_description_i_h */
