//
//  File:         abng_location_descr_rep_i.cc
//  Author:       Malgorzata Steinder (gosia@ics.agh.edu.pl)
//  Date Created: 17.12.97
//
// General Description:
//   Implementation file for the ABng_Loc_Descr_Rep_I class
//

#define EXCEPTIONS

#include <rw/cstring.h>
#include <iostream.h>

#include "abng_location_descr_i.h"
#include "sensor_observer.h"
#include "location_descr_observer.h"

#include <rw/tvdlist.h>

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_ABng_Core_Management_ABng_Location_Description_Rep(ABng_Loc_Descr_Rep_I)
#else
DEF_TIE_ABng_Core_Management_ABng_Location_Description_Rep(ABng_Loc_Descr_Rep_I)
#endif
#elif defined OMNIORB
  // DEF_TIE__tie_ABng_Location_Description_Rep
#endif


ostream& operator << (ostream& str, ABng_Core_System::ABng_Location_Description::Description& descr);


//
// 		Constructor
//              ___________


  ABng_Loc_Descr_Rep_I::ABng_Loc_Descr_Rep_I (
#ifdef ObjectStore
    os_database* db_p
#elif defined FilePersistency
    const char *repository_name_p
#endif
  ) : Bootstrap_listener(),
      Basic_Rep_I<ALD_Types, ALD_SDO_Types>(hash_abng_loc_descr,
#ifdef ObjectStore
    db_p
#elif defined FilePersistency
    repository_name_p
#endif
 ) {
  _instance = this;
  create_corba_objects();
}

//
// 		Destructor
//		__________

ABng_Loc_Descr_Rep_I::~ABng_Loc_Descr_Rep_I() {

  destroy_corba_objects();
  delete ps_dev;
}

void ABng_Loc_Descr_Rep_I::init( int init_v) {
  Basic_Rep_I<ALD_Types, ALD_SDO_Types>::init( init_v);
  try {
    Bootstrap::register_server_in_NS(
        Bootstrap::SRV_ABng_Location_Description_Repository(),
	my_rep
    );
    
    register_callback( Bootstrap::SRV_Location_Description_Repository());
    register_callback( Bootstrap::SRV_Sensor_Repository());
    ps_dev= new ProcessControlImpl( "abng_location_description_repository", this);
  }
  catch ( CORBA::Exception &ex) {
    cerr << "Caught exception while contacting server manager" << ex << endl;
    exit( 1);
  }
}


void restore_loc_descr(
        ABng_Loc_Descr_I *current_abng_loc_descr,
	void* apply_data
) {

#ifdef DEBUG
  cerr << "restore_loc_descr" << endl;
#endif
  try {
     Domain_Repository::Location_Description::Pattern pattern;
     pattern.is_any = 0;
     pattern.item_id_pattern.is_any = 0;
     pattern.item_id_pattern.value =
         current_abng_loc_descr->get_loc_descr_id();
     pattern.type_name_pattern.is_any = 1;
     pattern.property_list_pattern.is_any = 1;
     pattern.property_list_pattern.constraints = 
         Domain_Repository::UNRESTRICTED;

     Domain_Repository::Location_Description_Rep_var* 
       location_description_rep
         = (Domain_Repository::Location_Description_Rep_var*)apply_data;
     Domain_Repository::Location_Descriptions* 
       _descrs = (*location_description_rep)->find(pattern);

     if(_descrs->length() == 1)
        current_abng_loc_descr->set_loc_descr((*_descrs)[0]);

     delete _descrs;

  } catch(CORBA::Exception &ex) {
    cerr << ex << endl;
  }
#ifdef DEBUG
  cerr << "restore_loc_descr>>" << endl;
#endif
}

void restore_sensor_list(
        ABng_Loc_Descr_I *current_abng_loc_descr,
	void* apply_data
) {
  Domain_Repository::Repository_Item_Ids _sensor_ids = 
      current_abng_loc_descr->get_sensor_ids();
  ABng_Core_System::Sensors* _sensors;
  ABng_Core_System::Sensor::Pattern pattern;
  pattern.is_any = 0;
  pattern.item_id_pattern.is_any = 0;
  pattern.id_pattern.is_any = 1;

  ABng_Core_Management::Sensor_Rep_var* sensor_rep =
     (ABng_Core_Management::Sensor_Rep_var*)apply_data;

  ABng_Core_System::ABng_Location_Description::Description d =
     current_abng_loc_descr->get_descr();
  ABng_Core_System::Sensors *sensor_list = & d.sensor_list;

  for(int i = 0, current = 0; i < _sensor_ids.length(); i ++) {
     pattern.item_id_pattern.value = _sensor_ids[i];

     // We let the exception propagate to other functions
     _sensors = (*sensor_rep)->find(pattern);
    
     if(_sensors->length() == 1) {
          (*sensor_list)[current] = 
           ABng_Core_System::Sensor::_duplicate((*_sensors)[0]);
          _sensor_ids[current] = _sensor_ids[i];
        current ++;
     }
     else current_abng_loc_descr->remove_sensor(_sensor_ids[i]);
  }
  (*sensor_list).length(current);
  _sensor_ids.length(current);
  current_abng_loc_descr->set_descr(d);
}

void 
 ABng_Loc_Descr_Rep_I::reference_resolved(
                                      const ABng_Server_Manager::Name &name,
				      CORBA::Object_var server_ref,
				      ABng_Server_Manager::uid callback_id) {
  auto_lock local_lock(*this);

  if ( name == Bootstrap::SRV_Location_Description_Repository()) {
    try {
     location_description_rep = 
        Domain_Repository::Location_Description_Rep::_narrow( server_ref);
    }
    catch (...) {
       location_description_rep = 
             Domain_Repository::Location_Description_Rep::_nil();
       return;
    }

    Location_Descr_Observer_I::instance()->reference_resolved(name, server_ref);
    
    try {
      ABng_Core_System::ABng_Location_Description::Pattern pattern;
      pattern.is_any = 1;
      ABng_Core_System::ABng_Location_Descriptions* descrs = 
        private_find(pattern);

#ifdef DEBUG
      cerr << "Items in database " << (unsigned) (*descrs).length() << endl;
#endif
      for(int i = 0; i < (*descrs).length(); i ++) {
#ifdef DEBUG
	cerr << i << endl;
#endif
         Item_I *item = deref_item((*descrs)[i]);
         restore_loc_descr(item, &location_description_rep);
         if(CORBA::is_nil(item->get_descr().loc_descr)) {
#ifdef DEBUG
	   cerr << "private_remove" << endl;
#endif
	   private_remove((*descrs)[i]);
	 }
	 else {
#ifdef DEBUG
	   cerr << "public_notify_update" << endl;
#endif
	   public_notify_update( item);
	 }
      }
    } catch(CORBA::Exception &ex) {
      cerr << "CORBA::Exception" << endl;
      return;
    } catch(...) {
      cerr << "Exception" << endl;
      return;
    }
#ifdef DEBUG
    cerr << "Items updated" << endl;
#endif
    save();
  } 
  else {
      if ( name == Bootstrap::SRV_Sensor_Repository()) {
         try {
             sensor_rep = 
                ABng_Core_Management::Sensor_Rep::_narrow( server_ref);
         }
         catch (...) {
             sensor_rep = ABng_Core_Management::Sensor_Rep::_nil();
         }

         Sensor_Observer_I::instance()->reference_resolved(name, server_ref);

         if(sensor_rep != NULL)
               my_item_list.apply(restore_sensor_list, &sensor_rep);
         save();
      } 
  }
}

ABng_Loc_Descr_Rep_I* 
 ABng_Loc_Descr_Rep_I::_instance = 0;

ABng_Loc_Descr_Rep_I* 
 ABng_Loc_Descr_Rep_I::instance() {
   return _instance;
}

void remove_s(ABng_Loc_Descr_I * it, void * apply_data) {
  ABng_Core_System::Sensor_ptr s = (ABng_Core_System::Sensor_ptr)apply_data;

  it->remove(s);
}

void ABng_Loc_Descr_Rep_I::remove_sensor(
    Domain_Repository::Repository_Item_Id id
) {
    
    RWTPtrHashTableIterator<Item_I> iterator(my_item_list);
    while(iterator()) {
      try {
       iterator.key()->remove_sensor(id);
      } catch(...) {
      }
    }
}


//
// Private functions
//



int
ABng_Loc_Descr_Rep_I::remove_permitted(ABng_Loc_Descr_I * abng_loc_descr_i_p) {
    
   return 1;
}
           
int ABng_Loc_Descr_Rep_I::validate_update_data(
  Item_ptr item_p, 
  const ABng_Core_System::ABng_Location_Description::Description& 
      data_record_pattern
) {

  cout << "ABng_Loc_Descr_Rep_I::validate_update_data" << endl;

  Basic_Rep_I<ALD_Types, ALD_SDO_Types>::validate_update_data(
    item_p, data_record_pattern);

  Item_I *item = deref_item(item_p);
  if(item->get_loc_descr_id() != data_record_pattern.loc_descr->item_id())
    throw Domain_Repository::Invalid_Description();
   
  return validate_update_add_data(item, data_record_pattern);
}


int ABng_Loc_Descr_Rep_I::validate_update_add_data(
  Item_I* item, 
  const ABng_Core_System::ABng_Location_Description::Description& 
      data_record_pattern
) {

  cout << "validate_update_add_data" << endl;

  Domain_Repository::Repository_Item_Ids new_sensor_ids(
      data_record_pattern.sensor_list.length()
  );
  new_sensor_ids.length(data_record_pattern.sensor_list.length());
  for(int j = 0; j < data_record_pattern.sensor_list.length(); j ++) {
     try {
         new_sensor_ids[j] = data_record_pattern.sensor_list[j]->item_id();
     } catch(...) {
	 cerr << "Cannot get sensor item_id" << endl;
         throw Domain_Repository::Invalid_Description();
     }
  }

  for(j = 0; j < (int)new_sensor_ids.length() - 1; j ++) {
     for(int i = j + 1; i < new_sensor_ids.length(); i ++) {
	if(new_sensor_ids[j] == new_sensor_ids[i]) {
	    cerr << "Two sensors with the same id are not allowed" << endl;
            throw Domain_Repository::Invalid_Description();
        }
     }
  }

  ABng_Core_System::ABng_Location_Description::Pattern pattern;
  pattern.is_any = 0;
  pattern.item_id_pattern.is_any = 1;
  pattern.loc_descr_pattern.is_any = 0;
  pattern.loc_descr_pattern.item_id_pattern.is_any = 1;
  pattern.loc_descr_pattern.type_name_pattern.is_any = 0;
  try {
    Domain_Repository::Location_Description::Description * d;
    d = data_record_pattern.loc_descr->descr();
    pattern.loc_descr_pattern.type_name_pattern.value = d->type_name;
     delete d;
  } catch(...) {
     cerr << "Unable to get type_name" << endl;
     throw Domain_Repository::Invalid_Description();
  }
  pattern.loc_descr_pattern.property_list_pattern.is_any = 1;


  ABng_Core_System::ABng_Location_Descriptions* descrs;
  try {
      descrs = private_find(pattern);
  } catch(...) {
      cout << "Exception in find???" << endl;
      return 0;
  }

  cout << "I have found " << (*descrs).length() << " items with the type_name " << pattern.loc_descr_pattern.type_name_pattern.value << endl;
  
  for(j = 0; j < new_sensor_ids.length(); j++) {
    Domain_Repository::Repository_Item_Id id = new_sensor_ids[j];
    for(int i = 0; i < (*descrs).length(); i ++) {
        Item_I *item1 = deref_item((*descrs)[i]);
        if(item) if(*item1 == *item) continue;
        if(item1->contains_sensor(id))
	  throw Domain_Repository::Duplicate_Data_Record();
    }
  }

  return 1;
}

int ABng_Loc_Descr_Rep_I::validate_add_data(
  const ABng_Core_System::ABng_Location_Description::Description& 
      data_record_pattern
) {

  cout << "ABng_Loc_Descr_Rep_I::validate_add_data" << endl;

  Basic_Rep_I<ALD_Types, ALD_SDO_Types>::validate_add_data(data_record_pattern);

  return validate_update_add_data(0, data_record_pattern);
}


void ABng_Loc_Descr_Rep_I::public_notify_update(ABng_Loc_Descr_I * item) {

  RWTValHashTable<Observer_ptr>* observer_list_p = 0;
  item_ids_to_observers_list.findValue(item->item_id(), observer_list_p);
  
  notify_observers(*observer_list_p, item, Domain_Repository::OBJECT_UPDATED);
#ifdef FilePersistency
  // store updated state of the repository
  save();
#endif

}

unsigned 
 ABng_Loc_Descr_Rep_I::hash_abng_loc_descr(
        const ABng_Loc_Descr_I& abng_loc_descr_i_p
) {
	return abng_loc_descr_i_p.get_loc_descr_id();
}


int
 ABng_Loc_Descr_Rep_I::MatchABng_Loc_DescrApplyDataStruct::match_loc_descr(
    Domain_Repository::Location_Description::Pattern& pattern,
    Domain_Repository::Repository_Item_Id _id
) {

   if(pattern.property_list_pattern.is_any)
       pattern.property_list_pattern.constraints = 
            Domain_Repository::UNRESTRICTED;

   if(!_loc_value_descrs) {
       if(CORBA::is_nil((Domain_Repository::Location_Description_Rep_ptr)
                         _location_description_rep
                       )
       ) throw Domain_Repository::Destination_Unreachable();

       try {

          _loc_value_descrs = _location_description_rep->find_values(pattern);

       } catch(...) {
          throw Domain_Repository::Destination_Unreachable();
       }
   }

   for(int i = 0; i < _loc_value_descrs->length(); i ++)
       if((*_loc_value_descrs)[i].item_id == _id) return 1;

   return 0;
}


int
ABng_Loc_Descr_Rep_I::MatchABng_Loc_DescrApplyDataStruct::match_sensor(
    ABng_Core_System::Sensor::Pattern& pattern,
    Domain_Repository::Repository_Item_Ids* _ids,
    int j,
    int& matching_i
) {

    if(!_sensor_value_descrs[j]) {
      cout << "Find " << j << endl;
      if(CORBA::is_nil((ABng_Core_Management::Sensor_Rep_ptr)_sensor_rep)) {
	cout << "NIL" << endl;
              throw Domain_Repository::Destination_Unreachable();
      }

        try {

            _sensor_value_descrs[j] = _sensor_rep->find_values(pattern);

        } catch(...) {
            cout << "Exception!!" << endl;
            throw Domain_Repository::Destination_Unreachable();
        }
    }
    if(!_sensor_value_descrs[j]->length()) {
      cout << "No items for this pattern found" << endl;
      return 0;
    }

    cout << "From: " << matching_i << endl;
    for(int ii = 0; ii < _sensor_value_descrs[j]->length(); ii ++)
      cout << (*_sensor_value_descrs[j])[ii].item_id << " ";
    cout << endl;
    for(ii = 0; ii < _ids->length(); ii ++)
      cout << (*_ids)[ii] << " ";
    cout << endl;

    // At least one from the returned sensor ids must be in the sequence _ids
    if(!matching_i) {
      for(int i = 0; i <  _sensor_value_descrs[j]->length(); i ++)
         for(int k = matching_i; k < _ids->length(); k ++)
             if((*_sensor_value_descrs[j])[i].item_id == (*_ids)[k]) {
                matching_i = k;
                return 1;
             }
    }
    else {
      for(int i = 0; i <  _sensor_value_descrs[j]->length(); i ++) {
	cout << (*_sensor_value_descrs[j])[i].item_id << " " << matching_i << endl;
           if((*_sensor_value_descrs[j])[i].item_id == (*_ids)[matching_i]) 
               return 1;
      }
    }
    cout << "The test failed --- " << endl;
    return 0;
}

int 
ABng_Loc_Descr_Rep_I::MatchABng_Loc_DescrApplyDataStruct::match_sensors_pattern(ABng_Loc_Descr_I *current_abng_loc_descr
) {
   if(!_data_record_pattern.sensor_list_pattern.is_any) {

      ABng_Core_System::Sensor::Sensors_Pattern * 
       pdp = & _data_record_pattern.sensor_list_pattern;
      Domain_Repository::Repository_Item_Ids* 
       pd = & current_abng_loc_descr->get_sensor_ids();

      if(pdp->constraints == Domain_Repository::RESTRICTED_ORDERING_AND_NUMBER
         || pdp->constraints == Domain_Repository::RESTRICTED_NUMBER
      ) if(pdp->sensors_pattern.length() != pd->length()) return 0;

      if(pdp->constraints == Domain_Repository::RESTRICTED_ORDERING_AND_NUMBER
         || pdp->constraints == Domain_Repository::RESTRICTED_ORDERING
      ) {
          int start_of_matching = 0;
          if(pdp->sensors_pattern.length() > pd->length()) return 0;

          while(start_of_matching <= (int)(pd->length()
                                     - pdp->sensors_pattern.length())
          ) {
             int i, j;
             for(i = 0, j = start_of_matching; 
                 i < pdp->sensors_pattern.length() && j < pd->length(); 
                 i++, j++) {
  
	         if(!pdp->sensors_pattern[i].is_any) {
                  
	            if(!pdp->sensors_pattern[i].item_id_pattern.is_any) {

                       if((*pd)[j] != 
                          pdp->sensors_pattern[i].item_id_pattern.value
                       )  break;
                    }
                
                   // Item Id matches or is irrelevant.
                   // What about other patterns? 
		   // (there is only one other pattern)
                   if(!pdp->sensors_pattern[i].id_pattern.is_any) {
                   
		      // Does any of sensor ids match one of those which
                      // match the pattern?
                      if(!match_sensor(pdp->sensors_pattern[i], pd, i, j)
                      ) break;
                   }
                }
            }
            if(i == pdp->sensors_pattern.length()) break;
            start_of_matching++;
         }
         if(start_of_matching > pd->length()-pdp->sensors_pattern.length())
            return 0;
      }
      else {
	if(pdp->constraints == Domain_Repository::RESTRICTED_NUMBER) {
           int * matched_pd = new int[pd->length()];
           for(int i = 0; i < pd->length(); i ++) matched_pd[i] = 0;

           for(i = 0; i < pdp->sensors_pattern.length(); i++) {
	     if(!pdp->sensors_pattern[i].is_any) {
                int j = 0;
	        if(!pdp->sensors_pattern[i].item_id_pattern.is_any) {
		  for(; j < pd->length(); j ++) {
                     if((*pd)[j] != 
                          pdp->sensors_pattern[i].item_id_pattern.value
                     )  continue;
                     if(!matched_pd[j]) {
                         matched_pd[j] = 1;
                         break;
                     }
                  }
                  // If none of sensor ids equals to  that in the pattern
                  if(j == pd->length()) { 
                     delete matched_pd; 
                     return 0; 
                  }
                }
                
                // Item Id matches or is irrelevant.
                // What about other patterns? 
                if(!pdp->sensors_pattern[i].id_pattern.is_any) {
                   if(!match_sensor(pdp->sensors_pattern[i], pd, i, j)
		   )  { delete matched_pd; return 0; }
                   matched_pd[j] = 1;
                }
             }
          }
          delete matched_pd;
        } else {
          for(int i = 0; i < pdp->sensors_pattern.length(); i++) {
	    if(!pdp->sensors_pattern[i].is_any) {
                
	        if(!pdp->sensors_pattern[i].item_id_pattern.is_any) {
		     for(int j = 0; j < pd->length(); j ++)
                       if((*pd)[j] == 
                          pdp->sensors_pattern[i].item_id_pattern.value
                       )  break;
                     // If none of sensor ids equals to  that in the pattern
                     if(j == pd->length()) return 0;
                }
                
                // Item Id matches or is irrelevant.
                // What about other patterns? (there is only one other pattern)
                if(!pdp->sensors_pattern[i].id_pattern.is_any) {
                   
		   // Does any of sensor ids match one of those which
                   // match the pattern?
                   int j = 0;
                   if(!match_sensor(pdp->sensors_pattern[i], pd, i, j)
                   ) return 0;
                }
             }
          }
        }
      }
   }
   return 1;
}

void
 ABng_Loc_Descr_Rep_I::match_abng_loc_descr(
	ABng_Loc_Descr_I *current_abng_loc_descr,
	void* apply_data
) {
        struct MatchABng_Loc_DescrApplyDataStruct* match_struct =
           (struct MatchABng_Loc_DescrApplyDataStruct*)apply_data;

        // drp == data_record_pattern!!!!!
	ABng_Core_System::ABng_Location_Description::Pattern 
        drp = match_struct->_data_record_pattern;

        //
	// check general patten matching rule
        //
	if (drp.is_any){
	     gather_item (current_abng_loc_descr, 
                          match_struct->_found_abng_loc_descrs
             );
	     return;
	}
	
        //
	// check matching of item_id
        //
	if (!drp.item_id_pattern.is_any && 
            (drp.item_id_pattern.value 
               != current_abng_loc_descr->get_descr().item_id
            )
        ) return;
	
        //
	// check matching of loc_descr
        //
	if (!drp.loc_descr_pattern.is_any) {
           if (!drp.loc_descr_pattern.item_id_pattern.is_any && 
                   drp.loc_descr_pattern.item_id_pattern.value 
                   != current_abng_loc_descr->get_loc_descr_id()
           ) return;

           // Now, if other patterns matter...

           if(!drp.loc_descr_pattern.type_name_pattern.is_any ||
              !drp.loc_descr_pattern.property_list_pattern.is_any
	      ) {
		// Check if loc_descr id of the current item 
                // belongs to matching location descriptions. If not, return.
	        if(!match_struct->match_loc_descr(
                           drp.loc_descr_pattern,
                           current_abng_loc_descr->get_loc_descr_id()
                   )
                ) return;
           }
        }

	
        //
        // check matching of sensor_list
        //
        if(!match_struct->match_sensors_pattern(current_abng_loc_descr))
           return;
	
        //
	// this abng_loc_descr matches the pattern
        //
	gather_item(
                current_abng_loc_descr, 
		match_struct->_found_abng_loc_descrs
        );
	
}


 
ABng_Loc_Descr_Rep_I::MatchABng_Loc_DescrApplyDataStruct::~MatchABng_Loc_DescrApplyDataStruct() {

      if(_loc_value_descrs) delete _loc_value_descrs;
        for(int i = 0; i <  _sensor_value_descrs.length(); i ++)
          if( _sensor_value_descrs[i]) delete  _sensor_value_descrs[i];
} 

ABng_Loc_Descr_Rep_I::MatchABng_Loc_DescrApplyDataStruct::MatchABng_Loc_DescrApplyDataStruct(
        ABng_Core_System::ABng_Location_Descriptions* found_abng_loc_descrs,
        const ABng_Core_System::ABng_Location_Description::Pattern& 
            data_record_pattern,
        Domain_Repository::Location_Description_Rep_var 
            location_description_rep,
        ABng_Core_Management::Sensor_Rep_var sensor_rep
    ):  _found_abng_loc_descrs(found_abng_loc_descrs),
        _data_record_pattern(data_record_pattern),
        _loc_value_descrs(0),
        _location_description_rep(location_description_rep),
        _sensor_rep(sensor_rep)
   {
         if(data_record_pattern.is_any) return;
         if(data_record_pattern.sensor_list_pattern.is_any) return;
         _sensor_value_descrs.reshape(
            data_record_pattern.sensor_list_pattern.sensors_pattern.length()
         );
         for(int i = 0; i < _sensor_value_descrs.length(); i ++)
             _sensor_value_descrs[i] = 0;
    }
//
// Implementation of IDL operations of the ABng_Location_Description_Rep interface
//                                         __________

ABng_Core_Management::ABng_Location_Description_Rep *
 ABng_Loc_Descr_Rep_I::rep(ENV_DEF1(env)) {
return ABng_Core_Management::ABng_Location_Description_Rep::_duplicate(my_rep);
}

ABng_Core_System::ABng_Location_Descriptions* 
   ABng_Loc_Descr_Rep_I::private_find (
        const ABng_Core_System::ABng_Location_Description::Pattern& 
            data_record_pattern
        ENV_DEF(env)
   ) {

        MatchABng_Loc_DescrApplyDataStruct 
         apply_data(new ABng_Core_System::ABng_Location_Descriptions(
                          my_item_list.entries()
                        ),
                    data_record_pattern,
                    location_description_rep,
                    sensor_rep
         );

        my_item_list.apply(match_abng_loc_descr, (void*)&apply_data);

        // pointer to found_abng_loc_descrs was stored inside apply_data 
        // thus return it
        return apply_data._found_abng_loc_descrs;
}

void ABng_Loc_Descr_Rep_I::collect_values(ABng_Loc_Descr_I *lt, 
  ABng_Core_System::ABng_Location_Description::Value_Description& ld) {

  ld = *(lt->value_descr());
}

void ABng_Loc_Descr_Rep_I::init_corba_object(ABng_Loc_Descr_I *item_p) {

  // create the TIE (CORBA) object for the C++ object
#ifdef ORBIX
#ifdef ObjectStore
  ObjectStoreTIE_ABng_Core_System_ABng_Location_Description(ABng_Loc_Descr_I)* 
    new_abng_loc_descr_tie =
    new(my_db, ObjectStoreTIE_ABng_Core_System_ABng_Location_Description(
      ABng_Loc_Descr_I)::get_os_typespec()) 
      ObjectStoreTIE_ABng_Core_System_ABng_Location_Description(
        ABng_Loc_Descr_I
	) (item_p);
#else
  TIE_ABng_Core_System_ABng_Location_Description(ABng_Loc_Descr_I) *
    new_abng_loc_descr_tie = 
    new TIE_ABng_Core_System_ABng_Location_Description(ABng_Loc_Descr_I) (item_p);
#endif
#elif defined OMNIORB
  ABng_Core_System::ABng_Location_Description_ptr
    new_abng_loc_descr_tie = 
    new TIE_CLASS( ABng_Core_System,ABng_Location_Description)<ABng_Loc_Descr_I,0>(item_p);  
  CONNECT_IMPL( ABng_Core_System::ABng_Location_Description_var, new_abng_loc_descr_tie);
#endif
  item_p->set_corba_object(new_abng_loc_descr_tie);
}

DB_Header * ABng_Loc_Descr_Rep_I::create_db_header() {

  return new DB_Header( "ABng_Location_Description Repository", "1.0", "\
#       <file>::=       <next_item_id>\n\
#                       <number_of_items>\n\
#                       <data_records>\n\
#\n\
#       <data_records>::=  | <data_record> <data_records>\n\
#\n\
#       <data_record>::=        <repository_item_id>\n\
#                               <loc_descr_item_id>\n\
#                               <sensor_ids>\n\
#\n\
#       <sensor_ids>::=  \"<\" <seq_maximum> <seq_length>\n\
#                              <snsor_ids_buffer> \">\"\n\
#\n\
#       <sensor_ids_buffer>::=  | <sensor_id> <sensor_ids_buffer>\n\
#\n\
#       <sensor_id>::= <repository_item_id>\n\
#\n\
#       <loc_descr_item_id>::=   <repository_item_id>\n\
#\n\
#\n");
}

void ABng_Loc_Descr_Rep_I::construct_data_object(
    ABng_Core_System::System_Static_Data_Observer::System_Static_Data_Object& 
      data_object_p,
    ABng_Core_System::ABng_Location_Description_ptr item_p) {


  // construct union to store reference of added entity
  data_object_p.abng_loc_descr_obj(
    ABng_Core_System::ABng_Location_Description::_duplicate(item_p));
}

void ABng_Loc_Descr_Rep_I::create_corba_objects() {

#ifdef ORBIX
#ifdef FilePersistency
  // create a TIE for the object
  my_rep = 
    new TIE_ABng_Core_Management_ABng_Location_Description_Rep(
      ABng_Loc_Descr_Rep_I) (this);
#endif /* FilePersistency */
#elif defined OMNIORB
  my_rep =
    new TIE_CLASS(ABng_Core_Management,ABng_Location_Description_Rep)<ABng_Loc_Descr_Rep_I,0>(this);
  CONNECT_IMPL( ABng_Core_System::ABng_Location_Description_Rep_var,my_rep);
  
#endif

 write_ior(my_rep, 
    "./ABng_Location_Description_Rep_iiop.ref");
}

void ABng_Loc_Descr_Rep_I::destroy_corba_objects() {

  CORBA::release(my_rep);
}


