// $Id: away_i.cc,v 1.1.1.1 1998/07/29 14:16:23 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/away_rep/away_i.cc,v $
//
// $Log: away_i.cc,v $
// Revision 1.1.1.1  1998/07/29 14:16:23  mencnar
// initial revision
//
//
// File:          away_i.cc
// Author:        Daniel Mencnarowski (mencnar@ics.agh.edu.pl)
// Date Created:  07.07.98
//
// General Description:
//  Implementation file for the Away_I class
//

#ifdef ORBIX
#define EXCEPTIONS
#endif

#include "away_rep_i.hh"

#ifdef ObjectStore
static os_typespec *char_type = new os_typespec("char");
#endif

int operator == ( const ABng_Poller::AB_Poller::Badge_Id &first,
                  const ABng_Poller::AB_Poller::Badge_Id &second) {
  if (first.length() != second.length()) return 0;
  int size= first.length();
  for (int i=0; i<size; i++)
    if (first[i] != second[i]) return 0;
  return 1;
}

static inline void normalize_time( Domain_Repository::Time &t) {
  while (t.microseconds>1000000) {
    t.microseconds-=1000000;
    t.seconds++;
  }
  while (t.microseconds<0) {
    t.microseconds+= 1000000;
    t.seconds--;
  }
  if (t.seconds<0) {
    if (t.microseconds>0) {
      t.seconds++;
      t.microseconds-= 1000000;
    }
  }
}

int operator < (const Domain_Repository::Time &t1,
		const Domain_Repository::Time &t2) {

  Domain_Repository::Time xt1( t1), xt2( t2);
  normalize_time(xt1);
  normalize_time(xt2);
  return ( xt1.seconds < xt2.seconds) ? 1 : 
    ((xt1.seconds>xt2.seconds) ? 0 : (xt1.microseconds<xt2.microseconds));
}


//
// Implementation of constructor, destructor and operators
//                   _______________________________________________________

  Away_I::Away_I (
	const ABng_Core_Management::Away_Rep_Manager::Change_Description& descr_init
	) : 
    my_guard( *(Away_Rep_I::instance())),
    badge_holder_item_id( 0),
    pending_delete( 0)
{
#ifdef DEBUG
  cerr << "Away_I constructor (description) " << (int)this << endl;
#endif
   // init my_corba_object
  my_corba_object = (ABng_Core::Away_ptr)NULL;

  set_descr( descr_init);
}

#ifdef FilePersistency
   Away_I::Away_I (
	istream &iostr
	) :
     my_guard( *(Away_Rep_I::instance())),
     pending_delete( 0)
{
#ifdef DEBUG
  cerr << "Away_I stream constructor" << endl;
#endif
  // read description from the file
  restore( iostr);
  
  // init my_corba_object
  my_corba_object = (ABng_Core::Away_ptr)NULL;
}
#endif

Away_I::~Away_I (
  )
{
#ifdef DEBUG
  cerr << "Away_I destructor " << (int) this << endl;
#endif
}

Away_I::operator== (
	const Away_I& away_p
	)
{
  return this == &away_p;
}

//
// Implementation of methods enabling access to the state
//

void
  Away_I::set_descr (
	const ABng_Core_Management::Away_Rep_Manager::Change_Description& descr_init
  )
{
  ABng_Core::Away::Description new_descr;

  // init my_descr
  new_descr.item_id= my_descr.item_id;
  new_descr.start_time= descr_init.start_time;
  new_descr.stop_time= descr_init.stop_time;
  new_descr.cancel_from_time= descr_init.cancel_from_time;
  new_descr.message= CORBA::string_dup( descr_init.message);
  new_descr.contact= CORBA::string_dup( descr_init.contact);
  new_descr.cancel_by_badge= descr_init.cancel_by_badge;
  new_descr.active= 0; // active flag always ignored

  // Commit !
  //
  ABng_Core_System::Badge_Holder_var tmp_holder_var=
    ABng_Core_System::Badge_Holder::_duplicate( descr_init.away_holder);
  change_holder( tmp_holder_var,
		 new_descr);
  my_descr= new_descr;
}

void Away_I::fast_change_holder( ABng_Core_System::Badge_Holder_var new_holder_var,
				 ABng_Core_System::Badge_Holder::Description &bhd,
				 ABng_Core_System::Badge_Holder::Value_Description &bhvd,
				 ABng_Core::Away::Description &new_descr) {

  if (! CORBA::is_nil( (ABng_Core_System::Badge_Holder_ptr) new_holder_var) &&
      bhd.item_id != 0) {

    new_descr.type_specific_object= bhd.type_specific_object;
    my_holder= new_holder_var;
    my_badge= ABng_Core_System::Badge::_duplicate( bhd.my_badge);
    if (bhvd.my_badge.item_id != 0) {
      my_badge_item_id= bhvd.my_badge.item_id;
      badge_id= bhvd.my_badge.id;
    }
  }
}

void Away_I::change_holder( ABng_Core_System::Badge_Holder_var new_holder_var,
			    ABng_Core::Away::Description &new_descr) {
  
  if (! CORBA::is_nil( (ABng_Core_System::Badge_Holder_ptr) new_holder_var)) {
    try {
      Domain_Repository::Repository_Item_Id tmp_holder_id= 
	new_holder_var->item_id();
      ABng_Core_System::Badge_var tmp_badge_var= 
	new_holder_var->my_badge();
      Domain_Repository::Repository_Item_Id tmp_badge_item_id=
	tmp_badge_var->item_id();
      
      ABng_Core::Locatable_Type_Specific_Object_var ltso=
	new_holder_var->type_specific_object();
	
      new_descr.type_specific_object= *(ltso.operator->());

      ABng_Core_System::Badge_Id_var tmp_badge_id= tmp_badge_var->id();

      tmp_badge_var->attach_system_static_data_observer(
				       Away_Rep_I::instance()->get_ssdo());
      new_holder_var->attach_system_static_data_observer(
				       Away_Rep_I::instance()->get_ssdo());

      // Commit !
      //
      badge_holder_item_id= tmp_holder_id;
      my_badge_item_id= tmp_badge_item_id;
      my_holder= new_holder_var;
      my_badge= tmp_badge_var;
      badge_id= *(tmp_badge_id.operator->());
    }
    catch (Domain_Repository::Destination_Unreachable) {
      throw;
    }
    catch (...) {
      throw Domain_Repository::Destination_Unreachable("badge_holder_repository");
    }
  }
  else throw Domain_Repository::Invalid_Description();  
}

void
   Away_I::set_corba_object (
	ABng_Core::Away_ptr corba_object_p
   )
{
  my_corba_object = corba_object_p;
}

ABng_Core::Away_ptr
   Away_I::get_corba_object (
   ) 
{
  return my_corba_object;
}

void 
   Away_I::set_item_id(
	const Domain_Repository::Repository_Item_Id item_id_p
   )
{
  my_descr.item_id = item_id_p;
}

void Away_I::resolve_badge_holder( ABng_Core_Management::Badge_Holder_Rep_var holder_rep) {
  ABng_Core_System::Badge_Holder::Pattern pat;

  Domain_Repository::User::Pattern user_pattern;
  pat.is_any= 0;
  pat.item_id_pattern.is_any= 0;
  pat.item_id_pattern.value= badge_holder_item_id;
  pat.my_badge_pattern.is_any= 1;
  pat.current_badge_pattern.is_any= 1;
  pat.home_sensor_pattern.is_any= 1;
  pat.type_specific_object_pattern.is_any= 1;
  pat.type_specific_object_pattern.object_pattern.user_obj_pattern( user_pattern);
  pat.assigned_pattern.is_any= 1;
  pat.visitor_pattern.is_any= 1;

#ifdef DEBUG
  cerr << "resolve_badge_holder (item_id= " << badge_holder_item_id << ")" << endl;
#endif
  try {
    ABng_Core_System::Badge_Holders_var ret_val( 
		holder_rep->find( pat) );
    if (ret_val->length() == 1) { 
      ABng_Core_System::Badge_Holder_var tmp_holder_var= 
	ABng_Core_System::Badge_Holder::_duplicate( ret_val[0]);
      change_holder( tmp_holder_var,
		     my_descr);
#ifdef DEBUG
      cerr << "Resolved" << endl;
#endif
    }
  }
  catch (CORBA::COMM_FAILURE &ex) { 
    throw;
  }
  catch (...) {}
}

//
// Implementation of IDL operations of the Away_Observed_E interface
//                                         _____________________________

ABng_Core::Away_Data_Observed_E::Away_Data_Observers*
   Away_I::away_data_observer_list (
        ENV_DEF1(env)) {

     return Away_Rep_I::instance()->away_data_observer_list(
				       my_descr.item_id USE_ENV(env));
}

void 
   Away_I::attach_away_data_observer (
	ABng_Core::Away_Data_Observer_ptr obs
	ENV_DEF(env)) {

#ifdef DEBUG
  cerr << "Away_I::attach_away_data_observer " << (int) obs << " "
#ifdef ORBIX
       << obs->_refCount()
#endif
       << endl;
#endif

  // ignore observed_id_list parameter
  //  initialize the sequence for this badge item id
  Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

  // put this away item it into the sequance
  observed_id_list_p.length(1);
  observed_id_list_p[0] = my_descr.item_id;

  // register observer into the repository list
  Away_Rep_I::instance()->attach_away_data_observer(obs,
		observed_id_list_p USE_ENV(env));
}

void Away_I::detach_away_data_observer (
	ABng_Core::Away_Data_Observer_ptr obs
	ENV_DEF(env)) {
    
    
  // ignore observed_id_list parameter
  //  initialize the sequence for this away item id
  Domain_Repository::Repository_Item_Ids observed_id_list_p(1);
  
  // put this  item it into the sequance
  observed_id_list_p.length(1);
  observed_id_list_p[0] = my_descr.item_id;

  // register observer into the repository list
  try {
    Away_Rep_I::instance()->detach_away_data_observer(obs,
						      observed_id_list_p USE_ENV(env));
  }
  catch ( Domain_Repository::Unknown_Observer &ex) {
    throw ex;
  }
  catch ( ...) {}
}

//
// Implementation of IDL operations of the Away interface
//                                         ______
 
ABng_Core::Away::Description* Away_I::descr( ENV_DEF1(env)) {

  ABng_Guarded::auto_lock local_lock( my_guard);

  if (badge_holder_item_id != 0 &&
      CORBA::is_nil( (ABng_Core_System::Badge_Holder_ptr) my_holder)) 
    throw Domain_Repository::Destination_Unreachable("badge_holder_rep");

  // duplicate my_descr
  ABng_Core::Away::Description* my_descr_duplicated =
    new ABng_Core::Away::Description(my_descr);

  return my_descr_duplicated;
}

ABng_Core::Away::Value_Description*
  Away_I::value_descr (
      	ENV_DEF1(env)) {

  ABng_Guarded::auto_lock local_lock( my_guard);

  if (badge_holder_item_id != 0 &&
      CORBA::is_nil( (ABng_Core_System::Badge_Holder_ptr) my_holder)) 
    throw Domain_Repository::Destination_Unreachable("badge_holder_rep");
  
  ABng_Core::Away::Value_Description *ret_descr= 
    new ABng_Core::Away::Value_Description();

  try {
    ret_descr->item_id= my_descr.item_id;
    ret_descr->start_time= my_descr.start_time;
    ret_descr->stop_time= my_descr.stop_time;
    ret_descr->cancel_from_time= my_descr.cancel_from_time;
    ret_descr->message= CORBA::string_dup( my_descr.message);
    ret_descr->contact= CORBA::string_dup( my_descr.contact);
    ret_descr->cancel_by_badge= my_descr.cancel_by_badge;
    ret_descr->active= my_descr.active;

    ABng_Core_System::Badge_Holder::Value_Description_var bhval_var=
      my_holder->value_descr();
    ret_descr->type_specific_object_value= bhval_var->type_specific_object;

    return ret_descr;
  }
  catch (Domain_Repository::Destination_Unreachable ) {
    delete ret_descr;
    throw;
  }
  catch (...) {
    delete ret_descr;
    throw Domain_Repository::Destination_Unreachable("badge_holder_rep");
  }
  return NULL; // never get here
}
 
Domain_Repository::Repository_Item_Id 
   Away_I::item_id (
	ENV_DEF1(env)) {
  // no lock here: item_id is immutable

  return my_descr.item_id;
}

ABng_Core::Locatable_Type_Specific_Object* Away_I::type_specific_object(
	ENV_DEF1(env)
	) {
  
  ABng_Guarded::auto_lock local_lock( my_guard);

  return new ABng_Core::Locatable_Type_Specific_Object( my_descr.type_specific_object);
}

Domain_Repository::Time Away_I::start_time(
        ENV_DEF1(env)
	) {
  ABng_Guarded::auto_lock local_lock( my_guard);

  return my_descr.start_time;
}

Domain_Repository::Time Away_I::stop_time(
        ENV_DEF1(env)
	) {

  ABng_Guarded::auto_lock local_lock( my_guard);

  return my_descr.stop_time;
}

Domain_Repository::Time Away_I::cancel_from_time(
        ENV_DEF1(env)
	) {

  ABng_Guarded::auto_lock local_lock( my_guard);

  return my_descr.cancel_from_time;
}

char * Away_I::message(
        ENV_DEF1(env)
	) {

  ABng_Guarded::auto_lock local_lock( my_guard);

  return CORBA::string_dup( my_descr.message);
}

char * Away_I::contact(
        ENV_DEF1(env)
	) {

  ABng_Guarded::auto_lock local_lock( my_guard);

  return CORBA::string_dup( my_descr.contact);
}

CORBA::Boolean Away_I::cancel_by_badge(
        ENV_DEF1(env)
	) {

  ABng_Guarded::auto_lock local_lock( my_guard);

  return my_descr.cancel_by_badge;
}

CORBA::Boolean Away_I::active(
        ENV_DEF1(env)
	) {

  ABng_Guarded::auto_lock local_lock( my_guard);

  return my_descr.active;
}

#ifdef FilePersistency
//
//		Store and restore state methods
//		_________________


void
   Away_I::save (
	ostream &iostr
   )
{
  iostr << dec << my_descr.item_id << ' ';
  iostr << badge_holder_item_id << ' ';
  iostr << my_descr.start_time.seconds << ' ' 
	<< my_descr.start_time.microseconds << ' ';
  iostr << my_descr.stop_time.seconds << ' ' 
	<< my_descr.stop_time.microseconds << ' ';
  iostr << my_descr.cancel_from_time.seconds << ' ' 
	<< my_descr.cancel_from_time.microseconds << ' ';
  save_string( iostr, my_descr.message); iostr << ' ';
  save_string( iostr, my_descr.contact); iostr << ' ';
  iostr << (int) my_descr.cancel_by_badge;
  iostr << endl;
}

void
   Away_I::restore (
	istream &iostr
   )
{
  iostr >> dec >> my_descr.item_id;
  iostr >> badge_holder_item_id;
  iostr >> my_descr.start_time.seconds
	>> my_descr.start_time.microseconds;
  iostr >> my_descr.stop_time.seconds 
	>> my_descr.stop_time.microseconds;
  iostr >> my_descr.cancel_from_time.seconds 
	>> my_descr.cancel_from_time.microseconds;
  char *tmp= NULL;
  restore_string( iostr, tmp);
  my_descr.message= tmp;
  restore_string( iostr, tmp);
  my_descr.contact= tmp;
  int tmpint;
  iostr >> tmpint;
  my_descr.cancel_by_badge= (CORBA::Boolean) tmpint;
  my_descr.active= 0;
  if (!iostr) {
    cerr << "Away_I error: reading record from a stream";
    exit(1);
  }
}
#endif /* FilePersistency */



