// $Id: away_rep_i.hh,v 1.1.1.1 1998/07/29 14:16:23 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/away_rep/away_rep_i.hh,v $
//
// $Log: away_rep_i.hh,v $
// Revision 1.1.1.1  1998/07/29 14:16:23  mencnar
// initial revision
//
//
// File:          away_rep_i.hh
// Author:        Daniel Mencnarowski (mencnar@ics.agh.edu.pl)
// Date Created:  07.01.98
//
// General Description:
//   Header file for Away_Rep implementation class
//
 

#ifndef away_rep_i_hh
#define away_rep_i_hh

#include <stdio.h>
#include <stdlib.h>
#include <fstream.h>
#include <errno.h>
#include <string.h>

#ifdef ObjectStore
#include <oosa.h>
#endif


#include "abng_core_management.hh"
#include "away_i.hh"
#include "bootstrap_n.hh"
#include "abng_guarded.hh"
#include "observed_i.h"
#include "basic_rep_i.h"
#include "process_status.hh"
#include "basic_abobserver_i.h"

/*
#include "rw/tphasht.h"
#include "rw/tvhasht.h"
#include "rw/tphdict.h"
#include "rw/tvhdict.h"
*/
#include <rw/tvdlist.h>


class ADO_Types {

public:

  typedef ABng_Core::Away_Data_Observed_R Observed;
  typedef ABng_Core::Away_Data_Observer Observer;
  typedef ABng_Core::Away_Data_Observer_ptr Observer_ptr;
  typedef ABng_Core::Away_Data_Observed_E::Away_Data_Observers Observers;
};

class Away_Types {

public:

  typedef ABng_Core::Away Item;  
  typedef ABng_Core::Away_ptr Item_ptr;  
  typedef ABng_Core::Aways Items;  
  typedef Away_I Item_I;  
  typedef ABng_Core_Management::Away_Rep_Manager::Change_Description Initial_Data;
  typedef ABng_Core::Away::Description Description;
  typedef ABng_Core::Away_Data_Observer::Away_Data_Object Data_Object;
};


class Away_Rep_I  : public Basic_AB_Observer_I,
		    public Bootstrap_listener,
		    public Basic_Rep_I<Away_Types, ADO_Types>,
		    public ServerStatusInterface 
{ 		      
public:
  Away_Rep_I (
#ifdef ObjectStore
    os_database* db_p
#elif defined FilePersistency
    const char *repository_name_p
#endif
  );
		      
  virtual ~Away_Rep_I ();

  virtual void init( int init_v);
 
  // get instance of this class 
  //  - Singleton pattern (E. Gamm & al.,"Design Paterns", p. 127)
  static Away_Rep_I* 
    instance ();
 
 private:

  ABng_Core_Management::Badge_Holder_Rep_var r_badge_holder_rep;
  ABng_Core_Management::Badge_Rep_var r_badge_rep;
  Away_I *last_active_away_set;
  
  
  // calculate the hash value of the away
  static unsigned 
   hash_away(
	const Away_I& away_i_p
   ); 

  // put the current away into the container
  static void
   gather_away(
	Away_I* current_away,
	void* apply_data
   );

  // definition of structure containing parameters of match_away method
  struct MatchAwayApplyDataStruct {
    RWTValDlist<Away_I*> _found_aways;
    const ABng_Core::Away::Pattern& _data_record_pattern;

    MatchAwayApplyDataStruct(
   	const ABng_Core::Away::Pattern& _data_record_pattern_p):
		_data_record_pattern(_data_record_pattern_p) {}
  };

  // if the current away matches the pattern puts it into the container
  static void
   match_away(
	Away_I* current_away,
	void* apply_data
   );
 
  // put the current observer into the container
  static void
   gather_observers(
	Away_I* current_away,
	void* apply_data
   );

  virtual Description get_null_description( Item_I *item);
  virtual Value_Description get_null_value_description( Item_I *item);
 
 protected:
  // ctor:
  Away_Rep_I (
#ifdef ObjectStore
	os_database* db
#elif defined FilePersistency
#endif
  );
  void resolve_badge_holders();
  void attach_badges();

  void select_active_message( Domain_Repository::Repository_Item_Id holder_id,
			      int dont_remove);
  void heart_beat();
  static void static_heart_beat( void *args);
  ABng_S_Guarded heart_beat_sleeper;
  omni_condition heart_beat_cond;
  void wake_up_heart_beat();
  int heart_beat_needed;

  void my_update( Away_I *item);
  void my_remove( Away_I *item);
public:
  void reference_resolved( const ABng_Server_Manager::Name &name,
			   CORBA::Object_var server_ref,
			   ABng_Server_Manager::uid callback_id);

  //
  // IDL operations of the Away_Observed_R interface
  //                       _____________________________

  virtual ABng_Core::Away_Data_Observed_R::Observer_Entries*
   away_data_observer_list (
        ENV_DECL1(env)
   ) 
	throw (CORBA::SystemException) {
        return observer_list(USE_ENV1(env));
  }
             
  virtual ABng_Core::Away_Data_Observed_E::
     Away_Data_Observers*
   add_away_data_observer_list (
	ENV_DECL1(env)
   ) 
	throw (CORBA::SystemException) {
        return add_observer_list( USE_ENV1(env));
   }

  virtual void 
  attach_away_data_observer (
	ABng_Core::Away_Data_Observer_ptr obs,  
	const Domain_Repository::Repository_Item_Ids& observed_id_list
	ENV_DECL(env)
   )
	throw (CORBA::SystemException, 
               Domain_Repository::Unknown_Repository_Item_Ids
        ) {
        attach_observer(obs, observed_id_list USE_ENV(env));
  }

  virtual void 
   detach_away_data_observer (
        ABng_Core::Away_Data_Observer_ptr obs, 
        const Domain_Repository::Repository_Item_Ids& observed_id_list
        ENV_DECL(env)
   ) 
        throw (CORBA::SystemException, 
               Domain_Repository::Unknown_Repository_Item_Ids, 
               Domain_Repository::Unknown_Observer,
               Domain_Repository::Unmatched_Observer_Item_Ids
        ) {
        detach_observer( obs, observed_id_list USE_ENV(env));
  }



  virtual void 
   attach_add_away_data_observer (
	ABng_Core::Away_Data_Observer_ptr obs
	ENV_DECL(env)
   )
	throw (CORBA::SystemException) {
        attach_add_observer(obs USE_ENV(env));
  }
 
  virtual void 
   detach_add_away_data_observer (
        ABng_Core::Away_Data_Observer_ptr obs
        ENV_DECL(env)
   ) 
        throw (CORBA::SystemException, 
               Domain_Repository::Unknown_Observer) {
        detach_add_observer(obs USE_ENV(env));
  }

  // Suplementary operation

  virtual ABng_Core::Away_Data_Observed_E::
     Away_Data_Observers*
   away_data_observer_list (
	Domain_Repository::Repository_Item_Id item_id_p
	ENV_DECL(env)
   ) 
	throw (CORBA::SystemException) {
        return get_item_observers(item_id_p USE_ENV(env));
  }
  
  //
  // IDL operations of the System_Static_Data_Observer interface
  //

  virtual void 
   notify (
        Domain_Repository::Repository_Item_Id id,
        const ABng_Core_System::System_Static_Data_Observer::System_Static_Data_Object& sd, 
        Domain_Repository::Change_Type change
        ENV_DECL(env)
   );

  virtual void 
   abort (
        ENV_DECL1(env)
   );

  //
  // IDL operations of the Away_Rep+Away_Rep_Manager interface
  //                       __________

  
  virtual ABng_Core::Away_Rep_ptr rep( ENV_DECL1(env)) {
    return ABng_Core::Away_Rep::_duplicate( my_rep);
  }

  virtual ABng_Core::Aways* 
   away_list (
	ENV_DECL1(env)
   ) 
    throw (CORBA::SystemException) {
    return item_list( USE_ENV1(env));
  }

  virtual ABng_Core::Aways* 
   find (
	const ABng_Core::Away::Pattern& data_record_pattern
	ENV_DECL(env)
   );

  virtual ABng_Core::Aways* 
   private_find (
	const ABng_Core::Away::Pattern& data_record_pattern
	ENV_DECL(env)
   );

  virtual ABng_Core::Away::Value_Descriptions* 
   find_values (
	const ABng_Core::Away::Pattern& data_record_pattern
        ENV_DECL(env)) {

     return Basic_Rep_I<Away_Types, ADO_Types>::find_values( 
		  data_record_pattern USE_ENV(env));
  }

  virtual ABng_Core::Away::Query_Results*
   complex_query(
		 const ABng_Core::Away::Entity_Set &es,
		 const Domain_Repository::Query_Options &qo,
		 ABng_Core::Away_Data_Observer_ptr obs
		 ENV_DECL(env)) {

     return Basic_Rep_I<Away_Types, ADO_Types>::complex_query(
		   es,qo,obs USE_ENV(env));
  }

  virtual ABng_Core::Away_ptr 
   add (
	const ABng_Core_Management::Away_Rep_Manager::Change_Description& data_record
	ENV_DECL(env)
	);

  virtual void 
   remove (
	ABng_Core::Away_ptr object_to_remove
	ENV_DECL(env)
	);
  
  virtual void 
   update (
	ABng_Core::Away_ptr object_to_update, 
	const ABng_Core_Management::Away_Rep_Manager::Change_Description& 
              data_record_pattern
	ENV_DECL(env)
	); 

  virtual ABng_Core_System::Badge_Holder_ptr deref_away(
    ABng_Core::Away_ptr away_object
    ENV_DEF(env)) {

    Away_I *away_i= deref_item( away_object);
    if (away_i) 
      return ABng_Core_System::Badge_Holder::_duplicate( away_i->get_badge_holder());
    else return ABng_Core_System::Badge_Holder::_nil();
  }

  virtual void sighting(
    const char *network_group_name,
    ABng_Poller::AB_Poller::Network_Id network, 
    ABng_Poller::AB_Poller::Station_Id station, 
    const ABng_Poller::AB_Poller::Badge_Id& badge,
    ABng_Poller::AB_Poller::Sighting_Code code,
    const ABng_Poller::AB_Poller::Field_Id& field,
    ABng_Poller::AB_Poller::Byte click_num,
    ABng_Poller::AB_Poller::Byte attributes,
    ABng_Poller::AB_Poller::Seq_Number seq,
    const ABng_Poller::AB_Poller::Time_Stamp& ts
    ENV_DECL(env) );


  ABng_Core_System::System_Static_Data_Observer_ptr get_ssdo() {
    return my_ssdo;
  }

#ifdef ObjectStore
  //
  // for ObjectStore
  //

  static os_typespec* get_os_typespec();

#endif

  virtual void init_corba_object(Away_I *item_p);
  virtual DB_Header *create_db_header();


  virtual void construct_data_object(
    ABng_Core::Away_Data_Observer::Away_Data_Object& data_object_p,
    ABng_Core::Away_ptr item_p);


  virtual void collect_values( Away_I *lt, 
			       ABng_Core::Away::Value_Description& ld);

  // pointer to an unique istance of the class
  static Away_Rep_I* _instance;

  // repository
  ABng_Core::Away_Rep* my_rep;
  ABng_Core_Management::Away_Rep_Manager* my_rep_manager;
  ABng_Core_System::System_Static_Data_Observer* my_ssdo;
  ABng_Poller::AB_Observer_ptr my_poller_observer_obj;

  ProcessControlImpl *ps_dev;

  virtual void create_corba_objects();
  virtual void destroy_corba_objects();  

    
};

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_ABng_Core_Away_Rep(Away_Rep_I)
DEF_ObjectStoreTIE_ABng_Core_Management_Away_Rep_Manager(Away_Rep_I)
DEF_ObjectStoreTIE_ABng_Core_System_System_Static_Data_Observer(Away_Rep_I)
DEF_ObjectStoreTIE_ABng_Poller_AB_Observer(Away_Rep_I)
#else
DEF_TIE_ABng_Core_Away_Rep(Away_Rep_I)
DEF_TIE_ABng_Core_Management_Away_Rep_Manager(Away_Rep_I)
DEF_TIE_ABng_Core_System_System_Static_Data_Observer(Away_Rep_I)
DEF_TIE_ABng_Poller_AB_Observer(Away_Rep_I)
#endif
#elif defined OMNIORB

#endif
#endif /* away_rep_i_hh */

//


