// $Id: badge_i.hh,v 1.1.1.1 1998/06/17 14:43:58 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/badge_rep/badge_i.hh,v $
//
// $Log: badge_i.hh,v $
// Revision 1.1.1.1  1998/06/17 14:43:58  mencnar
//
// Revision 1.1  1998/01/13 10:43:00  mencnar
// Initial revision
//
//
// File:          badge_i.cc
// Author:        Daniel Mencnarowski (mencnar@ics.agh.edu.pl)
// Date Created:  07.01.98
//
// General Description:
//  Header file for the implementation class of the Badge interface 
//

#ifndef badge_i_h
#define badge_i_h

#include <stdio.h>
#include <fstream.h>
#include <errno.h>
#include <string.h>

#ifdef ObjectStore
#include <oosa.h>
#endif

#ifdef FilePersistency
#include <rw/rwfile.h>
#endif

#include "abng_core_management.hh"
#include "abng_poller.hh"
#include "abng_guarded.hh"
#include "compatibility.hh"

int operator == ( const Domain_Repository::Octets &first,
                  const Domain_Repository::Octets &second);

class Badge_I
{
 private:
  // state of the IDL Badge object
  ABng_Core_System::Badge::Description my_descr;
  Domain_Repository::Repository_Item_Id badge_holder_id;
  ABng_Guarded &my_guard;

  // reference of CORBA object encapsulated this C++ object
  ABng_Core_System::Badge_ptr my_corba_object;

 public:
  // ctor
   Badge_I (
	const ABng_Core_System::Badge::Description& descr_init
   );

   Badge_I (
	const Badge_I&
   );

  Domain_Repository::Repository_Item_Id holder_item_id() {
    return badge_holder_id;
  }

#ifdef FilePersistency
   Badge_I (
	istream &iostr
   );
#endif

  virtual 
   ~Badge_I ();

   operator==(	
	const Badge_I&
   );

  //
  // methods enabling access to the state
  //
  void
   set_descr (
	const ABng_Core_System::Badge::Description& descr_update
   );
 
  const ABng_Core_System::Badge::Description&
   get_descr () const;

  void
   set_corba_object (
	ABng_Core_System::Badge_ptr corba_object_p
   );

  ABng_Core_System::Badge_ptr
   get_corba_object ();

  void 
   set_item_id(
	const Domain_Repository::Repository_Item_Id item_id_p
   );

  //
  // IDL operations of the System_Static_Data_Observed_E interface
  //                       _____________________________

  virtual ABng_Core_System::System_Static_Data_Observed_E::System_Static_Data_Observers*
    system_static_data_observer_list (
	ENV_DECL1( env)
    );
                
  virtual void 
    attach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs
	ENV_DECL(env)
    );

  virtual void 
    detach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs
        ENV_DECL(env)
    );

  void resolve_badge_holder( ABng_Core_Management::Badge_Holder_Rep_var holder_rep);


  //
  // IDL operations of the Scheduler_Request_Queue interface
  //                       _______________________

  virtual ABng_Core_System::Scheduler_Request_Queue::Scheduler_Requests*
   list_user_requests (
	const char * user_id
	ENV_DECL(env)
   );
          
  virtual ABng_Core_System::Scheduler_Request_Queue::Scheduler_Requests* 
   list_all_requests (
	ENV_DECL1(env)
   );

  void
   remove_request (
	CORBA::Long request_id
	ENV_DECL(env)
   );

  //
  // IDL operations of the Badge_Commander interface
  //                       ________________

  void prepare_params( const ABng_Core_System::Sensors& sensors,
		       ABng_Core_System::Badge_Ids &bids,
		       ABng_Core_System::Sensor_Ids &sids);

  void map_exceptions( CORBA::Exception &ex);


  virtual CORBA::Long issue_command (
		const ABng_Core_System::Sensors& sensors, 
		const ABng_Core_System::Octets& cmd, 
		CORBA::Boolean has_reply, 
		CORBA::Octet reply_code, 
		const ABng_Core::Command_Context& ctx, 
		ABng_Core_System::Scheduler_Caller_ptr reply_to
		ENV_DECL(env));

   virtual CORBA::Long challenge (
	       const ABng_Core_System::Sensors& sensors,
	       const ABng_Core_System::Octets& random, 
	       const ABng_Core::Command_Context& ctx, 
	       ABng_Core_System::Scheduler_Caller_ptr reply_to
	       ENV_DECL(env));

  virtual CORBA::Long sound (
	       const ABng_Core_System::Sensors& sensors,
	       const ABng_Core_System::Octets& sound, 
	       const ABng_Core::Command_Context& ctx, 
	       ABng_Core_System::Scheduler_Caller_ptr reply_to
	       ENV_DECL(env));

  virtual CORBA::Long set_state (
              const ABng_Core_System::Sensors& sensors, 
	      CORBA::Octet status,
	      const ABng_Core_System::Octets &sound,
	      const ABng_Core::Command_Context& ctx, 
	      ABng_Core_System::Scheduler_Caller_ptr reply_to
	      ENV_DECL(env));

  virtual CORBA::Long get_state (
               const ABng_Core_System::Sensors& sensors, 
	       const ABng_Core::Command_Context& ctx, 
	       ABng_Core_System::Scheduler_Caller_ptr reply_to
	       ENV_DECL(env));

  virtual CORBA::Long set_home (
               const ABng_Core_System::Sensors& sensors, 
	       const ABng_Core_System::Octets& address, 
	       const ABng_Core::Command_Context& ctx, 
	       ABng_Core_System::Scheduler_Caller_ptr reply_to
	       ENV_DECL(env));

  virtual CORBA::Long get_home (
               const ABng_Core_System::Sensors& sensors, 
	       const ABng_Core::Command_Context& ctx, 
	       ABng_Core_System::Scheduler_Caller_ptr reply_to
	       ENV_DECL(env));


  //
  // IDL operations of the Badge interface
  //                       ______
 
  virtual ABng_Core_System::Badge::Description* 
   descr (
	ENV_DECL1(env)
   );

  virtual ABng_Core_System::Badge::Value_Description* 
   value_descr (
	ENV_DECL1(env)
   );
 
  virtual Domain_Repository::Repository_Item_Id 
   item_id (
	ENV_DECL1(env)
   );

  virtual ABng_Core_System::Badge_Id*
   id (
	ENV_DECL1(env)
   );

  virtual ABng_Core_System::Badge_Holder_ptr holder (
                 ENV_DECL1(env));


#ifdef ObjectStore
  //
  // method for ObjectStore
  //

  static os_typespec* get_os_typespec();

#elif defined FilePersistency
  // methods to store and restore object of this class
  virtual void
   save (
	ostream & istr
   );

  virtual void
   restore (
	istream &ostr
   );
#endif
};   

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_ABng_Core_System_Badge(Badge_I)
#else
DEF_TIE_ABng_Core_System_Badge(Badge_I)
#endif
#elif defined OMNIORB
  // DEF_TIE__tie_Badge
#endif

// implementation of this class need method "instance" of class Sensor_Rep_I
#include "badge_rep_i.hh"

#endif /* badge_i_hh */



