//
//  File:         location_type_i.cc
//  Author:       Malgorzata Steinder (gosia@ics.agh.edu.pl)
//  Date Created: 12.12.97
//
// General Description:
//   Implementation file for the Location_Type_I class
//

#include "location_type_i.h"

#ifdef ObjectStore
static os_typespec *char_type = new os_typespec("char");
#endif

//
// Implementation of constructor, copy constructor, destructor and operators
//                   _______________________________________________________

ostream& operator << (ostream& str, Domain_Repository::Location_Type::Description& descr) {
  str << "name = " << descr.name << endl;
  str << "property_description_list = { " << endl;
  str << "                              _maximum = " 
      << descr.property_description_list.maximum() << endl;
  str << "                              _length = " 
      << descr.property_description_list.length() << endl;
  str << "                              _buffer = {" << endl;
  for(int i = 0; i < descr.property_description_list.length(); i ++) {
       str << "                                          ";
       str << "{ name = " << descr.property_description_list[i].name << ", " 
           << "type = " ;
#ifdef ORBIX
       str << (char*)*(descr.property_description_list[i].type);
#elif defined OMNIORB
       serialize_TypeCode( str, descr.property_description_list[i].type);
#endif
       str << " } " << endl;
  }
  str << "                                        }" << endl;
  str << "                             } " << endl;
  return str;
}

  Location_Type_I::Location_Type_I (
	const Domain_Repository::Location_Type::Description& descr_init
  )
{  

        set_descr(descr_init);

        //
	// init my_corba_object
        //
	my_corba_object = (Domain_Repository::Location_Type_ptr)NULL;
}

  Location_Type_I::Location_Type_I (
	const Location_Type_I& location_type_p
  )
{
        //
      	// copy my_descr
        //

        my_descr = location_type_p.my_descr;
 
        //
	// init my_corba_object
        //
	my_corba_object = location_type_p.my_corba_object;
}                        

#ifdef FilePersistency
   Location_Type_I::Location_Type_I (
	istream& f_p
   )
{
	// read description from the file
	restore(f_p);

	// init my_corba_object
	my_corba_object = (Domain_Repository::Location_Type_ptr)NULL;
}
#endif

  Location_Type_I::~Location_Type_I (
  )
{
}

  Location_Type_I::operator== (
	const Location_Type_I& location_type_p
  )
{

	if (strcmp(my_descr.name, location_type_p.my_descr.name))
		return 0;

	/*        if(my_descr.property_description_list.length() !=
           location_type_p.my_descr.property_description_list.length())
                return 0;

        for(int i = 0; i < my_descr.property_description_list.length(); i++
	) {
            if(strcmp(
                   my_descr.property_description_list[i].name,
                   location_type_p.my_descr.property_description_list[i].name
               )
            ) return 0;

            if(!my_descr.property_description_list[i].type->equal(
	        location_type_p.my_descr.property_description_list[i].type
               )
            ) return 0;
        }
	*/
	return 1;
}

//
// Implementation of methods enabling access to the state
//

void
  Location_Type_I::set_descr (
	const Domain_Repository::Location_Type::Description& descr_update
  )
{
        //
        // check if valid
        //
        long max_length = descr_update.property_description_list.length();
        for(int i = 0; i < max_length - 1; i ++)
	    for(int j = i + 1; j < max_length; j ++) {
                  if(!strcmp(descr_update.property_description_list[i].name,
                             descr_update.property_description_list[j].name
                      )
		   ) throw Domain_Repository::Invalid_Description();
            }

        my_descr.name = descr_update.name;

        my_descr.property_description_list = 
            descr_update.property_description_list;

}

const Domain_Repository::Location_Type::Description&
   Location_Type_I::get_descr (
   ) const
{
	return my_descr;  
}

void
   Location_Type_I::set_corba_object (
	const Domain_Repository::Location_Type_ptr corba_object_p
   )
{
	my_corba_object = corba_object_p;
}

const Domain_Repository::Location_Type_ptr
   Location_Type_I::get_corba_object (
   ) 
{
	return my_corba_object;
}

void 
   Location_Type_I::set_item_id(
	const Domain_Repository::Repository_Item_Id item_id_p
   )
{
	my_descr.item_id = item_id_p;
}

//
// Implementation of IDL operations of the Static_Data_Observed_E interface
//                                         _____________________________

Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
   Location_Type_I::static_data_observer_list (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	return Location_Type_Rep_I::instance()->static_data_observer_list(
		my_descr.item_id USE_ENV(env));
}
                
void 
   Location_Type_I::attach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs
	ENV_DEF(env)
   )
		throw (CORBA::SystemException)
{
  	// initialize the sequence for this location_type item id
	Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

	// put this location_type item id into the sequance
	observed_id_list_p.length(1);
        observed_id_list_p[0] = my_descr.item_id;
	
	// register observer into the repository list
	Location_Type_Rep_I::instance()->attach_static_data_observer(obs,
		observed_id_list_p USE_ENV(env));
	
}

void 
   Location_Type_I::detach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Observer)
{
	// initialize the sequence for this location_type item id
	Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

	// put this location_type item it into the sequance
        observed_id_list_p.length(1);
	observed_id_list_p[0] = my_descr.item_id;
	

	// register observer into the repository list
	Location_Type_Rep_I::instance()->detach_static_data_observer(obs,
		observed_id_list_p USE_ENV(env));
	
}


//
// Implementation of IDL operations of the Location_Type interface
//                                         ______
 
Domain_Repository::Location_Type::Description* 
  Location_Type_I::descr (ENV_DEF1(env)) throw (CORBA::SystemException)
{
	// duplicate my_descr
	Domain_Repository::Location_Type::Description* my_descr_duplicated =
		new Domain_Repository::Location_Type::Description(my_descr);

	return my_descr_duplicated;
}


Domain_Repository::Location_Type::Value_Description* 
  Location_Type_I::value_descr (ENV_DEF1(env)) throw (CORBA::SystemException)
{
	// duplicate my_descr
	
	return descr();
}
 
Domain_Repository::Repository_Item_Id 
   Location_Type_I::item_id (ENV_DEF1(env)) 
		throw (CORBA::SystemException)
{
	return my_descr.item_id;
}



Domain_Repository::Location_Type::Location_Type_Name 
    Location_Type_I::name (ENV_DEF1(env)) throw (CORBA::SystemException)
{
    return CORBA::string_dup(my_descr.name);
}

Domain_Repository::Property_Descriptions*
    Location_Type_I::property_description_list (ENV_DEF1(env)
    ) 
    throw (CORBA::SystemException)
{

    Domain_Repository::Property_Descriptions* property_description_list = 
        new Domain_Repository::Property_Descriptions(
                        my_descr.property_description_list
            );

     return property_description_list;
}

#ifdef FilePersistency
//
//		Store and restore state methods
//		_________________

void
   Location_Type_I::save (
	ostream &ostr
   )
{
#ifdef DEBUG
	cerr << "Location_Type_I: saving location_type state to file" << endl; 
#endif

        ostr << my_descr.item_id << " <" << strlen(my_descr.name) <<
                " " << my_descr.name << ">";
        
        ostr << " <";
        ostr << my_descr.property_description_list.maximum() << " ";
        ostr << my_descr.property_description_list.length();

        for(int i = 0; i < my_descr.property_description_list.length(); i++
        ) {
	  ostr << " <<" << strlen(my_descr.property_description_list[i].name)
               << " " << my_descr.property_description_list[i].name << ">";
#ifdef ORBIX
          ostr << " <" 
               << strlen((char*)(*my_descr.property_description_list[i].type))
	       << " " << (char*)(*my_descr.property_description_list[i].type) 
               << ">";
#elif defined OMNIORB
	  serialize_TypeCode( ostr, my_descr.property_description_list[i].type);
#endif
	  ostr << " >";
        }
        ostr << ">" << endl;

#ifdef DEBUG
	cerr << "Location_Type_I: state saved" << endl; 
#endif
}

static char* get_str(istream& istr) {
        char c;
        do { istr >> c; } while (c != '<');
        long _tmpl;
        istr >> _tmpl;
        char * _name = new char[_tmpl + 1];
	c= istr.get(); // space separator
        for(int i = 0; i < _tmpl; i ++) 
	  _name[i]= istr.get();
	_name[_tmpl] = '\0';
        do { istr >> c; } while (c != '>');
        return _name;
}

istream& operator >> (istream& istr,
                       Domain_Repository::Property_Description &descr) {
        char c;
        do { istr >> c; } while (c != '<');
        char* _name;
        _name = get_str(istr);
        descr.name = CORBA::string_dup(_name);
        delete [] _name;

#ifdef ORBIX
        _name = get_str(istr);
        descr.type = new CORBA::TypeCode(_name);
        delete [] _name;
#elif defined OMNIORB
	CORBA::TypeCode_ptr tmp_tc;
	deserialize_TypeCode( istr, tmp_tc);
	descr.type= tmp_tc;
#endif
        do { istr >> c; } while (c != '>');

        return istr;
}




void
   Location_Type_I::restore (
	istream &istr
   )
{

#ifdef DEBUG
        cerr << "Location_Type_I:: restore" << endl;
#endif

        int _tmpl;
        istr >> my_descr.item_id;
        
        char* _name;
        
        _name = get_str(istr);
        my_descr.name = CORBA::string_dup(_name);
        delete [] _name;
        char c;
        do { istr >> c; } while (c != '<');   
        istr >> _tmpl;
        my_descr.property_description_list = 
            Domain_Repository::Property_Descriptions(_tmpl);

        istr >> _tmpl;
        my_descr.property_description_list.length(_tmpl);

        for(int i = 0; i < my_descr.property_description_list.length(); i ++
	) 
               istr >> my_descr.property_description_list[i];

        
        do { istr >> c; } while (c != '>');


#ifdef DEBUG
        cerr << "Location_Type_I  restored" << endl;
#endif
}
#endif /* FilePersistency */




