//
//  File:         location_type_i.h
//  Author:       Malgorzata Steinder (gosia@ics.agh.edu.pl)
//  Date Created: 12.12.97
//
// General Description:
//   Header file for the implementation class of the Location_Type interface
//

#ifndef location_type_i_h
#define location_type_i_h

#include <stdio.h>
#include <fstream.h>
#include <errno.h>
#include <string.h>

#ifdef ObjectStore
#include <oosa.h>
#endif

#ifdef FilePersistency
#include <rw/rwfile.h>
#endif

#include "domain_rep_management.hh"
#include "compatibility.hh"

class Location_Type_I
{
 private:
  //
  // the state of the IDL Location_Type object
  //
  Domain_Repository::Location_Type::Description my_descr;

  //
  // the reference of CORBA object encapsulated this C++ object
  //
  Domain_Repository::Location_Type_ptr my_corba_object;

 public:
   //
   // ctors
   //
   Location_Type_I (
	const Domain_Repository::Location_Type::Description& descr_init
   );

   Location_Type_I (
	const Location_Type_I&
   );

#ifdef FilePersistency
   Location_Type_I (
	istream & from
   );
#endif

  virtual 
   ~Location_Type_I ();

   operator==(	
	const Location_Type_I&
   );

  //
  // methods enabling access to the state
  //
  void
    set_descr (
	const Domain_Repository::Location_Type::Description& descr_update
    );
 
  const Domain_Repository::Location_Type::Description&
    get_descr () const;

  void
    set_corba_object (
	const Domain_Repository::Location_Type_ptr corba_object_p
    );

  const Domain_Repository::Location_Type_ptr
    get_corba_object ();

  void 
    set_item_id(
	const Domain_Repository::Repository_Item_Id item_id_p
    );

  //
  // IDL operations of the Static_Data_Observed_E interface
  //                       _____________________________

  virtual Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
    static_data_observer_list (ENV_DECL1( env)) throw (CORBA::SystemException);
                
  virtual void 
    attach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs 
	ENV_DECL( env)
    )
    throw (CORBA::SystemException);

  virtual void 
    detach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs 
	ENV_DECL( env)
    ) 
    throw ( CORBA::SystemException, 
            Domain_Repository::Unknown_Observer
          );


  //
  // IDL operations of the Location_Type interface
  //                       ______
 
  virtual Domain_Repository::Location_Type::Description* 
    descr (ENV_DECL1( env)) throw (CORBA::SystemException);

  virtual Domain_Repository::Location_Type::Value_Description* 
    value_descr (ENV_DECL1( env)) throw (CORBA::SystemException);
 
  virtual Domain_Repository::Repository_Item_Id 
    item_id (ENV_DECL1( env)) throw (CORBA::SystemException);

  virtual Domain_Repository::Location_Type::Location_Type_Name 
    name (ENV_DECL1( env)) throw (CORBA::SystemException);

  virtual Domain_Repository::Property_Descriptions*
    property_description_list (ENV_DECL1( env)) throw (CORBA::SystemException);


#ifdef ObjectStore
  //
  // method for ObjectStore
  //

  static os_typespec* get_os_typespec();

#elif defined FilePersistency
  // methods to store and restore object of this class
  virtual void
   save (
	ostream & istr
   );

  virtual void
   restore (
	istream &ostr
   );
#endif
};   

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_Domain_Repository_Location_Type(Location_Type_I)
#else
DEF_TIE_Domain_Repository_Location_Type(Location_Type_I)
#endif
#elif defined OMNIORB
// DEF_TIE__tie_Location_Type
#endif

// implementation of this class need method "instance" of class Location_Type_Rep_I
#include "location_type_rep_i.h"

#endif /* location_type_i_h */
