//
//  File:         location_type_rep_i.h
//  Author:       Malgorzata Steinder (gosia@ics.agh.edu.pl)
//  Date Created: 17.12.97
//
// General Description:
//   Header file for Location_Type_Rep implementation class
//

#ifndef location_type_rep_i_h
#define location_type_rep_i_h

#include <stdlib.h>
#include <fstream.h>
#include <string.h>

#ifdef ObjectStore
#include <oosa.h>
#endif

#include "domain_rep_management.hh"
#include "location_type_i.h"
#include "bootstrap_n.hh"
#include "basic_rep_i.h"
#include "process_status.hh"

class LT_SDO_Types {

public:

  typedef Domain_Repository::Static_Data_Observed_R Observed;
  typedef Domain_Repository::Static_Data_Observer Observer;
  typedef Domain_Repository::Static_Data_Observer_ptr Observer_ptr;
  typedef Domain_Repository::Static_Data_Observed_E::Static_Data_Observers 
    Observers;
};

class LT_Types {

public:

  typedef Domain_Repository::Location_Type Item;  
  typedef Domain_Repository::Location_Type_ptr Item_ptr;  
  typedef Domain_Repository::Location_Types Items;  
  typedef Location_Type_I Item_I;  
  typedef Domain_Repository::Location_Type::Description Initial_Data;
  typedef Domain_Repository::Static_Data_Observer::Static_Data_Object 
    Data_Object;
};

class Location_Type_Rep_I  : public Bootstrap_listener,
			     public Basic_Rep_I<LT_Types, LT_SDO_Types>,
			     public ServerStatusInterface { 

public:

  Location_Type_Rep_I (
#ifdef ObjectStore
    os_database* db_p
#elif defined FilePersistency
    const char *repository_name_p
#endif
  );

  virtual ~Location_Type_Rep_I ();

  void init( int init_v);

  // get instance of this class 
  //  - Singleton pattern (E. Gamm & al.,"Design Paterns", p. 127)
  static Location_Type_Rep_I* 
    instance ();

  //
  // IDL operations of the Static_Data_Observed_R interface
  //                       _____________________________

  virtual Domain_Repository::Static_Data_Observed_R::Observer_Entries*
   static_data_observer_list (
	ENV_DECL1(env)
   ) 
	throw (CORBA::SystemException) {
        return observer_list(USE_ENV1(env));
  }
             
  virtual Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
   add_static_data_observer_list (
	ENV_DECL1(env)
   ) 
	throw (CORBA::SystemException) {
        return add_observer_list(USE_ENV1(env));
   }
           
  virtual void 
  attach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs,  
	const Domain_Repository::Repository_Item_Ids& observed_id_list
	ENV_DECL(env)
   )
	throw (CORBA::SystemException, 
               Domain_Repository::Unknown_Repository_Item_Ids
        ) {
        attach_observer(obs, observed_id_list USE_ENV(env));
  }

  virtual void 
   detach_static_data_observer (
        Domain_Repository::Static_Data_Observer_ptr obs, 
        const Domain_Repository::Repository_Item_Ids& observed_id_list
        ENV_DECL(env)
   ) 
        throw (CORBA::SystemException, 
               Domain_Repository::Unknown_Repository_Item_Ids, 
               Domain_Repository::Unknown_Observer,
               Domain_Repository::Unmatched_Observer_Item_Ids
        ) {
        detach_observer( obs, observed_id_list USE_ENV(env));
  }



  virtual void 
   attach_add_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs
	ENV_DECL(env)
   )
	throw (CORBA::SystemException) {
        attach_add_observer(obs USE_ENV(env));
  }

  virtual void 
   detach_add_static_data_observer (
        Domain_Repository::Static_Data_Observer_ptr obs
        ENV_DECL(env)
   ) 
        throw (CORBA::SystemException, 
               Domain_Repository::Unknown_Observer) {
        detach_add_observer(obs USE_ENV(env));
  }

  // Suplementary operation

  virtual Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
   static_data_observer_list (
	Domain_Repository::Repository_Item_Id item_id_p
	ENV_DECL(env)
   ) 
	throw (CORBA::SystemException) {
        return get_item_observers(item_id_p USE_ENV(env));
  }


  //
  // IDL operations of the Location_Type_Rep interface
  //                       __________

  virtual Domain_Repository::Location_Type_Rep *
   rep(
	ENV_DECL1(env)
   ) 
	throw (CORBA::SystemException);
  
  virtual Domain_Repository::Location_Types* 
   location_type_list (
	ENV_DECL1(env)
   ) 
    throw (CORBA::SystemException) {
    return item_list(USE_ENV1(env));
  }

  virtual Domain_Repository::Location_Types* 
   find (
	const Domain_Repository::Location_Type::Pattern& data_record_pattern
	ENV_DECL(env)
   )
	throw (CORBA::SystemException,
               Domain_Repository::Search_Not_Implemented
        ) {   // no locking here 
		   return Basic_Rep_I<LT_Types, LT_SDO_Types>::find( data_record_pattern USE_ENV(env));}

  
  virtual Domain_Repository::Location_Types* 
   private_find (
        const Domain_Repository::Location_Type::Pattern& data_record_pattern
        ENV_DECL(env)
   )
        throw (CORBA::SystemException,
               Domain_Repository::Search_Not_Implemented
        );

  virtual Domain_Repository::Location_Type::Value_Descriptions* 
   find_values (
	const Domain_Repository::Location_Type::Pattern& data_record_pattern
	ENV_DECL(env)
   ) 
	throw (CORBA::SystemException,
               Domain_Repository::Search_Not_Implemented
	       ) {
return Basic_Rep_I<LT_Types, LT_SDO_Types>::find_values( data_record_pattern USE_ENV(env));}

  virtual Domain_Repository::Location_Type::Query_Results*
   complex_query(
		 const Domain_Repository::Location_Type::Entity_Set &es,
		 const Domain_Repository::Query_Options &qo,
		 Domain_Repository::Static_Data_Observer_ptr obs
		 ENV_DECL(env)) {

     return Basic_Rep_I<LT_Types, LT_SDO_Types>::complex_query(
		   es,qo,obs USE_ENV(env));
  }

  virtual Domain_Repository::Location_Type_ptr 
   add (
	const Domain_Repository::Location_Type::Description& data_record
	ENV_DECL(env)
   )
	 throw (CORBA::SystemException, 
                Domain_Repository::Duplicate_Data_Record,
                Domain_Repository::Invalid_Description
		) {
    return Basic_Rep_I<LT_Types, LT_SDO_Types>::add(data_record USE_ENV(env));
  }

  virtual void 
   remove (
	Domain_Repository::Location_Type_ptr object_to_remove
        ENV_DECL(env)
   )
	throw (CORBA::SystemException, 
               Domain_Repository::Unknown_Object_Ref,
               Domain_Repository::Operation_Not_Allowed
	       ) {
    
     Basic_Rep_I<LT_Types, LT_SDO_Types>::remove(object_to_remove USE_ENV(env));
  }

  virtual void 
   update (
	Domain_Repository::Location_Type_ptr object_to_update, 
	const Domain_Repository::Location_Type::Description& 
              data_record_pattern
	ENV_DECL(env)
   ) 
	throw (CORBA::SystemException, 
	       Domain_Repository::Unknown_Object_Ref,
	       Domain_Repository::Duplicate_Data_Record,
               Domain_Repository::Invalid_Description
	       ) {

    Basic_Rep_I<LT_Types, LT_SDO_Types>::update(object_to_update, 
      data_record_pattern USE_ENV(env));
  }

  // set my_rep
  void set_rep(Domain_Repository::Location_Type_Rep*);

#ifdef ObjectStore
  static os_typespec* get_os_typespec();
#endif

 protected:

  // calculate the hash value of the location_type
  static unsigned hash_location_type(
    const Location_Type_I& location_type_i_p); 

  // definition of structure containing parameters of match_location_type method
  struct MatchLocation_TypeApplyDataStruct {
    Domain_Repository::Location_Types* _found_location_types;
    const Domain_Repository::Location_Type::Pattern& _data_record_pattern;

    // its constructor
    MatchLocation_TypeApplyDataStruct(
	Domain_Repository::Location_Types* 
             _found_location_types_p,
   	const Domain_Repository::Location_Type::Pattern& _data_record_pattern_p):
	 	_found_location_types(_found_location_types_p),
		_data_record_pattern(_data_record_pattern_p) {}
  };

  // if the current location_type matches the pattern puts it into the container
  static void
    match_location_type(Location_Type_I* current_location_type,
      void* apply_data);

  Domain_Repository::Location_Description_Rep_var location_description_rep;

  static const char * ref_server_names[];

  void reference_resolved(const ABng_Server_Manager::Name &name,
			  CORBA::Object_var server_ref,
			  ABng_Server_Manager::uid callback_id);

  virtual int remove_permitted(Location_Type_I *);

  virtual void init_corba_object(Location_Type_I *item_p);
  virtual DB_Header *create_db_header();

  virtual void construct_data_object(
    Domain_Repository::Static_Data_Observer::Static_Data_Object& data_object_p,
    Domain_Repository::Location_Type_ptr item_p);


  virtual int validate_update_data(Item_ptr item_p, 
    const Domain_Repository::Location_Type::Description& data_record_pattern);

  virtual void collect_values(Location_Type_I *lt, 
    Domain_Repository::Location_Type::Value_Description& ld);

  virtual void create_corba_objects();
  virtual void destroy_corba_objects();

  // Attributes
  // pointer to an unique istance of the class
  static Location_Type_Rep_I* _instance;

  // repository
  Domain_Repository::Location_Type_Rep_ptr my_rep;
  Domain_Repository_Management::Location_Type_Rep_Manager_ptr my_rep_manager;

  ProcessControlImpl *ps_dev;
};

#endif /* location_type_rep_i_h */














