// $Id: sched_reply.hh,v 1.2 1998/06/18 10:21:56 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/scheduler/sched_reply.hh,v $
//
// $Log: sched_reply.hh,v $
// Revision 1.2  1998/06/18 10:21:56  mencnar
// moved to CVS
//
// Revision 1.1  1997/10/23 09:21:08  mencnar
// Initial revision
//
//

#ifndef __sched_reply_h__
#define __sched_reply_h__

//--------------------------------------------------------------Sched_Reply
//
class Sched_Reply {
public:
 
  enum Reply_kind {
    	RK_challenge, 	RK_sound,	RK_set_state,
	RK_get_state, 	RK_set_home, 	RK_get_home,
	RK_get_station_state,
	RK_badge_unknown, RK_sensor_unknown
  };
  const Reply_kind reply_kind;
  CORBA::Octet reply_code; // valid only if reply_kind == RK_*_unknown
  Seq_Number seq_number; // valid only for badge replies
  const Sched_Barrier_Id barrier_id;
  ABng_Core::Time time_stamp;
 
  Sched_Reply( 	Reply_kind rk,
 		const ABng_Core::Time &ts,
		const Sched_Barrier_Id &bar) :
	reply_kind( rk), time_stamp( ts), barrier_id( bar) {
    unsigned long s,ns;
    omni_thread::get_time( &s, &ns);
    time_stamp.seconds= s;
    time_stamp.microseconds= ns/1000;
  }

  virtual ~Sched_Reply() {} // safe destruction of derived classes

  virtual void send_reply( Sched_Request &req, int repeat_num)= 0;
  virtual Sched_Barrier_Id min_barrier_id() = 0;
};

//--------------------------------------------------------Sched_Reply_Badge
//
class Sched_Reply_Badge : public Sched_Reply {
public:
   
  Sched_Reply_Badge(
		Reply_kind rk,
		const ABng_Core::Time &ts,
		const Sched_Barrier_Id &bar,
		Seq_Number seq) :
	Sched_Reply( rk, ts, bar) {
    seq_number= seq;
  }
  virtual Sched_Barrier_Id min_barrier_id();
};

//-------------------------------------------------Sched_Reply_Badge_unknown
//
class Sched_Reply_Badge_unknown : public Sched_Reply_Badge {
public:
  const ABng_Poller::AB_Poller::Octets data;

  Sched_Reply_Badge_unknown( const ABng_Poller::AB_Poller::Octets  &data,
			     const char *group_name);
	      
  void send_reply( Sched_Request &req, int repeat_num);

  static Sched_Barrier_Id extract_barrier_id( const ABng_Poller::AB_Poller::Octets &data,
					      const char *group_name);
  static Seq_Number extract_sequence_number( const ABng_Core_System::Octets &data);
  static ABng_Core::Time get_time_object();
};

//----------------------------------------------------Sched_Reply_challenge
//
class Sched_Reply_challenge : public Sched_Reply_Badge {
public:
  const ABng_Poller::AB_Poller::Octets random;
  
  Sched_Reply_challenge(
		const ABng_Core::Time &ts,
		const Sched_Barrier_Id &bar,
		Seq_Number seq,
		ABng_Poller::AB_Poller::Octets rn) :
	Sched_Reply_Badge( RK_challenge, ts, bar, seq),
	random( rn) {}
  void send_reply( Sched_Request &req, int repeat_num);
};

//-------------------------------------------------------Sched_Reply_sound
//
class Sched_Reply_sound : public Sched_Reply_Badge {
public:
  const ABng_Poller::AB_Poller::Octets sound;
  
  Sched_Reply_sound(
		const ABng_Core::Time &ts,
		const Sched_Barrier_Id &bar,
 		Seq_Number seq,
		const ABng_Poller::AB_Poller::Octets  &snd ) :
	Sched_Reply_Badge( RK_sound, ts, bar, seq),
	sound( snd) {}

  void send_reply( Sched_Request &req, int repeat_num);
};

//---------------------------------------------------Sched_Reply_set_state
//
class Sched_Reply_set_state : public Sched_Reply_Badge {
public:
  const ABng_Poller::AB_Poller::Byte status;
  const Tones sound;
  
  Sched_Reply_set_state(
		const ABng_Core::Time &ts,
		const Sched_Barrier_Id &bar,
		Seq_Number seq,
		const Tones &snd,
		ABng_Poller::AB_Poller::Byte stat) :
	Sched_Reply_Badge( RK_set_state, ts, bar, seq),
	status( stat), sound( snd) {}
  void send_reply( Sched_Request &req, int repeat_num);
};

//----------------------------------------------------Sched_Reply_get_state
//
class Sched_Reply_get_state : public Sched_Reply_Badge {
public:
  const ABng_Poller::AB_Poller::Byte major_version;
  const ABng_Poller::AB_Poller::Byte minor_version;
  const ABng_Poller::AB_Poller::Byte status;

  Sched_Reply_get_state(
		const ABng_Core::Time &ts,
		const Sched_Barrier_Id &bar,
		const Seq_Number &seq,
		ABng_Poller::AB_Poller::Byte maj_ver,
		ABng_Poller::AB_Poller::Byte min_ver,
		ABng_Poller::AB_Poller::Byte stat) :
	Sched_Reply_Badge( RK_get_state, ts, bar, seq),
	major_version( maj_ver), minor_version( min_ver), status( stat) {}
  void send_reply( Sched_Request &req, int repeat_num);
};

//-----------------------------------------------------Sched_Reply_set_home
//
class Sched_Reply_set_home : public Sched_Reply_Badge {
public:
  const ABng_Poller::AB_Poller::Address address;

  Sched_Reply_set_home(
		const ABng_Core::Time &ts,
		const Sched_Barrier_Id &bar,
		Seq_Number seq,
		ABng_Poller::AB_Poller::Address adr) :
	Sched_Reply_Badge( RK_set_home, ts, bar, seq),
	address( adr) {}
  void send_reply( Sched_Request &req, int repeat_num);
};

//-----------------------------------------------------Sched_Reply_get_home
//
class Sched_Reply_get_home : public Sched_Reply_Badge {
public:
  const ABng_Poller::AB_Poller::Address address;

  Sched_Reply_get_home(
		const ABng_Core::Time &ts,
		const Sched_Barrier_Id &bar,
		Seq_Number seq,
		ABng_Poller::AB_Poller::Address adr) :
	Sched_Reply_Badge( RK_get_home, ts, bar, seq),
	address( adr) {}
  void send_reply( Sched_Request &req, int repeat_num);
};

//-------------------------------------------------Sched_Reply_Sensor_unknown
//
class Sched_Reply_Sensor_unknown : public Sched_Reply {
public:
  const ABng_Poller::AB_Poller::Octets data;

  Sched_Reply_Sensor_unknown( const ABng_Poller::AB_Poller::Octets &data,
			      const char *group_name);
	      
  virtual Sched_Barrier_Id min_barrier_id();
  void send_reply( Sched_Request &req, int repeat_num);
  static Sched_Barrier_Id extract_barrier_id( const ABng_Poller::AB_Poller::Octets &data,
					      const char *group_name);
};

//----------------------------------------------Sched_Reply_get_station_state
//
class Sched_Reply_get_station_state : public Sched_Reply {
public:
  const ABng_Poller::AB_Poller::Byte major_version;
  const ABng_Poller::AB_Poller::Byte minor_version;
  const ABng_Poller::AB_Poller::Byte status1;
  const ABng_Poller::AB_Poller::Byte status2;

  Sched_Reply_get_station_state(
		const ABng_Core::Time &ts,
		const Sched_Barrier_Id &bar,
		ABng_Poller::AB_Poller::Byte maj_ver,
		ABng_Poller::AB_Poller::Byte min_ver,
		ABng_Poller::AB_Poller::Byte stat1,
		ABng_Poller::AB_Poller::Byte stat2) :
	Sched_Reply( RK_get_station_state, ts, bar), 
	major_version( maj_ver),	minor_version( min_ver),
	status1( stat1),		status2( stat2) {}
  virtual Sched_Barrier_Id min_barrier_id();
  void send_reply( Sched_Request &req, int repeat_num);
};

#endif 

//














