// $Id: sched_request.cc,v 1.6 1998/07/29 13:44:26 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/scheduler/sched_request.cc,v $
//
// $Log: sched_request.cc,v $
// Revision 1.6  1998/07/29 13:44:26  mencnar
// now uses a synchronised time
//
// Revision 1.5  1998/06/18 10:21:56  mencnar
// moved to CVS
//
// Revision 1.4  1997/11/03 09:28:30  mencnar
// Fixes: deadlock ( interval timed wait)
//        memory leak ( handling replies )
//        #ifdef DEBUG macros
//
// Revision 1.3  1997/10/24 08:06:11  mencnar
// User identification using Orbix principal
// Now throws ABng_Core::Request_Time_Invalid if request time is earlier than
// current time
//
// Revision 1.2  1997/10/23 14:28:10  mencnar
// Added comments
//
// Revision 1.1  1997/10/23 09:27:51  mencnar
// Initial revision
//
//


#include "sched_main.hh"
#include "sched_stat.hh"
#include <iostream.h>
#include "sched_debug.hh"
#include <stdio.h>


//-----------------------------------------------------Sched_Complex_Request
//
Sched_Complex_Request::
  Sched_Complex_Request( SchedulerImpl &a_schimpl,
			 const ABng_Core::Command_Context &a_context,
			 const Scheduler_Caller_var &a_caller,
			 const MY_PRINCIPAL_VAR _principal) :
    schimpl( a_schimpl), context( a_context), reply_to( a_caller),
    principal( _principal) {

  reply_to_is_valid= ! CORBA::is_nil( reply_to.operator->());

  req_id= schimpl.create_request_id();

  ABng_Server_Manager::Time curr_time= Bootstrap::gettimeofday();
  request_time.seconds= curr_time.seconds;
  request_time.microseconds= curr_time.microseconds;

#ifdef ORBIX
  user_name= CORBA::string_dup(((char *) ((CORBA::Principal_ptr) principal))); 

#elif defined OMNIORB
  user_name= CORBA::string_dup("unknown");
#endif


  if (context.at.seconds!=0 || context.at.microseconds!=0)
    if (Sched_Request::compare_time( context.at, request_time) < 0)
      throw ABng_Core::Invalid_Request_Time();
}

//____________________________________________________________________________
// start all threads in sub-requests
//
void Sched_Complex_Request::start() {
  int size= subreq.entries();

  for (int i=0; i<size; i++)
    subreq[i]->start();
}

//____________________________________________________________________________
// signal all threads to terminate,
// join them, the allocated space will be reclaimed
//
void Sched_Complex_Request::cancel() {
  int size= subreq.entries();
  int temp;
  void *xtemp= &temp;

#ifdef DEBUG
  cout << "Cancelling requests" << endl;
#endif

  for (int i=0; i<size; i++)
    subreq[i]->cancel();

#ifdef DEBUG
  cout << "Joining threads" << endl;
#endif

  for (int j=0; j<size; j++)
    subreq[j]->join( &xtemp);

#ifdef DEBUG
  cout << "All threads joined" << endl;
#endif
}

//____________________________________________________________________________
// return poller object for the specified location ( network, station, group)
//
ABng_Poller::AB_Poller_var Sched_Complex_Request::
poller( const Sched_Barrier_Id &loc) {
 return schimpl.poller( loc);
}

//____________________________________________________________________________
// free space of all sub-requests
// note: the sub-requests must be cancelled ( with cancel method),
//       and joined ( with join method)
//

Sched_Complex_Request::~Sched_Complex_Request() {
  auto_lock( *this);
  int size= subreq.entries();

#ifdef DEBUG
  cerr << "~Sched_Complex_Request" << endl;
#endif

  for (int i=0; i<size; i++) {
    delete subreq.removeFirst();
  }
}

//____________________________________________________________________________
// a sub-request calls this method to indicate it terminated request processing,
// if all sub-requests complete execution, the complex request is removed
// from requests' queue
//
void Sched_Complex_Request::subrequest_completed() {
  auto_lock( *this);

  if (subreq_counter >0 )
    if (--subreq_counter == 0) {
#ifdef DEBUG
      cout << "All subrequests completed execution" << endl;
#endif
      if (reply_to_is_valid ) {
	try {
	  reply_to->request_completed( req_id);
	}
	catch (...) {}
      }
      schimpl.request_queue.clean( *this);
    }
}


//-------------------------------------------------------Sched_Badge_Request
//
// Creates complex request for badge commands
// Checks all sensor/badge ids for their validity
//  (may throw an appropriate exception at this point)
// Next, creates necessary sub-requests
// The caller is responsible to call start() method after successful
// creation of object
//
Sched_Badge_Request::Sched_Badge_Request(
		       SchedulerImpl &a_schimpl, 
		       const ABng_Core::Command_Context &a_context,
		       const Scheduler_Caller_var &caller,
		       const MY_PRINCIPAL_VAR _principal,
		       const ABng_Core_System::Badge_Ids &_badges,
		       const ABng_Core_System::Sensor_Ids &_sensors) :
    Sched_Complex_Request( a_schimpl, a_context, caller, _principal),
    badges( _badges), sensors( _sensors) {

  if (badges.length()==0) badges= schimpl.get_all_badge_ids();
  if (badges.length()==0) throw ABng_Core::Invalid_Argument();

  RWTValDlist<Sched_Barrier_Id> sensor_ids;
 
  for (int x=0; x<sensors.length(); x++)
    schimpl.sb_repository.check_valid( sensors[x]);
  for (int y=0; y<badges.length(); y++)
    schimpl.sb_repository.check_valid( badges[y]);

  for (int j=0; j<sensors.length(); j++) {
    sensor_ids.append( Sched_Barrier_Id( (Network_Id) sensors[j].network_num,
					 (Station_Id) sensors[j].sensor_num,
					 RWCString( (const char *) sensors[j].group_name)));
  }

  subreq_counter= badges.length();
  for (int i=0; i<badges.length(); i++) {
    Sched_Barrier_Id badge_id( badges[i]);

    subreq.append(
          new Sched_Request( *this, 
			     schimpl.location_rep.get_barrier( badge_id),
			     sensor_ids, sensors.length() == 0));
  }
}

//____________________________________________________________________________
// Fills Scheduler_Request object with description of the request
// Derived classes may override this method, to (re)set certain fields
//
void Sched_Badge_Request::describe_request( Scheduler_Request &dreq) {

  dreq.uid= user_name;
  dreq.request_id= req_id;
  dreq.sensors= sensors;
  dreq.badges= badges;
  dreq.command_ctx= context;
  dreq.request_time= request_time;
}

int Sched_Badge_Request::accepts_reply( Sched_Request &req, Sched_Reply &reply) {
  if (reply.min_barrier_id() == req.barrier.barrier_id )
    if (reply.reply_kind >= Sched_Reply::RK_challenge &&
	reply.reply_kind <= Sched_Reply::RK_get_home ||
	reply.reply_kind == Sched_Reply::RK_badge_unknown) return 1;
  return 0;
}

//-------------------------------------------------Sched_Badge_Request_issue
//
void Sched_Badge_Request_issue::send_command( Sched_Request &req) {
  Sched_Barrier_Id &pos= req.last_seen();
  ABng_Poller::AB_Poller::Octets oct;

  oct.length(  2 /* sensor id */ +
		1 /* command code/mode */ +
		1 + req.barrier.barrier_id.badge_id.length() +
		1 /* data+badge_id length */ +
		data.length() -1);
  oct[0]= pos.station_id;  // station id
  oct[1]= data[0];         // command code/mode
  oct[2]= 1+req.barrier.barrier_id.badge_id.length() + data.length();
  oct[3]= req.barrier.barrier_id.badge_id.length();
  
  int size= req.barrier.barrier_id.badge_id.length();
  for (int i=0; i<size; i++)
    oct[4+i]= req.barrier.barrier_id.badge_id[i];
  for (int j=1; j<data.length(); j++)
    oct[3+size+j]= data[j];
  oct[3+size+data.length()]= pos.network_id;
#ifdef DEBUG
  cout << "issue command " << oct << endl;
#endif
  poller( pos)->issue_command( oct);
}

void Sched_Badge_Request_issue::describe_request( Scheduler_Request &dreq) {
  Sched_Badge_Request::describe_request( dreq);
  dreq.arguments= data;
  dreq.command_name= (const char *) "generic badge command";
}

int Sched_Badge_Request_issue::accepts_reply( Sched_Request &req, Sched_Reply &reply) {
  if (Sched_Badge_Request::accepts_reply( req, reply))
    if (reply.reply_kind == Sched_Reply::RK_badge_unknown)
      if ((reply.reply_code == reply_code)) return 1;
  return 0;
}

//---------------------------------------------------Sched_Request_challenge
//
void Sched_Request_challenge::send_command( Sched_Request &req) {
  Sched_Barrier_Id &pos= req.last_seen();

  poller( pos)->challenge( pos.network_id, pos.station_id,
					   req.barrier.barrier_id.badge_id,
					   random, req.get_seq_number());
}

void Sched_Request_challenge::describe_request( Scheduler_Request &dreq) {
  Sched_Badge_Request::describe_request( dreq);
  dreq.arguments= random;
  dreq.command_name= (const char *) "challenge";
}

int Sched_Request_challenge::accepts_reply( Sched_Request &req, Sched_Reply &reply) {
  if (Sched_Badge_Request::accepts_reply( req, reply))
    if (reply.reply_kind == Sched_Reply::RK_challenge) return 1;
  return 0;
}

//---------------------------------------------------Sched_Request_sound
//
void Sched_Request_sound::send_command( Sched_Request &req) {
  Sched_Barrier_Id &pos= req.last_seen();
#ifdef DEBUG
  cout << "seq_number " << (int) req.get_seq_number() << endl;
#endif
   poller( pos)->sound( pos.network_id, pos.station_id,
					req.barrier.barrier_id.badge_id,
					sound, req.get_seq_number());
}

void Sched_Request_sound::describe_request( Scheduler_Request &dreq) {
  Sched_Badge_Request::describe_request( dreq);
  dreq.arguments= sound;
  dreq.command_name= (const char *) "sound";
}

int Sched_Request_sound::accepts_reply( Sched_Request &req, Sched_Reply &reply) {
#ifdef DEBUG
  cout << "? sound_request accepts reply" << endl;
#endif
  if (Sched_Badge_Request::accepts_reply( req, reply))
    if (reply.reply_kind == Sched_Reply::RK_sound) return 1;
  return 0;
}

//---------------------------------------------------Sched_Request_set_state
//
void Sched_Request_set_state::send_command( Sched_Request &req) {
  Sched_Barrier_Id &pos= req.last_seen();

 poller( pos)->set_state( pos.network_id, pos.station_id,
					  req.barrier.barrier_id.badge_id,
					  status, sound, req.get_seq_number());
}

void Sched_Request_set_state::describe_request( Scheduler_Request &dreq) {
  Sched_Badge_Request::describe_request( dreq);
  dreq.arguments= sound;
  dreq.command_name= (const char *) "set_state";
}

int Sched_Request_set_state::accepts_reply( Sched_Request &req, Sched_Reply &reply) {
  if (Sched_Badge_Request::accepts_reply( req, reply))
    if (reply.reply_kind == Sched_Reply::RK_set_state) return 1;
  return 0;
}

//---------------------------------------------------Sched_Request_get_state
//
void Sched_Request_get_state::send_command( Sched_Request &req) {
  Sched_Barrier_Id &pos= req.last_seen();

  poller( pos)->get_state( pos.network_id, pos.station_id,
					   req.barrier.barrier_id.badge_id,
					   req.get_seq_number());
}

void Sched_Request_get_state::describe_request( Scheduler_Request &dreq) {
  Sched_Badge_Request::describe_request( dreq);
  dreq.command_name= (const char *) "get_state";
}

int Sched_Request_get_state::accepts_reply( Sched_Request &req, Sched_Reply &reply) {
  if (Sched_Badge_Request::accepts_reply( req, reply))
    if (reply.reply_kind == Sched_Reply::RK_get_state) return 1;
  return 0;
}


//----------------------------------------------------Sched_Request_set_home
//
void Sched_Request_set_home::send_command( Sched_Request &req) {
  Sched_Barrier_Id &pos= req.last_seen();

  poller( pos)->set_home( pos.network_id, pos.station_id,
					  req.barrier.barrier_id.badge_id,
					  address, req.get_seq_number());
}

void Sched_Request_set_home::describe_request( Scheduler_Request &dreq) {
  Sched_Badge_Request::describe_request( dreq);
  dreq.arguments= address;
  dreq.command_name= (const char *) "set_home";
}

int Sched_Request_set_home::accepts_reply( Sched_Request &req, Sched_Reply &reply) {
  if (Sched_Badge_Request::accepts_reply( req, reply))
    if (reply.reply_kind == Sched_Reply::RK_set_home) return 1;
  return 0;
}

//----------------------------------------------------Sched_Request_get_home
//
void Sched_Request_get_home::send_command( Sched_Request &req) {
  Sched_Barrier_Id &pos= req.last_seen();

  poller( pos)->get_home( pos.network_id, pos.station_id,
					  req.barrier.barrier_id.badge_id,
					  req.get_seq_number());
}

void Sched_Request_get_home::describe_request( Scheduler_Request &dreq) {
  Sched_Badge_Request::describe_request( dreq);
  dreq.command_name= (const char *) "get_home";
}

int Sched_Request_get_home::accepts_reply( Sched_Request &req, Sched_Reply &reply) {
  if (Sched_Badge_Request::accepts_reply( req, reply))
    if (reply.reply_kind == Sched_Reply::RK_get_home) return 1;
  return 0;
}

//------------------------------------------------------Sched_Sensor_Request
//
// Creates complex request object for sensor commands
// Checks sensor ids ( may throw an exception at this point)
// Next creates necesseary subrequests
// The caller is responsible to call start() method after successful
// creation
//
 
Sched_Sensor_Request::Sched_Sensor_Request(
		       SchedulerImpl &a_schimpl, 
		       const ABng_Core::Command_Context &a_context,
		       const Scheduler_Caller_var &caller,
		       const MY_PRINCIPAL_VAR _principal,
		       const ABng_Core_System::Sensor_Ids &_sensors) :
    Sched_Complex_Request( a_schimpl, a_context, caller, _principal),
    sensors( _sensors) {

  if (sensors.length()==0) sensors= schimpl.get_all_sensor_ids();
  if (sensors.length()==0) throw ABng_Core::Invalid_Argument();

  for (int x=0; x<sensors.length(); x++)
    schimpl.sb_repository.check_valid( sensors[x]);

  subreq_counter= sensors.length();
  for (int i=0; i<sensors.length(); i++) {
    RWTValDlist<Sched_Barrier_Id> sensor_ids;

    Sched_Barrier_Id barid( (Network_Id) sensors[i].network_num,
			    (Station_Id) sensors[i].sensor_num,
			    (const char *) sensors[i].group_name);
    
    subreq.append(
	 new Sched_Request( *this,
			    schimpl.location_rep.get_barrier( barid),
			    sensor_ids, 1));
  }
}

//____________________________________________________________________________
// Fills Scheduler_Request object with description of the request
// Derived classes may override this method, to (re)set certain fields
//
void Sched_Sensor_Request::describe_request( Scheduler_Request &dreq) {

#ifdef ORBIX
  CORBA::String_var un= (char *) ((CORBA::Principal_ptr) principal); 

  dreq.uid= (const char*) un;
#elif defined OMNIORB
  dreq.uid= (const char*) "unknown";
#endif
  dreq.request_id= req_id;
  dreq.sensors= sensors;
  dreq.command_ctx= context;
  dreq.request_time= request_time;
}


//-------------------------------------------------Sched_Badge_Request_issue
//
void Sched_Sensor_Request_issue::send_command( Sched_Request &req) {
  Sched_Barrier_Id &pos= req.last_seen();
  ABng_Poller::AB_Poller::Octets oct;

  oct.length(  2 /* sensor id */ +
		data.length());
  oct[0]= pos.station_id;  // station id
  for (int j=0; j<data.length(); j++)
    oct[1+j]= data[j];
  oct[1+data.length()]= pos.network_id;
#ifdef DEBUG
  cout << "issue command " << oct << endl;
#endif

  poller( pos)->issue_command( oct);
}

void Sched_Sensor_Request_issue::describe_request( Scheduler_Request &dreq) {
  Sched_Sensor_Request::describe_request( dreq);
  dreq.arguments= data;
  dreq.command_name= (const char *) "generic sensor command";
}

int Sched_Sensor_Request_issue::accepts_reply( Sched_Request &req, Sched_Reply &reply) {
  if (reply.min_barrier_id() == req.barrier.barrier_id)
    if (reply.reply_kind == Sched_Reply::RK_badge_unknown)
      if (reply.reply_code == reply_code) return 1;
  return 0;
}

//------------------------------------------------Sched_Request_reset_buffer
void Sched_Request_reset_buffer::send_command( Sched_Request &req) {
  Sched_Barrier_Id &pos= req.last_seen();
  
  poller( pos)->reset_buffer( pos.network_id, pos.station_id);
}

void Sched_Request_reset_buffer::describe_request( Scheduler_Request &dreq) {
  Sched_Sensor_Request::describe_request( dreq);
  dreq.command_name= (const char *) "reset_buffer";
}

//-------------------------------------------------Sched_Request_reset_fifos
//
void Sched_Request_reset_fifos::send_command( Sched_Request &req) {
  Sched_Barrier_Id &pos= req.last_seen();
 
  poller( pos)->reset_fifos( pos.network_id, pos.station_id);
}

void Sched_Request_reset_fifos::describe_request( Scheduler_Request &dreq) {
  Sched_Sensor_Request::describe_request( dreq);
  dreq.command_name= (const char *) "reset_fifos";
}

//--------------------------------------------------Sched_Request_set_output
//
void Sched_Request_set_output::send_command( Sched_Request &req) {
  Sched_Barrier_Id &pos= req.last_seen();

  poller( pos)->set_output( pos.network_id, pos.station_id,
                            output_value);
}

void Sched_Request_set_output::describe_request( Scheduler_Request &dreq) {
  Sched_Sensor_Request::describe_request( dreq);
  dreq.command_name= (const char *) "set_output";
}

//---------------------------------------------Sched_Request_set_persistence
//
void Sched_Request_set_persistence::send_command( Sched_Request &req) {
  Sched_Barrier_Id &pos= req.last_seen();

  poller( pos)->set_output( pos.network_id, pos.station_id,
						mode);
}

void Sched_Request_set_persistence::describe_request( Scheduler_Request &dreq) {
  Sched_Sensor_Request::describe_request( dreq);
  dreq.command_name= (const char *) "set_persistence";
}

//-------------------------------------------Sched_Request_get_station_state
//
void Sched_Request_get_station_state::send_command( Sched_Request &req) {
  Sched_Barrier_Id &pos= req.last_seen();

  poller( pos)->get_station_state( pos.network_id, pos.station_id);
}

void Sched_Request_get_station_state::describe_request( Scheduler_Request &dreq) {
  Sched_Sensor_Request::describe_request( dreq);
  dreq.command_name= (const char *) "get_station_state";
}

int Sched_Request_get_station_state::accepts_reply( Sched_Request &req, Sched_Reply &reply) {
  if (reply.min_barrier_id() == req.barrier.barrier_id)
    if (reply.reply_kind == Sched_Reply::RK_get_station_state) return 1;

  return 0;
}

//-------------------------------------------------------------Sched_Request
//
// Utility method : returns -1, if t1<t2
//                          +1, if t2>t2
//                           0, if t1==t2
//
int Sched_Request::compare_time(
	const ABng_Core::Time &t1,
	const ABng_Core::Time &t2) {
  ABng_Core::Time x1(t1), x2(t2);

  x1.seconds+= x1.microseconds/1000000;
  x2.seconds+= x2.microseconds/1000000;
  if (x1.seconds<x2.seconds) return -1;
  else if (x1.seconds>x2.seconds) return +1;
  else if (x1.microseconds<x2.microseconds) return -1;
  else if (x1.microseconds>x2.microseconds) return +1;
  else return 0;
}

//___________________________________________________________________________
// Signals the subrequest to terminate ASAP
//
void Sched_Request::cancel() {
  guard.lock();
  cancel_flag= 1;
  cond_sleep.signal();
  cond_sighting_arrived.signal();
  barrier.cond_reply_arrived.signal();
  guard.unlock();
}

void * Sched_Request::static_worker( void *arg) {
  return ((Sched_Request *) arg)->worker();
}


// Note: this method is called only from Sched_Barrier::reply_arrived
//  -- no locking required at this point
//
void Sched_Request::reply( Sched_Reply &x_reply) {
  if (my_reply) delete my_reply;
  my_reply= &x_reply;
}

//___________________________________________________________________________
//
// Sends requests, waits for replies, notifies callers
// Main scheduler routine for request dispatching
//
void * Sched_Request::worker() {
  ABng_Core::Command_Context &ctx= complex_request.context;

  sched_stat.inc_subrequest_counter();
  
  if (!(ctx.at.seconds == 0 && ctx.at.microseconds == 0)) {
    ABng_Server_Manager::Time cmt= Bootstrap::gettimeofday();
    struct timeval tv;
    gettimeofday( &tv, NULL);
    ABng_Server_Manager::Time diff;
    Domain_Repository::Time ctxat= ctx.at;

    ctxat.seconds+= (diff.seconds= tv.tv_sec - cmt.seconds);
    ctxat.microseconds+= (diff.microseconds= tv.tv_usec - cmt.microseconds);
    while (ctxat.microseconds>1000000) {
      ctxat.microseconds-=1000000;
      ctxat.seconds++;
    }
    while (ctxat.microseconds<0) {
      ctxat.microseconds+= 1000000;
      ctxat.seconds--;
    }

    barrier.wait_till( ctxat, *this);
    if (cancelled()) return NULL;
  }
  barrier.insert_request( *this);

#ifdef DEBUG
  cout << "Reuqest inserted to sighting queue" << endl;
#endif

  for (int i=0; i<ctx.repeat; i++) {

    seq_number= barrier.alloc_seq_number();
    // Allocates sequence number for request
    //

    for (int j=0; j<ctx.retries; j++) {
      ABng_Core::Time timeout;
      timeout= ctx.waiting_for_sighting_timeout;
#ifdef DEBUG
      cout << "Wait for sighting on barrier" << endl;
#endif
      int waitret= barrier.wait_for_sighting( timeout, *this);
#ifdef DEBUG
      cout << "wait_for_sighting returned " << waitret << endl;
#endif

      if (waitret == -1) { // request cancelled
	barrier.remove_request( *this);
	return NULL;
      }
      if (waitret == 0) { // timeout expired
	sched_stat.inc_sighting_errors_counter();
#ifdef DEBUG
	cout << "Sending notification" << endl;
#endif
	try {
	  if (ctx.interest == ABng_Core::Error ||
	      ctx.interest == ABng_Core::Error_And_Reply ||
	      ctx.interest == ABng_Core::Error_And_Request_Invoked)
	    if (complex_request.reply_to_is_valid)
	      complex_request.reply_to->
		error_occured( (ABng_Core_System::Sensor_Id) last_pos,
			       barrier.barrier_id.badge_id,
			       ABng_Core::SIGHTING_TIMEOUT,
			       complex_request.req_id, i,j);
	}
	catch (...) {}
#ifdef DEBUG
	cout << "done" << endl;
#endif
	continue; // next retry
      }
      //      omni_thread::sleep( 1);
#ifdef DEBUG
      cout << "Sending command" << endl;
#endif

      try {
	complex_request.send_command( *this);
      }
      catch (...) {
#ifdef DEBUG
	try {
	  throw;
	}
	catch (CORBA::Exception &ex) {
	  cerr << ex;
	}
	catch (...) {}
#endif
	try {
	  if (ctx.interest == ABng_Core::Error ||
	      ctx.interest == ABng_Core::Error_And_Reply ||
	      ctx.interest == ABng_Core::Error_And_Request_Invoked)
	    if (complex_request.reply_to_is_valid)
	      complex_request.reply_to->
		error_occured( (ABng_Core_System::Sensor_Id) last_pos,
			       barrier.barrier_id.badge_id,
			       ABng_Core::POLLER_UNREACHABLE,
			       complex_request.req_id, i,j);
	}
	catch (...) {}
	break;
      }

      sched_stat.inc_command_sent_counter();
      
      if (!complex_request.hasReply()) { // request has no reply
	if (ctx.interest == ABng_Core::Request_Invoked ||
	    ctx.interest == ABng_Core::Error_And_Request_Invoked)
	
	  if (complex_request.reply_to_is_valid) {
	    try {
	      complex_request.reply_to->
	       request_processed( (ABng_Core_System::Sensor_Id) last_pos,
				  barrier.barrier_id.badge_id,
				  last_sighting_time, complex_request.req_id,
				  i);
	    }
	    catch (...) {}
	  }
	barrier.end_request();
	break; // next repeat
      }

      // command has reply
#ifdef DEBUG
      cout << "Wait for reply on barrier " << endl;
#endif
      timeout= ctx.waiting_for_reply_timeout;

      waitret= barrier.wait_for_reply( timeout, *this);
#ifdef DEBUG
      cout << "wait_for_reply returned " << waitret << endl;
#endif

      if (waitret == -1) { // request cancelled
	barrier.end_request();
	barrier.remove_request( *this);
	return NULL;
      }      
      if (waitret == 0) { // reply timed out
	sched_stat.inc_reply_errors_counter();
#ifdef DEBUG
	cout << "Sending notification" << endl;
#endif
	try {
	  if (ctx.interest == ABng_Core::Error ||
	      ctx.interest == ABng_Core::Error_And_Reply ||
	      ctx.interest == ABng_Core::Error_And_Request_Invoked)
	    if (complex_request.reply_to_is_valid)
	      complex_request.reply_to->
		error_occured( (ABng_Core_System::Sensor_Id) last_pos,
			       barrier.barrier_id.badge_id,
			       ABng_Core::REPLY_TIMEOUT,
			       complex_request.req_id, i,j);

	}
	catch (...) {}
#ifdef DEBUG
	cout << "done" << endl;
#endif
	barrier.end_request();
	continue; // next retry
      }
      
      // reply arrived
      sched_stat.inc_reply_counter();

      if (ctx.interest == ABng_Core::Reply ||
	  ctx.interest == ABng_Core::Error_And_Reply) {
	try {
#ifdef DEBUG
	  cout << "Seding reply" << endl;
#endif
	  if (complex_request.reply_to_is_valid)
	    my_reply->send_reply( *this, i);
#ifdef DEBUG
	  cout << "Done" << endl;
#endif
	}
	catch (...) {}
      }
      else if (ctx.interest == ABng_Core::Request_Invoked ||
	       ctx.interest == ABng_Core::Error_And_Request_Invoked) {
	try {
	  if (complex_request.reply_to_is_valid)
	    complex_request.reply_to->
	      request_processed( (ABng_Core_System::Sensor_Id) last_pos,
		 barrier.barrier_id.badge_id,
		 my_reply->time_stamp, complex_request.req_id, i); 
	}
	catch (...) {}
      }
      barrier.end_request();
      delete my_reply;
      my_reply= NULL;

      // request sent, reply arrived
      // jump to next repeat
      break;
    }
  
    if (i+1<ctx.repeat)
    {
      unsigned long s,ns;
      omni_thread::get_time( &s, &ns,
			     ctx.interval.seconds, ctx.interval.microseconds*1000);
      ABng_Core::Time till;
      till.seconds= s;
      till.microseconds= ns/1000;
      barrier.wait_till( till, *this);
    }
    if (cancelled()) {
      barrier.remove_request( *this);
      return NULL;
    }
  }
  barrier.remove_request( *this);
#ifdef DEBUG
  cout << "Request processing terminated" << endl;
#endif
  complex_request.subrequest_completed( );
  return NULL;
}





































