// $Id: locatable_rep_i.cc,v 1.2 1998/07/29 14:57:06 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/sighting_processors/locatable_rep_i.cc,v $
//
// $Log: locatable_rep_i.cc,v $
// Revision 1.2  1998/07/29 14:57:06  mencnar
// performance tuning,
// receive sightings calls moved to dyn_dispatcher.* for future optimisation
//
// Revision 1.1.1.1  1998/06/18 13:37:37  mencnar
//
// Revision 1.1  1998/03/06 13:24:38  jasz
// Initial revision
//
//
// File:          locatable_rep_i.cc
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  26.01.98
//
// General Description:
//  Declaration of the implementation class for the "locatable part" of View,
//  VSP and View_Manager interfaces.

#define EXCEPTIONS

#include "locatable_rep_i.h"
#include "location_rep_i.h"
#include "view_i.h"

#include <rw/rwdate.h>
#include <rw/rwtime.h>
#include <rw/tvdlist.h>

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_ABng_Core_Locatable(Locatable_I)
  //DEF_ObjectStoreTIE_ABng_Core_Internals_Locatable_Sighting_Processor(Locatable_I)
#else
DEF_TIE_ABng_Core_Locatable(Locatable_I)
  //DEF_TIE_ABng_Core_Internals_Locatable_Sighting_Processor(Locatable_I)
#endif
#endif

Locatable_Rep_I::Locatable_Rep_I(
#ifdef ObjectStore
    os_database* db_p,
#elif defined FilePersistency
    const char *repository_name_p,
#endif
    const char *view_name_p,
    long sightings_dump_period_p,
    unsigned long burst_delay,
    int
  ) : No_Update_Basic_Rep_I<Locatable_Types, VDO_Types>(hash_locatable,
#ifdef ObjectStore
    db_p
#elif defined FilePersistency
    repository_name_p
#endif
  ),
  my_bh_rep(ABng_Core_Management::Badge_Holder_Rep::_nil()),
  my_msp(ABng_Core_Internals::Main_Sighting_Processor::_nil()),
  my_dynamic_observed(
#ifdef FilePersistency
    (const char *) (RWCString( (const char *) repository_name_p)+".dynamic"),
#endif
    burst_delay
  ),
  my_badge_ids_to_locatables(Comparable_Badge_Id::hash),
  my_badge_item_ids_to_locatables(hash_item_id),
  my_sensor_item_ids_to_locatables(hash_item_id),
  my_view_name(CORBA::string_dup("")),
  sightings_dump_period( sightings_dump_period_p) {

  _instance = this;
  if(view_name_p) {
    my_view_name = CORBA::string_dup(view_name_p);
  }
}

// Cold start constructor
Locatable_Rep_I::Locatable_Rep_I(
#ifdef ObjectStore
  os_database* db_p,
#elif defined FilePersistency
  const char *repository_name_p,
#endif
  const char *view_name_p,
  long sightings_dump_period_p,
  unsigned long burst_delay
  ) : No_Update_Basic_Rep_I<Locatable_Types, VDO_Types>(hash_locatable,
#ifdef ObjectStore
    db_p
#elif defined FilePersistency
    repository_name_p
#endif
  ),
  my_bh_rep(ABng_Core_Management::Badge_Holder_Rep::_nil()),
  my_msp(ABng_Core_Internals::Main_Sighting_Processor::_nil()),
  my_dynamic_observed(
#ifdef FilePersistency
    (const char *) (RWCString( (const char *) repository_name_p)+".dynamic"),
#endif
    burst_delay
  ),
  my_badge_ids_to_locatables(Comparable_Badge_Id::hash),
  my_badge_item_ids_to_locatables(hash_item_id),
  my_sensor_item_ids_to_locatables(hash_item_id),
  my_view_name(CORBA::string_dup("")),
  sightings_dump_period( sightings_dump_period_p) {

  _instance = this;
  if(view_name_p) {
    my_view_name = CORBA::string_dup(view_name_p);
  }
}

Locatable_Rep_I::~Locatable_Rep_I() {}

Locatable_Rep_I *Locatable_Rep_I::_instance = 0;

Locatable_Rep_I *Locatable_Rep_I::instance() {

  return _instance;
}

// Operations of the View interface.

ABng_Core_System::Badge_Holder_ptr Locatable_Rep_I::deref(
    ABng_Core::Locatable_ptr lct_object
    ENV_DEF(env)) {

  Locatable_I *lct_i= deref_item( lct_object);
  if (lct_i) 
    return ABng_Core_System::Badge_Holder::_duplicate( lct_i->get_badge_holder());
}


// Get the location type name.
Domain_Repository::Location_Type::Location_Type_Name Locatable_Rep_I::type (
    ENV_DEF1( IT_env) ) {

  return CORBA::string_dup(
                 Location_Rep_I::instance()->get_location_type_name());
}

// Get all sightings received since the given time.
ABng_Core::Sighting_Descriptions* Locatable_Rep_I::get_sightings (
  const ABng_Core_System::Time& since
  ENV_DEF(env)) {

  Item_List_Iterator iterator(my_item_list);
  int len = my_item_list.entries();
  ABng_Core::Sighting_Descriptions *sightings = 
    new ABng_Core::Sighting_Descriptions(len);

  for(int i = 0; iterator(); ) {
    ABng_Core::Sighting_Description &sd= iterator.key()->local_get_sighting();
    if (sd.update_time >= since) {
      sightings->length( i+1);
      (*sightings)[i++] = sd;
    }
  }
  return sightings; 
}

// The implementation of the notify IDL operation of 
// the System_Static_Data_Observer interface.

void Locatable_Rep_I::notify(
  Domain_Repository::Repository_Item_Id id,
  const ABng_Core_System::System_Static_Data_Observer::System_Static_Data_Object& sd, 
  Domain_Repository::Change_Type change
  ENV_DEF(env)) {
    
#ifdef DEBUG
  cerr << "Locatable_Rep_I::notify" << endl;
#endif

  // Determine what has been changed.
  switch(sd._d()) {
    case (ABng_Core_System::System_Static_Data_Observer::BADGE_HOLDER_SD) : {
      // Something has changed in BH_Rep. 
      // Find a locatable corresponding to the changed BH.
      Locatable_I locatable_pattern(id);
      Locatable_I *loc;
      if(!(loc = my_item_list.find(&locatable_pattern))) {
	// There is no locatable corresponding to that BH.
	break;
      }
      // Determine a change kind.
      switch(change) {
        case Domain_Repository::OBJECT_REMOVED : {
	  // Remove the locatable corresponding to the removed BH.
	  private_remove(loc->get_corba_object());
	  break;
	}
        case Domain_Repository::OBJECT_UPDATED : {
	  // Some attributes in the corresponding BH have changed.
	  // Check which of them has been modified and take 
	  // an appropriate action.
	  refresh(loc);
	  break;
	}
        case Domain_Repository::OBJECT_ADDED : {
	  // The ADD_OBJECT event is ignored.
	}
        default : {
	  break;
	}
      }
      break;
    }

    case (ABng_Core_System::System_Static_Data_Observer::BADGE_SD) : {
      // Something in Badge_Rep has changed.
      // Determine a change kind.
      switch(change) {
        case Domain_Repository::OBJECT_UPDATED : {
	  // The id of one of the badges has been modified.
	  // This might be a badge of a locatable. 
	  // Update your database if necessary.
	  replace_badge(sd.badge_obj());
	  break;
	}
        default : {
	  // Other events are ignored.
	  break;
	}
      }
      break;
    }
    case (ABng_Core_System::System_Static_Data_Observer::SENSOR_SD) : {
      // Something in Sensor_Rep has changed.
      // Determine a change kind.
     switch(change) {
        case Domain_Repository::OBJECT_UPDATED : {
	  // The id of one of the sensors has been modified.
	  // This might be a badge of a locatable.
	  // Update your database if necessary.
	  replace_sensor(sd.sensor_obj());
	  break;
	}
        default : {
	  // Other events are ignored.
	  break;
	}
      }
      break;
    }
    default : {
      break;
    }
  }
}

// Get a locatable associated with the badge with the given id.

Locatable_I *Locatable_Rep_I::resolve_badge(
  const ABng_Core_System::Badge_Id& bid) {

  Comparable_Badge_Id cbid(bid);
  Comparable_Badge_Id ret_key;
  Locatable_I *res= NULL; // !! initialize the variable
  my_badge_ids_to_locatables.findKeyAndValue(cbid, ret_key, res);
  return res;
}

ABng_Core_System::Badge_Ids Locatable_Rep_I::locatables2badge_ids(
    const ABng_Core::Locatables *locs) {

  ABng_Core::Locatables_var auto_obj;

  if ( locs->length() == 0) {
    auto_obj= (ABng_Core::Locatables*) (locs= locatable_list());
    // empty list of locatables -> all locatables in the view
  }

  ABng_Core_System::Badge_Ids ret( locs->length());
  ret.length( locs->length());

  for ( int i=0; i<locs->length(); i++) {
    Locatable_I *locatable_i= deref_item( (*locs)[i]);
    if (!locatable_i) break;

    ret[ i]= locatable_i->get_bd()->id;
  }
  return ret;
}

ABng_Core::Locatables Locatable_Rep_I::badge_ids2locatables(
    const ABng_Core_System::Badge_Ids *bids) {

  ABng_Core::Locatables ret( bids->length());

  ret.length( bids->length());

  for ( int i=0; i<bids->length(); i++) {
    
    Locatable_I *locatable_i= resolve_badge( (*bids)[i]);
    if (! locatable_i) break; // any other ideas ?

    ABng_Core::Locatable_ptr locatable_obj= locatable_i->get_corba_object();
    ret[ i]= ABng_Core::Locatable::_duplicate( locatable_obj);
  }
  return ret;
}

ABng_Core::Locatable_var Locatable_Rep_I::badge_id2locatable(
    const ABng_Core_System::Badge_Id &bid) 
  throw( ABng_Core::Invalid_Argument) {

  Locatable_I *locatable_i= resolve_badge( bid);
  if (! locatable_i) throw ABng_Core::Invalid_Argument();
  return ABng_Core::Locatable::_duplicate( locatable_i->get_corba_object());
}


// Notify dynamic data observers registered with the locatable 
// about a change of sighting data.

void Locatable_Rep_I::dynamic_notify(Locatable_I *locatable_p) {
  // "save" should be called every "sightings_dump_period" seconds
  RWTime current_time;
  if ( current_time >= last_save_time+sightings_dump_period) {
    last_save_time= current_time;
    save();
  }

  // Get a list of DDOs attached to the locatable.
  Basic_Observed_I<DDO_Types>::Observer_List *observer_list_p = 
    my_dynamic_observed.get_item_observer_list(locatable_p->item_id());

  // Send a current sighting of the locatable to them.
  Basic_Observed_I<DDO_Types>::Observer_List_Iterator 
    iterator(*observer_list_p);

  RWTValDlist<DDO_Types::Observer_ptr> dead_observers;

  // Notify all observers of this entity
  while (iterator()){
    if (!my_dynamic_observed.dynamic_notify_observer( iterator.key(),
						      locatable_p->local_get_sighting()))
      dead_observers.insert(iterator.key());
  }

  RWTValDlistIterator<DDO_Types::Observer_ptr> dead_iterator(dead_observers);
  
  // Remove dead observers.
  while (dead_iterator()) {
    try {
      my_dynamic_observed.private_detach_observer(
	 dead_iterator.key(),
	 my_dynamic_observed.get_item_ids_list(dead_iterator.key()));
    } 
    catch(...) {
    }
  }
  if (dead_observers.entries()) my_dynamic_observed.save_observers();
}

// Detach the location for the locatable.

void Locatable_Rep_I::remove_location(Location_I *location_p) {

  Item_List_Iterator iterator(my_item_list);
  while(iterator()) {
    iterator.key()->remove_location(location_p);
  }
}

// Refresh all the information that could have changed in the BH_Rep.
// This method should be called after the BH_Rep is launched.

void Locatable_Rep_I::resolve_bh_rep(
  ABng_Core_Management::Badge_Holder_Rep_var bh_rep_p) {

  // Update your copy of BH_Rep reference.
  if(CORBA::is_nil(
    (ABng_Core_Management::Badge_Holder_Rep_ptr) bh_rep_p)) {
    return;
  }
  // Update your copy of ABng_LD_Rep reference.
  my_bh_rep = bh_rep_p;
  ABng_Core_System::Badge_Holder::Entity_Set es;
  {
    Item_List_Iterator iterator(my_item_list);

    int len= my_item_list.entries();
    Domain_Repository::Repository_Item_Ids ids(len);
    ids.length(len);
  
    for (int i=0; iterator(); i++)
      ids[i]= iterator.key()->get_badge_holder_item_id();
    
    es.s_ids(ids);
  }
  Domain_Repository::Query_Options qo;
  qo.return_references= 1;
  qo.return_descriptions= 0;
  qo.return_value_descriptions= 0;
  qo.do_attach= 1;
  qo.do_detach= 0;
  ABng_Core_System::Badge_Holder::Query_Results_var qr;
  try {
    qr= my_bh_rep->complex_query( es, qo, View_I::instance()->get_ssdo_ptr());
  }
  catch (...) {
    return;
  }

  Item_List_Iterator iterator(my_item_list);
  int stop = 0;
  int ref_res = 0;
  // Assume the eager MSP notification mechanism.
  // MSP will be notified immediatelly after a discovery that a badge  
  // of a locatable has changed.
  int lazy_msp_notif = 0;
  if(my_item_list.entries() == 0) {
    // There is no locatables in the repository.
    // This means that MSP will be provided with badge lists of 
    // all locatablea. To improve this assume the lazy MSP notification 
    // mechanism. The MSP will be notified only once after all local copies 
    // od badge ids are updated.
    lazy_msp_notif = 1;
  }

  RWTValDlist<Locatable_I*> del_locatables;

  // Go though the locatable list.
  for (int i=0;
       iterator() && !stop;
       i++) {

    // Convert item foreign item ids stored within the locatable 
    // into references.
    //    Locatable_I::Resolution_Result res = 
    //      iterator.key()->resolve_ids(my_bh_rep);
    Locatable_I::Resolution_Result res =
      iterator.key()->fast_resolve_ids( qr->r_references[i]);

#ifdef DEBUG
    cerr << "Locatable_I::resolve_ids result " << (int) res << endl;
#endif
    switch(res) {
      case Locatable_I::SUCCESS : {
	// Convertion has succeeded. Refresh the locatable. 
        // Notify the MSP if required and necessary.
	ref_res = refresh(iterator.key(), !lazy_msp_notif);
#ifdef DEBUG
	cerr << "Locatable_Rep_I::refresh result " << (int) ref_res << endl;
#endif
       	switch(ref_res) {
	  case Locatable_I::COMM_FAILURE : {
	    // Something went wrong. Assume the same problem will occur 
	    // for the next locatables again and leave the loop.
	    stop = 1;
	    break;
	  }
	  default : {
	    break;
	  }
	}
	break;
      }
      case Locatable_I::RES_COMM_FAILURE : {
	// Something went wrong during convertion. Assume the same problem 
	// will occur for the next locatables again and leave the loop.
	stop = 1;
	break;
      }
      case Locatable_I::INCONSISTENCY : {
	// The corresponding BH has been removed.
	// Remove the locatable.
	del_locatables.insert( iterator.key());
	break;
      case Locatable_I::IGNORE:
	break;
      }
      default : {
	break;
      }
    }
  }
  for (RWTValDlistIterator<Locatable_I*> del_iter( del_locatables);
       del_iter(); )
    private_remove( del_iter.key()->get_corba_object());

  if(lazy_msp_notif) {
    // Update the sensor list of the view in the MSP.
    ABng_Core_System::Badge_Ids empty_bids(0);
    empty_bids.length(0);
    ABng_Core_System::Badge_Ids_var bids = get_badge_list();
    try {
      if (!CORBA::is_nil( (ABng_Core_Internals::Main_Sighting_Processor_ptr) 
			  my_msp)) {
	my_msp->remove_badges(my_view_name, empty_bids);
	my_msp->add_badges(my_view_name, *(bids.operator->()));
      }
    }
    catch(...) {
    }
  }
}

void Locatable_Rep_I::resolve_badge_rep(
  ABng_Core_Management::Badge_Rep_var badge_rep_p) {

  if(CORBA::is_nil(
    (ABng_Core_Management::Badge_Rep_ptr) badge_rep_p)) {
    return;
  }
  // Update your copy of ABng_LD_Rep reference.
  my_badge_rep = badge_rep_p;

  // we don't need to refresh information about badges
  // as such an update should be on the way from badge_holder repository
  //

  Item_List_Iterator iterator(my_item_list);
  int len= my_item_list.entries();
  Domain_Repository::Repository_Item_Ids ids(len);
  ids.length(len);
  
  for (int i=0; iterator(); i++)
    ids[i]= iterator.key()->get_bd()->item_id;

  ABng_Core_System::Badge::Entity_Set es;
  es.s_ids(ids);
  Domain_Repository::Query_Options qo;
  qo.return_references= 0;
  qo.return_descriptions= 0;
  qo.return_value_descriptions= 0;
  qo.do_attach= 1;
  qo.do_detach= 0;
  try {
    ABng_Core_System::Badge::Query_Results_var qr=
      my_badge_rep->complex_query( es, qo, View_I::instance()->get_ssdo_ptr());
  }
  catch (...) {}
}

void Locatable_Rep_I::resolve_msp(
  ABng_Core_Internals::Main_Sighting_Processor_var msp_p) {

  // Just update the local copy of the MSP reference.
  my_msp = msp_p;
}

ABng_Core_System::Badge_Ids *Locatable_Rep_I::get_badge_list() {

  // Construct a sequence containing all badges belonging to the view.
  int len = my_badge_ids_to_locatables.entries();
  ABng_Core_System::Badge_Ids *res = 
    new ABng_Core_System::Badge_Ids(len);
  res->length(len);
  Badge_Id_Locatable_Dictionary_Iterator 
    iterator(my_badge_ids_to_locatables);
  for(int i = 0; i < len, iterator(); i++) {
    (*res)[i] = iterator.key();
  }
  res->length(len);
  return res;
}

// Calculate the hash value of the locatable.

unsigned Locatable_Rep_I::hash_locatable(const Locatable_I& locatable_i_p) {

  return locatable_i_p.get_badge_holder_item_id();
}

// Add helper methods

void Locatable_Rep_I::init_corba_object(Locatable_I *item_p) {

  // create the TIE (CORBA) object for the C++ object
#ifdef ORBIX
#ifdef ObjectStore
  ObjectStoreTIE_ABng_Core_Locatable(Locatable_I)* 
    new_locatable_type_tie =
    new(my_db, ObjectStoreTIE_ABng_Core_Locatable(
      Locatable_I)::get_os_typespec()) 
      ObjectStoreTIE_ABng_Core_Locatable(
        Locatable_I
	) (item_p);
  /*
  ObjectStoreTIE_ABng_Core_Internals_Locatable_Sighting_Processor(Locatable_I)* 
    new_locatable_type_tie =
    new(my_db, ObjectStoreTIE_ABng_Core_Internals_Locatable_Sighting_Processor (
      Locatable_I)::get_os_typespec()) 
      ObjectStoreTIE_ABng_Core_Internals_Locatable_Sighting_Processor(
        Locatable_I
	) (item_p);
  */
#else
  TIE_ABng_Core_Locatable(Locatable_I) *
    new_locatable_tie = 
    new TIE_ABng_Core_Locatable(Locatable_I) (item_p);
  /*
  TIE_ABng_Core_Internals_Locatable_Sighting_Processor(Locatable_I) *
    new_lesp_tie = 
    new TIE_ABng_Core_Internals_Locatable_Sighting_Processor(Locatable_I) (
      item_p);
  */
#endif
#elif defined OMNIORB
  ABng_Core::Locatable_ptr new_locatable_tie=
    new TIE_CLASS(ABng_Core,Locatable)<Locatable_I,0>(item_p);
  CONNECT_IMPL( ABng_Core::Locatable_var, new_locatable_tie);
  
#endif
  item_p->set_corba_object(new_locatable_tie);
  /*
  item_p->set_lesp_corba_object(new_lesp_tie);
  */
}

void Locatable_Rep_I::insert_item(Locatable_I* item_p) {

  // Call the "basic" insert.
  No_Update_Basic_Rep_I<Locatable_Types, VDO_Types>::insert_item(item_p);

  // Update the dynamic data observer registry.
  my_dynamic_observed.insert_new_item(item_p->item_id());

  // Get the description of the badge associated with the new locatable.
  Comparable_Badge_Descr *bd = item_p->get_bd();

  // Update both badge dictionaries.
  my_badge_ids_to_locatables.insertKeyAndValue(
    Comparable_Badge_Id(bd->id), item_p);
  my_badge_item_ids_to_locatables.insertKeyAndValue(
    bd->item_id, item_p);
  my_sensor_item_ids_to_locatables.insertKeyAndValue(
    item_p->get_home_sd()->item_id, item_p);
  // Notify MSP about adding a new badge.
  ABng_Core_System::Badge_Ids bids(1);
  bids.length(1);
  bids[0]= bd->id;
  try {
    if (!CORBA::is_nil( (ABng_Core_Internals::Main_Sighting_Processor_ptr) 
			my_msp))
      my_msp->add_badges(my_view_name,  bids);
  }
  catch(...) {
  }
  try {
    Domain_Repository::Repository_Item_Ids ids( 1);
    ids.length( 1);
    ids[0]= item_p->get_badge_holder_item_id();
    if (!CORBA::is_nil( (ABng_Core_Management::Badge_Holder_Rep_ptr) my_bh_rep))
      my_bh_rep->attach_system_static_data_observer( 
		       View_I::instance()->get_ssdo_ptr(),
		       ids);
  }
  catch (CORBA::Exception &ex) {
#ifdef DEBUG
    cerr << "Locatable_Rep_I::insert_item " << ex << endl;
#endif
  }
}

// Remove helper methods

void Locatable_Rep_I::remove_item(Locatable_I *item_p) {

  // Call the "basic" remove.
  No_Update_Basic_Rep_I<Locatable_Types, VDO_Types>::remove_item(item_p);

  // Update the dynamic data observer registry.
  my_dynamic_observed.detach_item_from_observers(item_p->item_id());

  // Get the description of the badge associated with the locatable.
  Comparable_Badge_Descr *bd = item_p->get_bd();

  // Update both badge dictionaries.
  my_badge_ids_to_locatables.remove(Comparable_Badge_Id(bd->id));
  my_badge_item_ids_to_locatables.remove(bd->item_id);
  my_sensor_item_ids_to_locatables.remove(
    item_p->get_home_sd()->item_id);
  // Notify MSP about removing sensors.
  ABng_Core_System::Badge_Ids bids(1);
  bids.length(1);
  try {
    if (!CORBA::is_nil( (ABng_Core_Internals::Main_Sighting_Processor_ptr) 
			my_msp))
      my_msp->remove_badges(my_view_name, bids);
  }
  catch(...) {
  }
  Location_Rep_I::instance()->remove_locatable(item_p);
}

// Find helper methods

ABng_Core::Locatables* 
  Locatable_Rep_I::private_find (
  const ABng_Core::Locatable::Pattern& data_record_pattern
  ENV_DEF( env)) 
{

#ifdef DEBUG
  struct timeval t1,t2;
  gettimeofday( &t1, NULL);
#endif
  
  MatchLocatableApplyDataStruct apply_data( data_record_pattern);

  my_item_list.apply( match_locatable, (void*)&apply_data);

  // remote find 
  if (apply_data._found_locatables.entries() &&
      (!data_record_pattern.is_any)) // not empty
    if (! data_record_pattern.type_specific_object_pattern.is_any) { 
      // remote pattern really matters

#ifdef DEBUG
      cerr << "Remote find" << endl;
#endif
      if (CORBA::is_nil( (ABng_Core_Management::Badge_Holder_Rep_ptr) my_bh_rep))
	throw Domain_Repository::Destination_Unreachable("badge_holder_rep");

      try {
	ABng_Core_System::Badge_Holder::Pattern bh_pattern;
	bh_pattern.is_any= 0;
	bh_pattern.item_id_pattern.is_any= 1;
	bh_pattern.my_badge_pattern.is_any= 1;
	bh_pattern.current_badge_pattern.is_any= 1;
	bh_pattern.home_sensor_pattern.is_any= 1;
	bh_pattern.type_specific_object_pattern= 
	  data_record_pattern.type_specific_object_pattern;
	bh_pattern.assigned_pattern.is_any=
	  bh_pattern.visitor_pattern.is_any= 1;

	ABng_Core_System::Badge_Holder::Value_Descriptions *
	  ret_val( my_bh_rep->find_values( bh_pattern));
	ABng_Core_System::Badge_Holder::Value_Descriptions_var
	  auto_ret_val( ret_val);

#ifdef DEBUG
	cerr << "Remote find: " << (unsigned) ret_val->length()
	     << " items matched in bh rep " << endl;
#endif
	for ( RWTValDlistIterator<Locatable_I*> iter( apply_data._found_locatables);
	      iter(); ) {

	  int i;
	  Locatable_I * it= iter.key();
	  for (i=0; i<ret_val->length(); i++)
	    if ( it->get_badge_holder_item_id() == 
		 (*ret_val)[i].item_id) break;

	  if (i==ret_val->length()) // no corresponding item found
	    iter.remove(); // remove current item
	}
      }
      catch (Domain_Repository::Destination_Unreachable &) {
	throw;
      }
      catch (CORBA::Exception &ex) {
#ifdef DEBUG
	cerr << "Locatable_Rep_I::private_find " << ex << endl;
#endif
	throw Domain_Repository::Destination_Unreachable("badge_holder_rep");	
      }	
    }
  //  ----
  ABng_Core::Locatables *ret= 
    new ABng_Core::Locatables( apply_data._found_locatables.entries());
  ret->length( apply_data._found_locatables.entries());

  int pos=0;
  for ( RWTValDlistIterator<Locatable_I*> iter( apply_data._found_locatables);
	iter(); )
    (*ret)[pos++]= ABng_Core::Locatable::_duplicate( iter.key()->get_corba_object());

#ifdef DEBUG
  gettimeofday(&t2,NULL);
  cerr << (t2.tv_sec-t1.tv_sec)*1000000L+(t2.tv_usec-t1.tv_usec) << endl;
#endif
  return ret;
}

// check for local patterns: item_id_pattern, home_location_pattern
//
void Locatable_Rep_I::match_locatable( Locatable_I *current_locatable,
				       void *_apply_data) {

  MatchLocatableApplyDataStruct &apply_data= 
    *(MatchLocatableApplyDataStruct*)_apply_data;

  if (apply_data._data_record_pattern.is_any) {
    apply_data._found_locatables.insert( current_locatable);
    return;
  }
  if (!apply_data._data_record_pattern.item_id_pattern.is_any &&
       apply_data._data_record_pattern.item_id_pattern.value != 
          current_locatable->get_descr().item_id) return;

  if (!apply_data._data_record_pattern.home_location_pattern.is_any) {
    if (CORBA::is_nil(current_locatable->get_descr().home_location)) return;
    
    MatchLocationApplyDataStruct
      location_apply_data( apply_data._data_record_pattern.home_location_pattern);

    Location_Rep_I::match_location( current_locatable->get_home_location(),
				    (void*)&location_apply_data);
    if (! location_apply_data._found_locations.entries()) return;
  }

  apply_data._found_locatables.insert( current_locatable);
}

void Locatable_Rep_I::collect_values(Locatable_I *lt, 
  ABng_Core::Locatable::Value_Description& ld) {

  ABng_Core::Locatable::Value_Description_var val_descr= lt->value_descr();
  ld= *(val_descr.operator->());
}

ABng_Core::Locatable::Description 
  Locatable_Rep_I::get_null_description( Item_I *item) {
  
  Description d;
  if (item) d.item_id= item->item_id();
  d.type_specific_object.user_obj( Domain_Repository::User::_nil());
  return d;
}

ABng_Core::Locatable::Value_Description 
  Locatable_Rep_I::get_null_value_description( Item_I *item) {

  Value_Description d;
  if (item) d.item_id= item->item_id();
  d.type_specific_object_value.user_description( 
				  Domain_Repository::User::Value_Description());
  return d;
}



// Common helper methods

void Locatable_Rep_I::construct_data_object(
  ABng_Core::View_Data_Observer::View_Data_Object& data_object_p,
  Locatable_ptr item_p) {

  // construct union to store reference of added entity
  data_object_p.locatable_obj(
    ABng_Core::Locatable::_duplicate(item_p));
}

void Locatable_Rep_I::save() {

  No_Update_Basic_Rep_I<Locatable_Types, VDO_Types>::save();
}

void Locatable_Rep_I::save(ostream &str) {

  No_Update_Basic_Rep_I<Locatable_Types, VDO_Types>::save(str);
  str << my_view_name << endl;
}

void Locatable_Rep_I::restore(istream &str) {

  No_Update_Basic_Rep_I<Locatable_Types, VDO_Types>::restore(str);
  str >> my_view_name;
}



DB_Header * Locatable_Rep_I::create_db_header() {

  // Yes, this constructs the wrong DB_Header with a duplicated description
  // section, but the second description is correct, at least.
  // Something in the DB_Header code should be changed.

  return new DB_Header( "Locatable Repository", "1.0", "\
#       <file>::=       <next_item_id>\n\
#                       <number_of_items>\n\
#                       <data_records>\n\
#                       <view_name>\n\
#\n\
#       <data_records>::=  | <data_record> <data_records>\n\
#\n\
#       <data_record>::=        <repository_item_id>\n\
#                               <abng_locatable_item_id>\n\
#                               <sighting_location_id>\n\
#                               <sighting_message_type>\n\
#                               <sighting_attributes>\n\
#                               <sighting_sequence_number>\n\
#                               <sighting_update_time_seconds>\n\
#                               <sighting_update_time_microseconds>\n\
#                               <sighting_quality>\n\
#\n\
#\n", 1);

}

// Refresh the locatable attributes representing information
// from other repositories (BH_Rep, Sensor_Rep) that could have 
// changed. If notify_msp is non-zero the MSP is immediately notified 
// if a badge associates with the locatable is chaned.

Locatable_I::Refreshing_Result Locatable_Rep_I::refresh(Locatable_I *item_p,
  int notify_msp) {

  // Ask the locatable to contact the corresponding BH, to determine
  // the set of relevant information which have been changed
  // and to refresh its attributes.
  Comparable_Badge_Descr *new_bd = 0;
  Comparable_Badge_Descr *old_bd = 0;
  Comparable_Sensor_Descr *new_home_sd = 0;
  Comparable_Sensor_Descr *old_home_sd = 0;
  Locatable_I::Refreshing_Result ref_res = 
    item_p->refresh(new_bd, old_bd, new_home_sd, old_home_sd);
#ifdef DEBUG
  cerr << "Locatable_I::refresh result " << (int) ref_res << endl;
#endif

  switch(ref_res) {
    case Locatable_I::CHANGED_INTERNALS : {
      // The locatable's badge has been updated.
      // Update the database. Notify MSP if required.
      if(old_bd != new_bd) {
      	update_badge(item_p, old_bd, new_bd, notify_msp);
      	delete old_bd;
      }
      if(old_home_sd != new_home_sd) {
	update_sensor(item_p, old_home_sd, new_home_sd);
	delete old_home_sd;
      }      
      break;
    }
    case Locatable_I::CHANGED_EXTERNALS : {
      // The reference of the BD has been updated.
      // Notify view data observers.
      Observer_List *observer_list_p = 
	get_item_observer_list(item_p->item_id());
      notify_observers(*observer_list_p, item_p, 
        Domain_Repository::OBJECT_UPDATED);
      break;
    }
    case Locatable_I::CHANGED_BOTH: {
      // Both the list of badges and the BH referece have been updated.
      // Update the database. Notify MSP if required.
      if(old_bd != new_bd) {
	update_badge(item_p, old_bd, new_bd, notify_msp);
      }
      if(old_home_sd != new_home_sd) {
	update_sensor(item_p, old_home_sd, new_home_sd);
      }
      // Notify observers.
      Observer_List *observer_list_p = 
	get_item_observer_list(item_p->item_id());
      notify_observers(*observer_list_p, item_p, 
        Domain_Repository::OBJECT_UPDATED);
      break;
    }
    case Locatable_I::COMM_FAILURE : {
      // The communication error has occured when refreshing attributes.
      break;
    }
    case Locatable_I::UNCHANGED : {
      // Nothing has changed.
      break;
    }
    default : {
      break;
    }
  }
  return ref_res;
}          

// Replace the local copy of the badge description with the current value.
void Locatable_Rep_I::replace_badge(
  ABng_Core_System::Badge_ptr badge_p) {

  Comparable_Badge_Descr *new_cbd;
  Comparable_Badge_Descr *old_cbd;
  ABng_Core_System::Badge::Description *new_bd;
  Comparable_Badge_Id new_cbid;
  Comparable_Badge_Id old_cbid;
  Locatable_I *loc;

  // Get the badge description.
  try {
    new_bd = badge_p->descr();
#ifdef ORBIX
    if (CORBA::is_nil( new_bd->holder))
      new_bd->holder= NULL;
#endif
  }
  catch(...) {
    return;
  }

  new_cbd = new Comparable_Badge_Descr(new_bd);
  new_cbid = Comparable_Badge_Id(new_cbd->id);
  delete new_bd;

  // Find a locatable containing the sensor
  if(!my_badge_item_ids_to_locatables.findValue(new_cbd->item_id, loc)) {
    // Remove the badge id from the dictionary
    my_badge_ids_to_locatables.remove(new_cbid);
    delete new_cbd;
    return;
  }
  // Find the old description of the updated badge.
  old_cbd = loc->get_bd();
  if(!old_cbd) {
    // There is no description for this sensor.
    // This is an abnormal situation and it should not happen.
    // But, well, let's do something with it. 
    my_badge_item_ids_to_locatables.remove(new_cbd->item_id);
    my_badge_ids_to_locatables.remove(new_cbid);
    delete new_cbd;
    return;
  }

  old_cbid = old_cbd->id;
  if(old_cbid == new_cbid) {
    // The badge has not changed its id. 
    delete new_cbd;
    return;
  }
  // Replace the old badge id with the new one.
  loc->set_bd(new_cbd);
  // Update dictionaries.
  my_badge_ids_to_locatables.remove(old_cbid);
  my_badge_ids_to_locatables.insertKeyAndValue(new_cbid, loc);

  // Notify MSP about the change.
  ABng_Core_System::Badge_Ids old_ids(1);
  ABng_Core_System::Badge_Ids new_ids(1);
  old_ids[0] = old_cbd->id;
  old_ids.length(1);
  new_ids.length(1);
  new_ids[0] = new_cbd->id;
  try {
    if (!CORBA::is_nil( (ABng_Core_Internals::Main_Sighting_Processor_ptr) 
			my_msp)) {
      my_msp->remove_badges(my_view_name, old_ids);
      my_msp->add_badges(my_view_name, new_ids);
    }
  }
  catch (...) {
  }
  delete old_cbd;
}

// Replace the local copy of the home sensor description with the current value.
void Locatable_Rep_I::replace_sensor(
  ABng_Core_System::Sensor_ptr sensor_p) {

  Comparable_Sensor_Descr *new_csd;
  Comparable_Sensor_Descr *old_csd;
  ABng_Core_System::Sensor::Description *new_sd;
  Comparable_Sensor_Id new_csid;
  Comparable_Sensor_Id old_csid;
  Locatable_I *loc;

  // Get the badge description.
  try {
    new_sd = sensor_p->descr();
  }
  catch(...) {
    return;
  }

  new_csd = new Comparable_Sensor_Descr(new_sd);
  new_csid = Comparable_Sensor_Id(new_csd->id);
  delete new_sd;

  // Find a locatable containing the home sensor
  if(!my_sensor_item_ids_to_locatables.findValue(new_csd->item_id, loc)) {
    // Remove the badge id from the dictionary
    delete new_csd;
    return;
  }
  // Find the old description of the updated badge.
  old_csd = loc->get_home_sd();
  if(!old_csd) {
    // There is no description for this sensor.
    // This is an abnormal situation and it should not happen.
    // But, well, let's do something with it. 
    my_badge_item_ids_to_locatables.remove(new_csd->item_id);
    delete new_csd;
    return;
  }

  old_csid = old_csd->id;
  if(old_csid == new_csid) {
    // The sensor has not changed its id. 
    delete new_csd;
    return;
  }
  // Replace the old sensor id with the new one.
  loc->set_home_sd(new_csd);
  delete old_csd;
}

// Replace the local copy of locatable's badge description list with its 
// current value. Notify the MSP if it is required.

void Locatable_Rep_I::update_badge(	    
  Locatable_I *item_p,
  Comparable_Badge_Descr *old_bd, 
  Comparable_Badge_Descr *new_bd, 
  int notify_msp) {

  // Remove the old badge.
  my_badge_item_ids_to_locatables.remove(old_bd->item_id);
  my_badge_ids_to_locatables.remove(
    Comparable_Badge_Id(old_bd->id));

  // Insert the new badge.
  my_badge_item_ids_to_locatables.insertKeyAndValue(new_bd->item_id, item_p);
  my_badge_ids_to_locatables.insertKeyAndValue(
    Comparable_Badge_Id(new_bd->id), item_p);

  
  // Notify MSP.
  if(notify_msp) {
    ABng_Core_System::Badge_Ids old_ids(1);
    ABng_Core_System::Badge_Ids new_ids(1);
    old_ids.length(1);
    new_ids.length(1);
    old_ids[0] = old_bd->id;
    new_ids[0] = new_bd->id;
    try{
      if (!CORBA::is_nil( (ABng_Core_Internals::Main_Sighting_Processor_ptr) 
			  my_msp)) {
	my_msp->remove_badges(my_view_name, old_ids);
	my_msp->add_badges(my_view_name, new_ids);
      }
    }
    catch (...) {
    }
  }
}

// Replace the local copy of locatable's home sensor description with its 
// current value. Notify the MSP if it is required.

void Locatable_Rep_I::update_sensor(	    
  Locatable_I *item_p,
  Comparable_Sensor_Descr *old_sd, 
  Comparable_Sensor_Descr *new_sd) {

  // Remove the old badge.
  my_sensor_item_ids_to_locatables.remove(old_sd->item_id);

  // Insert the new badge.
  my_sensor_item_ids_to_locatables.insertKeyAndValue(new_sd->item_id, item_p);
}






