// $Id: location_i.h,v 1.1.1.1 1998/06/18 13:37:37 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/sighting_processors/location_i.h,v $
//
// $Log: location_i.h,v $
// Revision 1.1.1.1  1998/06/18 13:37:37  mencnar
//
// Revision 1.1  1998/03/06 13:47:26  jasz
// Initial revision
//
//
// File:          location_i.h
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  26.01.98
//
// General Description:
//  Declaration the implementation class of the Location and
//  LSP interfaces. To improve performance the LSP interface is not 
//  implemented as a CORBA object but as a usual C++ object.

#ifndef __LOCATION_I_H__
#define __LOCATION_I_H__

#include <iostream.h>
#include <rw/tvhset.h>
#include <rw/tphset.h>

#ifdef ObjectStore
#include <oosa.h>
#endif

#include "abng_core_management.hh"
#include  "abng_guarded.hh"
#include "comparable_sensor_descr.h"
#include "compatibility.hh"


int operator >= ( const ABng_Core::Time &t1,
		  const ABng_Core::Time &t2);

class Locatable_I;

class Location_I {

public:

  typedef RWTPtrHashSet<Comparable_Sensor_Descr> Sensor_Descr_List;
  typedef RWTPtrHashSetIterator<Comparable_Sensor_Descr> 
    Sensor_Descr_List_Iterator;
 
  enum Event_Descriptor {
    CAME_IN,
    CAME_OUT
  };

  enum Resolution_Result {
    SUCCESS,
    RES_COMM_FAILURE,
    INCONSISTENCY,
    IGNORE
  };

  enum Refreshing_Result {
    COMM_FAILURE,
    CHANGED_EXTERNALS,
    CHANGED_INTERNALS,
    CHANGED_BOTH,
    UNCHANGED,
    IGNORE_REFRESH
  };

  // The cold start constructor.
  Location_I(
    ABng_Core_System::ABng_Location_Description_ptr abng_loc);

  // The constructor used to make a search pattern.
  // It should not be used to create real repository items.
  Location_I(Domain_Repository::Repository_Item_Id abng_loc_id_p);

  // The hot start constructor.
#ifdef FilePersistency
   Location_I(istream &iostr);
#endif

  virtual ~Location_I();

  virtual operator==(const Location_I& loc_p) {
    return my_abng_ld_item_id == loc_p.my_abng_ld_item_id;}

  // Operations of the Dynamic_Data_Observed_E interface:

  virtual ABng_Core::Dynamic_Data_Observed_E::Dynamic_Data_Observers* 
    dynamic_data_observer_list(
    ENV_DECL1(env) );

  virtual void attach_dynamic_data_observer(
    ABng_Core::Dynamic_Data_Observer_ptr obs
    ENV_DECL(env) );
                
  virtual void detach_dynamic_data_observer(
    ABng_Core::Dynamic_Data_Observer_ptr obs
    ENV_DECL(env) );

  // Operations of the View_Data_Observed_E interface:

  virtual ABng_Core::View_Data_Observed_E::View_Data_Observers* 
    view_data_observer_list(
    ENV_DECL1(env) );
                
  virtual void attach_view_data_observer(
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DECL(env) );

  virtual void detach_view_data_observer(
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DECL(env) );

  // Operations of the Location_Commander interface:

  virtual CORBA::Long issue_location_command(
    const ABng_Core::Octets& cmd,
    CORBA::Boolean has_reply,
    CORBA::Octet reply_code,
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env));
           
  virtual CORBA::Long issue_locatable_command(
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core::Octets& cmd, 
    CORBA::Boolean has_reply,
    CORBA::Octet reply_code,
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env));
           
  virtual CORBA::Long challenge(
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core::Octets& random, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env));
           
  virtual CORBA::Long sound(
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core::Octets& sound, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env));
           
  virtual CORBA::Long set_state(
    const ABng_Core::Locatables& locatable_list, 
    CORBA::Octet status, 
    const ABng_Core::Octets& sound, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env));
           
  virtual CORBA::Long get_state(
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env));
           
  virtual CORBA::Long set_home(
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core::Octets& address, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env));
           
  virtual CORBA::Long get_home(
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env));
           
  virtual CORBA::Long reset_buffer(
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env));
           
  virtual CORBA::Long reset_fifos(
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env));
           
  virtual CORBA::Long get_station_state(
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env));
           
  virtual CORBA::Long set_output(
   CORBA::Octet output_value, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env));
           
  virtual CORBA::Long set_persistence(CORBA::Boolean mode, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env));

  // Operations of Request_Queue:
  virtual ABng_Core::Request_Queue::Requests* list_user_requests(
    const char * user_id
    ENV_DECL( IT_env));

  virtual ABng_Core::Request_Queue::Requests* list_all_requests(
    ENV_DECL1( IT_env));

  virtual void remove_request(CORBA::Long request_id
    ENV_DECL( IT_env));

  // Operations of the Location interface:

  virtual Domain_Repository::Repository_Item_Id item_id(
    ENV_DECL1(env) ) {
    return my_descr.item_id;
  }

  virtual ABng_Core::Location::Description* descr(
    ENV_DECL1(env));
  virtual ABng_Core::Location::Value_Description* value_descr(
    ENV_DECL1( IT_env));

  virtual Domain_Repository::Location_Description_ptr loc_descr(
    ENV_DECL1(env));

  virtual ABng_Core::Sighting_Descriptions* get_sightings(
    const ABng_Core::Time& since
    ENV_DECL(env));

  // Operations of the LSP interface:

  /*
  virtual ABng_Core::Location_ptr my_location(
    CORBA::Environment &IT_env=CORBA::default_environment) 
    throw(CORBA::SystemException);

  virtual ABng_Core_System::ABng_Location_Description_ptr 
    my_abng_location_description(
    CORBA::Environment &IT_env=CORBA::default_environment) 
    throw(CORBA::SystemException);

  virtual ABng_Core_Internals::Locatable_Sighting_Processors* badge_sp_list(
    CORBA::Environment &IT_env=CORBA::default_environment) 
    throw(CORBA::SystemException);
  
  virtual void process_sighting(
    ABng_Core_Internals::Locatable_Sighting_Processor_ptr locatable_sp, 
    ABng_Core_Internals::Location_Sighting_Processor::Event_Descriptor event, 
    CORBA::Environment &IT_env=CORBA::default_environment) 
    throw(CORBA::SystemException);
  */

  virtual void process_sighting(
    Locatable_I *locatable_sp, 
    Event_Descriptor event
    ENV_DECL(env));

  // Non-IDL public methods.

  // Accessors:
  virtual void set_descr(
    const ABng_Core::Location::Description& descr_update);

  const ABng_Core::Location::Description& get_descr() const {return my_descr;}

  virtual void set_corba_object(
    ABng_Core::Location_ptr location_corba_object_p) {
    my_location_corba_object = location_corba_object_p;
  }

  ABng_Core::Location_ptr get_corba_object() {
    return my_location_corba_object;
  }

  /*
  virtual void set_lsp_corba_object(
    ABng_Core_Internals::Location_Sighting_Processor_ptr 
    lsp_corba_object_p) {
    my_lsp_corba_object = lsp_corba_object_p;
  }

  ABng_Core_Internals::Location_Sighting_Processor_ptr 
    get_lsp_corba_object() {
    return ABng_Core_Internals::Location_Sighting_Processor::_duplicate(
      my_lsp_corba_object);
  }
  */

  virtual void set_item_id(
    const Domain_Repository::Repository_Item_Id item_id_p) {
    my_descr.item_id = item_id_p;
  }

  virtual ABng_Core_System::ABng_Location_Description_var get_abng_ld() {
    return my_abng_ld;
  }

  virtual void set_abng_ld(
    ABng_Core_System::ABng_Location_Description_var abng_ld_p) {
    my_abng_ld = abng_ld_p;
  }

  virtual Domain_Repository::Repository_Item_Id get_abng_ld_item_id() const {
    return my_abng_ld_item_id;
  }

  virtual Sensor_Descr_List *get_sds() {
    return &my_sds;
  }

  // Detach the locatable from the location.
  virtual void remove_locatable(const Locatable_I *locatable_p) {
    my_locatables.remove(locatable_p);
  }

  // Remove the sensor description
  virtual void remove_sd(const Comparable_Sensor_Descr *sd_p) {
    Comparable_Sensor_Descr *res = my_sds.remove(sd_p);
    if(res) delete res;
  }
 
  // Insert the sensor description
  virtual void insert_sd(Comparable_Sensor_Descr *sd_p) {
    my_sds.insert(sd_p);
  }

  // Find the sensor description
  virtual Comparable_Sensor_Descr *find_sd(Comparable_Sensor_Descr *sd_p) {
    return my_sds.find(sd_p);
  }

  ABng_Core::Locations create_location_list();

#ifdef ObjectStore
  // method for ObjectStore
  static os_typespec* get_os_typespec();
#elif defined FilePersistency
  // methods to store and restore object of this class
  virtual void save(ostream &iostr);
  virtual void restore(istream &iostr);
#endif

  // Refresh all the information that could have changed in the 
  // corresponding ABng_LD.
  virtual Resolution_Result resolve_ids(
    ABng_Core_Management::ABng_Location_Description_Rep_ptr abng_ld_rep_p);

  // Refresh the location attributes corresponding to information
  // from other repositories (ABng_LD_Rep, Sensor_Rep) that could have 
  // changed. If the sensor list associated with the location has been
  // updated return both the old and the new sensor lists.
  virtual Refreshing_Result refresh(
    Sensor_Descr_List& new_sds, Sensor_Descr_List& old_sds);

protected:

  typedef RWTPtrHashSet<Locatable_I> Locatable_List;
  typedef RWTPtrHashSetIterator<Locatable_I> Locatable_List_Iterator;


  ABng_Core::Location_ptr my_location_corba_object;
  // ABng_Core_Internals::Location_Sighting_Processor_var my_lsp_corba_object;

  ABng_Core::Location::Description my_descr;
  ABng_Core_System::ABng_Location_Description_var my_abng_ld;
  Domain_Repository::Repository_Item_Id my_abng_ld_item_id;
  Sensor_Descr_List my_sds;
  // The list of locatables currently seen in the location.
  Locatable_List my_locatables;
};

#endif  /* __LOCATION_H__ */


