// $Id: location_rep_i.cc,v 1.2 1998/07/29 14:57:08 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/sighting_processors/location_rep_i.cc,v $
//
// $Log: location_rep_i.cc,v $
// Revision 1.2  1998/07/29 14:57:08  mencnar
// performance tuning,
// receive sightings calls moved to dyn_dispatcher.* for future optimisation
//
// Revision 1.1.1.1  1998/06/18 13:37:37  mencnar
//
// Revision 1.1  1998/03/06 13:41:02  jasz
// Initial revision
//
//
// File:          location_rep_i.cc
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  26.01.98
//
// General Description:
//  Definition of the implementation class for the "location part" of the View,
//  VSP and View_Manager interfaces.

#define EXCEPTIONS

#include <rw/rwdate.h>
#include <rw/rwdate.h>
#include <rw/tvdlist.h>
#include <rw/rwstring.h>
#include "location_rep_i.h"
#include "locatable_rep_i.h"
#include "locatable_i.h"
#include "view_i.h"
#include "dyn_dispatcher.hh"

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_ABng_Core_Location(Location_I)
  //DEF_ObjectStoreTIE_ABng_Core_Internals_Location_Sighting_Processor(Location_I)
#else
DEF_TIE_ABng_Core_Location(Location_I)
  //DEF_TIE_ABng_Core_Internals_Location_Sighting_Processor(Location_I)
#endif
#endif

// Hot start constructor
Location_Rep_I::Location_Rep_I(
#ifdef ObjectStore
  os_database* db_p,
#elif defined FilePersistency
  const char *repository_name_p,
#endif
  unsigned long burst_delay
  ) : No_Update_Basic_Rep_I<L_Types, VDO_Types>(hash_location,
#ifdef ObjectStore
    db_p
#elif defined FilePersistency
    repository_name_p
#endif
  ),
  my_abng_ld_rep(ABng_Core_Management::ABng_Location_Description_Rep::_nil()),
  my_msp(ABng_Core_Internals::Main_Sighting_Processor::_nil()),
  my_dynamic_observed(
#ifdef FilePersistency
    (const char *) (RWCString( (const char *) repository_name_p)+".dynamic"),
#endif
    burst_delay
  ),
  my_sensor_ids_to_locations(Comparable_Sensor_Id::hash),
  my_sensor_item_ids_to_locations(hash_item_id),
  my_view_name(CORBA::string_dup("")),
  my_location_type_name(CORBA::string_dup(""))  {

  _instance = this;
}

// Cold start constructor
Location_Rep_I::Location_Rep_I(
#ifdef ObjectStore
  os_database* db_p,
#elif defined FilePersistency
  const char *repository_name_p,
#endif
  const char *view_name_p,
  const char *location_type_name_p,
  unsigned long burst_delay
  ) : No_Update_Basic_Rep_I<L_Types, VDO_Types>(hash_location,
#ifdef ObjectStore
    db_p
#elif defined FilePersistency
    repository_name_p
#endif
  ),
  my_abng_ld_rep(ABng_Core_Management::ABng_Location_Description_Rep::_nil()),
  my_msp(ABng_Core_Internals::Main_Sighting_Processor::_nil()),
  my_dynamic_observed(
#ifdef FilePersistency
    (const char *) (RWCString( (const char *) repository_name_p)+".dynamic"),
#endif
    burst_delay
  ),
  my_sensor_ids_to_locations(Comparable_Sensor_Id::hash),
  my_sensor_item_ids_to_locations(hash_item_id),
  my_view_name(CORBA::string_dup("")),
  my_location_type_name(CORBA::string_dup("")) {

  _instance = this;
  if(view_name_p) {
    my_view_name = CORBA::string_dup(view_name_p);
  }
  if(location_type_name_p) {
    my_location_type_name = CORBA::string_dup(location_type_name_p);
  }
}

Location_Rep_I::~Location_Rep_I() {}

Location_Rep_I *Location_Rep_I::_instance = 0;

Location_Rep_I *Location_Rep_I::instance() {

  return _instance;
}

ABng_Core_System::ABng_Location_Description_ptr Location_Rep_I::deref(
    ABng_Core::Location_ptr lct_object
    ENV_DEF(env)) {

  Location_I *lct_i= deref_item( lct_object);
  if (lct_i) 
    return ABng_Core_System::ABng_Location_Description::_duplicate( 
							lct_i->get_abng_ld());
}


// The implementation of the notify IDL operation of 
// the System_Static_Data_Observer interface.

void Location_Rep_I::notify(
  Domain_Repository::Repository_Item_Id id,
  const ABng_Core_System::System_Static_Data_Observer::System_Static_Data_Object& sd, 
  Domain_Repository::Change_Type change
  ENV_DEF(env) ) {
    
#ifdef DEBUG
  cerr << "Location_Rep_I::notify" << endl;
#endif

  // Determine what has been changed.
  switch(sd._d()) {
    case (ABng_Core_System::System_Static_Data_Observer::ABNG_LOCATION_DESCRIPTION_SD) : {
      // Something has changed in ABng_LD_Rep. 
      // Find a location corresponding to the changed ABng_LD.
      Location_I location_pattern(id);
      Location_I *loc;
      if(!(loc = my_item_list.find(&location_pattern))) {
	// There is no location corresponding to that ABng_LD.
	break;
      }
      // Determine a change kind.
      switch(change) {
        case Domain_Repository::OBJECT_REMOVED : {
	  // Remove the location corresponding to the removed ABng_LD.
	  private_remove(loc->get_corba_object());
	  break;
	}
        case Domain_Repository::OBJECT_UPDATED : {
	  // Some attributes of the corresponding ABng_LD have changed.
	  // Check which of them has been modified and take 
	  // an appropriate action.
	  refresh(loc);
	  break;
	}
        case Domain_Repository::OBJECT_ADDED : {
	  // The ADD_OBJECT event is ignored.
	}
        default : {
	  break;
	}
      }
      break;
    }

    case (ABng_Core_System::System_Static_Data_Observer::SENSOR_SD) : {
      // Something in Sensor Rep has changed.
      // Determine a change kind.
      switch(change) {
        case Domain_Repository::OBJECT_UPDATED : {
	  // The id of one of the sensors has been modified.
	  // Update your database.
	  replace_sensor(sd.sensor_obj());
	  break;
	}
        default : {
	  // Other events are ignored.
	  break;
	}
      }
      break;
    }
    default : {
      break;
    }
  }
}

// Get a location containing the sensor with the given id.

Location_I *Location_Rep_I::resolve_sensor(
  const ABng_Core_System::Sensor_Id& sid) {

  Comparable_Sensor_Id csid(sid);
  Location_I *res= NULL; // !! initialize the variable
  my_sensor_ids_to_locations.findValue(csid, res);
  return res;
}

// Notify dynamic data observers registered with the new or/and old location
// of the given locatable about a change of sighting data.

void Location_Rep_I::dynamic_notify(Location_I *new_location, 
  Location_I *old_location, Locatable_I *locatable_p) {

  auto_lock local_lock( *this);

  // Get a list of new location DDOs.
  Basic_Observed_I<DDO_Types>::Observer_List *new_location_observers = 
    my_dynamic_observed.get_item_observer_list(new_location->item_id());

  my_dynamic_observed.dynamic_notify_observers( *new_location_observers,
						locatable_p->local_get_sighting());

  if(old_location && old_location != new_location) {
    // The old location is given and it is different than the new one.
    Basic_Observed_I<DDO_Types>::Observer_List* old_location_observers = 
      my_dynamic_observed.get_item_observer_list(old_location->item_id());

    Basic_Observed_I<DDO_Types>::Observer_List old_obs= *old_location_observers;

    // Get a list of old location DDOs.
    Basic_Observed_I<DDO_Types>::Observer_List_Iterator
      old_iterator(old_obs);
    
    ABng_Core::Dynamic_Data_Observer_ptr obj_to_remove;
    int do_remove= 0;

    while (old_iterator()) {
      if(new_location_observers->contains(old_iterator.key())) {
	do_remove= 1;
	obj_to_remove= old_iterator.key();
      }
    }
    if (do_remove)
      old_obs.remove( obj_to_remove);

    my_dynamic_observed.dynamic_notify_observers( old_obs,
						  locatable_p->local_get_sighting());
  }
}

// Detach locatable from its current location.

void Location_Rep_I::remove_locatable(const Locatable_I *locatable_p) {

  Item_List_Iterator iterator(my_item_list);
  while(iterator()) {
    iterator.key()->remove_locatable(locatable_p);
  }
}

// Refresh all the information that could have changed in the ABng_LD_Rep.
// This method should be called after the ABng_LD_Rep is launched.

void Location_Rep_I::resolve_abng_ld_rep(
    ABng_Core_Management::ABng_Location_Description_Rep_var abng_ld_rep_p) {

#ifdef DEBUG
  cerr << "Location_Rep_I::resolve_abng_ld_rep (1)" << endl;
#endif
  if(CORBA::is_nil(
    (ABng_Core_Management::ABng_Location_Description_Rep_ptr)
    abng_ld_rep_p)) {
    return;
  }
#ifdef DEBUG
  cerr << "Location_Rep_I::resolve_abng_ld_rep (2)" << endl;
#endif

  // Update your copy of ABng_LD_Rep reference.
  my_abng_ld_rep = abng_ld_rep_p;

  ABng_Core_System::ABng_Location_Description::Entity_Set es;
  {
    Item_List_Iterator iterator(my_item_list);

    int len= my_item_list.entries();
    Domain_Repository::Repository_Item_Ids ids(len);
    ids.length(len);
  
    for (int i=0; iterator(); i++)
      ids[i]= iterator.key()->get_abng_ld_item_id();
    
    es.s_ids(ids);
  }
  Domain_Repository::Query_Options qo;
  qo.return_references= 0;
  qo.return_descriptions= 0;
  qo.return_value_descriptions= 0;
  qo.do_attach= 1;
  qo.do_detach= 0;
  ABng_Core_System::ABng_Location_Description::Query_Results_var qr;
  try {
    qr= my_abng_ld_rep->complex_query( es, qo, View_I::instance()->get_ssdo_ptr());
  }
  catch (...) {
#ifdef DEBUG
    cerr << "Location_Rep_I::resolve_abng_ld_rep complex_query exception" << endl;
#endif
    return;
  }

  Item_List_Iterator iterator(my_item_list);
  int stop = 0;
  int ref_res = 0;
  // Assume the eager MSP notification mechanism.
  // MSP will be notified immediatelly after a discovery that a sensor list 
  // of a locatable has  changed.
  int lazy_msp_notif = 0;
  if(my_item_list.entries() == 0) {
    // There is no item locations in the repository.
    // This means that MSP will be provided with sensor lists of 
    // all locations. To improve this assume the lazy MSP notification 
    // mechanism. The MSP will be notified only once after all local copies 
    // od sensor lists are updated.
    lazy_msp_notif = 1;
  }

#ifdef DEBUG
  cerr << "Location_Rep_I::resolve_abng_ld_rep ...attached" << endl;
#endif

  // Go through the location list.
  while(iterator() && !stop) {

    // Convert item foreign item ids stored in the location into references.
    Location_I::Resolution_Result res = 
      iterator.key()->resolve_ids(my_abng_ld_rep);
#ifdef DEBUG
    cerr << "Location_I::resolve_ids result " << (int) res << endl;
#endif
    switch(res) {
      case Location_I::SUCCESS : {
	// Convertion has succeeded. Refresh the location. 
        // Notify the MSP if required and necessary.
	ref_res = refresh(iterator.key(), !lazy_msp_notif);
	switch(ref_res) {
	  case Location_I::COMM_FAILURE : {
	    // Something went wrong. Assume the same problem will acour 
	    // for the next locations again and leave the loop.
	    stop = 1;
	    break;
	  }
	  default : {
	    break;
	  }
	}
	break;
      }
      case Location_I::RES_COMM_FAILURE : {
	// Something went wrong during convertion. Assume the same problem 
	// will acour for the next locations again and leave the loop.
	stop = 1;
	break;
      }
      case Location_I::INCONSISTENCY : {
	// The corresponding ABng_LD has been removed.
	// Remove the location.
	private_remove(iterator.key()->get_corba_object());
	break;
      }
      default : {
	break;
      }
    }
  }
  if(lazy_msp_notif) {
    // Update the sensor list of the view in the MSP.
    ABng_Core_System::Sensor_Ids empty_sids(0);
    empty_sids.length(0);
    ABng_Core_System::Sensor_Ids_var sids = get_sensor_list();
    try {
      // Remove all sensors currently registered in MSP.
      my_msp->remove_sensors(my_view_name, empty_sids);
      // Register new sensors.
      my_msp->add_sensors(my_view_name, *(sids.operator->()));
    }
    catch(...) {
    }
  }
}

void Location_Rep_I::resolve_sensor_rep(
  ABng_Core_Management::Sensor_Rep_var sensor_rep_p) {

  if(CORBA::is_nil(
    (ABng_Core_Management::Sensor_Rep_ptr) sensor_rep_p)) {
    return;
  }
  // Update your copy of ABng_LD_Rep reference.
  my_sensor_rep = sensor_rep_p;
  
  // we don't need to refresh information about sensors
  // as such an update should be on the way from abng_location_descriptions rep.
  //

  Item_List_Iterator iterator(my_item_list);
  int stop= 0;
  while(iterator() && !stop) {
    try {
      Sensor_Descr_List *sds= iterator.key()->get_sds();
      Domain_Repository::Repository_Item_Ids rids( sds->entries());
      rids.length( sds->entries());
      int i=0;

      for ( Sensor_Descr_List_Iterator iter( *sds);
	    iter(); )
	rids[ i++]= iter.key()->item_id;

      my_sensor_rep->attach_system_static_data_observer( 
		       View_I::instance()->get_ssdo_ptr(),
		       rids);
    }
    catch (CORBA::COMM_FAILURE) {
      stop= 1;
      break;
    }
  }
}

void Location_Rep_I::resolve_msp(
  ABng_Core_Internals::Main_Sighting_Processor_var msp_p) {

  // Just update the local copy of the MSP reference.
  my_msp = msp_p;
}

ABng_Core_System::Sensor_Ids *Location_Rep_I::get_sensor_list() {

  // Construct a sequence containing all sensors belonging to the view.
  int len = my_sensor_ids_to_locations.entries();
  ABng_Core_System::Sensor_Ids *res = 
    new ABng_Core_System::Sensor_Ids(len);
  res->length(len);
  Sensor_Id_Location_Dictionary_Iterator 
    iterator(my_sensor_ids_to_locations);
  for(int i = 0; i < len, iterator(); i++) {
    (*res)[i] = iterator.key();
  }
  return res;
}


ABng_Core_System::Sensor_Ids Location_Rep_I::locations2sensor_ids(
    const ABng_Core::Locations *locs) {

  ABng_Core::Locations_var auto_obj;

  if ( locs->length() == 0) {
    auto_obj= (ABng_Core::Locations *) (locs= location_list());
    // empty list of locations -> all locations in the view
  }

  ABng_Core_System::Sensor_Ids ret;
  int ret_ptr= 0;

  for ( int i=0; i<locs->length(); i++) {
    Location_I *location_i= deref_item( (*locs)[i]);
    if (!location_i) break;
    
    Sensor_Descr_List *sds= location_i->get_sds();
    ret.length( ret.length() + sds->entries());
    for ( Sensor_Descr_List_Iterator iter( *sds);
	  iter(); )
      ret[ ret_ptr++]= iter.key()->id;
  }
  return ret;
}

ABng_Core::Location_var Location_Rep_I::sensor_id2location(
    const ABng_Core_System::Sensor_Id &sid) 
  throw( ABng_Core::Invalid_Argument) {

  Location_I *location_i= resolve_sensor( sid);
  if (! location_i) throw ABng_Core::Invalid_Argument();
#ifdef DEBUG
#ifdef ORBIX
  cerr << "location ref_count " << location_i->get_corba_object()->_refCount() << endl;
#endif
#endif
  return ABng_Core::Location::_duplicate( location_i->get_corba_object());
}

ABng_Core::Locations Location_Rep_I::sensor_ids2locations(
    const ABng_Core_System::Sensor_Ids *sids) {

  ABng_Core::Locations ret;

  for ( int i=0; i<sids->length(); i++) {
    
    Location_I *location_i= resolve_sensor( (*sids)[i]);
    if (! location_i) break; // any other ideas ?

    ABng_Core::Location_ptr location_obj= location_i->get_corba_object();
    int j;
    for ( j=0; j<ret.length(); j++)
      if (ret[j] == location_obj) break;
    if ( j == ret.length()) break; /* duplicated location object */

    ret.length( ret.length()+1);
    ret[ ret.length()-1]= ABng_Core::Location::_duplicate( location_obj);
  }
  return ret;
}

Location_I * Location_Rep_I::locatable_came_in( 
                   Domain_Repository::Repository_Item_Id rid,
		   Locatable_I *locatable) {

  Location_I *location= my_item_deref[ rid];
  if ( !location) return NULL;
  location->process_sighting( locatable, 
			      Location_I::CAME_IN);
  return location;
}

// Calculate the hash value of the location.
unsigned Location_Rep_I::hash_location(const Location_I& location_i_p) {

  return location_i_p.get_abng_ld_item_id();
}

// Add helper methods

void Location_Rep_I::init_corba_object(Location_I *item_p) {

  // Originally, a location provided two IDL interfaces: Location and LSP
  // This has been changed. To improve performance LSP is not implemented 
  // as a CORBA object.
#ifdef ORBIX
#ifdef ObjectStore
  ObjectStoreTIE_ABng_Core_Location(Location_I)* 
    new_location_type_tie =
    new(my_db, ObjectStoreTIE_ABng_Core_Location(
      Location_I)::get_os_typespec()) 
      ObjectStoreTIE_ABng_Core_Location(
        Location_I
	) (item_p);
  /*  ObjectStoreTIE_ABng_Core_Internals_Location_Sighting_Processor(Location_I)* 
    new_lsp_type_tie =
    new(my_db, ObjectStoreTIE_ABng_Core_Internals_Location_Sighting_Processor (
      Location_I)::get_os_typespec()) 
      ObjectStoreTIE_ABng_Core_Internals_Location_Sighting_Processor(
        Location_I
	) (item_p);
	*/
#else
  TIE_ABng_Core_Location(Location_I) *
    new_location_tie = 
    new TIE_ABng_Core_Location(Location_I) (item_p);
  /*
  TIE_ABng_Core_Internals_Location_Sighting_Processor(Location_I) *
    new_lsp_tie = 
    new TIE_ABng_Core_Internals_Location_Sighting_Processor(Location_I) (
      item_p);
  */
#endif
#elif defined OMNIORB
  ABng_Core::Location_ptr new_location_tie=
    new TIE_CLASS(ABng_Core,Location)<Location_I,0>(item_p);
  CONNECT_IMPL( ABng_Core::Location_var, new_location_tie);
#endif
  item_p->set_corba_object(new_location_tie);
  /*
  item_p->set_lsp_corba_object(new_lsp_tie);
  */
}

void Location_Rep_I::insert_item(Location_I* item_p) {

  // Call the "basic" insert.
  No_Update_Basic_Rep_I<L_Types, VDO_Types>::insert_item(item_p);

  // Update the dynamic data observer registry.
  my_dynamic_observed.insert_new_item(item_p->item_id());

  // Get the list of sensors associated with the new location.
  Location_I::Sensor_Descr_List *sds = item_p->get_sds();
  Location_I::Sensor_Descr_List_Iterator iterator(*sds);

  // Update both sensor dictionaries.
  ABng_Core_System::Sensor_Ids sids(sds->entries());
  sids.length(sds->entries());
  int i = 0;
  while(iterator()) {
    my_sensor_ids_to_locations.insertKeyAndValue(
      Comparable_Sensor_Id(iterator.key()->id), item_p);
    my_sensor_item_ids_to_locations.insertKeyAndValue(
      iterator.key()->item_id, item_p);
    sids[i++] = iterator.key()->id;
  }
  // Notify MSP about adding new sensors.
  try {
    if (!CORBA::is_nil( (ABng_Core_Internals::Main_Sighting_Processor_ptr) 
			my_msp))
      my_msp->add_sensors(my_view_name,  sids);
  }
  catch(...) {
  }
  try {
    Domain_Repository::Repository_Item_Ids ids( 1);
    ids.length( 1);
    ids[0]= item_p->get_abng_ld_item_id();
    if (!CORBA::is_nil( (ABng_Core_Management::ABng_Location_Description_Rep_ptr) my_abng_ld_rep))
      my_abng_ld_rep->attach_system_static_data_observer( 
		       View_I::instance()->get_ssdo_ptr(),
		       ids);
  }
  catch (CORBA::Exception &ex) {
#ifdef DEBUG
    cerr << "Location_Rep_I::insert_item " << ex << endl;
#endif
  }
}

// Remove helper methods

void Location_Rep_I::remove_item(Location_I *item_p) {

  // Call the "basic" remove.
  No_Update_Basic_Rep_I<L_Types, VDO_Types>::remove_item(item_p);

  // Update the dynamic data observer registry.
  my_dynamic_observed.detach_item_from_observers(item_p->item_id());

  // Get the list of sensors associated with the new location.
  Location_I::Sensor_Descr_List *sds = item_p->get_sds();
  Location_I::Sensor_Descr_List_Iterator iterator(*sds);

  // Update both sensor dictionaries.
  ABng_Core_System::Sensor_Ids sids(sds->entries());
  int i = 0;
  while(iterator()) {
    my_sensor_ids_to_locations.remove(
      Comparable_Sensor_Id(iterator.key()->id));
    my_sensor_item_ids_to_locations.remove(
      iterator.key()->item_id);
    sids[i++] = iterator.key()->id;
  }
  sids.length(sds->entries());
  // Notify MSP about removing sensors.
  try {
    if (!CORBA::is_nil( (ABng_Core_Internals::Main_Sighting_Processor_ptr) 
			my_msp))
      my_msp->remove_sensors(my_view_name,  sids);
  }
  catch(...) {
  }
  Locatable_Rep_I::instance()->remove_location(item_p);
}

// Find helper methods

ABng_Core::Locations* 
  Location_Rep_I::private_find (
  const ABng_Core::Location::Pattern& data_record_pattern
  ENV_DEF( env)) {

#ifdef DEBUG
  cerr << "Location_Rep_I::private_find; is_any = " 
       << (int) data_record_pattern.is_any << endl;
#endif

  MatchLocationApplyDataStruct apply_data( data_record_pattern);

  my_item_list.apply( match_location, (void*)&apply_data);

  // remote find will go here (someday); let's do it today: done today
  if (apply_data._found_locations.entries() &&
      (!data_record_pattern.is_any)) // not empty
    if (! data_record_pattern.loc_descr_pattern.is_any) { 
      // remote pattern really matters

      if (CORBA::is_nil( (ABng_Core_Management::ABng_Location_Description_Rep_ptr) 
			 my_abng_ld_rep)) 
	throw Domain_Repository::Destination_Unreachable("abng_location_description_rep");
      try {
	ABng_Core_System::ABng_Location_Description::Pattern abng_ld_pattern;
	abng_ld_pattern.is_any= 0;
	abng_ld_pattern.item_id_pattern.is_any= 1;
	abng_ld_pattern.loc_descr_pattern= data_record_pattern.loc_descr_pattern;
	abng_ld_pattern.sensor_list_pattern.is_any= 1;

	ABng_Core_System::ABng_Location_Description::Value_Descriptions *
	  ret_val( my_abng_ld_rep->find_values( abng_ld_pattern));
	ABng_Core_System::ABng_Location_Description::Value_Descriptions_var
	  auto_ret_val( ret_val);
	
	for ( RWTValDlistIterator<Location_I*> iter( apply_data._found_locations);
	      iter(); ) {
	  
	  int i;
	  Location_I * it= iter.key();

	  for (i=0; i<ret_val->length(); i++)
	    if ( it->get_abng_ld_item_id() ==
		 (*ret_val)[i].item_id) break;

	  if (i==ret_val->length()) // no corresponding item found
	    iter.remove(); // remove current item
	}
      }
      catch (CORBA::Exception &ex) {
#ifdef DEBUG
	cerr << "Location_Rep_I::private_find " << ex << endl;
#endif
	throw Domain_Repository::Destination_Unreachable("abng_location_description_rep");	
      }	
    }
  //  ----

  ABng_Core::Locations *ret= 
    new ABng_Core::Locations( apply_data._found_locations.entries());
  ret->length( apply_data._found_locations.entries());
#ifdef DEBUG
  cerr << "Location_Rep_I::private_find; found "
       << (int) apply_data._found_locations.entries() 
       << " location(s)" << endl;
#endif
  int pos= 0;
  for ( RWTValDlistIterator<Location_I*> iter( apply_data._found_locations);
	iter(); 
	pos++)
    (*ret)[pos]= ABng_Core::Location::_duplicate( iter.key()->get_corba_object());
  return ret;
}

void Location_Rep_I::match_location( Location_I *current_location,
				     void *_apply_data) {

  MatchLocationApplyDataStruct &apply_data= 
    *(MatchLocationApplyDataStruct*)_apply_data;

  if (apply_data._data_record_pattern.is_any) {
    apply_data._found_locations.insert( current_location);
    return;
  }
  if (!apply_data._data_record_pattern.item_id_pattern.is_any &&
       apply_data._data_record_pattern.item_id_pattern.value != 
          current_location->get_descr().item_id) return;

  apply_data._found_locations.insert( current_location);
}

void Location_Rep_I::collect_values(Location_I *lt, 
  ABng_Core::Location::Value_Description& ld) {

  ABng_Core::Location::Value_Description_var val_descr= lt->value_descr();
  ld= *(val_descr.operator->());
}

// Common helper methods

void Location_Rep_I::construct_data_object(
  ABng_Core::View_Data_Observer::View_Data_Object& data_object_p,
  Location_ptr item_p) {

  // construct union to store reference of added entity
  data_object_p.location_obj(
    ABng_Core::Location::_duplicate(item_p));
}

void Location_Rep_I::save() {

  No_Update_Basic_Rep_I<L_Types, VDO_Types>::save();
}

void Location_Rep_I::save(ostream &str) {

  No_Update_Basic_Rep_I<L_Types, VDO_Types>::save(str);
  str << my_view_name << endl;
  str << my_location_type_name;
}

void Location_Rep_I::restore(istream &str) {

  No_Update_Basic_Rep_I<L_Types, VDO_Types>::restore(str);
  str >> my_view_name;
  str >> my_location_type_name;
}

DB_Header * Location_Rep_I::create_db_header() {

  // Yes, this constructs the wrong DB_Header with a duplicated description
  // section, but the second description is correct, at least.
  // Something in the DB_Header code should be changed.
  return new DB_Header( "Location Repository", "1.0", "\
#       <file>::=       <next_item_id>\n\
#                       <number_of_items>\n\
#                       <data_records>\n\
#                       <view_name>\n\
#                       <location_type_name>\n\
#\n\
#       <data_records>::=  | <data_record> <data_records>\n\
#\n\
#       <data_record>::=        <repository_item_id>\n\
#                               <abng_location_item_id>\n\
#\n\
#\n", 1);
}

// Refresh the location attributes representing information
// from other repositories (ABng_LD_Rep, Sensor_Rep) that could have 
// changed. If notify_msp is non-zero the MSP is immediately notified 
// if a sensor list installed in the location is chaned.

Location_I::Refreshing_Result Location_Rep_I::refresh(Location_I *item_p,
  int notify_msp) {

  // Ask the location to contact the corresponding ABng_LD, to determine
  // the set of relevant information which have been changed
  // and to refresh its attributes.
  Location_I::Sensor_Descr_List new_sds(Comparable_Sensor_Descr::hash);
  Location_I::Sensor_Descr_List old_sds(Comparable_Sensor_Descr::hash);

  Location_I::Refreshing_Result ref_res = 
    item_p->refresh(old_sds, new_sds);

  switch(ref_res) {
    case Location_I::CHANGED_INTERNALS : {
      // The location sensor list has been updated.
      // Update the database. Notify MSP if required.
      update_sensors(item_p, old_sds, new_sds, notify_msp);
      break;
    }
    case Location_I::CHANGED_EXTERNALS : {
      // The reference of the LD has been updated.
      // Notify view data observers.
      Observer_List *observer_list_p = 
	get_item_observer_list(item_p->item_id());
      notify_observers(*observer_list_p, item_p, 
        Domain_Repository::OBJECT_UPDATED);
      break;
    }
    case Location_I::CHANGED_BOTH: {
      // Both the list of sensors and the LD referece have been updated.
      // Update the database. Notify MSP if required.
      update_sensors(item_p, old_sds, new_sds, notify_msp);
      // Notify observers.
      Observer_List *observer_list_p = 
	get_item_observer_list(item_p->item_id());
      notify_observers(*observer_list_p, item_p, 
        Domain_Repository::OBJECT_UPDATED);
      break;
    }
    case Location_I::COMM_FAILURE : {
      // The communication error has occured when refreshing attributes.
      break;
    }
    case Location_I::UNCHANGED : {
      // Nothing has changed.
      break;
    }
    default : {
      break;
    }
  }
  return ref_res;
}          

// Replace the local copy of the sensor description with the current value.
void Location_Rep_I::replace_sensor(
  ABng_Core_System::Sensor_ptr sensor_p) {

  Comparable_Sensor_Descr *new_csd;
  Comparable_Sensor_Descr *old_csd;
  ABng_Core_System::Sensor::Description *new_sd;
  Comparable_Sensor_Id new_csid;
  Comparable_Sensor_Id old_csid;
  Location_I *loc;

  // Get the sensor description.
  try {
    new_sd = sensor_p->descr();
  }
  catch(...) {
    return;
  }

  new_csd = new Comparable_Sensor_Descr(new_sd);
  new_csid = Comparable_Sensor_Id(new_csd->id);
  delete new_sd;

  // Find a location containing the sensor
  if(!my_sensor_item_ids_to_locations.findValue(new_csd->item_id, loc)) {
    // Remove the sensor id from the dictionary
    my_sensor_ids_to_locations.remove(new_csid);
    delete new_csd;
    return;
  }
  // Find the old description of the updated sensor.
  old_csd = loc->find_sd(new_csd);
  if(!old_csd) {
    // There is no description for this sensor.
    // This is an abnormal sitruation and it should not happen,
    // i.e. this means that there is an inconsistency between the dictionary
    // the location's sensor list. But, well, let's do something with it. 
    my_sensor_item_ids_to_locations.remove(new_csd->item_id);
    my_sensor_ids_to_locations.remove(new_csid);
    delete new_csd;
    return;
  }

  old_csid = old_csd->id;
  if(old_csid == new_csid) {
    // The sensor has not changed its id. 
    // This is also very unlikely to happen.
    delete new_csd;
    return;
  }
  // Replace the old sensor id with the new one.
  loc->remove_sd(new_csd);
  loc->insert_sd(new_csd);
  // Update dictionaries.
  my_sensor_ids_to_locations.remove(old_csid);
  my_sensor_ids_to_locations.insertKeyAndValue(new_csid, loc);

  // Notify MSP about the change.
  ABng_Core_System::Sensor_Ids old_ids(1);
  ABng_Core_System::Sensor_Ids new_ids(1);
  old_ids.length(1);
  new_ids.length(1);
  old_ids[0] = old_csd->id;
  new_ids[0] = new_csd->id;
  try {
    if (!CORBA::is_nil( (ABng_Core_Internals::Main_Sighting_Processor_ptr) 
			my_msp)) {
      my_msp->remove_sensors(my_view_name, old_ids);
      my_msp->add_sensors(my_view_name, new_ids);
    }
  }
  catch (...) {
  }
  delete old_csd;
}

// Replace the local copy of location's sensor description list with its 
// current value. Notify the MSP if it is required.

void Location_Rep_I::update_sensors(	    
    Location_I *item_p,
    Location_I::Sensor_Descr_List& old_sds, 
    Location_I::Sensor_Descr_List& new_sds,
    int notify_msp) {

  int i;
  Location_I::Sensor_Descr_List_Iterator old_iterator(old_sds);
  Location_I::Sensor_Descr_List_Iterator new_iterator(new_sds);
  ABng_Core_System::Sensor_Ids old_sids(old_sds.entries());
  ABng_Core_System::Sensor_Ids new_sids(new_sds.entries());
  old_sids.length( old_sds.entries());
  new_sids.length( new_sds.entries());
  i = 0;
  // Remove old sensors
  while(old_iterator()) {
    my_sensor_item_ids_to_locations.remove(old_iterator.key()->item_id);
    my_sensor_ids_to_locations.remove(
      Comparable_Sensor_Id(old_iterator.key()->id));
    old_sids[i++] = old_iterator.key()->id;
  }
  i = 0;
  // Insert new sensors
  while(new_iterator()) {
    my_sensor_item_ids_to_locations.insertKeyAndValue(
      new_iterator.key()->item_id, item_p);	
    my_sensor_ids_to_locations.insertKeyAndValue(
      Comparable_Sensor_Id(new_iterator.key()->id), item_p);	
    new_sids[i++] = new_iterator.key()->id;
  }
  // Notify MSP.
  if(notify_msp) {
    try{
      if (!CORBA::is_nil( (ABng_Core_Internals::Main_Sighting_Processor_ptr) 
			  my_msp)) {
	if (old_sids.length())
	  my_msp->remove_sensors(my_view_name, old_sids);
	my_msp->add_sensors(my_view_name, new_sids);
      }
    }
    catch (...) {
    }
  }
  old_sds.clearAndDestroy();
  // NOTE:
  // This method updates the location's sensor list with its current copy
  // regadless of the kind of its change. Even if only one sensor
  // has been changed, removed or added the whole list is updated.
  // This may be is not very efficient, but determining what exactly
  // has been changed would be probably worse.
}
