// -*- Mode: C++; -*-
// $Id: location_rep_i.h,v 1.2 1998/07/29 14:57:09 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/sighting_processors/location_rep_i.h,v $
//
// $Log: location_rep_i.h,v $
// Revision 1.2  1998/07/29 14:57:09  mencnar
// performance tuning,
// receive sightings calls moved to dyn_dispatcher.* for future optimisation
//
// Revision 1.1.1.1  1998/06/18 13:37:37  mencnar
//
// Revision 1.1  1998/03/06 13:47:33  jasz
// Initial revision
//
//
// File:          location_rep_i.h
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  26.01.98
//
// General Description:
//  Declaration of the implementation class for the "location part" of View,
//  VSP and View_Manager interfaces.

#ifndef __LOCATION_REP_I_H__
#define __LOCATION_REP_I_H__

#include <stdlib.h>
#include <fstream.h>
#include <string.h>
#include <rw/tvhdict.h>
#include <rw/tvdlist.h>

#ifdef ObjectStore
#include <oosa.h>
#endif

#include "abng_core_management.hh"
#include "location_i.h"
#include "basic_rep_i.h"
#include "observed_i.h"
#include "view_rep_types.h"
#include "comparable_sensor_id.h"
#include "dyn_dispatcher.hh"

class L_Types {

public:

  typedef ABng_Core::Location Item;  
  typedef ABng_Core::Location_ptr Item_ptr;  
  typedef ABng_Core::Locations Items;  
  typedef Location_I Item_I;  
  typedef ABng_Core_System::ABng_Location_Description_ptr Initial_Data;
  typedef ABng_Core::View_Data_Observer::View_Data_Object Data_Object;
};

class Location_Rep_I : public No_Update_Basic_Rep_I<L_Types, VDO_Types> { 

public:

  // Hot start constructor
  Location_Rep_I (
#ifdef ObjectStore
    os_database* db_p,
#elif defined FilePersistency
    const char *repository_name_p,
#endif
    unsigned long burst_delay
  );

  // Cold start constructor
  Location_Rep_I (
#ifdef ObjectStore
    os_database* db_p,
#elif defined FilePersistency
    const char *repository_name_p,
#endif
    const char *view_name_p,
    const char *location_type_name_p,
    unsigned long burst_delay
  );

  virtual ~Location_Rep_I ();
  static Location_Rep_I *instance ();

  void set_burst_delay( unsigned long new_delay) {
    my_dynamic_observed.set_burst_delay( new_delay);
  }

  void load_dynamic_observers() { my_dynamic_observed.load_observers(); }
  
  // Operations of the Dynamic_Data_Observed_R interface
  
  virtual ABng_Core::Dynamic_Data_Observed_R::Observer_Entries* 
    dynamic_data_observer_list (
    ENV_DECL1(env)) {

    return my_dynamic_observed.observer_list( USE_ENV1(env));
  }

  virtual void attach_dynamic_data_observer (
    ABng_Core::Dynamic_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    my_dynamic_observed.attach_observer(obs, observed_id_list USE_ENV(env));
  }

  virtual void detach_dynamic_data_observer (
    ABng_Core::Dynamic_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    my_dynamic_observed.detach_observer(obs, observed_id_list USE_ENV(env));
  }

  // Suplementary operation

  virtual ABng_Core::Dynamic_Data_Observed_E::Dynamic_Data_Observers*
    dynamic_data_observer_list (
    Domain_Repository::Repository_Item_Id item_id_p
    ENV_DECL(env)) {
       
    return my_dynamic_observed.get_item_observers(item_id_p USE_ENV(env));
  }

  // Operations of the Domain_Repository::View_Data_Observed_R interface

  virtual ABng_Core::View_Data_Observed_R::Observer_Entries* 
    view_data_observer_list (
    ENV_DECL1(env)) {
   
    return observer_list( USE_ENV1(env));
  }

  virtual ABng_Core::View_Data_Observed_E::View_Data_Observers* 
    add_view_data_observer_list (
    ENV_DECL1(env)) {
   
    return add_observer_list( USE_ENV1(env));
  }

  virtual void attach_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    attach_observer(obs, observed_id_list USE_ENV(env));
  }

  virtual void detach_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    detach_observer( obs, observed_id_list USE_ENV(env));
  }

  virtual void attach_add_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DECL(env)) {

    attach_add_observer(obs USE_ENV(env));
  }

  virtual void detach_add_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DECL(env)) {

    detach_add_observer(obs USE_ENV(env));
  }

  // Suplementary operation

  virtual ABng_Core::View_Data_Observed_E::View_Data_Observers*
    view_data_observer_list (
    Domain_Repository::Repository_Item_Id item_id_p
    ENV_DECL(env)) {
       
    return get_item_observers(item_id_p USE_ENV(env));
  }

  // IDL operations of the View and View_Manager interfaces
  
  virtual ABng_Core::Locations* location_list (
    ENV_DECL1(env)) {

    return item_list( USE_ENV1(env));
  }

  virtual ABng_Core::Locations* find (
    const ABng_Core::Location::Pattern& data_record_pattern
    ENV_DECL(env)) {

    return No_Update_Basic_Rep_I<L_Types, VDO_Types>::find(
      data_record_pattern USE_ENV(env));
  }


  virtual ABng_Core::Location::Value_Descriptions* find_values (
    const ABng_Core::Location::Pattern& data_record_pattern
    ENV_DECL(env) ) {
  
    return No_Update_Basic_Rep_I<L_Types, VDO_Types>::find_values(
      data_record_pattern USE_ENV(env));
  }

  virtual ABng_Core::Location::Query_Results*
   complex_query(
		 const ABng_Core::Location::Entity_Set &es,
		 const Domain_Repository::Query_Options &qo,
		 ABng_Core::View_Data_Observer_ptr obs
		 ENV_DECL(env)) {

     return No_Update_Basic_Rep_I<L_Types, VDO_Types>::complex_query(
		   es,qo,obs USE_ENV(env));
  }

  virtual ABng_Core::Location_ptr add (
    const ABng_Core_System::ABng_Location_Description_ptr& new_object
    ENV_DECL(env)) {
#ifdef DEBUG
#ifdef ORBIX
    cerr << "Location::add _refCount " << new_object->_refCount() << endl;
#endif
#endif
    ABng_Core::Location_ptr ret=
      No_Update_Basic_Rep_I<L_Types, VDO_Types>::add(new_object USE_ENV(env));
#ifdef DEBUG
#ifdef ORBIX
    cerr << "Location:add(2) _refCount " << new_object->_refCount() << endl;
#endif
#endif
    return ret;
  }

  virtual void remove (
    ABng_Core::Location_ptr object_to_remove
    ENV_DECL(env)) {

    No_Update_Basic_Rep_I<L_Types, VDO_Types>::remove(object_to_remove USE_ENV(env));
  }

  virtual ABng_Core_System::ABng_Location_Description_ptr deref(
    ABng_Core::Location_ptr lct_object
    ENV_DECL(env));

  // Operations of the ABng_Core_System::System_Static_Data_Observer interface
  // Used for intra-core notification mechanism
  void notify(
    Domain_Repository::Repository_Item_Id id,
    const ABng_Core_System::System_Static_Data_Observer::System_Static_Data_Object& sd, 
    Domain_Repository::Change_Type change
    ENV_DECL(env));

  // Non-IDL public methods used by Locatable_I and View_I classes:

  // Accessors:
  ABng_Core_Management::ABng_Location_Description_Rep_var 
    get_abng_ld_rep() {
    return my_abng_ld_rep;
  }
  virtual char *get_view_name() {
    return my_view_name;
  }
  virtual char *get_location_type_name() {
    return my_location_type_name;
  }

  // Get a location containing the sensor with the given id.
  Location_I *resolve_sensor(const ABng_Core_System::Sensor_Id& sid);

  // Notify dynamic data observers registered with the new or/and old locations
  // of the given locatable.
  virtual void dynamic_notify(Location_I *new_location, 
    Location_I *old_location, Locatable_I *locatable_p);

  // Detach locatable from its current location.
  virtual void remove_locatable(const Locatable_I *locatable_p);

  // Refresh all the information that could have changed in the ABng_LD_Rep.
  // This method should be called after the ABng_LD_Rep is launched.
  virtual void resolve_abng_ld_rep(
    ABng_Core_Management::ABng_Location_Description_Rep_var abng_ld_rep_p);

  virtual void resolve_sensor_rep(
    ABng_Core_Management::Sensor_Rep_var sensor_rep_p);

  // Send the list of all sensors belonging to the view to the MSP.
  // This method should be called after the MSP is launched.
  virtual void resolve_msp(
    ABng_Core_Internals::Main_Sighting_Processor_var msp_p);

  // Generate a list (sequence) of all sensors belonging to the view.
  virtual ABng_Core_System::Sensor_Ids *get_sensor_list();

  ABng_Core_System::Sensor_Ids locations2sensor_ids(
    const ABng_Core::Locations *locs);
  ABng_Core::Locations sensor_ids2locations(
    const ABng_Core_System::Sensor_Ids *sids);
  ABng_Core::Location_var sensor_id2location(
    const ABng_Core_System::Sensor_Id &sid)
    throw( ABng_Core::Invalid_Argument);

  Location_I * locatable_came_in( Domain_Repository::Repository_Item_Id rid,
				  Locatable_I *locatable);

#ifdef ObjectStore
  static os_typespec* get_os_typespec();
#endif

  // definition of structure containing parameters of match_location method

  struct MatchLocationApplyDataStruct {
    RWTValDlist<Location_I*> _found_locations;
    const ABng_Core::Location::Pattern& _data_record_pattern;

    MatchLocationApplyDataStruct(
	   const ABng_Core::Location::Pattern& _data_record_pattern_p) :
      _data_record_pattern( _data_record_pattern_p) {}
  };

  // if the current location matches the pattern puts it into the container
  static void
    match_location( Location_I* current_location,
		     void *apply_data);

 protected:

  // Calculate the hash value of the location.
  static unsigned hash_location(const Location_I& location_i_p); 


  // Add helper methods
  virtual void init_corba_object(Item_I *item_p);
  virtual void insert_item(Item_I* item_p);

  // Remove helper methods
  virtual void remove_item(Item_I* item_p);

  // Find helper methods
  virtual ABng_Core::Locations* private_find (
    const ABng_Core::Location::Pattern& data_record_pattern
    ENV_DECL(env));
  virtual void collect_values(Location_I *lt, 
   ABng_Core::Location::Value_Description& ld);

  // Common helper methods
  virtual void construct_data_object(
    ABng_Core::View_Data_Observer::View_Data_Object& data_object_p,
    ABng_Core::Location_ptr item_p);
  virtual void save();
  virtual void save(ostream &);
  virtual void restore (istream &);
  virtual DB_Header *create_db_header();
  virtual void create_corba_objects() {}
  virtual void destroy_corba_objects() {}

  // Helper metods used for intra-core notification mechanism:

  // Refresh the location attributes corresponding to information
  // from other repositories (ABng_LD_Rep, Sensor_Rep) that could have 
  // changed. If notify_msp is non-zero the MSP is immediately notified 
  // if a sensor list installed in the location is changed.
  virtual Location_I::Refreshing_Result refresh(Location_I *item_p,
    int notify_msp = 1);

  // Replace the local copy of the sensor description with the current value.
  virtual void replace_sensor(ABng_Core_System::Sensor_ptr sensor_p);

  // Replace the local copy of location's sensor descriptions with the 
  // current value. Notify the MSP if it is required.
  virtual void update_sensors(Location_I *item_p,
    Location_I::Sensor_Descr_List& old_sds, 
    Location_I::Sensor_Descr_List& new_sds,
    int notify_msp = 1);

  static Location_Rep_I* _instance;

  // References of related objects.
  ABng_Core_Management::ABng_Location_Description_Rep_var my_abng_ld_rep;
  ABng_Core_Management::Sensor_Rep_var my_sensor_rep;
  ABng_Core_Internals::Main_Sighting_Processor_var my_msp;

  // The registry of dynamic data observers.
  DynDispatcher my_dynamic_observed;

  // Dictionaries necessary to serve events originating from MSP
  // (sightings) and Sensor_Rep.
  typedef RWTValHashDictionary<Domain_Repository::Repository_Item_Id, 
    Location_I*> Sensor_Item_Id_Location_Dictionary;
  typedef RWTValHashDictionaryIterator<Domain_Repository::Repository_Item_Id, 
    Location_I*> Sensor_Item_Id_Location_Dictionary_Iterator;

  typedef RWTValHashDictionary<Comparable_Sensor_Id, 
    Location_I*> Sensor_Id_Location_Dictionary;
  typedef RWTValHashDictionaryIterator<Comparable_Sensor_Id, 
    Location_I*> Sensor_Id_Location_Dictionary_Iterator;

  Sensor_Item_Id_Location_Dictionary my_sensor_item_ids_to_locations;
  Sensor_Id_Location_Dictionary my_sensor_ids_to_locations;

  CORBA::String_var my_view_name;
  CORBA::String_var my_location_type_name;

};

#endif /* __LOCATION_REP_I_H__ */



