// -*- Mode: C++; -*-

#ifndef __MSP_I_H__
#define __MSP_I_H__

#include "basic_abobserver_i.h"
#include <rw/tvhset.h>
#include <rw/tphdict.h>
#include <rw/tvhdict.h>
#include <rw/cstring.h>
#include "bootstrap_n.hh"
#include "const.h"
#include "comparable_sensor_id.h"
#include "comparable_badge_id.h"
#include "process_status.hh"

struct TimeSeqNumber {
  ABng_Server_Manager::Time sighting_time;
  ABng_Core::Seq_Number seq_number;

  TimeSeqNumber() {}
  TimeSeqNumber( ABng_Server_Manager::Time _sighting_time,
		 ABng_Core::Seq_Number _seq_number) :
    sighting_time( _sighting_time),
    seq_number( _seq_number) {}
};

class VSP_Entry {

public:

  VSP_Entry(ABng_Core_Internals::View_Sighting_Processor_var vsp_p,
    unsigned want_duplicates_p = (unsigned) 0) :
    my_vsp(vsp_p), 
    my_sensor_list(Comparable_Sensor_Id::hash, hash_table_size), 
    my_badge_list(Comparable_Badge_Id::hash, hash_table_size),
    badge_ids_to_seq_numbers(
			     Comparable_Badge_Id::hash, hash_table_size),
    my_wants_duplicates(want_duplicates_p) {}

  virtual ~VSP_Entry() {}

  virtual void add_sensors(
    const ABng_Core_System::Sensor_Ids& sensor_id_list);
  virtual void remove_sensors(
    const ABng_Core_System::Sensor_Ids& sensor_id_list);
  virtual int contains_sensor(
    const ABng_Core_System::Sensor_Id& sid);
  virtual ABng_Core_System::Sensor_Ids *get_sensors();
  virtual void add_badges(
    const ABng_Core_System::Badge_Ids& badge_id_list);
  virtual void remove_badges(
    const ABng_Core_System::Badge_Ids& badge_id_list);
  virtual int contains_badge(const ABng_Core_System::Badge_Id& bid);
  virtual ABng_Core_System::Badge_Ids *get_badges();
  
  virtual int wants_duplicates() {
    return my_wants_duplicates;
  }

  virtual int is_duplicated(const Comparable_Badge_Id& bid, 
			    TimeSeqNumber seq_num);
 
  virtual ABng_Core_Internals::View_Sighting_Processor_var get_vsp() {
    return my_vsp;
  }
  //  protected:
    
  ABng_Core_Internals::View_Sighting_Processor_var my_vsp;

  typedef RWTValHashSet<Comparable_Sensor_Id> Sensor_Id_List;
  typedef RWTValHashTableIterator<Comparable_Sensor_Id> 
    Sensor_Id_List_Iterator;
  typedef RWTValHashSet<Comparable_Badge_Id> Badge_Id_List;
  typedef RWTValHashTableIterator<Comparable_Badge_Id> 
    Badge_Id_List_Iterator;

  Sensor_Id_List  my_sensor_list;
  Badge_Id_List my_badge_list;

  typedef RWTValHashDictionary<Comparable_Badge_Id, TimeSeqNumber>
    Badge_Id_Seq_Number_Dictionary;
  typedef RWTValHashDictionaryIterator<
    Comparable_Badge_Id, TimeSeqNumber>
    Badge_Id_Seq_Number_Dictionary_Iterator;
  
  Badge_Id_Seq_Number_Dictionary badge_ids_to_seq_numbers;

  int my_wants_duplicates;
};

class MSP_I : public Basic_AB_Observer_I, 
	      protected Bootstrap_listener,
	      public ServerStatusInterface {

  ProcessControlImpl *ps_dev;
public:

  MSP_I();
  virtual ~MSP_I();
  
  virtual void sighting(
    const char *network_group_name,
    ABng_Poller::AB_Poller::Network_Id network, 
    ABng_Poller::AB_Poller::Station_Id station, 
    const ABng_Poller::AB_Poller::Badge_Id& badge,
    ABng_Poller::AB_Poller::Sighting_Code code,
    const ABng_Poller::AB_Poller::Field_Id& field,
    ABng_Poller::AB_Poller::Byte click_num,
    ABng_Poller::AB_Poller::Byte attributes,
    ABng_Poller::AB_Poller::Seq_Number seq,
    const ABng_Poller::AB_Poller::Time_Stamp& ts
    ENV_DECL(env) );
  
  virtual ABng_Core_Internals::Main_Sighting_Processor::
    View_Sighting_Processors *view_sp_list (
    ENV_DECL1(env) ) 
    throw (CORBA::SystemException);
  
  virtual void add_vsp (
    const char *view_name,
    ABng_Core_Internals::View_Sighting_Processor_ptr vsp, 
    const ABng_Core_System::Badge_Ids& badge_id_list, 
    const ABng_Core_System::Sensor_Ids& sensor_id_list, 
    CORBA::Boolean wants_duplicates
    ENV_DECL(env) )
    throw (CORBA::SystemException,
	   Domain_Repository::Unknown_Object_Ref,
	   Domain_Repository::Duplicate_Data_Record);
  
  virtual void find_vsp (
    const char *view_name,
    ABng_Core_Internals::View_Sighting_Processor_ptr& vsp,
    ABng_Core_System::Badge_Ids*& badge_id_list, 
    ABng_Core_System::Sensor_Ids*& sensor_id_list, 
    CORBA::Boolean& wants_duplicates
    ENV_DECL(env) ) 
    throw (CORBA::SystemException, 
	   ABng_Core::Unknown_View_Name);

  virtual void remove_vsp (
    const char *view_name
    ENV_DECL(env) ) 
    throw (CORBA::SystemException, 
	   ABng_Core::Unknown_View_Name);

  virtual void add_badges (
    const char *view_name,
    const ABng_Core_System::Badge_Ids& badge_id_list
    ENV_DECL(env) ) 
    throw (CORBA::SystemException, 
	   ABng_Core::Unknown_View_Name);
  
  virtual void remove_badges (
    const char *view_name,
    const ABng_Core_System::Badge_Ids& badge_id_list
    ENV_DECL(env) ) 
    throw (CORBA::SystemException, 
	   ABng_Core::Unknown_View_Name, 
	   ABng_Core_System::Unknown_Badge_Id);
  
  virtual void add_sensors (
    const char *view_name,
    const ABng_Core_System::Sensor_Ids& sensor_id_list
    ENV_DECL(env) ) 
    throw (CORBA::SystemException, 
	   ABng_Core::Unknown_View_Name);
  
  virtual void remove_sensors (
    const char *view_name,
    const ABng_Core_System::Sensor_Ids& sensor_id_list
    ENV_DECL(env) )
    throw (CORBA::SystemException, 
	   ABng_Core::Unknown_View_Name, 
	   ABng_Core_System::Unknown_Sensor_Id);
  
  virtual void test();
  
  virtual void print_contents();
  
  ABng_Core_Internals::Main_Sighting_Processor_ptr get_msp_obj() {
    return my_msp_obj;}
  
protected:
        
  typedef RWTPtrHashDictionary<RWCString, VSP_Entry> 
    Name_VSP_Dictionary;
  typedef RWTPtrHashDictionaryIterator<RWCString, VSP_Entry> 
    Name_VSP_Dictionary_Iterator;

  Name_VSP_Dictionary names_to_vsps;

  ABng_Core_Internals::Main_Sighting_Processor_ptr my_msp_obj;
  ABng_Poller::AB_Observer_ptr my_poller_observer_obj;
    
 void reference_resolved( const ABng_Server_Manager::Name &server_name, 
  			   CORBA::Object_var server_ref,
			   ABng_Server_Manager::uid callback_id);
};

#endif /* __MSP_I_H__ */









