// $Id: view_i.h,v 1.2 1998/07/29 14:57:11 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/sighting_processors/view_i.h,v $
//
// $Log: view_i.h,v $
// Revision 1.2  1998/07/29 14:57:11  mencnar
// performance tuning,
// receive sightings calls moved to dyn_dispatcher.* for future optimization
//
// Revision 1.1.1.1  1998/06/18 13:37:37  mencnar
//
// Revision 1.1  1998/03/06 13:48:40  jasz
// Initial revision
//
//
// File:          view_i.h
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  10.01.98
//
// General Description:
//   Declaration of the View_I class implementing the View ABng component.


#ifndef __VIEW_I_H__
#define __VIEW_I_H__

#include <iostream.h>
#include <rw/tphasht.h>
#include <rw/tvhasht.h>
#include <rw/tphdict.h>
#include <rw/tvhdict.h>

#include "abng_core_management.hh"
#include "bootstrap_n.hh"
#include "abng_guarded.hh"
#include "location_i.h"
#include "locatable_i.h"
#include "location_rep_i.h"
#include "locatable_rep_i.h"
#include "request_mapper.h"
#include "process_status.hh"

#ifdef ObjectStore
#include <oosa.h>
#endif

#ifdef FilePersistency
#include <fstream.h>
#endif

class View_I : public Bootstrap_listener, 
	       public ABng_Guarded,
	       public ServerStatusInterface 
{

  static void register_my_servers( void *args);
  ABng_S_Guarded repos_guard;
  omni_condition repos_cond;
  unsigned long last_repos_resolved, last_repos_resolved_us;

public:

  // Constants for quality attribute updates.
  static const unsigned MIN_QUALITY;
  static const unsigned MAX_QUALITY;
  static const unsigned QUALITY_STEP;

  // View_I is a singleton class.
  static View_I *instance();

  // The hot start constructor. Passing a view name is necessary as it
  // determines the directory where the databases for this view are placed. 
  View_I(
#ifdef ObjectStore
    os_database *view_db_p,
    os_database *location_rep_db_p,
    os_database *locatable_rep_db_p,
#elif defined FilePersistency
    const char *view_db_p,
    const char *location_rep_db_p,
    const char *locatable_rep_db_p,
#endif
    const char *view_name_p,
    long sightings_dump_period);

  // The cold start constructor.
  View_I(
#ifdef ObjectStore
    os_database *view_db_p,
    os_database *location_rep_db_p,
    os_database *locatable_rep_db_p,
#elif defined FilePersistency
    const char *view_db_p,
    const char *location_rep_db_p,
    const char *locatable_rep_db_p,
#endif
    const char *view_name_p,
    const char *location_type_name_p,
    int wants_duplicates_p,
    long sightings_dump_period);

  virtual ~View_I();
 
  // Operations of the Dynamic_Data_Observed_R interface.
  
  virtual ABng_Core::Dynamic_Data_Observed_R::Observer_Entries* 
    location_dynamic_data_observer_list (
    ENV_DECL1(env)) {

    return my_location_rep.dynamic_data_observer_list( USE_ENV1(env));
  }

  virtual void attach_location_dynamic_data_observer (
    ABng_Core::Dynamic_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    my_location_rep.attach_dynamic_data_observer(
      obs, observed_id_list USE_ENV(env));
  }

  virtual void detach_location_dynamic_data_observer (
    ABng_Core::Dynamic_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    my_location_rep.detach_dynamic_data_observer(
      obs, observed_id_list USE_ENV(env));
  }

  virtual ABng_Core::Dynamic_Data_Observed_R::Observer_Entries* 
    locatable_dynamic_data_observer_list (
    ENV_DECL1(env)) {

    return my_locatable_rep.dynamic_data_observer_list( USE_ENV1(env));
  }

  virtual void attach_locatable_dynamic_data_observer (
    ABng_Core::Dynamic_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    my_locatable_rep.attach_dynamic_data_observer(
      obs, observed_id_list USE_ENV(env));
  }

  virtual void detach_locatable_dynamic_data_observer (
    ABng_Core::Dynamic_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    my_locatable_rep.detach_dynamic_data_observer(
      obs, observed_id_list USE_ENV(env));
  }

  // Operations of the Domain_Repository::View_Data_Observed_R interface.

  virtual ABng_Core::View_Data_Observed_R::Observer_Entries* 
    location_view_data_observer_list (
    ENV_DECL1(env)) {

    return my_location_rep.view_data_observer_list( USE_ENV1(env));
  }
   
  virtual ABng_Core::View_Data_Observed_E::View_Data_Observers* 
    location_add_view_data_observer_list (
    ENV_DECL1(env)) {

    return my_location_rep.add_view_data_observer_list( USE_ENV1(env));
  }

  virtual void attach_location_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    my_location_rep.attach_view_data_observer(
      obs, observed_id_list USE_ENV(env));
  }

  virtual void detach_location_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    my_location_rep.detach_view_data_observer(
      obs, observed_id_list USE_ENV(env));
  }

  virtual void attach_location_add_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DECL(env)) {

    my_location_rep.attach_add_view_data_observer(
      obs USE_ENV(env));
  }

  virtual void detach_location_add_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DECL(env)) {

    my_location_rep.detach_add_view_data_observer(
      obs USE_ENV(env));
  }

  virtual ABng_Core::View_Data_Observed_R::Observer_Entries* 
    locatable_view_data_observer_list (
    ENV_DECL1(env)) {

    return my_locatable_rep.view_data_observer_list( USE_ENV1(env));
  }
   
  virtual ABng_Core::View_Data_Observed_E::View_Data_Observers* 
    locatable_add_view_data_observer_list (
    ENV_DECL1(env)) {

    return my_locatable_rep.add_view_data_observer_list( USE_ENV1(env));
  }

  virtual void attach_locatable_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    my_locatable_rep.attach_view_data_observer(
      obs, observed_id_list USE_ENV(env));
  }

  virtual void detach_locatable_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    my_locatable_rep.detach_view_data_observer(
      obs, observed_id_list USE_ENV(env));
  }

  virtual void attach_locatable_add_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DECL(env)) {

    my_locatable_rep.attach_add_view_data_observer(obs USE_ENV(env));
  }


  virtual void detach_locatable_add_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DECL(env)) {

    my_locatable_rep.detach_add_view_data_observer(obs USE_ENV(env));
  }

  // Operations of Request_Queue:

  virtual ABng_Core::Request_Queue::Requests* list_user_requests (
    const char * user_id
    ENV_DECL( IT_env));

  virtual ABng_Core::Request_Queue::Requests* list_all_requests (
    ENV_DECL1( IT_env));

  virtual void remove_request (CORBA::Long request_id
    ENV_DECL( IT_env));

  // Operations of Caller:

  virtual void challenge_reply (const ABng_Core_System::Sensor_Id& sensor, 
                                const ABng_Core_System::Badge_Id& badge, 
                                const ABng_Core_System::Octets& random, 
                                const ABng_Core_System::Time& ts,
                                CORBA::Long request_id,
                                CORBA::Long repeat_num
                                ENV_DECL( IT_env) );

  virtual void sound_reply (const ABng_Core_System::Sensor_Id& sensor, 
                            const ABng_Core_System::Badge_Id& badge, 
                            const ABng_Core_System::Octets& sound, 
                            const ABng_Core_System::Time& ts, 
                            CORBA::Long request_id, 
                            CORBA::Long repeat_num
                            ENV_DECL( IT_env) );

  virtual void set_state_reply (const ABng_Core_System::Sensor_Id& sensor, 
                                const ABng_Core_System::Badge_Id& badge, 
                                CORBA::Octet status, 
                                const ABng_Core_System::Octets& sound, 
                                const ABng_Core_System::Time& ts, 
                                CORBA::Long request_id, 
                                CORBA::Long repeat_num
                                ENV_DECL( IT_env) );

  virtual void get_state_reply (const ABng_Core_System::Sensor_Id& sensor, 
                                const ABng_Core_System::Badge_Id& badge, 
                                CORBA::Octet major_version, 
                                CORBA::Octet minor_version, 
                                CORBA::Octet status, 
                                const ABng_Core_System::Time& ts, 
                                CORBA::Long request_id, 
                                CORBA::Long repeat_num
                                ENV_DECL( IT_env) );

  virtual void set_home_reply (const ABng_Core_System::Sensor_Id& sensor, 
                               const ABng_Core_System::Badge_Id& badge, 
                               const ABng_Core_System::Octets& address, 
                               const ABng_Core_System::Time& ts, 
                               CORBA::Long request_id, 
                               CORBA::Long repeat_num
                               ENV_DECL( IT_env) );

  virtual void get_home_reply (const ABng_Core_System::Sensor_Id& sensor, 
                               const ABng_Core_System::Badge_Id& badge, 
                               const ABng_Core_System::Octets& address, 
                               const ABng_Core_System::Time& ts, 
                               CORBA::Long request_id, 
                               CORBA::Long repeat_num
                               ENV_DECL( IT_env) );

  virtual void get_station_state_reply (const ABng_Core_System::Sensor_Id& sensor,
                                        CORBA::Octet major_version, 
                                        CORBA::Octet minor_version, 
                                        CORBA::Octet status1, 
                                        CORBA::Octet status2, 
                                        const ABng_Core_System::Time& ts, 
                                        CORBA::Long request_id, 
                                        CORBA::Long repeat_num
                                        ENV_DECL( IT_env) );

  virtual void sensor_command_reply( const ABng_Core_System::Sensor_Id& sensor,
                                     const ABng_Core_System::Octets &reply_body,
                                     const ABng_Core_System::Time &ts,
                                     CORBA::Long request_id,
                                     CORBA::Long repeat_num
                                     ENV_DECL( IT_env) );

  virtual void badge_command_reply( const ABng_Core_System::Sensor_Id& sensor,
                                    const ABng_Core_System::Badge_Id& badge,
                                    const ABng_Core_System::Octets &reply_body,
                                    const ABng_Core_System::Time &ts,
                                    CORBA::Long request_id,
                                    CORBA::Long repeat_num
                                    ENV_DECL( IT_env) );

  virtual void null_message (const ABng_Core_System::Sensor_Id& sensor, 
                             const ABng_Core_System::Time& ts, 
                             CORBA::Long request_id,
                             CORBA::Long repeat_num
                             ENV_DECL( IT_env) );

  virtual void error_occured (const ABng_Core_System::Sensor_Id& sensor,
			      const ABng_Core_System::Badge_Id& badge,
			      ABng_Core::Command_Error_Code error_code,
			      CORBA::Long request_id,
			      CORBA::Long repeat_num,
			      CORBA::Long retry_num
			      ENV_DECL( IT_env) );

  virtual void request_processed (const ABng_Core_System::Sensor_Id& sensor,
                                  const ABng_Core_System::Badge_Id& badge,
                                  const ABng_Core_System::Time& ts, 
                                  CORBA::Long request_id,
                                  CORBA::Long repeat_num
                                  ENV_DECL( IT_env) );

  virtual void request_completed( CORBA::Long request_id
				  ENV_DECL( IT_env) );
				  

  // Operations of Commander:

  virtual CORBA::Long issue_location_command (
    const ABng_Core::Locations& location_list, 
    const ABng_Core_System::Octets& cmd, 
    CORBA::Boolean has_reply,
    CORBA::Octet reply_code,
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env) );

  virtual CORBA::Long issue_locatable_command (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core_System::Octets& cmd, 
    CORBA::Boolean has_reply,
    CORBA::Octet reply_code,    
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to 
    ENV_DECL( IT_env) );

  virtual CORBA::Long challenge (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core_System::Octets& random, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to 
    ENV_DECL( IT_env) );

  virtual CORBA::Long sound (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core_System::Octets& sound, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to 
    ENV_DECL( IT_env) );

  virtual CORBA::Long set_state (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Locatables& locatable_list, 
    CORBA::Octet status, 
    const ABng_Core_System::Octets& sound, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to 
    ENV_DECL( IT_env) );

  virtual CORBA::Long get_state (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to 
    ENV_DECL( IT_env) );

    virtual CORBA::Long set_home (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core_System::Octets& address, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to 
    ENV_DECL( IT_env) );

    virtual CORBA::Long get_home (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Locatables& locatable_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to 
    ENV_DECL( IT_env) );

  virtual CORBA::Long reset_buffer (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to 
    ENV_DECL( IT_env) );

  virtual CORBA::Long reset_fifos (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to 
    ENV_DECL( IT_env) );

  virtual CORBA::Long get_station_state (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to 
    ENV_DECL( IT_env) );

  virtual CORBA::Long set_output (
    const ABng_Core::Locations& location_list, 
    CORBA::Octet output_value, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to 
    ENV_DECL( IT_env) );

  virtual CORBA::Long set_persistence (
    const ABng_Core::Locations& location_list, 
    CORBA::Boolean mode, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to 
    ENV_DECL( IT_env) );

  // Operations of the View interface.

  virtual char * view_name (
    ENV_DECL1(env)) {
      return CORBA::string_dup(my_view_name);
    }

  virtual Domain_Repository::Location_Type::Location_Type_Name type (
    ENV_DECL1( IT_env) );

  virtual ABng_Core::Locations* location_list (
    ENV_DECL1(env)) {

    return my_location_rep.location_list( USE_ENV1(env));
  }

  virtual ABng_Core::Locatables* locatable_list (
    ENV_DECL1(env)) {
    return my_locatable_rep.locatable_list( USE_ENV1(env));
  }

  virtual CORBA::Boolean wants_duplicates (
    ENV_DECL1(env)) {
    return my_wants_duplicates;
  }

  virtual ABng_Core::Locations* find_location (
    const ABng_Core::Location::Pattern& data_record_pattern
    ENV_DECL(env)) {

    return my_location_rep.find(data_record_pattern USE_ENV(env));
  }

  virtual ABng_Core::Location::Value_Descriptions* find_location_values (
    const ABng_Core::Location::Pattern& data_record_pattern
    ENV_DECL(env)) {

    return my_location_rep.find_values(data_record_pattern USE_ENV(env));
  }

  virtual ABng_Core::Location::Query_Results*
   complex_query_location(
		 const ABng_Core::Location::Entity_Set &es,
		 const Domain_Repository::Query_Options &qo,
		 ABng_Core::View_Data_Observer_ptr obs
		 ENV_DECL(env)) {

    return my_location_rep.complex_query( es, qo, obs);
  }

  virtual ABng_Core::Locatables* find_locatable (
    const ABng_Core::Locatable::Pattern& data_record_pattern
    ENV_DECL(env)) {

    return my_locatable_rep.find(data_record_pattern USE_ENV(env));
  }

  virtual ABng_Core::Locatable::Value_Descriptions* find_locatable_values (
    const ABng_Core::Locatable::Pattern& data_record_pattern
    ENV_DECL(env)) {

    return my_locatable_rep.find_values(data_record_pattern USE_ENV(env));
  }

  virtual ABng_Core::Locatable::Query_Results*
   complex_query_locatable(
		 const ABng_Core::Locatable::Entity_Set &es,
		 const Domain_Repository::Query_Options &qo,
		 ABng_Core::View_Data_Observer_ptr obs
		 ENV_DECL(env)) {

    return my_locatable_rep.complex_query( es, qo, obs);
  }

  virtual ABng_Core::Sighting_Descriptions* get_sightings (
    const ABng_Core_System::Time& since
    ENV_DECL(env));

  // Operations of the View_Manager interface.

  virtual ABng_Core::Location_ptr add_location (
    ABng_Core_System::ABng_Location_Description_ptr new_data
    ENV_DECL(env)) {

    return my_location_rep.add(new_data USE_ENV(env));
  }

  virtual void remove_location (
    ABng_Core::Location_ptr object_to_remove
    ENV_DECL(env)) {

    my_location_rep.remove(object_to_remove USE_ENV(env));
  }

  virtual ABng_Core::Locatable_ptr add_locatable (
    ABng_Core_System::Badge_Holder_ptr new_data
    ENV_DECL(env)) {

    return my_locatable_rep.add(new_data USE_ENV(env));
  }

  virtual void remove_locatable (
    ABng_Core::Locatable_ptr object_to_remove
    ENV_DECL(env)) {

    my_locatable_rep.remove(object_to_remove USE_ENV(env));
  }

  virtual ABng_Core_System::Badge_Holder_ptr deref_locatable(
    ABng_Core::Locatable_ptr lct_object
    ENV_DECL(env)) {

    return my_locatable_rep.deref( lct_object);
  }

  virtual ABng_Core_System::ABng_Location_Description_ptr deref_location(
    ABng_Core::Location_ptr lct_object
    ENV_DECL(env)) {

    return my_location_rep.deref( lct_object);
  }

  // Operations of the View_Sighting_Processor interface

  virtual ABng_Core::View_ptr my_view (
    ENV_DECL1(env)) {
      return ABng_Core::View::_duplicate(my_view_obj);}

  virtual CORBA::ULong burst_delay_time( ENV_DECL1(env));
  virtual void burst_delay_time( CORBA::ULong new_burst_time
				 ENV_DECL(env));
  virtual CosTypedEventChannelAdmin::TypedEventChannel_ptr sighting_channel( ENV_DECL1(env));
  virtual void sighting_channel( CosTypedEventChannelAdmin::TypedEventChannel_ptr ch
				 ENV_DECL(env));

  /*
  virtual ABng_Core_Internals::Location_Sighting_Processors* 
    location_sp_list (
    CORBA::Environment &IT_env=CORBA::default_environment) 
    throw (CORBA::SystemException);

  virtual ABng_Core_Internals::Locatable_Sighting_Processors* 
    locatable_sp_list (
    CORBA::Environment &IT_env=CORBA::default_environment)    
    throw (CORBA::SystemException);
  */
  
  virtual ABng_Core_System::Sensor_Ids* sensor_id_list (
    ENV_DECL1(env));

  virtual ABng_Core_System::Badge_Ids* badge_id_list (
    ENV_DECL1(env));

  virtual void process_sighting (
    const ABng_Core_Internals::Internal_Sighting_Description& isd
    ENV_DECL(env));

  // Operations of the SSDO interface.
  
  void notify(
    Domain_Repository::Repository_Item_Id id,
    const ABng_Core_System::System_Static_Data_Observer::System_Static_Data_Object& sd, 
    Domain_Repository::Change_Type change
    ENV_DECL(env));

  void abort(
    ENV_DECL1(env)) {}

  // Methods invoked after ABng_LD_Rep, BH_Rep and MSP are discovered. 
  virtual void resolve_abng_ld_rep(
    ABng_Core_Management::ABng_Location_Description_Rep_var abng_ld_rep_p);
  virtual void resolve_bh_rep(
    ABng_Core_Management::Badge_Holder_Rep_var bh_rep_p);
  virtual void resolve_msp(
    ABng_Core_Internals::Main_Sighting_Processor_var msp_p);
  void View_I::resolve_scheduler(
    ABng_Core_System::Scheduler_var scheduler_p);

  virtual void resolve_sensor_rep(
    ABng_Core_Management::Sensor_Rep_var sensor_rep_p);
  virtual void resolve_badge_rep(
    ABng_Core_Management::Badge_Rep_var badge_rep_p);

  // Create all implemented CORBA objects.
  virtual void create_corba_objects();
  // Destroy all implemented CORBA objects.
  virtual void destroy_corba_objects();

  virtual void init(int cold_start = 0);

  ABng_Core_System::System_Static_Data_Observer_ptr get_ssdo_ptr();


private:
  // Helper methods used by implementation of Commander/Caller interfaces

  ABng_Core::Request_Queue::Requests* map_scheduler_requests(
    ABng_Core_System::Scheduler_Request_Queue::Scheduler_Requests *sreqs);
  void map_scheduler_exception();
  ABng_Core_System::Scheduler_var get_scheduler_ref();

protected:

#ifdef FilePersistency
  // Methods to store and restore anobject of this class
  virtual DB_Header *create_db_header();
  virtual void save ();
  virtual void save( ostream &);
  virtual void restore ( istream &);
#endif

 void reference_resolved( const ABng_Server_Manager::Name &server_name, 
  			   CORBA::Object_var server_ref,
			   ABng_Server_Manager::uid callback_id);

  static View_I* inst;
  CORBA::String_var my_view_name;
  CORBA::Boolean my_wants_duplicates;
#ifdef ObjectStore
  os_database *my_view_db;
#elif defined FilePersistency
  char *my_view_db;
#endif
  Location_Rep_I my_location_rep;
  Locatable_Rep_I my_locatable_rep;

  unsigned long r_burst_delay_time;
  CosTypedEventChannelAdmin::TypedEventChannel_var channel;
  CosTypedEventChannelAdmin::TypedProxyPushConsumer_var ppushcons;
  ABng_Core::Dynamic_Data_Observer_var channel_observer;

  Request_Mapper request_mapper;
  ABng_Core::View_ptr my_view_obj;
  ABng_Core_Management::View_Manager_ptr my_view_manager_obj;
  ABng_Core_Internals::View_Sighting_Processor_ptr my_vsp_obj;
  ABng_Core_System::System_Static_Data_Observer_ptr my_ssdo_obj;
  ABng_Core_System::Scheduler_Caller_ptr my_scheduler_caller_obj;

  ABng_Core_System::Scheduler_var my_scheduler;
  ABng_Core_Internals::Main_Sighting_Processor_var my_msp;
  ProcessControlImpl *ps_dev;
};
#endif /* __VIEW_I_H__ */












