// $Id: view_server.cc,v 1.1.1.1 1998/06/18 13:37:37 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/sighting_processors/view_server.cc,v $
//
// $Log: view_server.cc,v $
// Revision 1.1.1.1  1998/06/18 13:37:37  mencnar
//
// Revision 1.1  1998/03/06 13:41:56  jasz
// Initial revision
//
//
// File:          view_server_i.cc
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  26.01.98
//
// General Description:
//  Definition of the View server class.

#include "view_server.h"

char View_Server::cwd_buffer[255];

View_Server::View_Server(
#ifdef ObjectStore
  const char *view_db_name_p, 
#elif FilePersistency
  const char *view_db_name_p,
  const char *location_rep_db_name_p,
  const char *locatable_rep_db_name_p,
#endif
  const char *current_directory_p) :
  Rep_Server(view_db_name_p, current_directory_p) {

#ifdef FilePersistency
  my_location_rep_db_name = strdup(location_rep_db_name_p);
  my_locatable_rep_db_name = strdup(locatable_rep_db_name_p);
#endif
}

View_Server::~View_Server() {

#ifdef FilePersistency
  delete [] my_location_rep_db_name;
  delete [] my_locatable_rep_db_name;
#endif
}

void View_Server::init(int argc, char **argv) {

  
  prog_name = strdup(argv[0]);
  init_corba(argc, argv);
  cold_start = 0;
  long sightings_dump_period= 5; // minutes
  int c;
  char *view_name = 0;
  char *location_type_name = 0;
  int wants_duplicates = 0;
#ifdef ObjectStore
  while (( c = getopt (argc, argv, "t:d:swhp:")) != -1) {
#else 
  while (( c = getopt (argc, argv, "t:d:swhp:")) != -1) {
#endif
    switch (c) {
      case 's': {
	cold_start++;
	break;
      }
      case 'w': {
	wants_duplicates = 1;
	break;
      }
#ifdef ORBIX
      case 't': {
	my_timeout = atol( optarg);
	break;
      }
#endif /* ORBIX */
      case 'd': {
	// set the data directory
	my_current_directory = optarg;
	break;
      }
      case 'p': {
	sightings_dump_period= atoi( optarg);
	break;
      }
      case 'h': {
	print_info();
	exit(1);
      }
      default: {
	print_info();
	exit(1);
      }
    }
  }
  if(cold_start) {
    if(optind != argc - 2) {
      print_info();
      exit(1);
    }
    location_type_name = argv[optind + 1];
  }
  if(!cold_start && optind != argc - 1) {
    print_info();
    exit(1);
  }
  view_name = argv[optind];
#ifdef ORBIX
  set_orbix_conf();
#endif
#ifdef ObjectStore
  os_database *db;
  db = init_os(my_db_name);
#else
  check_directory(my_current_directory);
  char *dir = concat_path_name(my_current_directory, view_name);
  my_current_directory = dir;
  check_directory(my_current_directory);
  char *full_view_db_name = 
    concat_path_name(my_current_directory, file_name);
  char *full_location_rep_db_name =  
    concat_path_name(my_current_directory, my_location_rep_db_name);
  char *full_locatable_rep_db_name =  
    concat_path_name(my_current_directory, my_locatable_rep_db_name);

  Bootstrap::init(orb_ptr);
  if(cold_start) {
    my_view = new View_I(full_view_db_name, 
      full_location_rep_db_name,
      full_locatable_rep_db_name,
      view_name,
      location_type_name,
      wants_duplicates,
      sightings_dump_period*60);
  }
  else {
    my_view = new View_I(full_view_db_name, 
      full_location_rep_db_name,
      full_locatable_rep_db_name,
      view_name,
      sightings_dump_period*60);    
  }
  delete [] full_view_db_name;
  delete [] full_location_rep_db_name;
  delete [] full_locatable_rep_db_name;
#endif
  my_view->init(cold_start);
}


void View_Server::print_info() {

  cout << "Usage:\t" << prog_name << " [options] view_name" << endl;
  cout << "\t" << prog_name << " -s [options] view_name type_name" << endl << endl;
  cout << "The following options may be used:" << endl;

  cout << "-d directory\tSpecify the database directory." << endl;
  cout << "-p period\nSpecify a numer of minutes to wait for sightings dump"
       << "\t\t(default: 5 minutes)" << endl;
  cout << "-s\t\tUsed when the server is launched for the first time (cold" << endl;
  cout << "\t\tstart). When this option is used the type name of locations " << endl;
  cout << "belonging to the view must be given as the type_name parameter." << endl;

#ifdef ORBIX
  cout << "-t timeout\tSpecify a number of miliseconds to wait between Orbix events." << endl;
#endif ORBIX

  cout << "-w\t\tRequire to pass duplicated locatable messages to the view." << endl;

  cout << "\t\tCan be only used in conjunction with -s." << endl;
}

void View_Server::final() {}
