// $Id: abngel.cc,v 1.1 1998/11/30 14:33:34 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/utils/event_log_util/abngel.cc,v $
//
// $Log: abngel.cc,v $
// Revision 1.1  1998/11/30 14:33:34  mencnar
// Initial release
//
//
//

#include <iostream.h>
#include <strstream.h>
#include <rw/rwtime.h>
#include <rw/rwdate.h>
#include <rw/tvhasht.h>
#include <event_log.hh>
#include "bootstrap_n.hh"
#include <stdlib.h>


static RWTime offset_time( RWDate(1,1,1970),0,0,0,RWZone::utc());

int match_source_name= 0;
int match_all_ids= 0;

void usage( char *prog_name) {
  cerr << "Usage: " << prog_name << endl
       << "\t-s \"logname [logname [logname ...]]\"\t\t-start event logs" << endl
       << "\t-t \"logname [logname [logname ...]]\"\t\t-stop event logs" << endl
       << "\t-r \"logname [logname [logname ...]]\"\t\t-reset event logs" << endl
       << "\t-g \"logname [logname [logname ...]]\"\t\t-get buffer length" << endl
       << "\t-l \"logname [logname ...] length [length ...]\"\t-set buffer length" << endl
       << "\t-c \"logname1 logname2\"\t\t\t\t-compare event logs" << endl
       << "\t-i \"logname [logname ...]\"\t\t\t-list event logs" << endl
       << "\t-d logname\t\t-show duration of events" << endl
       << "\t-n \t\t\t-do not ignore event source name in comparison" << endl
       << "\t-a \t\t\t-show all time durations for multiple event ids" << endl;
  exit(1);
}


struct EventWrapper {
  EventLog::EventItem ev;
  CORBA::ULong id;
  
  EventWrapper() {}
  EventWrapper( const EventLog::EventItem &an_ev,
		CORBA::ULong an_id) :
    ev( an_ev), id( an_id) {}
};

unsigned hashEvent( const EventWrapper &evw) {
  return (unsigned) (evw.ev.unique_id.ts.seconds+evw.ev.unique_id.ts.microseconds);
}


int operator == ( const EventLog::EventItem &ev1,
		  const EventLog::EventItem &ev2) {

  return (ev1.unique_id.ts.seconds == ev2.unique_id.ts.seconds &&
	  ev1.unique_id.ts.microseconds == ev2.unique_id.ts.microseconds &&
	  (!match_source_name || 
	   !strcmp(ev1.unique_id.sender, ev2.unique_id.sender)));
}

int operator == ( const EventWrapper &evw1,
		  const EventWrapper &evw2) {
  return (evw1.ev == evw2.ev);
}
		  
EventLog_var *log_refs( const char *names, unsigned &len) {
  char buf[256];
  {
    istrstream istr(names);
    len= 0;
    while (istr >> buf)
      ++len;
  }
  EventLog_var *ret= new EventLog_var[len];
  unsigned i= 0;
  istrstream istr(names);
  while (istr >> buf) {
    ABng_Server_Manager::Name nm= Bootstrap::CTX_ABng();
    nm= (nm+"EventLog")+buf;
    CORBA::Object_var obj;
    try {
      obj= Bootstrap::resolve(nm);
      if (!CORBA::is_nil(obj))
	if (obj->_non_existent()) obj= NULL;
    }
    catch (...) {
      obj= NULL;
    }
    if (CORBA::is_nil( obj)) {
      cerr << "Cannot contact event log " << buf << endl;
      exit(1);
    }
    EventLog_var el;
    try {
      el= EventLog::_narrow( obj);
    }
    catch (...) {}
    ret[i++]= el;
  }
  return ret;
}

void start_event_log( const char *names) {
  unsigned len;
  EventLog_var *logs= log_refs( names, len);
  for (unsigned i=0; i<len; i++)
    logs[i]->start();
  delete []logs;
}

void stop_event_log( const char *names) {
  unsigned len;
  EventLog_var *logs= log_refs( names, len);
  for (unsigned i=0; i<len; i++)
    logs[i]->stop();
  delete []logs;
}

void reset_event_log( const char *names) {
  unsigned len;
  EventLog_var *logs= log_refs( names, len);
  for (unsigned i=0; i<len; i++)
    logs[i]->reset();
  delete []logs;
}

void list_event_log( const char *names) {
  unsigned len;
  EventLog_var *logs= log_refs( names, len);
  for (unsigned i=0; i<len; i++) {
    cout << "\t\tLog : " << endl;
    CORBA::ULong pos= 0;
    EventLog::Events_var ev;
    do {
      ev= logs[i]->get_events( pos, 100);
      for (CORBA::ULong x= 0; x<ev->length(); x++) {
	cout << (unsigned long) (x+pos) << '\t'
	     << (const char *) ev[x].unique_id.sender << ' '
	     << (unsigned long) ev[x].unique_id.event_id << ' '
	     << RWTime( offset_time.seconds()+ev[x].unique_id.ts.seconds)
	     << ' ' << ((ev[x].unique_id.ts.microseconds)/1000.0) << endl
	     << RWTime( offset_time.seconds()+ev[x].receive_ts.seconds)
	     << ' ' << ((ev[x].receive_ts.microseconds)/1000.0) << endl;
      }
      pos+= ev->length();
    }
    while (ev->length()>0);
  }
  delete []logs;
}

EventLog::Events * read_log( EventLog_ptr log) {
  EventLog::Events *ret= new EventLog::Events();
  CORBA::ULong pos= 0;
  EventLog::Events_var ev;
  do {
    ev= log->get_events( pos, 100);
    ret->length( ret->length() + ev->length());
    for (CORBA::ULong i=0; i<ev->length(); i++)
      (*ret)[pos+i]= ev[i];
    pos+= ev->length();
  } while (ev->length());
  return ret;
}

void duration_event_log( const char *names) {
  unsigned len;
  EventLog_var *logs= log_refs( names, len);
  EventLog::Events_var *evlogs= new EventLog::Events_var[len];
  {
    for (unsigned i=0; i<len; i++)
      evlogs[i]= read_log( logs[i]);
  }
  for (CORBA::ULong i=0; i<len; i++) {
    for (CORBA::ULong j=1; j<evlogs[i]->length(); j++) {
      if (evlogs[i][j-1] == evlogs[i][j]) {
	EventLog::EventItem ei= evlogs[i][j];
	while (ei.receive_ts.microseconds <
	       evlogs[i][j-1].receive_ts.microseconds) {
	  ei.receive_ts.microseconds += 1000000;
	  ei.receive_ts.seconds -= 1;
	}
	ei.receive_ts.microseconds-= evlogs[i][j-1].receive_ts.microseconds;
	ei.receive_ts.seconds-= evlogs[i][j-1].receive_ts.seconds;
	cout << (unsigned long) (ei.receive_ts.seconds*1000000+
				 ei.receive_ts.microseconds)
	     << endl;
      }
    }
    cout << endl;
  }
  delete []evlogs;
  delete []logs;
}

void compare_event_log( const char *names) {
  unsigned len;
  EventLog_var *logs= log_refs( names, len);
  EventLog::Events_var *evlogs= new EventLog::Events_var[len];
  {
    for (unsigned i=0; i<len; i++)
      evlogs[i]= read_log( logs[i]);
  }

  CORBA::ULong src_el_len= evlogs[0]->length();
  EventLog::TimeStamps_var *edt= new EventLog::TimeStamps_var[len-1];
  RWTValHashTable<EventWrapper> base_ev( hashEvent);
  for (CORBA::ULong xi=0; xi<src_el_len; xi++) {
    base_ev.insert( EventWrapper(evlogs[0][xi], xi));
  }

  {
    for (unsigned i=1; i<len; i++) {
      CORBA::ULong el_len= evlogs[i]->length();
      edt[i-1]= new EventLog::TimeStamps(src_el_len);
      edt[i-1]->length(src_el_len);
      for (CORBA::ULong xd=0; xd<src_el_len; xd++)
	edt[i-1][xd].seconds= edt[i-1][xd].microseconds= 0;
      
      for (CORBA::ULong xj=0; xj<el_len; xj++) {

	if (xj>0) 
	  if (!match_all_ids && evlogs[i][xj-1] == evlogs[i][xj]) continue;

	// find corresponding event
	CORBA::ULong xf;
	EventWrapper ew;
	if (base_ev.find( EventWrapper(evlogs[i][xj], 0), ew))
	  xf= ew.id;
	else xf= src_el_len;

	/*
	for (xf=0; xf<src_el_len; xf++)
	  if (evlogs[i][xj] == evlogs[0][xf])  break;
	*/

	if (xf != src_el_len) {
	  if (evlogs[i][xj].receive_ts.microseconds < 
	      evlogs[0][xf].receive_ts.microseconds) {
	    evlogs[i][xj].receive_ts.microseconds+= 1000000;
	    evlogs[i][xj].receive_ts.seconds--;
	  }
	  edt[i-1][xf].microseconds= 
	    evlogs[i][xj].receive_ts.microseconds-
	    evlogs[0][xf].receive_ts.microseconds;
	  edt[i-1][xf].seconds=
	    evlogs[i][xj].receive_ts.seconds-
	    evlogs[0][xf].receive_ts.seconds;
	}
	//
      }
    }
  }
  for (unsigned i=1; i<len; i++) {
    for (CORBA::ULong xf=0; xf<src_el_len; xf++) {
      CORBA::ULong t= edt[i-1][xf].seconds*1000000+edt[i-1][xf].microseconds;
      if (t) cout << (unsigned long) t << endl;
    }
  }
  delete []logs;
  delete []evlogs;
  delete []edt;
}

void get_length_event_log( const char *names) {
  unsigned len;
  EventLog_var *logs= log_refs( names, len);
  for (unsigned i=0; i<len; i++)
    cout << logs[i]->max_length() << endl;
  delete []logs;
}

void set_length_event_log( const char *names, char ***vals, int *nvals) {
  unsigned len;
  EventLog_var *logs= log_refs( names, len);
  (*vals)+= 2;
  (*nvals)-= 2;
  int rd= 1;
  CORBA::ULong l= atol( **vals);
  for (unsigned i=0; i<len; i++) {
    if (*nvals > 0 && rd) {
      CORBA::ULong xl= atol( **vals);
      if (xl==0) rd= 0;
      else {
	l= xl;
	(*vals)++;
	(*nvals)--;
      }
    }
    logs[i]->max_length( l);
  }
}

int main( int argc, char **argv) {
  CORBA::ORB_ptr orb_ptr= CORBA::ORB_init( argc, argv, ORB_IDENTIFIER);
  CORBA::BOA_ptr boa_ptr= orb_ptr->BOA_init( argc, argv, BOA_IDENTIFIER);

#ifdef ORBIX
  orb_ptr->setDiagnostics( 0);
#endif

  char *prog_name= argv[0];
  if (argc<2) usage( prog_name);

  Bootstrap::init( orb_ptr);
  
  argc--;
  argv++;
  while (argc>1) {
    if (!strcmp(*argv,"-s")) {
      if (argc<=1) usage( prog_name);
      start_event_log( argv[1]);
      argc-=2;
      argv+=2;
    }
    else if (!strcmp(*argv,"-r")) {
      if (argc<=1) usage( prog_name);
      reset_event_log( argv[1]);
      argc-=2;
      argv+=2;
    }
    else if (!strcmp(*argv,"-t")) {
      if (argc<=1) usage( prog_name);
      stop_event_log( argv[1]);
      argc-=2;
      argv+=2;
    }
    else if (!strcmp(*argv,"-g")) {
      if (argc<=1) usage( prog_name);
      get_length_event_log( argv[1]);
      argc-=2;
      argv+=2;
    }
    else if (!strcmp(*argv,"-l")) {
      if (argc<=2) usage( prog_name);
      set_length_event_log( argv[1], &argv, &argc);
    }
    else if (!strcmp(*argv,"-c")) {
      if (argc<=1) usage( prog_name);
      compare_event_log( argv[1]);
      argc-=2;
      argv+=2;
    }
    else if (!strcmp(*argv,"-i")) {
      if (argc<=1) usage( prog_name);
      list_event_log( argv[1]);
      argc-=2;
      argv+=2;
    }
    else if (!strcmp(*argv,"-d")) {
      if (argc<=1) usage( prog_name);
      duration_event_log( argv[1]);
      argc-=2;
      argv+=2;
    }
    else if (!strcmp(*argv,"-n")) {
      match_source_name= 1;
      argc--;
      argv++;
    }
    else if (!strcmp(*argv,"-a")) {
      match_all_ids= 1;
      argc--;
      argv++;
    }
    else {
      cerr << *argv << endl;
      usage( prog_name);
    }
  }
  return 0;
}
