// $Id: regtest.cc,v 1.1 1998/11/30 14:36:00 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/utils/event_log_util/regtest.cc,v $
//
// $Log: regtest.cc,v $
// Revision 1.1  1998/11/30 14:36:00  mencnar
// Initial revision
//
//

#include <iostream.h>
#include <strstream.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <limits.h>
#include <signal.h>
#include <rw/tvdlist.h>
#include "abng_poller.hh"
#include "domain_rep_management.hh"
#include "abng_core_management.hh"
#include "bootstrap_n.hh"
#include "abng_core_types.h"
#include "db_utils.hh"

// Parameters:
//   badge range:
//    each badge will be added to Badge Repository
//    for each badge there will be created a user in User Repository
//    for each badge there will be created a badge holder in Badge Holder Rep.
//    for each badge there will be created a locatable in Locatable Repository
//
//   sensor range:
//    each sensor will be added to Sensor Repository
//    for each sensor there will be created an entry in Location Description Rep.
//    for each sensor there will be created an abng_location_description object
//    for each sensor there will be created a location in Location Repository
//



int operator == ( const ABng_Poller::Simple_AB_Poller::Octets &first,
		  const ABng_Poller::Simple_AB_Poller::Octets &second) {
  if (first.length() != second.length()) return 0;
  int size= first.length();
  for (int i=0; i<size; i++)
    if (first[i] != second[i]) return 0;
  return 1;
}

CORBA::ORB_ptr orb_ptr;
CORBA::BOA_ptr boa_ptr;
char *poller_name= NULL;
char *view_name= NULL;
int remove_mode= 0;

ABng_Poller::Simple_AB_Poller::Octets min_badge;
ABng_Poller::Simple_AB_Poller::Octets max_badge;

ABng_Poller::AB_Poller::Network_Id min_network_id;
ABng_Poller::AB_Poller::Station_Id min_station_id;
ABng_Poller::AB_Poller::Network_Id max_network_id;
ABng_Poller::AB_Poller::Station_Id max_station_id;

ABng_Core_System::Badge_Ids bids;
ABng_Core_System::Badges badges;
ABng_Core_System::Sensor_Ids sids;
ABng_Core_System::Sensors sensors;

Domain_Repository::Users users;
Domain_Repository::Location_Descriptions ldescrs;
ABng_Core_System::Badge_Holders bholders;
ABng_Core_System::ABng_Location_Descriptions alds;
ABng_Core::Locatables locatables;
ABng_Core::Locations locations;

ABng_Core_Management::Badge_Rep_var brep;
ABng_Core_Management::Sensor_Rep_var srep;
Domain_Repository::User_Rep_var urep;
Domain_Repository_Management::User_Rep_Manager_var urepman;
Domain_Repository::Location_Description_Rep_var ldrep;
Domain_Repository_Management::Location_Description_Rep_Manager_var ldrepman;
ABng_Core_Management::Badge_Holder_Rep_var bhrep;
ABng_Core_Management::ABng_Location_Description_Rep_var aldrep;
ABng_Core_Management::View_Manager_var viewman;
ABng_Core::View_var view;


void usage( const char *prog) {
  cout << "usage: " << prog << endl
       << "\t-poller_name name" << endl
       << "\t-view_name name" << endl
       << "\t-min_badge badge_id -max_badge badge_id" << endl
       << "\t-min_sensor group_id station_id -max_sensor group_id station_id" << endl
       << "\t-remove" << endl;
  exit(1);
}

void parse_params( char *prog_name, istream &istr) {
  char opt_name[80];

  while (istr >> opt_name) {
    if (!strcmp(opt_name,"-poller_name")) {
      istr >> opt_name;
      poller_name= strdup(opt_name);
    }
    else if (!strcmp(opt_name,"-view_name")) {
      istr >> opt_name;
      view_name= strdup(opt_name);
    }
    else if (!strcmp(opt_name, "-min_badge")) {
      if (!(istr >> min_badge)) usage( prog_name);
    }
    else if (!strcmp(opt_name, "-max_badge")) {
      if (!(istr >> max_badge)) usage( prog_name);
    }
    else if (!strcmp(opt_name, "-min_sensor")) {
      unsigned long nid,sid;
      istr >> nid >> sid;
      if (!istr) usage( prog_name);
      min_network_id= nid;
      min_station_id= sid;
    }
    else if (!strcmp(opt_name, "-max_sensor")) {
      unsigned long nid,sid;
      istr >> nid >> sid;
      if (!istr) usage( prog_name);
      max_network_id= nid;
      max_station_id= sid;
    }
    else if (!strcmp(opt_name, "-remove")) {
      remove_mode= 1;
    }
    else {
      cerr << "Unknown option " << opt_name << endl;
      exit(1);
    }
  }
}

void update_badge() {
  cerr << "Updating Badge_Rep..." << endl;
  badges.length( bids.length());
  for (CORBA::ULong i=0; i<bids.length(); i++) {
    cerr << "\b\b\b\b\b\b\b\b" << i;
    ABng_Core_System::Badge::Pattern p( Badge_Types::init_pattern());
    p.is_any= 0;
    p.id_pattern.is_any= 0;
    p.id_pattern.value= bids[i];
    ABng_Core_System::Badges_var blist= brep->find( p);
    if (blist->length() == 0) {
      if (remove_mode) continue;
      ABng_Core_System::Badge::Description bdesc;
      bdesc.id= bids[i];
      badges[i]= brep->add( bdesc);
    }
    else {
      // assume length is 1
      if (remove_mode) {
	brep->remove( blist[0]);
      }
      else badges[i]= blist[0];
    }
  }
  cerr << "..done" << endl;
}

void update_sensor() {
  cerr << "Updating Sensor_Rep..." << endl;
  sensors.length( sids.length());
  for (CORBA::ULong i=0; i<sids.length(); i++) {
    cerr << "\b\b\b\b\b\b\b\b" << i;
    ABng_Core_System::Sensor::Pattern p( Sensor_Types::init_pattern());
    p.is_any= 0;
    p.id_pattern.is_any= 0;
    p.id_pattern.sensor_num_pattern.is_any= 0;
    p.id_pattern.sensor_num_pattern.value= sids[i].sensor_num;
    p.id_pattern.network_num_pattern.is_any= 0;
    p.id_pattern.network_num_pattern.value= sids[i].network_num;
    p.id_pattern.network_grp_name_pattern.is_any= 0;
    p.id_pattern.network_grp_name_pattern.value= sids[i].group_name;

    ABng_Core_System::Sensors_var slist= srep->find( p);
    if (slist->length() == 0) {
      if (remove_mode) continue;
      ABng_Core_System::Sensor::Description sdesc;
      sdesc.id= sids[i];
      sensors[i]= srep->add( sdesc);
    }
    else {
      // assume length is 1
      if (remove_mode) {
	srep->remove( slist[0]);
      }
      else sensors[i]= slist[0];
    }
  }
  cerr << "..done" << endl;
}

void update_user() {
  cerr << "Updating User_Rep..." << endl;

  users.length( bids.length());

  for (CORBA::ULong i=0; i<bids.length(); i++) {
    ostrstream ostr;
    ostr << "USR_" << poller_name << '_';
    for (CORBA::ULong xi=0; xi<bids[i].length(); xi++)
      ostr << hex << (unsigned) bids[i][xi] << '_';
    ostr << '\0';
    char *buf= ostr.str();

    cerr << "\b\b\b\b\b\b\b\b" << i;
    Domain_Repository::User::Pattern p( User_Types::init_pattern());
    p.is_any= 0;
    p.uid_pattern.is_any= 0;
    p.uid_pattern.value= (const char *)buf;

    Domain_Repository::Users_var ulist= urep->find( p);
    if (ulist->length() == 0) {
      if (remove_mode) continue;
      Domain_Repository::User::Description udesc;
      udesc.uid= (const char*) buf;
      udesc.forename= (const char *) buf;
      users[i]= urepman->add( udesc);
    }
    else {
      // assume length is 1
      if (remove_mode) {
	urepman->remove( ulist[0]);
      }
      else users[i]= ulist[0];
    }
    ostr.rdbuf()->freeze(0);
  }
  cerr << "..done" << endl;
}

void update_badge_holder() {
  cerr << "Updating Badge_Holder_Rep..." << endl;

  bholders.length( bids.length());

  for (CORBA::ULong i=0; i<bids.length(); i++) {
    cerr << "\b\b\b\b\b\b\b\b" << i;
    ABng_Core_System::Badge_Holder::Pattern p( Badge_Holder_Types::init_pattern());
    p.is_any= 0;
    p.current_badge_pattern.is_any= 0;
    p.current_badge_pattern.item_id_pattern.is_any= 1;
    p.current_badge_pattern.id_pattern.is_any= 0;
    p.current_badge_pattern.id_pattern.value= bids[i];

    ABng_Core_System::Badge_Holders_var bhlist= bhrep->find( p);

    if (bhlist->length() == 0) {
      if (remove_mode) continue;
      ABng_Core_System::Badge_Holder::Description bhdesc;
      bhdesc.my_badge= bhdesc.current_badge= badges[i];
      bhdesc.type_specific_object.user_obj( users[i]);
      bhdesc.assigned= (CORBA::Boolean) 1;
      bhdesc.visitor= (CORBA::Boolean) 0;
      bholders[i]= bhrep->add( bhdesc);
    }
    else {
      // assume length is 1
      if (remove_mode) {
	bhrep->remove( bhlist[0]);
      }
      else bholders[i]= bhlist[0];
    }
  }
  cerr << "..done" << endl;
}

void update_location_description() {
  cerr << "Updating Location_Description_Rep..." << endl;

  ldescrs.length( sids.length());

  for (CORBA::ULong i=0; i<bids.length(); i++) {
    ostrstream ostr;
    ostr << "R_" << (const char *) sids[i].group_name
	 << '_' << sids[i].network_num
	 << '_' << sids[i].sensor_num
	 << '\0';
    char *buf= ostr.str();

    cerr << "\b\b\b\b\b\b\b\b" << i;
    
    Domain_Repository::Location_Description::Pattern p(
       			      Location_Description_Types::init_pattern());
    p.is_any= 0;
    p.type_name_pattern.is_any= 0;
    p.type_name_pattern.value= (const char *) "room";

    Domain_Repository::Property_Patterns pp;
    pp.length(1);
    pp[0].is_any= 0;
    pp[0].name_pattern.is_any= 0;
    pp[0].name_pattern.value= (const char*) "number";
#ifdef ORBIX
    const char *fs= buf;
#else
    CORBA::Any::from_string fs( buf, 0);
#endif
    pp[0].value_pattern.is_any= 0;
    pp[0].value_pattern.value <<= fs;
    p.property_list_pattern.is_any= 0;
    p.property_list_pattern.value= pp;
    

    Domain_Repository::Location_Descriptions_var ldlist= ldrep->find(p);

    if (ldlist->length() == 0) {
      if (remove_mode) continue;
      Domain_Repository::Location_Description::Description lddesc;
      lddesc.type_name= (const char *) "room";
      lddesc.property_list.length(6);

      lddesc.property_list[0].name= (const char *) "number";
#ifdef ORBIX
      const char *fs0= buf;
#else
      CORBA::Any::from_string fs0( buf, 0);
#endif
      lddesc.property_list[0].value <<= fs0;

      lddesc.property_list[1].name= (const char *) "description";
#ifdef ORBIX
      const char *fs1= "";
#else
      CORBA::Any::from_string fs1( "", 0);
#endif
      lddesc.property_list[1].value <<= fs1;

      lddesc.property_list[2].name= (const char *) "phones";
      Domain_Repository::Strings fs2;
      lddesc.property_list[2].value <<= fs2;
      
      lddesc.property_list[3].name= (const char *) "people";
      Domain_Repository::Ulongs fs3;
      lddesc.property_list[3].value <<= fs3;

      lddesc.property_list[4].name= (const char *) "equipment";
      lddesc.property_list[4].value <<= fs3;

      lddesc.property_list[5].name= (const char *) "color";
#ifdef ORBIX
      const char *fs5= "";
#else
      CORBA::Any::from_string fs5( "", 0);
#endif
      lddesc.property_list[5].value <<= fs5;
      
      ldescrs[i]= ldrepman->add( lddesc);
    }
    else {
      // assume length is 1
      if (remove_mode) {
	ldrepman->remove( ldlist[0]);
      }
      else ldescrs[i]= ldlist[0];
    }
    ostr.rdbuf()->freeze(0);
  }
  cerr << "..done" << endl;
}

void update_abng_location_description() {
  cerr << "Updating ABng_Location_Description_Rep..." << endl;

  alds.length( sids.length());

  for (CORBA::ULong i=0; i<sids.length(); i++) {
    ostrstream ostr;
    ostr << "R_" << (const char *) sids[i].group_name
	 << '_' << sids[i].network_num
	 << '_' << sids[i].sensor_num
	 << '\0';
    char *buf= ostr.str();

    cerr << "\b\b\b\b\b\b\b\b" << i;
    ABng_Core_System::ABng_Location_Description::Pattern p( ABng_Location_Description_Types::init_pattern());


    Domain_Repository::Location_Description::Pattern ldp(
       			      Location_Description_Types::init_pattern());
    ldp.is_any= 0;
    ldp.type_name_pattern.is_any= 0;
    ldp.type_name_pattern.value= (const char *) "room";

    Domain_Repository::Property_Patterns pp;
    pp.length(1);
    pp[0].is_any= 0;
    pp[0].name_pattern.is_any= 0;
    pp[0].name_pattern.value= (const char*) "number";
#ifdef ORBIX
    const char *fs= buf;
#else
    CORBA::Any::from_string fs( buf, 0);
#endif
    pp[0].value_pattern.is_any= 0;
    pp[0].value_pattern.value <<= fs;
    ldp.property_list_pattern.is_any= 0;
    ldp.property_list_pattern.value= pp;

    p.is_any= 0;
    p.loc_descr_pattern= ldp;

    ABng_Core_System::ABng_Location_Descriptions_var aldlist= aldrep->find( p);

    if (aldlist->length() == 0) {
      if (remove_mode) continue;
      ABng_Core_System::ABng_Location_Description::Description alddesc;
      ABng_Core_System::Sensors slist(1);
      slist.length(1);
      slist[0]= sensors[i];

      alddesc.loc_descr= ldescrs[i];
      alddesc.sensor_list= slist;
      alds[i]= aldrep->add( alddesc);
    }
    else {
      // assume length is 1
      if (remove_mode) {
	aldrep->remove( aldlist[0]);
      }
      else alds[i]= aldlist[0];
    }
    ostr.rdbuf()->freeze(0);
  }
  cerr << "..done" << endl;  
}

void update_location() {
  cerr << "Updating Location_Rep..." << endl;

  locations.length( sids.length());

  for (CORBA::ULong i=0; i<sids.length(); i++) {
    ostrstream ostr;
    ostr << "R_" << (const char *) sids[i].group_name
	 << '_' << sids[i].network_num
	 << '_' << sids[i].sensor_num
	 << '\0';
    char *buf= ostr.str();

    cerr << "\b\b\b\b\b\b\b\b" << i;

    ABng_Core::Location::Pattern p;
    p.is_any= 0;
    p.item_id_pattern.is_any= 1;
    
    Domain_Repository::Location_Description::Pattern ldp(
       			      Location_Description_Types::init_pattern());
    ldp.is_any= 0;
    ldp.type_name_pattern.is_any= 0;
    ldp.type_name_pattern.value= (const char *) "room";

    Domain_Repository::Property_Patterns pp;
    pp.length(1);
    pp[0].is_any= 0;
    pp[0].name_pattern.is_any= 0;
    pp[0].name_pattern.value= (const char*) "number";
#ifdef ORBIX
    const char *fs= buf;
#else
    CORBA::Any::from_string fs( buf, 0);
#endif
    pp[0].value_pattern.is_any= 0;
    pp[0].value_pattern.value <<= fs;
    ldp.property_list_pattern.is_any= 0;
    ldp.property_list_pattern.value= pp;
    p.loc_descr_pattern= ldp;

    ABng_Core::Locations_var loclist= view->find_location( p);

    if (loclist->length() == 0) {
      if (remove_mode) continue;
      locations[i]= viewman->add_location( alds[i]);
    }
    else {
      // assume length is 1
      if (remove_mode) {
	viewman->remove_location( loclist[0]);
      }
      else locations[i]= loclist[0];
    }    

    ostr.rdbuf()->freeze(0);
  }
  cerr << "..done" << endl;
}

void update_locatable() {
  cerr << "Updating Locatable_Rep..." << endl;

  locatables.length( bids.length());

  for (CORBA::ULong i=0; i<bids.length(); i++) {
    ostrstream ostr;
    ostr << "USR_" << poller_name << '_';
    for (CORBA::ULong xi=0; xi<bids[i].length(); xi++)
      ostr << hex << (unsigned) bids[i][xi] << '_';
    ostr << '\0';
    char *buf= ostr.str();

    cerr << "\b\b\b\b\b\b\b\b" << i;

    ABng_Core::Locatable::Pattern p;
    p.is_any= 0;
    p.item_id_pattern.is_any= 1;
    Domain_Repository::User::Pattern up( User_Types::init_pattern());
    up.is_any= 0;
    up.uid_pattern.is_any= 0;
    up.uid_pattern.value= (const char *)buf;
    p.type_specific_object_pattern.is_any= 0;
    p.type_specific_object_pattern.object_pattern.user_obj_pattern( up);

    p.assigned_pattern.is_any= 1;
    p.visitor_pattern.is_any= 1;
    
    Domain_Repository::Location_Description::Pattern ldp(
       			      Location_Description_Types::init_pattern());
    p.home_location_pattern.is_any= 1;
    p.home_location_pattern.loc_descr_pattern= ldp;

    ABng_Core::Locatables_var loclist;
    try {
      loclist= view->find_locatable( p);
    }
    catch (...) {
      cerr << "find_locatable" << endl;
      throw;
    }

    if (loclist->length() == 0) {
      if (remove_mode) continue;
      try {
	locatables[i]= viewman->add_locatable( bholders[i]);
      }
      catch (Domain_Repository::Duplicate_Data_Record &ex) {
	cerr << "add_locatable duplicated" << endl;
      }
      catch (...) {
	cerr << "add_locatable" << endl;
      }
    }
    else {
      // assume length is 1
      if (remove_mode) {
	viewman->remove_locatable( loclist[0]);
      }
      else locatables[i]= loclist[0];
    }    

    ostr.rdbuf()->freeze(0);
  }
  cerr << "..done" << endl;
}

int main( int argc, char **argv) {
  orb_ptr= CORBA::ORB_init( argc, argv, ORB_IDENTIFIER);
  boa_ptr= orb_ptr->BOA_init( argc, argv, BOA_IDENTIFIER);

#ifdef ORBIX
  orb_ptr->setDiagnostics( 0);
#endif

  Bootstrap::init(orb_ptr);

  ostrstream params;

  if (argc<2) usage( argv[0]);
  if (strcmp(argv[1],"-")) {
    for (int i=1; i<argc; i++)
      params << argv[i] << ' ';
    params << '\0';

    istrstream istr( params.str());
    parse_params( argv[0], istr);
    params.rdbuf()->freeze(0);
  }
  else {
    parse_params( argv[0], cin);
  }

  if (poller_name == NULL) usage( argv[0]);
  if (view_name == NULL) usage( argv[0]);

  if (min_badge.length() != max_badge.length() || min_badge.length() == 0)
    usage( argv[0]);
  
  for (unsigned xb=0; xb<min_badge.length(); xb++)
    if (min_badge[xb] > max_badge[xb]) {
      cout << "Invalid badge range" << endl;
      usage( argv[0]);
    } else if (min_badge[xb] < max_badge[xb]) break;
  
  {
    ABng_Poller::Simple_AB_Poller::Octets cbadge( min_badge);

    unsigned xi= 0;
    while (!(cbadge == max_badge)) {
      ++xi;
      for (int i=cbadge.length()-1; i>=0; --i) {
	if (cbadge[i] == 0xff)
	  cbadge[i]= 0;
	else {
	  ++cbadge[i];
	  break;
	}
      }
    }
    CORBA::ULong num_badges= xi;

    bids= ABng_Core_System::Badge_Ids(num_badges);
    bids.length(num_badges);
    
    xi= 0;
    cbadge= min_badge;
    while (!(cbadge == max_badge)) {
      bids[xi++]= cbadge;
      for (int i=cbadge.length()-1; i>=0; --i) {
	if (cbadge[i] == 0xff)
	  cbadge[i]= 0;
	else {
	  ++cbadge[i];
	  break;
	}
      }
    }  
  }
  
  unsigned xi=0;
  {
    ABng_Poller::AB_Poller::Network_Id nid= min_network_id;
    ABng_Poller::AB_Poller::Station_Id sid= min_station_id;

    while (nid != max_network_id || sid != max_station_id) {
      ++xi;
      if (sid == 0xff) {
	sid= 0;
	++nid;
      }
      else ++sid;
    }
  }
     
  CORBA::ULong num_stations= xi;
  
  sids= ABng_Core_System::Sensor_Ids(num_stations);
  sids.length(num_stations);
  xi= 0;
  ABng_Poller::AB_Poller::Network_Id nid= min_network_id;
  ABng_Poller::AB_Poller::Station_Id sid= min_station_id;
  
  while (nid != max_network_id || sid != max_station_id) {
    if (sid == 0xff) {
      sid= 0;
      ++nid;
    }
    else ++sid;
    sids[xi].network_num= nid;
    sids[xi].sensor_num= sid;
    sids[xi++].group_name= (const char *) poller_name;
  }

  cerr << "Resolving repositories" << endl;

  cerr << "Badge_Rep...";
  try {
    brep = ABng_Core_Management::Badge_Rep::_narrow( Bootstrap::resolve( Bootstrap::SRV_Badge_Repository()));
  }
  catch (CORBA::Exception &ex) {
    cerr << ex << endl;
    exit(1);
  }
  cerr << "done" << endl;
  
  cerr << "Sensor_Rep...";
  try {
    srep = ABng_Core_Management::Sensor_Rep::_narrow( Bootstrap::resolve( Bootstrap::SRV_Sensor_Repository()));
  }
  catch (CORBA::Exception &ex) {
    cerr << ex << endl;
    exit(1);
  }
  cerr << "done" << endl;
  
  cerr << "User_Rep...";
  try {
    urepman = Domain_Repository_Management::User_Rep_Manager::_narrow( Bootstrap::resolve( Bootstrap::SRV_User_Repository_Manager()));
    urep= urepman->rep();
  }
  catch (CORBA::Exception &ex) {
    cerr << ex << endl;
    exit(1);
  }
  cerr << "done" << endl;

  cerr << "Location_Description_Rep...";
  try {
    ldrepman = Domain_Repository_Management::Location_Description_Rep_Manager::_narrow( Bootstrap::resolve( Bootstrap::SRV_Location_Description_Repository_Manager()));
    ldrep= ldrepman->rep();
  }
  catch (CORBA::Exception &ex) {
    cerr << ex << endl;
    exit(1);
  }
  cerr << "done" << endl;

  cerr << "Badge_Holder_Rep...";
  try {
    bhrep = ABng_Core_Management::Badge_Holder_Rep::_narrow( Bootstrap::resolve( Bootstrap::SRV_Badge_Holder_Repository()));
  }
  catch (CORBA::Exception &ex) {
    cerr << ex << endl;
    exit(1);
  }
  cerr << "done" << endl;
  
  cerr << "ABng_Location_Description_Rep...";
  try {
    aldrep = ABng_Core_Management::ABng_Location_Description_Rep::_narrow( Bootstrap::resolve( Bootstrap::SRV_ABng_Location_Description_Repository()));
  }
  catch (CORBA::Exception &ex) {
    cerr << ex << endl;
    exit(1);
  }
  cerr << "done" << endl;
  
  cerr << "View...";
  try {
    viewman = ABng_Core_Management::View_Manager::_narrow( Bootstrap::resolve( Bootstrap::CTX_ABng_Core_Management_View_Managers()+view_name));
    view= viewman->my_view();
  }
  catch (CORBA::Exception &ex) {
    cerr << ex << endl;
    exit(1);
  }
  cerr << "done" << endl;

  try {
    update_badge();
    update_sensor();
    update_user();
    update_location_description();
    
    if (!remove_mode) {
      update_badge_holder();
      update_abng_location_description();
      update_location();
      update_locatable();
    }
    // else: badge holders will be deleted as notifies for users removed
    // propagates to badge holder repository and locatable repository
    //

  }
  catch (Domain_Repository::Destination_Unreachable &du) {
    cerr << "Destination unreachable " << du.server_name << endl;
    exit(1);
  }
  catch (CORBA::Exception &ex) {
    cerr << ex << endl;
    exit(1);
  }
  return 0;
}




