// $Id: view_mgr_cl.cc,v 1.2 1998/11/30 14:18:28 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/utils/sighting_processors_util/view_mgr_cl.cc,v $
//
// $Log: view_mgr_cl.cc,v $
// Revision 1.2  1998/11/30 14:18:28  mencnar
// Added support for listing dynamic observers
//
// Revision 1.1.1.1  1998/06/18 14:44:37  mencnar
//
// Revision 1.1  1998/03/30 08:02:10  mencnar
// Initial revision
//
//
//----------------------------------------------------------------
// File:          view_mgr_cl.cc
// Author:        Daniel Mencnarowski (mencnar@ics.agh.edu.pl)
// Date Created:  23.03.98
//
// General Description:
//   Test client, manages a view
//


#include <rw/rwtime.h>
#include <rw/rwdate.h>
#include "abng_core_management.hh"
#include "bootstrap_n.hh"
#include <stdlib.h>
#include <iostream.h>



ABng_Core_Management::View_Manager_var view_manager;
ABng_Core::View_var view;
ABng_Core_Management::Badge_Holder_Rep_var bh_repository;
ABng_Core_System::Badge_Holder_var bh;
ABng_Core_Management::ABng_Location_Description_Rep_var abng_ld_repository;
ABng_Core_System::ABng_Location_Description_var abng_ld;

void usage( char *prog_name) {
  cout << prog_name << endl
       << "\t-v view_name" << endl
       << "\t-a abng_location_description_item_id\t-add location" << endl
       << "\t-y location_item_id\t\t\t-remove location" << endl
       << "\t-b badge_holder_item_id\t\t\t-add locatable" << endl
       << "\t-z locatable_item_id\t\t\t-remove locatable" << endl
       << "\t-t locatable_item_id\t\t\t-get sighting" << endl
       << "\t-r \t\t\t\t\t-list users of a view" << endl
       << "\t-u sighting_period\t\t\t-hours since last sighting" << endl
       << "\t-s location_item_id\t\t\t-get sightings" << endl
       << "\t-x                 \t\t\t-get sightings (view method)" << endl
       << "\t-l \t\t\t\t\t-list view content" << endl
       << "\t-o \t\t\t\t\t-list view observers" << endl
       << "\t-q burst_delay_time\t\t\t-set burst delay time" << endl
       << "\t-w \t\t\t\t\t-get burst delay time" << endl
       << "\t-c \t\t\t\t\t-get sighting channel" << endl
       << "\t-e IOR\t\t\t\t-set sighting channel" << endl;
}

ostream & operator << ( ostream &ostr, 
			const ABng_Core::Time &time) {

  RWTime offset( RWDate( 1, 1,1970), 0,0,0, RWZone::utc());
  RWTime t( offset.seconds()+time.seconds);
  ostr << t << " " << time.microseconds << " us (local time)";
  return ostr;
}

ostream & show_any( ostream &ostr, CORBA::Any &any_value) {
  switch ( any_value.type()->kind()) {
  case CORBA::tk_string:
    {
#ifdef OMNIORB
      char *str= NULL;
      CORBA::Any::to_string ts( str, any_value.type()->length());
      any_value >>= ts;
      if (str) {
	ostr << str;
	CORBA::string_free( str);
      }
#else
      char * str= NULL;
      any_value >>= str;
      ostr << (const char*) str;
#endif
    }
    break;
  case CORBA::tk_long:
    {
      CORBA::Long l;
      any_value >>= l;
      ostr << l;
    }
    break;
  case CORBA::tk_ulong:
    {
      CORBA::ULong l;
      any_value >>= l;
      ostr << l;
    }
    break;   
  case CORBA::tk_short:
    {
      CORBA::Short s;
      any_value >>= s;
      ostr << s;
    }
    break;  
  case CORBA::tk_sequence:
#ifdef OMNIORB
    {
      CORBA::ULong seqlen= any_value.type()->length();
      
      if (any_value.type()->content_type()->kind() == CORBA::tk_ulong) {
	Domain_Repository::Repository_Item_Ids *pids= NULL;
	if (any_value >>= pids) ;
	else ostr << "FAILED";
	ostr << "@" << seqlen << " ";
	for (int i=0; i< pids->length(); i++)
	  ostr << (*pids)[i] << ' ';
      }
      else if (any_value.type()->content_type()->kind() == CORBA::tk_string) {
	ABng_Server_Manager::Name *pids= NULL;
	if (any_value >>=pids) ;
	else ostr << "FAILED";
	ostr << "@" << seqlen << " ";
	for (int i=0; i<pids->length(); i++)
	  ostr << (*pids)[i] << ' ';
      }
    }
    break;
#else

  {  // Orbix specific
    if(any_value.type()->equal(Domain_Repository::_tc_Ulongs)) {
      Domain_Repository::Ulongs* v = 
	(Domain_Repository::Ulongs*)any_value.value();
      for(int i = 0; i < v->length(); i ++) ostr << (*v)[i] << " ";
      break;
    }
    if(any_value.type()->equal(Domain_Repository::_tc_Strings)) {
      Domain_Repository::Strings* v = 
	(Domain_Repository::Strings*)any_value.value();
      for(int i = 0; i < v->length(); i ++) ostr << (*v)[i] << " ";
      break;
    }
  }
  break;
#endif

  default:
    ostr << "Unsupported type code kind " << (int) any_value.type()->kind();
    break;
  }
  return ostr;
}

ABng_Core::Locatable::Pattern get_locatable_pattern( int item_id) {
  ABng_Core::Locatable::Pattern loc_pattern;
  Domain_Repository::User::Pattern user_pattern;

  loc_pattern.is_any= 0;
  loc_pattern.item_id_pattern.is_any= 0;
  loc_pattern.item_id_pattern.value= item_id;
  loc_pattern.home_location_pattern.is_any= 1;
  loc_pattern.home_location_pattern.loc_descr_pattern.property_list_pattern.constraints=
    Domain_Repository::UNRESTRICTED;
  loc_pattern.type_specific_object_pattern.is_any= 1;
  loc_pattern.type_specific_object_pattern.object_pattern.user_obj_pattern( user_pattern);
  return loc_pattern;
}

ostream & operator << ( ostream &ostr, const ABng_Core::Sighting_Description &sd) {
  ostr << "\tview_name\t\t" << sd.view_name << endl;
  ostr << "\tmy_locatable\t\t" << sd.my_locatable << endl;
  ostr << "\tmy_locatable_type\t";

  switch (sd.my_locatable_type) {
  case ABng_Core::LOCATABLE_USER:
    ostr << "LOCATABLE_USER" << endl << flush;
    try {
      ABng_Core::Locatable::Pattern loc_pattern= get_locatable_pattern( sd.my_locatable);
      ABng_Core::Locatables_var ret_val( view->find_locatable( loc_pattern));
      if (ret_val->length() != 1) {
	cerr << "\t\t" << ret_val->length() << " locatable(s) found" << endl;
      }
      cout << "Description :" << flush;
      ABng_Core::Locatable::Value_Description_var val_descr= ret_val[0]->value_descr();
      Domain_Repository::User::Value_Description &user_descr=
	val_descr->type_specific_object_value.user_description();

      ostr << "\t(" << user_descr.uid
	   << ") " << user_descr.forename << " " << user_descr.surname
	   << endl;
    }
    catch (Domain_Repository::Destination_Unreachable &ex) {
      cerr << ex << " " << ex.server_name << endl;
    }
    catch (CORBA::Exception &ex) {
      cerr << "Caught exception " << ex << endl;
    }
    break;
  case ABng_Core::LOCATABLE_EQUIPMENT:
    ostr << "LOCATABLE_EQUIPMENT" << endl;
    break;
  default:
    ostr << "unknown" << endl;
    break;
  }
  ostr << "\tmy_location\t\t" << sd.my_location << endl << flush;
  try {
    ABng_Core::Location::Pattern loc_pattern;
    loc_pattern.is_any= 0;
    loc_pattern.item_id_pattern.is_any= 0;
    loc_pattern.item_id_pattern.value= sd.my_location;
    loc_pattern.loc_descr_pattern.is_any= 1;
    loc_pattern.loc_descr_pattern.property_list_pattern.constraints=
      Domain_Repository::UNRESTRICTED;

    ABng_Core::Locations_var ret_val( view->find_location( loc_pattern));
    if (ret_val->length() != 1) {
	cerr << "\t\t" << ret_val->length() << " location(s) found" << endl;
    }
    else {
      cout << "Description :" << endl << flush;
      ABng_Core::Location::Value_Description_var val_descr= ret_val[0]->value_descr();
      cout << "\t\tlocation type name " << val_descr->loc_value_descr.type_name << endl;
      for (int i=0; i<val_descr->loc_value_descr.property_list.length(); i++ ) {
	cout << "\t\t#" << i << " " << val_descr->loc_value_descr.property_list[i].name << " ";
	show_any( cout, val_descr->loc_value_descr.property_list[i].value) << endl;
      }
    }
  }
  catch (Domain_Repository::Destination_Unreachable &ex) {
    cerr << ex << " " << ex.server_name << endl;
  }
  catch (CORBA::Exception &ex) {
    cerr << "Caught exception " << ex << endl;
  }
  
  ostr << "\tmessage_type (hex) " << hex << (unsigned) sd.message_type << endl;
  ostr << "\tattributes (hex) " << hex << (unsigned) sd.attributes << endl;
  ostr << "\tsequence_number (hex) " << hex << (unsigned) sd.sequence_number << endl;
  ostr << dec << "\tupdate_time\t\t" << sd.update_time << endl;
  ostr << "\tquality\t\t" << (unsigned) sd.q << endl;
  return ostr;
}

int main( int argc, char **argv) {
  CORBA::ORB_ptr orb= CORBA::ORB_init( argc, argv, ORB_IDENTIFIER);
#ifdef ORBIX
  orb->setDiagnostics(0);
#endif
  Bootstrap::init( orb);

  int c;
  CORBA::Object_var tmp_obj( NULL);
  ABng_Core::Time ts;
  ts.seconds= ts.microseconds= 0;

  if (argc<2) {
    usage( *argv);
    return 1;
  }

  while (( c = getopt( argc, argv, "v:a:b:y:z:lhs:t:rxu:owq:e:c")) != -1)
    switch (c) {
    case 'v':
      try {
	tmp_obj= Bootstrap::resolve( 
		   Bootstrap::CTX_ABng_Core_Management_View_Managers()+optarg);
	view_manager= ABng_Core_Management::View_Manager::_narrow( tmp_obj);
	view= view_manager->my_view();
      }
      catch ( CORBA::Exception &ex) {
	cerr << "Cannot bind to view/view manager" << endl;
	cerr << ex << endl;
	exit( 1);
      }
      break;

    case 'b': {
      if ( CORBA::is_nil( (ABng_Core::View_ptr) view)) {
	cerr << "View object not bound (use -v option)" << endl;
	exit( 1);
      }
      try {
	tmp_obj= Bootstrap::resolve( Bootstrap::SRV_Badge_Holder_Repository());
	bh_repository= 
	  ABng_Core_Management::Badge_Holder_Rep::_narrow( tmp_obj);
      }
      catch ( CORBA::Exception &ex) {
	cerr << "Cannot bind to badge holder repository" << endl;
	cerr << ex << endl;
	exit( 1);
      }
      ABng_Core_System::Badge_Holder::Pattern bh_pattern;
      Domain_Repository::User::Pattern user_pattern;
      bh_pattern.is_any= 0;
      bh_pattern.item_id_pattern.is_any= 0;
      bh_pattern.item_id_pattern.value= atoi( optarg);
      bh_pattern.my_badge_pattern.is_any= 1;
      bh_pattern.current_badge_pattern.is_any= 1;
      bh_pattern.home_sensor_pattern.is_any= 1;
      bh_pattern.type_specific_object_pattern.is_any= 1;
      bh_pattern.type_specific_object_pattern.object_pattern.user_obj_pattern(user_pattern);
      bh_pattern.assigned_pattern.is_any= 1;
      bh_pattern.visitor_pattern.is_any= 1;

      try {
	ABng_Core_System::Badge_Holders_var ret_bh=
	  bh_repository->find( bh_pattern);
	if ( ret_bh->length() != 1) {
	  cerr << ret_bh->length() << " badge holders found" << endl;
	  exit( 1);
	}
	bh= ABng_Core_System::Badge_Holder::_duplicate( ret_bh[0]);
	view_manager->add_locatable( bh);
      }
      catch ( CORBA::Exception &ex) {
	cerr << "Caught " << ex << endl;
	exit( 1);
      }
      break;
    }

    case 'z':
      if ( CORBA::is_nil( (ABng_Core::View_ptr) view)) {
	cerr << "View object not bound (use -v option)" << endl;
	exit( 1);
      }
      try 
      {
	ABng_Core::Locatable::Pattern loc_pattern= get_locatable_pattern( atoi( optarg));

	ABng_Core::Locatables_var ret_val( view->find_locatable( loc_pattern));
	if (ret_val->length() != 1) {
	  cerr << ret_val->length() << " locatable(s) found" << endl;
	  exit( 1);
	}
	view_manager->remove_locatable( ret_val[0]);
      }
      catch (CORBA::Exception &ex) {
	cerr << "Caught exception " << ex << endl;
	exit( 1);	
      }
      break;

    case 'a': {
      if ( CORBA::is_nil( (ABng_Core::View_ptr) view)) {
	cerr << "View object not bound (use -v option)" << endl;
	exit( 1);
      }
      try {
	tmp_obj= Bootstrap::resolve( 
		    Bootstrap::SRV_ABng_Location_Description_Repository());
	abng_ld_repository= 
	  ABng_Core_Management::ABng_Location_Description_Rep::_narrow( 
								 tmp_obj);
      }
      catch ( CORBA::Exception &ex) {
	cerr << "Cannot bind to ABng location description repository" << endl;
	cerr << ex << endl;
	exit( 1);
      }
      ABng_Core_System::ABng_Location_Description::Pattern abng_ld_pattern;
      abng_ld_pattern.is_any= 0;
      abng_ld_pattern.item_id_pattern.is_any= 0;
      abng_ld_pattern.item_id_pattern.value= atoi( optarg);
      abng_ld_pattern.loc_descr_pattern.is_any= 1;
      abng_ld_pattern.loc_descr_pattern.property_list_pattern.constraints= 
	Domain_Repository::UNRESTRICTED;
      abng_ld_pattern.sensor_list_pattern.is_any= 1;
      abng_ld_pattern.sensor_list_pattern.constraints =
	Domain_Repository::UNRESTRICTED;
      try {
	ABng_Core_System::ABng_Location_Descriptions_var ret_abng_ld=
	  abng_ld_repository->find( abng_ld_pattern);
	if ( ret_abng_ld->length() != 1) {
	  cerr << ret_abng_ld->length() << " ABng location descriptions found" << endl;
	  exit( 1);
	}
	cerr << "Adding location" << endl;
	abng_ld= ABng_Core_System::ABng_Location_Description::_duplicate( 
							ret_abng_ld[0]);
	view_manager->add_location( abng_ld);
      }
      catch ( CORBA::Exception &ex) {
	cerr << "Caught " << ex << endl;
	exit( 1);
      }
      break;
    }
      
    case 'y':
      if ( CORBA::is_nil( (ABng_Core::View_ptr) view)) {
	cerr << "View object not bound (use -v option)" << endl;
	exit( 1);
      }
      try 
      {
	ABng_Core::Location::Pattern loc_pattern;
	loc_pattern.is_any= 0;
	loc_pattern.item_id_pattern.is_any= 0;
	loc_pattern.item_id_pattern.value= atoi( optarg);
	loc_pattern.loc_descr_pattern.is_any= 1;

	ABng_Core::Locations_var ret_val( view->find_location( loc_pattern));
	if (ret_val->length() != 1) {
	  cerr << ret_val->length() << " locations found" << endl;
	  exit( 1);
	}
	view_manager->remove_location( ret_val[0]);
      }
      catch (CORBA::Exception &ex) {
	cerr << "Caught exception " << ex << endl;
	exit( 1);	
      }
      break;

    case 'l':
      if ( CORBA::is_nil( (ABng_Core::View_ptr) view)) {
	cerr << "View object not bound (use -v option)" << endl;
	exit( 1);
      }
      try {
	cout << "view_name        " << view->view_name() << endl;
	cout << "type             " << view->type() << endl;
	cout << "wants_duplicates " << (int) view->wants_duplicates() << endl;
	ABng_Core::Locations *locations= view->location_list();
	cout << "Found " << locations->length() << " location(s)" << endl;
	for ( int i=0; i<locations->length(); i++) {
	  cout << "#" << i << endl;
	  try {
	    ABng_Core::Location::Value_Description_var descr=
	      (*locations)[i]->value_descr();
	    cout << " item_id      " << descr->item_id << endl;
	    for (int x=0; 
		 x<descr->loc_value_descr.property_list.length(); 
		 x++ ) {
	      cout << "\t#" << x << " " 
		   << descr->loc_value_descr.property_list[x].name 
		   << " ";
	      show_any( cout, 
			descr->loc_value_descr.property_list[x].value) << endl;
	    }
	  }
	  catch ( Domain_Repository::Destination_Unreachable &ex_du) {
	    cout << "!Destination unreachable : " << ex_du.server_name << endl;
	  }
	  catch ( CORBA::Exception &ex_1) {
	    cout << ex_1 << endl;
	  }
	}
	ABng_Core::Locatables *locatables= view->locatable_list();
	cout << "Found " << locatables->length() << " locatable(s)" << endl;
	for ( int j=0; j<locatables->length(); j++) {
	  cout << "#" << j << endl;
	  try {
	    ABng_Core::Locatable::Description_var descr=
	      (*locatables)[j]->descr();
	    cout << " item_id      " << descr->item_id << "   ";
	    switch (descr->type_specific_object._d()) {
	    case ABng_Core::LOCATABLE_USER: {
	      Domain_Repository::User::Description_var udesc=
		descr->type_specific_object.user_obj()->descr();
	      cout << (const char *) udesc->forename << " "
		   << (const char *) udesc->surname;
	    }
	    break;
	    case ABng_Core::LOCATABLE_EQUIPMENT:
	      Domain_Repository::Equipment::Description_var edesc=
		descr->type_specific_object.equipment_obj()->descr();
	      cout << (const char *) edesc->name;
	      break;
	    }
	    cout << endl;
	  }
	  catch ( Domain_Repository::Destination_Unreachable &du) {
	    cerr << "Destination_Unreachable " << du.server_name << endl;
	  }
	  catch ( CORBA::Exception &ex_1) {
	    cout << ex_1 << endl;
	  }
	}
      }
      catch ( CORBA::Exception &ex) {
	cerr << "Caught exception " << ex << endl;
	exit( 1);
      }
      break;
    case 's':
      if ( CORBA::is_nil( (ABng_Core::View_ptr) view)) {
	cerr << "View object not bound (use -v option)" << endl;
	exit( 1);
      }
      try 
      {
	{
	ABng_Core::Sighting_Descriptions_var 
	  sightings( view->get_sightings( ts));
	cout << sightings->length() << " sightings found" << endl;
	}

	ABng_Core::Location::Pattern loc_pattern;
	loc_pattern.is_any= 0;
	loc_pattern.item_id_pattern.is_any= 0;
	loc_pattern.item_id_pattern.value= atoi( optarg);
	loc_pattern.loc_descr_pattern.is_any= 1;
	loc_pattern.loc_descr_pattern.property_list_pattern.constraints=
	  Domain_Repository::UNRESTRICTED;

	ABng_Core::Locations_var ret_val( view->find_location( loc_pattern));
	if (ret_val->length() != 1) {
	  cerr << ret_val->length() << " locations found" << endl;
	  exit( 1);
	}
	ABng_Core::Sighting_Descriptions_var 
	  sightings( ret_val[0]->get_sightings( ts));
	cerr << sightings->length() << " sightings found" << endl;
	for (int i=0; i<sightings->length(); i++) {
	  cout << "#" << i << endl;
	  cout << sightings[i] << endl;
	}
      }
      catch (CORBA::Exception &ex) {
	cerr << "Caught exception " << ex << endl;
	exit( 1);	
      }
      break;
    case 'r':
      if ( CORBA::is_nil( (ABng_Core::View_ptr) view)) {
	cerr << "View object not bound (use -v option)" << endl;
	exit( 1);
      }
      try {
	ABng_Core::Locatable::Pattern loc_pattern= 
	  get_locatable_pattern( 0);
	loc_pattern.is_any= 1;

	ABng_Core::Locatables_var ret_val( view->find_locatable( loc_pattern));
	for (int i=0; i<ret_val->length(); i++) {
	  ABng_Core::Locatable::Value_Description_var val_descr= 
	    ret_val[i]->value_descr();

	  if (val_descr->type_specific_object_value._d() ==
	      ABng_Core::LOCATABLE_USER) {
	    Domain_Repository::User::Value_Description &user_descr=
	      val_descr->type_specific_object_value.user_description();

	    cout << val_descr->item_id << " "
		 << user_descr.forename << " " << user_descr.surname
		 << endl;
	  }
	}
      }
      catch (Domain_Repository::Destination_Unreachable &ex) {
	cerr << ex << " " << ex.server_name << endl;
      }
      catch (CORBA::Exception &ex) {
	cerr << "Caught exception " << ex << endl;
      }
      break;
    case 't':
      if ( CORBA::is_nil( (ABng_Core::View_ptr) view)) {
	cerr << "View object not bound (use -v option)" << endl;
	exit( 1);
      }
      try 
      {
	{
	  //	ABng_Core::Sighting_Descriptions_var 
	  //	  sightings( view->get_sightings( ts));
	  //	cerr << sightings->length() << " sightings found" << endl;
	}

	ABng_Core::Locatable::Pattern loc_pattern= get_locatable_pattern( atoi( optarg));

	ABng_Core::Locatables_var ret_val( view->find_locatable( loc_pattern));
	if (ret_val->length() != 1) {
	  cerr << ret_val->length() << " locatable(s) found" << endl;
	  exit( 1);
	}
	ABng_Core::Sighting_Description_var 
	  sighting( ret_val[0]->get_sighting());
	cout << *(sighting.operator->()) << endl;
      }
      catch (CORBA::Exception &ex) {
	cerr << "Caught exception " << ex << endl;
	exit( 1);	
      }
      break;
    case 'x':
      if ( CORBA::is_nil( (ABng_Core::View_ptr) view)) {
	cerr << "View object not bound (use -v option)" << endl;
	exit( 1);
      }
      try 
      {
	//	cerr << ts << endl;
	ABng_Core::Sighting_Descriptions_var 
	  sightings( view->get_sightings( ts));
	cerr << sightings->length() << " sightings found" << endl;
	for (int i=0; i<sightings->length(); i++) {
	  switch (sightings[i].my_locatable_type) {
	  case ABng_Core::LOCATABLE_USER:
	    cout << "#" << i << endl;
	    cout << sightings[i] << endl;
	    break;
	  }
	}
      }
      catch (CORBA::Exception &ex) {
	cerr << "Caught exception " << ex << endl;
	exit( 1);	
      }
      break;
    case 'u':
      {
	ts.seconds= ts.microseconds= 0;
	RWTime offset( RWDate( 1, 1,1970), 0,0,0, RWZone::utc());
	RWTime curr_time;
	ts.seconds= curr_time.seconds()-offset.seconds()-atoi(optarg)*3600;
	ts.microseconds= 0;
	//	cerr << ts << endl;
      }
      break;
    case 'q':
      if ( CORBA::is_nil( (ABng_Core::View_ptr) view)) {
	cerr << "View object not bound (use -v option)" << endl;
	exit( 1);
      }
      try {
	view_manager->burst_delay_time( atol(optarg));
      }
      catch (CORBA::Exception &ex) {
	cerr << "Caught exception " << ex << endl;
	exit( 1);
      }
      break;
    case 'w':
      if ( CORBA::is_nil( (ABng_Core::View_ptr) view)) {
	cerr << "View object not bound (use -v option)" << endl;
	exit( 1);
      }
      try {
	cout << (unsigned long) view_manager->burst_delay_time() << endl;
      }
      catch (CORBA::Exception &ex) {
	cerr << "Caught exception " << ex << endl;
	exit( 1);
      }
      break;
    case 'c':
      if ( CORBA::is_nil( (ABng_Core::View_ptr) view)) {
	cerr << "View object not bound (use -v option)" << endl;
	exit( 1);
      }
      try {
	CORBA::Object_var obj=
	  view_manager->sighting_channel();
	CORBA::String_var str= orb->object_to_string( obj);
	cout << (const char *) str << '#' << endl;
      }
      catch (CORBA::Exception &ex) {
	cerr << ex << endl;
	exit(1);
      }
      break;
    case 'e':
      if ( CORBA::is_nil( (ABng_Core::View_ptr) view)) {
	cerr << "View object not bound (use -v option)" << endl;
	exit( 1);
      }
      try {

	CORBA::Object_var obj= orb->string_to_object( optarg);
	CosTypedEventChannelAdmin::TypedEventChannel_var ch=
	  CosTypedEventChannelAdmin::TypedEventChannel::_narrow( obj);
	view_manager->sighting_channel( ch);
      }
      catch (CORBA::Exception &ex) {
	cerr << ex << endl;
	exit(1);
      }
      break;
    case 'o':
      if ( CORBA::is_nil( (ABng_Core::View_ptr) view)) {
	cerr << "View object not bound (use -v option)" << endl;
	exit( 1);
      }
      try {
	ABng_Core::Dynamic_Data_Observed_R::Observer_Entries_var oe;
	oe= view->location_dynamic_data_observer_list();
	cout << "Location observers: " << (unsigned) oe->length() << endl;
	{
	  for (CORBA::ULong x1= 0; x1<oe->length(); x1++) {
	    cout << '#' << (unsigned) x1 << endl;
	    CORBA::String_var str= orb->object_to_string( oe[x1].obs);
	    cout << (const char *) str << endl;
	    for (CORBA::ULong i=0; i<oe[x1].interest.length(); i++)
	      cout << (unsigned) oe[x1].interest[i] << ' ';
	    cout << endl;
	  }
	}
	oe= view->locatable_dynamic_data_observer_list();
	cout << "Locatable observers: " << (unsigned) oe->length() << endl;
	for (CORBA::ULong x1= 0; x1<oe->length(); x1++) {
	  cout << '#' << (unsigned) x1 << endl;
	  CORBA::String_var str= orb->object_to_string( oe[x1].obs);
	  cout << (const char *) str << endl;
	  for (CORBA::ULong i=0; i<oe[x1].interest.length(); i++)
	    cout << (unsigned) oe[x1].interest[i] << ' ';
	  cout << endl;
	}
      }
      catch (CORBA::Exception &ex) {
	cerr << "Caught exception " << ex << endl;
	exit( 1);	
      }
      break;
    default:
      usage( argv[0] );
      exit( 1);
      break;
    }
  exit( 0);
}


